@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -93,7 +93,7 @@ static Arg queue_args[] = {
   {"bottom",    XawChainBottom}
 };
 
-static Arg get_sizepos_args[] = {
+static Arg sizepos_args[] = {
   {"width",     (XtArgVal)NULL},
   {"height",    (XtArgVal)NULL},
   {"x",         (XtArgVal)NULL},
@@ -112,11 +112,13 @@ enum { da_hide };
 static int dialog_action;
 static int tick_stripchart_accumulator = 999999;
 static int tick_interval = 2;
-static int screenwidth, screenheight;
+static int maxposset = 0;
 static int minposset = 0;
 static int x_adjustment = -1;
 static int y_adjustment = -1;
+static Dimension screenwidth, screenheight;
 static Dimension original_x, original_y;
+static Dimension maxposx, maxposy;
 static Dimension minposx, minposy;
 static Dimension maxwidth, maxheight;
 static Widget outer_form_widget;
@@ -149,6 +151,23 @@ va_end(ap);
 }
 
 
+/*************************************************
+*            Dummy routine                       *
+*************************************************/
+
+/* Eximon calls some functions from the main exim code, and these in turn
+may call log_write in error cases. That just writes to stderr, but the log
+module calls smtp_closedown() on panic crashes, so we must provide a dummy
+function here. One of these days this should all be made much tidier, with
+less inter-dependence. */
+
+void smtp_closedown(char *message)
+{
+message = message;
+}
+
+
+
 /*************************************************
 *                SIGCHLD handler                 *
 *************************************************/
@@ -173,7 +192,7 @@ if (action_queue_update) tick_queue_accumulator = 999999;
 
 void updateAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
-scan_spool_input(TRUE, TRUE);
+scan_spool_input(TRUE);
 queue_display();
 tick_queue_accumulator = 0;
 }
@@ -200,7 +219,7 @@ queue_skip = NULL;
 XtDestroyWidget(unhide_widget);
 unhide_widget = NULL;
 
-scan_spool_input(TRUE, TRUE);
+scan_spool_input(TRUE);
 queue_display();
 tick_queue_accumulator = 0;
 }
@@ -237,19 +256,19 @@ Dimension width, height;
 XWindowAttributes a;
 Window w = XtWindow(toplevel_widget);
 
-/* Get the position of the top level widget. */
+/* Get the position and size of the top level widget. */
 
-get_sizepos_args[0].value = (XtArgVal)(&width);
-get_sizepos_args[1].value = (XtArgVal)(&height);
-get_sizepos_args[2].value = (XtArgVal)(&x);
-get_sizepos_args[3].value = (XtArgVal)(&y);
-XtGetValues(toplevel_widget, get_sizepos_args, 4);
+sizepos_args[0].value = (XtArgVal)(&width);
+sizepos_args[1].value = (XtArgVal)(&height);
+sizepos_args[2].value = (XtArgVal)(&x);
+sizepos_args[3].value = (XtArgVal)(&y);
+XtGetValues(toplevel_widget, sizepos_args, 4);
 
 /* Get the position of the widget's window relative to its parent; this
 gives the thickness of the window manager's furniture. At least it does
 in twm. For fwvm it gives zero. The size/movement function uses this data.
 I tried doing this before entering the main loop, but it didn't always
-work properly. Running it every time seems to be OK. */
+work properly with twm. Running it every time seems to be OK. */
 
 XGetWindowAttributes(X_display, XtWindow(toplevel_widget), &a);
 if (a.x != 0) x_adjustment = a.x;
@@ -261,40 +280,66 @@ where the positioning includes the furniture. */
 
 if (width == maxwidth && height == maxheight)
   {
+  maxposx = x;
+  maxposy = y;
+  maxposset = 1;
+
   if (minposset)
-    {
-    XMoveResizeWindow(X_display, w,
-      minposx - ((x_adjustment >= 0)? x_adjustment : 0),
-      minposy - ((y_adjustment >= 0)? y_adjustment : 0),
-      min_width, min_height);
-    minposset = 0;
-    }
-  else XResizeWindow(X_display, w, min_width, min_height);
+    xs_SetValues(toplevel_widget, 4,
+      "width",     min_width,
+      "height",    min_height,
+      "x",         minposx - ((x_adjustment >= 0)? x_adjustment : 0),
+      "y",         minposy - ((y_adjustment >= 0)? y_adjustment : 0));
+  else
+    xs_SetValues(toplevel_widget, 2,
+      "width",     min_width,
+      "height",    min_height);
   }
 
-/* Else always expand to maximum. If currently at minimum size, and we have to
-move it, remember where it was for coming back. If we don't have a value
-for the thickness of the furniture, the implication is that the coordinates
-position the application window, so we can't use (0,0) because that loses the
-furniture. Use screwed in values that seem to work with fvwm. */
+/* Else always expand to maximum. If currently at minimum size, remember where
+it was for coming back. If we don't have a value for the thickness of the
+furniture, the implication is that the coordinates position the application
+window, so we can't use (0,0) because that loses the furniture. Use screwed in
+values that seem to work with fvwm. */
 
 else
   {
   int xx = x;
   int yy = y;
-  if (x + maxwidth > screenwidth || y + maxheight + 10 > screenheight)
+
+  if (width == min_width && height == min_height)
     {
-    if (width == min_width && height == min_height)
-      {
-      minposx = x;
-      minposy = y;
-      minposset = 1;
-      }
-    if (x + maxwidth > screenwidth) xx = (x_adjustment >= 0)? 0 : 4;
-    if (y + maxheight + 10 > screenheight) yy = (y_adjustment >= 0)? 0 : 21;
-    XMoveResizeWindow(X_display, w, xx, yy, maxwidth, maxheight);
+    minposx = x;
+    minposy = y;
+    minposset = 1;
     }
-  else XResizeWindow(X_display, w, maxwidth, maxheight);
+
+  if ((int)(x + maxwidth) > (int)screenwidth ||
+      (int)(y + maxheight + 10) > (int)screenheight)
+    {
+    if (maxposset)
+      {
+      xx = maxposx - ((x_adjustment >= 0)? x_adjustment : 0);
+      yy = maxposy - ((y_adjustment >= 0)? y_adjustment : 0);
+      }
+    else
+      {
+      if ((int)(x + maxwidth) > (int)screenwidth)
+        xx = (x_adjustment >= 0)? 0 : 4;
+      if ((int)(y + maxheight + 10) > (int)screenheight)
+        yy = (y_adjustment >= 0)? 0 : 21;
+      }
+
+    xs_SetValues(toplevel_widget, 4,
+      "width",     maxwidth,
+      "height",    maxheight,
+      "x",         xx,
+      "y",         yy);
+    }
+
+  else xs_SetValues(toplevel_widget, 2,
+        "width",     maxwidth,
+        "height",    maxheight);
   }
 
 /* Ensure the window is at the top */
@@ -337,7 +382,7 @@ switch(dialog_action)
   queue_skip = sk;
   strcpy(sk->text, s);
   sk->reveal = time(NULL) + 60 * 60;
-  scan_spool_input(TRUE, TRUE);
+  scan_spool_input(TRUE);
   queue_display();
   tick_queue_accumulator = 0;
   break;
@@ -346,31 +391,6 @@ switch(dialog_action)
 
 
 
-/*************************************************
-*              Regexp error handler              *
-*************************************************/
-
-/* The regexp routines require an error handler. The function
-doesn't identify where the error happened, but since we
-compile all the expressions at the start, we can detect whether
-it's a compilation error or a runtime error. Give a message for
-the former and collapse, but ignore the latter. We maintain a
-pointer to regexps in the process of compilation so as to be
-able to say which one it was that failed. */
-
-void regerror(char *s)
-{
-if (!eximon_initialized)
-  {
-  printf("*** Error in regular expression: %s\n", s);
-  if (regexp_compiling != NULL)
-    printf("*** Failed expression: %s\n", regexp_compiling);
-  exit(99);
-  }
-}
-
-
-
 /*************************************************
 *              Ticker function                   *
 *************************************************/
@@ -395,7 +415,7 @@ so just in case, clean up any completed children here. */
 
 if (tick_queue_accumulator >= queue_update)
   {
-  scan_spool_input(TRUE, FALSE);
+  scan_spool_input(TRUE);
   queue_display();
   tick_queue_accumulator = 0;
   if (tick_stripchart_accumulator >= stripchart_update)
@@ -409,7 +429,7 @@ the stripchart. */
 
 else if (tick_stripchart_accumulator >= stripchart_update)
   {
-  scan_spool_input(FALSE, FALSE);
+  scan_spool_input(FALSE);
   tick_stripchart_accumulator = 0;
   }
 
@@ -448,15 +468,6 @@ XtAppAddTimeOut(X_appcon, tick_interval * 1000, ticker, 0);
 
 
 
-void action_hook(Widget w, XtPointer client_data, String name,
-  XEvent *event, String *string, Cardinal *n)
-{
-printf("action hook, name = %s Widget=%d queue_widget=%d\n",
-  name, (int)w, (int)queue_widget);
-}
-
-
-
 
 
 /*************************************************
@@ -477,11 +488,13 @@ Widget stripchart_form_widget,
 
 message_id_external = message_id_option + 1;
 message_id = message_id_external + 1;
+message_subdir[1] = 0;
 
 /* Some store needs getting for big_buffer, which is used for
 constructing file names and things. */
 
-big_buffer = (char *)malloc(1024);
+big_buffer_size = 1024;
+big_buffer = (char *)malloc(big_buffer_size);
 
 /* Set up the version string and date and output them */
 
@@ -610,7 +623,7 @@ xs_SetValues(stripchart_form_widget, 5,
 
 /* Create the queue count stripchart and its label. */
 
-create_stripchart(stripchart_form_widget, "queue");
+create_stripchart(stripchart_form_widget, queue_stripchart_name);
 
 /* If configured, create the size monitoring stripchart, but
 only if the OS supports statfs(). */
@@ -618,10 +631,15 @@ only if the OS supports statfs(). */
 if (size_stripchart != NULL)
   {
 #ifdef HAVE_STATFS
-  char *name = size_stripchart + (int)strlen(size_stripchart) - 1;
-  while (name > size_stripchart && *name == '/') name--;
-  while (name > size_stripchart && *name != '/') name--;
-  create_stripchart(stripchart_form_widget, name);
+  if (size_stripchart_name == NULL)
+    {
+    size_stripchart_name = size_stripchart + (int)strlen(size_stripchart) - 1;
+    while (size_stripchart_name > size_stripchart &&
+      *size_stripchart_name == '/') size_stripchart_name--;
+    while (size_stripchart_name > size_stripchart &&
+      *size_stripchart_name != '/') size_stripchart_name--;
+    }
+  create_stripchart(stripchart_form_widget, size_stripchart_name);
 #else
   printf("Can't create size stripchart: statfs() function not available\n");
 #endif
@@ -719,11 +737,11 @@ XtRealizeWidget(toplevel_widget);
 /* Find out the size of the initial window, and set that as its
 maximum. While we are at it, get the initial position. */
 
-get_sizepos_args[0].value = (XtArgVal)(&maxwidth);
-get_sizepos_args[1].value = (XtArgVal)(&maxheight);
-get_sizepos_args[2].value = (XtArgVal)(&original_x);
-get_sizepos_args[3].value = (XtArgVal)(&original_y);
-XtGetValues(toplevel_widget, get_sizepos_args, 4);
+sizepos_args[0].value = (XtArgVal)(&maxwidth);
+sizepos_args[1].value = (XtArgVal)(&maxheight);
+sizepos_args[2].value = (XtArgVal)(&original_x);
+sizepos_args[3].value = (XtArgVal)(&original_y);
+XtGetValues(toplevel_widget, sizepos_args, 4);
 
 xs_SetValues(toplevel_widget, 2,
   "maxWidth",  maxwidth,
