@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -117,9 +117,10 @@ static Arg item_14_arg[] = {
 
 static void popdownAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
-XawTextSinkDisplayText(queue_text_sink,
-  highlighted_x, highlighted_y,
-  highlighted_start, highlighted_end, 0);
+if (highlighted_x >= 0)
+  XawTextSinkDisplayText(queue_text_sink,
+    highlighted_x, highlighted_y,
+    highlighted_start, highlighted_end, 0);
 XtDestroyWidget(w);
 menu_is_up = FALSE;
 }
@@ -132,12 +133,22 @@ menu_is_up = FALSE;
 
 static void msglogAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+int i;
 char buffer[256];
-Widget text = text_create((char *)client_data, 200);
-FILE *f;
-sprintf(buffer, "%s/msglog/%s", spool_directory, (char *)client_data);
+Widget text = text_create((char *)client_data, text_depth);
+FILE *f = NULL;
+
+/* End up with the split version, so message looks right when non-exist */
+
+for (i = 0; i < (spool_is_split? 2:1); i++)
+  {
+  message_subdir[0] = (i != 0)? ((char *)client_data)[5] : 0;
+  sprintf(buffer, "%s/msglog/%s/%s", spool_directory, message_subdir,
+    (char *)client_data);
+  f = fopen(buffer, "r");
+  if (f != NULL) break;
+  }
 
-f = fopen(buffer, "r");
 if (f == NULL)
   text_showf(text, "%s: %s\n", buffer, strerror(errno));
 else
@@ -155,12 +166,20 @@ else
 
 static void bodyAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
+int i;
 char buffer[256];
-Widget text = text_create((char *)client_data, 200);
-FILE *f;
-sprintf(buffer, "%s/input/%s-D", spool_directory, (char *)client_data);
+Widget text = text_create((char *)client_data, text_depth);
+FILE *f = NULL;
+
+for (i = 0; i < (spool_is_split? 2:1); i++)
+  {
+  message_subdir[0] = (i != 0)? ((char *)client_data)[5] : 0;
+  sprintf(buffer, "%s/input/%s/%s-D", spool_directory, message_subdir,
+    (char *)client_data);
+  f = fopen(buffer, "r");
+  if (f != NULL) break;
+  }
 
-f = fopen(buffer, "r");
 if (f == NULL)
   text_showf(text, "Failed to open file: %s\n", strerror(errno));
 else
@@ -213,8 +232,11 @@ if (editing)
   {
   if ((pid = fork()) == 0)
     {
-    sprintf(buffer, "xterm -n %s -title %s -e %s %s %s", id, id,
-      exim_path, action, id);
+    sprintf(buffer, "xterm -n %s -title %s -e %s %s %s %s %s", id, id,
+      exim_path,
+      (alternate_config == NULL)? "" : "-C",
+      (alternate_config == NULL)? "" : alternate_config,
+      action, id);
     system(buffer);
     _exit(0);
     }
@@ -235,14 +257,16 @@ if (address_arg[0] != 0)
     qualify = qualify_domain;
     }
   }
-sprintf(buffer, "%s %s %s %s%s%s%s%s", exim_path, action, id,
-  quote, address_arg, at, qualify, quote);
+sprintf(buffer, "%s %s %s %s %s %s%s%s%s%s", exim_path,
+  (alternate_config == NULL)? "" : "-C",
+  (alternate_config == NULL)? "" : alternate_config,
+  action, id, quote, address_arg, at, qualify, quote);
 
 /* If we know we are going to need the window, create it now. */
 
 if (action_output || delivery)
   {
-  text = text_create(id, 200);
+  text = text_create(id, text_depth);
   text_showf(text, "%s\n", buffer);
   }
 
@@ -254,7 +278,7 @@ if (pipe(pipe_fd) != 0)
   {
   if (text == NULL)
     {
-    text = text_create(id, 200);
+    text = text_create(id, text_depth);
     text_showf(text, "%s\n", buffer);
     }
   text_show(text, "*** Failed to create pipe ***\n");
@@ -292,7 +316,7 @@ if (!delivery)
     {
     if (text == NULL)
       {
-      text = text_create(id, 200);
+      text = text_create(id, text_depth);
       text_showf(text, "%s\n", buffer);
       }
     while ((count = read(pipe_fd[0], buffer, 254)) > 0)
@@ -314,7 +338,7 @@ if (!delivery)
 
   if (rc == 0 && strcmp(action + (int)strlen(action) - 4, "-Mes") == 0)
     {
-    queue_item *q = find_queue(id, queue_noop);
+    queue_item *q = find_queue(id, queue_noop, 0);
     if (q != NULL) q->sender = string_copy(address_arg);
     }
 
@@ -329,7 +353,7 @@ because nothing will get written to the log to show that this has
 happened. (Other freezing/unfreezings get logged and picked up from
 there.) */
 
-qq = find_queue(id, queue_noop);
+qq = find_queue(id, queue_noop, 0);
 if (qq != NULL) qq->frozen = FALSE;
 
 /* New, asynchronous code runs in a subprocess for commands that
@@ -561,7 +585,7 @@ static void editsenderAction(Widget w, XtPointer client_data,
 queue_item *q;
 char *sender;
 strcpy(actioned_message, (char *)client_data);
-q = find_queue(actioned_message, queue_noop);
+q = find_queue(actioned_message, queue_noop, 0);
 sender = (q == NULL)? "" : q->sender;
 action_required = "-Mes";
 dialog_ref_widget = menushell;
@@ -611,10 +635,10 @@ static void headersAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
 char buffer[256];
 header_line *h, *next;
-Widget text = text_create((char *)client_data, 500);
+Widget text = text_create((char *)client_data, text_depth);
 
 sprintf(buffer, "%s-H", (char *)client_data);
-if (spool_read_header(buffer, TRUE) != spool_read_OK)
+if (spool_read_header(buffer, TRUE, FALSE) != spool_read_OK)
   {
   if (errno == ERRNO_SPOOLFORMAT)
     {
@@ -764,12 +788,17 @@ void menu_create(Widget w, XEvent *event, String *actargs, Cardinal *count)
 {
 int line;
 int i;
+char *s;
 XawTextPosition p;
-XtTranslations menu_trans;
 Widget src, menu_line, item_1, item_2, item_3, item_4,
   item_5, item_6, item_7, item_8, item_9, item_10, item_11,
   item_12, item_13, item_14;
-char *s;
+XtTranslations menu_trans = XtParseTranslationTable(
+  "<EnterWindow>:   highlight()\n\
+   <LeaveWindow>:   unhighlight()\n\
+   <BtnMotion>:     highlight()\n\
+   <BtnUp>:         MenuPopdown()notify()unhighlight()\n\
+  ");
 
 /* Get the sink and source and the current text pointer */
 
@@ -804,6 +833,10 @@ if (s[p] == 0)
   menushell = XtCreatePopupShell("menu", simpleMenuWidgetClass,
     queue_widget, menushell_arg, XtNumber(menushell_arg));
   XtAddCallback(menushell, "popdownCallback", popdownAction, NULL);
+  xs_SetValues(menushell, 2,
+    "cursor",       XCreateFontCursor(X_display, XC_arrow),
+    "translations", menu_trans);
+  highlighted_x = -1;
   return;
   }
 
@@ -839,13 +872,6 @@ menushell = XtCreatePopupShell("menu", simpleMenuWidgetClass,
   queue_widget, menushell_arg, XtNumber(menushell_arg));
 XtAddCallback(menushell, "popdownCallback", popdownAction, NULL);
 
-menu_trans = XtParseTranslationTable(
-  "<EnterWindow>:   highlight()\n\
-   <LeaveWindow>:   unhighlight()\n\
-   <BtnMotion>:     highlight()\n\
-   <BtnUp>:         MenuPopdown()notify()unhighlight()\n\
-  ");
-
 xs_SetValues(menushell, 2,
   "cursor",       XCreateFontCursor(X_display, XC_arrow),
   "translations", menu_trans);
