@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -17,7 +17,6 @@ queue and displaying the data therefrom. */
 *                 Static variables               *
 *************************************************/
 
-static int input_mtime = 0;   /* mtime for input directory */
 static int queue_total = 0;   /* number of items in queue */
 
 /* Table for turning base-62 numbers into binary */
@@ -102,7 +101,7 @@ free(p);
 *             Set up new queue item              *
 *************************************************/
 
-static queue_item *set_up(char *name)
+static queue_item *set_up(char *name, int dir_char)
 {
 int i, rc, save_errno;
 struct stat statdata;
@@ -117,6 +116,7 @@ q->destinations = NULL;
 strcpy(q->name, name);
 q->seen = TRUE;
 q->frozen = FALSE;
+q->dir_char = dir_char;
 q->sender = NULL;
 q->size = 0;
 
@@ -127,8 +127,9 @@ the recipients fields to be initialized. If there's a format error
 in the headers, we can still display info from the envelope. */
 
 message_size = 0;
+message_subdir[0] = dir_char;
 sprintf(buffer, "%s-H", name);
-rc =  spool_read_header(buffer, FALSE);
+rc =  spool_read_header(buffer, FALSE, TRUE);
 save_errno = errno;
 
 /* If we failed to read the envelope, compute the input time by
@@ -185,10 +186,18 @@ if (rc != spool_read_OK)
 /* Now set up the remaining data */
 
 q->frozen = deliver_freeze;
-q->sender = sender_address;
+
+if (user_null_sender && sender_address != NULL && sender_address[0] != 0)
+  {
+  q->sender = malloc((int)strlen(sender_address) + 6);
+  sprintf(q->sender, "<> (%s)", sender_address);
+  free(sender_address);
+  }
+else q->sender = sender_address;
+
 sender_address = NULL;
 
-sprintf(buffer, "%s/input/%s-D", spool_directory, name);
+sprintf(buffer, "%s/input/%s/%s-D", spool_directory, message_subdir, name);
 if (stat(buffer, &statdata) == 0) q->size = message_size + statdata.st_size;
 
 /* Scan and process the recipients list, skipping any that have already
@@ -267,7 +276,7 @@ while (p != NULL)
 
 
 
-queue_item *find_queue(char *name, int action)
+queue_item *find_queue(char *name, int action, int dir_char)
 {
 int first = 0;
 int last = queue_index_size - 1;
@@ -279,7 +288,7 @@ queue_item *p, *q, *qq;
 if (queue_total == 0)
   {
   if (action != queue_add) return NULL;
-  if ((qq = set_up(name)) != NULL)
+  if ((qq = set_up(name, dir_char)) != NULL)
     {
     int i;
     for (i = 0; i < queue_index_size; i++) queue_index[i] = qq;
@@ -295,7 +304,7 @@ as special cases. */
 if (strcmp(name, (queue_index[0])->name) < 0)
   {
   if (action != queue_add) return NULL;
-  if ((qq = set_up(name)) != NULL)
+  if ((qq = set_up(name, dir_char)) != NULL)
     {
     qq->next = queue_index[0];
     (queue_index[0])->prev = qq;
@@ -309,7 +318,7 @@ if (strcmp(name, (queue_index[0])->name) < 0)
 if (strcmp(name, (queue_index[queue_index_size-1])->name) > 0)
   {
   if (action != queue_add) return NULL;
-  if ((qq = set_up(name)) != NULL)
+  if ((qq = set_up(name, dir_char)) != NULL)
     {
     qq->prev = queue_index[queue_index_size-1];
     (queue_index[queue_index_size-1])->next = qq;
@@ -358,7 +367,7 @@ for (;;)
     {
     if (action == queue_add)
       {
-      if ((qq = set_up(name)) != NULL)
+      if ((qq = set_up(name, dir_char)) != NULL)
         {
         qq->next = p;
         qq->prev = p->prev;
@@ -390,40 +399,66 @@ for (;;)
 *        Scan the exim spool directory           *
 *************************************************/
 
-/* The scan happens only if the directory has been changed, unless forced. We
-count the entries to set the value for the queue stripchart, and set up data
-for the queue display window if the "full" option is given. */
+/* If we discover that there are subdirectories, set a flag so that the menu
+code knows to look for them. We count the entries to set the value for the
+queue stripchart, and set up data for the queue display window if the "full"
+option is given. */
 
-void scan_spool_input(int full, int force)
+void scan_spool_input(int full)
 {
-struct stat statdata;
+int i;
+int subptr;
+int subdir_max = 1;
+int count = 0;
+int indexptr = 1;
+queue_item *p;
+struct dirent *ent;
+DIR *dd;
 char input_dir[256];
+char subdirs[64];
+
+subdirs[0] = 0;
+stripchart_total[0] = 0;
 
 sprintf(input_dir, "%s/input", spool_directory);
-if (stat(input_dir, &statdata)) return;
+subptr = (int)strlen(input_dir);
+input_dir[subptr+2] = 0;               /* terminator for lengthened name */
 
-if (force || input_mtime != statdata.st_mtime)
+/* Loop for each spool file on the queue - searching any subdirectories that
+may exist. When initializing eximon, every file will have to be read. To show
+there is progress, output a dot for each one to the standard output. */
+
+for (i = 0; i < subdir_max; i++)
   {
-  queue_item *p;
-  int count = 0;
-  int indexptr = 1;
-  struct dirent *ent;
-  DIR *dd = opendir(input_dir);
-  if (dd == NULL) return;
+  int subdirchar = subdirs[i];      /* 0 for main directory */
+  if (subdirchar != 0)
+    {
+    input_dir[subptr] = '/';
+    input_dir[subptr+1] = subdirchar;
+    }
 
-  stripchart_total[0] = 0;
-
-  /* Loop for each spool file on the queue - there can be temporary
-  files with non-standard names that must also be ignored. Do a
-  quick check pro tem. When initializing eximon, every file will
-  have to be read. To show there is progress, output a dot for each
-  one to the standard output. If not doing a full queue scan (but
-  just counting) don't waste time searching the in-store queue. */
+  dd = opendir(input_dir);
+  if (dd == NULL) continue;
 
   while ((ent = readdir(dd)) != NULL)
     {
     char *name = ent->d_name;
-    if ((int)strlen(name) == SPOOL_NAME_LENGTH &&
+    int len = (int)strlen(name);
+
+    /* If we find a single alphameric sub-directory on the first
+    pass, add it to the list for subsequent scans, and remember that
+    we are dealing with a split directory. */
+
+    if (i == 0 && len == 1 && isalnum(*name))
+      {
+      subdirs[subdir_max++] = *name;
+      spool_is_split = TRUE;
+      continue;
+      }
+
+    /* Otherwise, if it is a header spool file, add it to the list */
+
+    if (len == SPOOL_NAME_LENGTH &&
         name[SPOOL_NAME_LENGTH - 2] == '-' &&
         name[SPOOL_NAME_LENGTH - 1] == 'H')
       {
@@ -432,64 +467,61 @@ if (force || input_mtime != statdata.st_mtime)
       if (!eximon_initialized) { printf("."); fflush(stdout); }
       strcpy(basename, name);
       basename[SPOOL_NAME_LENGTH - 2] = 0;
-      if (full) find_queue(basename, queue_add);
+      if (full) find_queue(basename, queue_add, subdirchar);
       }
     }
   closedir(dd);
+  }
 
-  /* If simply counting the number, we are done; same if there are no
-  items in the in-store queue. */
+/* If simply counting the number, we are done; same if there are no
+items in the in-store queue. */
 
-  if (!full || queue_total == 0) return;
+if (!full || queue_total == 0) return;
 
-  /* Now scan the queue and remove any items that were not
-  in the directory. At the same time, set up the index pointers
-  into the queue. Because we are removing items, the total that
-  we are comparing against isn't actually correct, but in a long
-  queue it won't make much difference, and in a short queue it
-  doesn't matter anyway!*/
+/* Now scan the queue and remove any items that were not in the directory. At
+the same time, set up the index pointers into the queue. Because we are
+removing items, the total that we are comparing against isn't actually correct,
+but in a long queue it won't make much difference, and in a short queue it
+doesn't matter anyway!*/
 
-  p = queue_index[0];
-  while (p != NULL)
+p = queue_index[0];
+while (p != NULL)
+  {
+  if (!p->seen)
     {
-    if (!p->seen)
+    queue_item *next = p->next;
+    if (p->prev == NULL) queue_index[0] = next;
+      else p->prev->next = next;
+    if (next == NULL)
       {
-      queue_item *next = p->next;
-      if (p->prev == NULL) queue_index[0] = next;
-        else p->prev->next = next;
-      if (next == NULL)
-        {
-        int i;
-        queue_item *q = queue_index[queue_index_size-1];
-        for (i = queue_index_size - 1; i >= 0; i--)
-          if (queue_index[i] == q) queue_index[i] = p->prev;
-        }
-      else next->prev = p->prev;
-      clean_up(p);
-      queue_total--;
-      p = next;
-      }
-    else
-      {
-      if (++count > (queue_total * indexptr)/(queue_index_size-1))
-        {
-        queue_index[indexptr++] = p;
-        }
-      p->seen = FALSE;  /* for next time */
-      p = p->next;
+      int i;
+      queue_item *q = queue_index[queue_index_size-1];
+      for (i = queue_index_size - 1; i >= 0; i--)
+        if (queue_index[i] == q) queue_index[i] = p->prev;
       }
+    else next->prev = p->prev;
+    clean_up(p);
+    queue_total--;
+    p = next;
     }
-
-  /* If a lot of messages have been removed at the bottom, we may not
-  have got the index all filled in yet. Make sure all the pointers
-  are legal. */
-
-  while (indexptr < queue_index_size - 1)
+  else
     {
-    queue_index[indexptr++] = queue_index[queue_index_size-1];
+    if (++count > (queue_total * indexptr)/(queue_index_size-1))
+      {
+      queue_index[indexptr++] = p;
+      }
+    p->seen = FALSE;  /* for next time */
+    p = p->next;
     }
+  }
 
-  input_mtime = statdata.st_mtime;
+/* If a lot of messages have been removed at the bottom, we may not
+have got the index all filled in yet. Make sure all the pointers
+are legal. */
+
+while (indexptr < queue_index_size - 1)
+  {
+  queue_index[indexptr++] = queue_index[queue_index_size-1];
   }
 }
 
@@ -500,20 +532,60 @@ if (force || input_mtime != statdata.st_mtime)
 *    Update the recipients list for a message    *
 *************************************************/
 
-/* We read it only if its update time differs from last time. */
+/* We read the spool file only if its update time differs from last time,
+or if there is a journal file in existence. */
+
+/* First, a local subroutine to scan the non-recipients tree and
+remove any of them from the address list */
+
+static void
+scan_tree(queue_item *p, tree_node *tn)
+{
+if (tn != NULL)
+  {
+  if (tn->left != NULL) scan_tree(p, tn->left);
+  if (tn->right != NULL) scan_tree(p, tn->right);
+  (void)find_dest(p, tn->name, dest_remove);
+  }
+}
+
+/* The main function */
 
 static void update_recipients(queue_item *p)
 {
 int i;
-char buffer[1024];
+FILE *jread;
 struct stat statdata;
+char buffer[1024];
 
-sprintf(buffer, "%s/input/%s-H", spool_directory, p->name);
-if (stat(buffer, &statdata) < 0 || p->update_time == statdata.st_mtime)
-  return;
+message_subdir[0] = p->dir_char;
+
+sprintf(buffer, "%s/input/%s/%s-J", spool_directory, message_subdir, p->name);
+jread = fopen(buffer, "r");
+if (jread == NULL)
+  {
+  sprintf(buffer, "%s/input/%s/%s-H", spool_directory, message_subdir, p->name);
+  if (stat(buffer, &statdata) < 0 || p->update_time == statdata.st_mtime)
+    return;
+  }
+
+/* Get the contents of the header file; if any problem, just give up. */
 
 sprintf(buffer, "%s-H", p->name);
-if (spool_read_header(buffer, FALSE) != spool_read_OK) return;
+if (spool_read_header(buffer, FALSE, TRUE) != spool_read_OK) return;
+
+/* If there's a journal file, add its contents to the non-recipients tree */
+
+if (jread != NULL)
+  {
+  while (fgets(big_buffer, big_buffer_size, jread) != NULL)
+    {
+    int n = (int)strlen(big_buffer);
+    big_buffer[n-1] = 0;
+    tree_add_nonrecipient(big_buffer+1, big_buffer[0] == 'Y');
+    }
+  fclose(jread);
+  }
 
 /* Scan and process the recipients list, removing any that have already
 been delivered, and removing visible names. In the nonrecipients tree,
@@ -548,6 +620,13 @@ if (recipients_list != NULL)
   free(recipients_list);
   }
 
+/* We also need to scan the tree of non-recipients, which might
+contain child addresses that are not in the recipients list, but
+which may have got onto the address list as a result of eximon
+noticing an == line in the log. Then free the tree, remember the
+update time, and we are done. */
+
+scan_tree(p, tree_nonrecipients);
 tree_free(tree_nonrecipients);
 p->update_time = statdata.st_mtime;
 }
@@ -645,7 +724,7 @@ while (p != NULL)
     text_showf(queue_widget, "%c%2d%c %s %s %-8s %s%s%s",
       (p->frozen)? '*' : ' ',
       t, u,
-      string_formatsize(p->size, big_buffer),
+      string_format_size(p->size, big_buffer),
       p->name,
       (p->sender == NULL)? "       " :
         (p->sender[0] == 0)? "<>     " : p->sender,
