@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include <stdio.h>
@@ -10,10 +10,11 @@
 #include <string.h>
 #include <time.h>
 
-#include "../regexp/regexp.h"
+#include "pcre/pcre.h"
 #include <sys/types.h>
 
 #include "macros.h"
+#include "config.h"
 #include "structs.h"
 #include "globals.h"
 
@@ -23,7 +24,7 @@ void version_init(void)
 int i = 0;
 char today[20];
 
-version_string = "1.26";
+version_string = "1.35";
 
 strcpy(today, __DATE__);
 if (today[4] == ' ') i = 1;
