@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with running Exim as a daemon */
@@ -11,22 +11,14 @@
 #include "exim.h"
 
 
-/* Still messing around with names in IPv6. The latest API draft uses
-IN6ADDR_ANY_INIT, but the Sun implementation hasn't caught up yet.
-This macro papers over things pro tem. */
-
-#if HAVE_IPV6 && !defined IN6ADDR_ANY_INIT && defined IPV6ADDR_ANY_INIT
-#define IN6ADDR_ANY_INIT IPV6ADDR_ANY_INIT
-#endif
-
 
 /*************************************************
 *          Host and net setup tables             *
 *************************************************/
 
-/* Setup work for lists of hosts and nets is done once at the start of
-the daemon, to save doing it for every call. These list the options that
-are so processed. */
+/* Setup work for lists of hosts, nets and addresses is done once at the start
+of the daemon, to save doing it for every call. These list the options that are
+so processed. */
 
 typedef struct {
   char **text;
@@ -38,15 +30,23 @@ typedef struct {
   ip_net_item **data;
 } net_setup;
 
+typedef struct {
+  char **text;
+  re_block **data;
+} address_setup;
+
 host_setup host_setup_list[] = {
+  { &helo_verify_except_hosts,   &helo_verify_except_hostlist },
   { &sender_verify_except_hosts, &sender_verify_except_hostlist },
   { &rfc1413_except_hosts,       &rfc1413_except_hostlist },
   { &sender_host_accept,         &sender_host_accept_hosts },
   { &sender_host_accept_relay,   &sender_host_accept_relay_hosts },
   { &sender_host_reject,         &sender_host_reject_hosts },
+  { &sender_host_reject_except,  &sender_host_reject_except_hosts },
   { &sender_host_reject_recipients, &sender_host_reject_recipients_hosts },
   { &sender_host_reject_relay,   &sender_host_reject_relay_hosts },
   { &smtp_etrn_hosts,            &smtp_etrn_hostlist },
+  { &smtp_expn_hosts,            &smtp_expn_hostlist },
   { &smtp_reserve_hosts,         &smtp_reserve_hostlist },
   { &receiver_unqualified_hosts, &receiver_unqualified_hostlist },
   { &receiver_verify_except_hosts, &receiver_verify_except_hostlist },
@@ -55,12 +55,15 @@ host_setup host_setup_list[] = {
 };
 
 net_setup net_setup_list[] = {
-  { &helo_verify_nets,           &helo_verify_netlist },
+  { &helo_verify_except_nets,    &helo_verify_except_netlist },
+  { &host_lookup_nets,           &host_lookup_netlist },
+  { &rbl_except_nets,            &rbl_except_netlist },
   { &sender_verify_except_nets,  &sender_verify_except_netlist },
   { &rfc1413_except_nets,        &rfc1413_except_netlist },
   { &sender_net_accept,          &sender_net_accept_nets },
   { &sender_net_accept_relay,    &sender_net_accept_relay_nets },
   { &sender_net_reject,          &sender_net_reject_nets },
+  { &sender_net_reject_except,   &sender_net_reject_except_nets },
   { &sender_net_reject_recipients, &sender_net_reject_recipients_nets },
   { &sender_net_reject_relay,    &sender_net_reject_relay_nets },
   { &smtp_etrn_nets,             &smtp_etrn_netlist },
@@ -71,13 +74,26 @@ net_setup net_setup_list[] = {
   { NULL, NULL}
 };
 
+address_setup address_setup_list[] = {
+  { &queue_remote_except,        &re_queue_remote_except },
+  { &receiver_verify_addresses,  &re_receiver_verify_addresses },
+  { &receiver_verify_senders,    &re_receiver_verify_senders },
+  { &receiver_verify_senders_except, &re_receiver_verify_senders_except },
+  { &recipients_reject_except,   &re_recipients_reject_except },
+  { &sender_accept,              &re_sender_accept },
+  { &sender_accept_recipients,   &re_sender_accept_recipients },
+  { &sender_reject,              &re_sender_reject },
+  { &sender_reject_recipients,   &re_sender_reject_recipients },
+  { NULL, NULL}
+};
+
 
 
 /*************************************************
 *               Local static variables           *
 *************************************************/
 
-static volatile BOOL sigalrm_seen;
+static volatile BOOL daemon_sigalrm_seen;
 static volatile BOOL sigchld_seen;
 static volatile BOOL sighup_seen;
 
@@ -106,10 +122,10 @@ Returns:  nothing
 */
 
 static void
-sigalrm_handler(int sig)
+daemon_sigalrm_handler(int sig)
 {
-sigalrm_seen = TRUE;
-signal(SIGALRM, sigalrm_handler);
+daemon_sigalrm_seen = TRUE;
+signal(SIGALRM, daemon_sigalrm_handler);
 }
 
 
@@ -178,35 +194,28 @@ static void
 handle_smtp_call(int listen_socket, int accept_socket,
   struct sockaddr *accepted)
 {
-FILE *in = NULL;
-FILE *out = fdopen(accept_socket, "w");
 pid_t pid;
 int dup_accept_socket = dup(accept_socket);
 
 /* Make the address available in ASCII representation. */
 
 sender_host_address = host_ntoa(-1, accepted, NULL);
+DEBUG(1) debug_printf("Connection request from %s\n", sender_host_address);
 
-/* Set up a default sender_fullhost, for cases when HELO is not present. */
-
-sender_fullhost = string_sprintf("[%s]", sender_host_address);
-
-DEBUG(1) debug_printf("Connection request from %s\n", sender_fullhost);
-
-/* Check that duplication of the socket worked, and if so, make
-an input file. */
+/* Set up the file for writing replies, then check that duplication of the
+socket worked, and if so, make an input file. */
 
+smtp_out = fdopen(accept_socket, "w");
 if (dup_accept_socket < 0)
   {
   DEBUG(1) debug_printf("Couldn't dup socket descriptor\n");
-  DEBUG(1) debug_printf("421 %s: Connection refused: %s\n", primary_hostname,
-    strerror(errno));
-  fprintf(out, "421 %s: Connection refused: %s\r\n", primary_hostname,
-    strerror(errno));
+  DEBUG(1) debug_printf("421 %s: Connection setup failed: %s\n",
+    primary_hostname, strerror(errno));
+  fprintf(smtp_out, "421 %s: Connection setup failed: %s\r\n",
+    primary_hostname, strerror(errno));
   goto ERROR_RETURN;
   }
-
-in = fdopen(dup_accept_socket, "r");
+smtp_in = fdopen(dup_accept_socket, "r");
 
 /* Check maximum number of connections. We do not check for reserved
 connections or unacceptable hosts here. That is done in the subprocess because
@@ -218,10 +227,10 @@ if (smtp_accept_max > 0 && smtp_accept_count >= smtp_accept_max)
     smtp_accept_count, smtp_accept_max);
   DEBUG(1) debug_printf("421 %s: Too many concurrent SMTP connections; "
     "please try again later.\n", primary_hostname);
-  fprintf(out, "421 %s: Too many concurrent SMTP connections; "
+  fprintf(smtp_out, "421 %s: Too many concurrent SMTP connections; "
     "please try again later.\r\n", primary_hostname);
   log_write(4, LOG_MAIN, "Connection from %s refused: too many connections",
-    sender_fullhost);
+    sender_host_address);
   goto ERROR_RETURN;
   }
 
@@ -240,10 +249,10 @@ if (smtp_load_reserve >= 0)
       (double)load_average/1000.0);
     DEBUG(1) debug_printf("421 %s: Too much load; please try again later.\n",
       primary_hostname);
-    fprintf(out, "421 %s: Too much load; please try again later.\r\n",
+    fprintf(smtp_out, "421 %s: Too much load; please try again later.\r\n",
       primary_hostname);
     log_write(4, LOG_MAIN, "Connection from %s refused: load average = %.2f",
-      sender_fullhost, (double)load_average/1000.0);
+      sender_host_address, (double)load_average/1000.0);
     goto ERROR_RETURN;
     }
   }
@@ -271,17 +280,15 @@ if (pid == 0)
 
   /* Attempt to get an id from the sending machine via the RFC 1413
   protocol. We do this in the sub-process in order not to hold up the
-  main process if there is any delay. */
+  main process if there is any delay. Then set up the fullhost information
+  in case there is no HELO/EHLO. */
 
   verify_get_ident(accept_socket);
+  host_build_sender_fullhost();
 
   DEBUG(1)
-    {
-    debug_printf("Process %d is handling incoming connection from [%s]",
-      getpid(), sender_host_address);
-    if (sender_ident != NULL) debug_printf(" (%s)\n", sender_ident);
-      else debug_printf("\n");
-    }
+    debug_printf("Process %d is handling incoming connection from %s\n",
+      getpid(), sender_fullhost);
 
   /* If there are too many child processes for immediate delivery,
   set the local_queue_only flag, which is initialized from the
@@ -300,18 +307,14 @@ if (pid == 0)
   A break in the connection causes the process to die (see accept.c). */
 
   sender_local = sender_host_unknown = FALSE;
-  smtp_input = TRUE;
 
   for (;;)
     {
     int rc;
     message_id[0] = 0;   /* Clear out any previous message_id */
 
-    set_process_info("handling incoming connection from [%s]%s%s%s",
-      sender_host_address,
-      (sender_ident == NULL)? "" : " (",
-      (sender_ident == NULL)? "" : sender_ident,
-      (sender_ident == NULL)? "" : ")");
+    set_process_info("handling incoming connection from %s",
+      sender_fullhost);
 
     DEBUG(1) debug_printf("ready for new message\n");
 
@@ -321,10 +324,10 @@ if (pid == 0)
     retry if no message was accepted (can happen for invalid message
     parameters). */
 
-    if ((rc = smtp_setup_msg(in, out, smtp_first)) > 0)
+    if ((rc = smtp_setup_msg(smtp_first)) > 0)
       {
       smtp_first = FALSE;
-      (void) accept_msg(in, out, FALSE);
+      (void) accept_msg(smtp_in, smtp_out, FALSE);
       if (message_id[0] == 0) continue;
       }
     else exit((rc == 0)? EXIT_SUCCESS : EXIT_FAILURE);
@@ -383,13 +386,19 @@ if (pid == 0)
 
     if (!local_queue_only && (pid = fork()) == 0)
       {
-      fclose(in);
-      fclose(out);
+      fclose(smtp_in);
+      fclose(smtp_out);
+      smtp_in = NULL;
       if (security_level >= 2)
         {
         int i = 0;
-        char *argv[8];
+        char *argv[8 + MAX_CLMACROS];
         argv[i++] = exim_path;
+        if (clmacro_count > 0)
+          {
+          memcpy(argv + i, clmacros, clmacro_count * sizeof(char *));
+          i += clmacro_count;
+          }
         if (debug_level > 0)
           argv[i++] = string_sprintf("-d%d", debug_level);
         if (dont_deliver) argv[i++] = "-N";
@@ -404,8 +413,8 @@ if (pid == 0)
         execv(argv[0], argv);
         log_write(0, LOG_PANIC_DIE, "exec of exim -Mc failed");
         }
-      /* No need to re-exec - put the SIGCHLD signal back to its default */
-      signal(SIGCHLD, SIG_DFL);
+
+      /* No need to re-exec */
       (void)deliver_message(message_id, FALSE, FALSE, FALSE);
       _exit(EXIT_SUCCESS);
       }
@@ -422,7 +431,7 @@ remember the pid for ticking off when the child completes. */
 if (pid < 0)
   {
   DEBUG(1) debug_printf("fork of SMTP accept process failed\n");
-  fprintf(out, "421: %s Connection refused: %s\r\n", primary_hostname,
+  fprintf(smtp_out, "421: %s Connection refused: %s\r\n", primary_hostname,
     strerror(errno));
   }
 else
@@ -446,16 +455,19 @@ else
 ERROR_RETURN:
 
 /* Close the streams associated with the socket which will also
-close the socket fds in the parent process. */
+close the socket fds in this process. */
 
-if (in != NULL) fclose(in);
-fclose(out);
+if (smtp_in != NULL)
+  {
+  fclose(smtp_in);
+  smtp_in = NULL;
+  }
+fclose(smtp_out);
 
 /* Free the store used for holding the incoming host address */
 
 store_free(sender_host_address);
-store_free(sender_fullhost);
-sender_host_address = sender_fullhost = NULL;
+sender_host_address = NULL;
 }
 
 
@@ -504,7 +516,10 @@ daemons running, as long as no more than one listens on a given TCP/IP port on
 the same interface(s). We put a non-standard port number in the file name to
 distinguish running/testing versions, and use different names for non-listening
 and non-queue-running daemons, but if someone runs two similar daemons
-simultaneously, then only the last started will get its pid written. */
+simultaneously, then only the last started will get its pid written.
+
+Note re use of sprintf: spool_directory and pid_file_path are checked on input
+to be < 200 characters. */
 
 if (smtp_port < 0)
   {
@@ -526,7 +541,7 @@ else
   }
 
 /* For daemons that aren't both listening and queue-running, add the option
-that they are doing to the file name. */
+which they are doing to the file name. */
 
 if (!daemon_listen)
   sprintf(buff+(int)strlen(buff), "-q%s", readconf_printtime(queue_interval));
@@ -537,17 +552,21 @@ terminal, if we are not debugging. Most modern Unixes seem to have setsid() for
 getting rid of the controlling terminal. For any OS that doesn't, setsid() can
 be #defined as a no-op, or as something else. */
 
-if (debug_level == 0 && !debug_trace_memory)
+if (debug_level <= 0 && !debug_trace_memory)
   {
   pid_t pid;
   int fd;
 
+  log_close_all();  /* Just in case anything was logged earlier */
+
   /* Ensure cached files for searches and expands, which might have been
   opened while reading the configuration, are closed before doing a general
-  close on all file descriptors. */
+  close on all file descriptors. Set log_stderr NULL so log doesn't try to
+  use it to copy paniclog output. */
 
   search_tidyup();
   for (fd = mac_maxfd; fd >= 0; fd--) close(fd);
+  log_stderr = NULL;
 
   /* Fork, in case current process is a process group leader (see 'man
   setsid' for an explanation). */
@@ -584,25 +603,12 @@ if (daemon_listen)
 
   host_setup *hset;
   net_setup *nset;
+  address_setup *aset;
 
-  /* If there are lists of nets and hosts + idents for various checks,
-  pre-process them now so that this work is not repeated for each message. For
-  a sender_{accept,reject} list, just do a match that won't work so as to get
-  any regular expressions compiled. */
-
-  if (sender_accept != NULL)
-    (void)match_isinlist("@", sender_accept, &re_sender_accept, FALSE);
-
-  if (sender_accept_recipients != NULL)
-    (void)match_isinlist("@", sender_accept_recipients,
-      &re_sender_accept_recipients, FALSE);
-
-  if (sender_reject != NULL)
-    (void)match_isinlist("@", sender_reject, &re_sender_reject, FALSE);
-
-  if (sender_reject_recipients != NULL)
-    (void)match_isinlist("@", sender_reject_recipients,
-      &re_sender_reject_recipients, FALSE);
+  /* If there are lists of nets and hosts + idents and addresses for various
+  checks, pre-process them now so that this work is not repeated for each
+  message. For a sender_{accept,reject} list, just do a match that won't work
+  so as to get any regular expressions compiled. */
 
   for (hset = host_setup_list; hset->text != NULL; hset++)
     if (*(hset->text) != NULL)
@@ -612,6 +618,10 @@ if (daemon_listen)
     if (*(nset->text) != NULL)
       verify_setup_netlist(*(nset->text), nset->data);
 
+  for (aset = address_setup_list; aset->text != NULL; aset++)
+    if (*(aset->text) != NULL)
+      (void)match_address_list("?@?", 2, *(aset->text), aset->data, -1, ':');
+
   /* The above may have caused databases to get opened and cached; in
   particular, the sender_{accept,reject} options may do this. So make
   sure that any cached open files are closed. */
@@ -631,6 +641,8 @@ if (daemon_listen)
     }
   else net_port = htons(smtp_port);
 
+  DEBUG(9) debug_printf("port = %d\n", smtp_port);
+
   /* If local_interfaces is set, create the list of local interfaces and
   set up a vector for holding the listening sockets. */
 
@@ -703,11 +715,12 @@ if (daemon_listen)
     memset(&sin, 0, sizeof(sin));
 
     /* This code has to be entirely different for IPv6 and IPv4 because
-    of the different struct layouts. */
+    of the different struct layouts. Sigh. */
 
     #if HAVE_IPV6
     sin.sin6_family = AF_INET6;
     sin.sin6_port = net_port;
+
     if (ipa->address[0] == 0)
       {
       sin.sin6_addr = anyaddr6;
@@ -715,7 +728,16 @@ if (daemon_listen)
       }
     else
       {
-      inet_pton(AF_INET6, ipa->address, &sin.sin6_addr);
+      int rc;
+      if (strchr(ipa->address, ':') != NULL)
+        rc = inet_pton(AF_INET6, ipa->address, &sin.sin6_addr);
+      else
+        {
+        rc = inet_pton(AF_INET, ipa->address, &sin.sin6_flowinfo);
+        sin.sin6_family = AF_INET;
+        }
+      if (rc != 1)
+        log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", ipa->address);
       DEBUG(9) debug_printf("listening on %s\n", ipa->address);
       }
     #else
@@ -837,7 +859,7 @@ signal(SIGHUP, sighup_handler);
 gone off. This will cause the first queue-runner to get kicked off straight
 away, and the alarm to be reset. */
 
-sigalrm_seen = (queue_interval > 0);
+daemon_sigalrm_seen = (queue_interval > 0);
 
 
 /* Log the start up of a daemon. */
@@ -880,18 +902,22 @@ if (f != NULL)
   fclose(f);
   DEBUG(2) debug_printf("pid written to %s\n", buff);
   }
-else DEBUG(2) debug_printf("failed to open pid file %s: %s\n", buff,
-  strerror(errno));
+else
+  DEBUG(2) debug_printf("%s\n", string_open_failed(errno, "pid file %s", buff));
 
 /* Close the log so it can be renamed and moved. This process doesn't write
 to the log again, unless it is about to die or exec and in the latter case
 it closes the log first. */
 
-log_close();
+log_close_all();
 
 DEBUG(2) debug_printf("daemon running with uid=%d gid=%d euid=%d egid=%d\n",
   (int)getuid(), (int)getgid(), (int)geteuid(), (int)getegid());
 
+/* Any messages accepted via this route are going to be SMTP. */
+
+smtp_input = TRUE;
+
 /* Enter the never-ending loop... */
 
 for (;;)
@@ -909,7 +935,7 @@ for (;;)
   start, before the first wait. This causes the first queue-runner to be
   started immediately. */
 
-  if (sigalrm_seen)
+  if (daemon_sigalrm_seen)
     {
     DEBUG(9) debug_printf("SIGALRM received\n");
 
@@ -940,24 +966,36 @@ for (;;)
         if (geteuid() != root_uid)
           {
           int i = 0;
-          char *argv[5];
+          char *argv[5 + MAX_CLMACROS];
+          char opt[16];
+          char *p = opt;
+
           argv[i++] = exim_path;
+          if (clmacro_count > 0)
+            {
+            memcpy(argv + i, clmacros, clmacro_count * sizeof(char *));
+            i += clmacro_count;
+            }
           if (config_changed)
             {
             argv[i++] = "-C";
             argv[i++] = config_filename;
             }
-          argv[i++] = queue_run_force?
-            (queue_run_local? "-qfl" : "-qf") :
-            (queue_run_local? "-ql" : "-q");
+          argv[i++] = opt;
+          *p++ = '-';
+          *p++ = 'q';
+          if (queue_2stage) *p++ = 'q';
+          if (queue_run_force) *p++ = 'f';
+          if (queue_run_local) *p++ = 'l';
+          *p = 0;
           argv[i++] = (char *)0;
           execv(argv[0], argv);
-          log_write(0, LOG_PANIC_DIE, "exec of exim -q failed");
+          log_write(0, LOG_PANIC_DIE, "exec of exim %s failed", opt);
           }
 
         /* No need to re-exec */
 
-        queue_run(NULL, NULL);
+        queue_run(NULL, NULL, FALSE);
         _exit(EXIT_SUCCESS);
         }
 
@@ -984,8 +1022,8 @@ for (;;)
 
     /* Reset the alarm time */
 
-    sigalrm_seen = FALSE;
-    signal(SIGALRM, sigalrm_handler);
+    daemon_sigalrm_seen = FALSE;
+    signal(SIGALRM, daemon_sigalrm_handler);
     alarm(queue_interval);
     }
 
@@ -1166,18 +1204,19 @@ for (;;)
 
   /* Handle being woken by SIGHUP. We know at this point that the result
   of accept() has been dealt with, so we can re-exec exim safely, first
-  closing the listening socket so that it can be reused. */
+  closing the listening sockets so that they can be reused. */
 
   if (sighup_seen)
     {
     int sk;
     log_write(0, LOG_MAIN, "pid %d: SIGHUP received: re-exec daemon",
       getpid());
-    log_close();
+    log_close_all();
     for (sk = 0; sk < listen_socket_count; sk++) close(listen_sockets[sk]);
     sighup_argv[0] = exim_path;
     execv(exim_path, sighup_argv);
-    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "pid %d: exec of %s failed", getpid());
+    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "pid %d: exec of %s failed",
+      getpid(), exim_path);
     }
 
   }   /* End of main loop */
