@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -18,20 +18,28 @@ optionlist aliasfile_director_options[] = {
       (void *)(offsetof(aliasfile_director_options_block, expand_gid)) },
   { "*expand_user",       opt_stringptr | opt_hidden,
       (void *)(offsetof(aliasfile_director_options_block, expand_uid)) },
+  { "*partial_match", opt_int | opt_hidden,
+      (void *)(offsetof(aliasfile_director_options_block, partial_match)) },
   { "*set_group",         opt_bool | opt_hidden,
       (void *)(offsetof(aliasfile_director_options_block, gid_set)) },
   { "*set_user",          opt_bool | opt_hidden,
       (void *)(offsetof(aliasfile_director_options_block, uid_set)) },
+  { "check_ancestor",     opt_bool,
+      (void *)(offsetof(aliasfile_director_options_block, check_ancestor)) },
   { "current_directory",     opt_stringptr,
       (void *)(offsetof(aliasfile_director_options_block, current_dir)) },
   { "directory",          opt_stringptr | opt_hidden,
       (void *)(offsetof(aliasfile_director_options_block, home_dir)) },
-  { "errors_to",          opt_stringptr,
-      (void *)(offsetof(aliasfile_director_options_block, errors_to)) },
+  { "directory2_transport",opt_transportptr,
+      (void *)(offsetof(aliasfile_director_options_block, directory2_transport)) },
+  { "directory_transport",opt_transportptr,
+      (void *)(offsetof(aliasfile_director_options_block, directory_transport)) },
   { "expand",             opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, expand)) },
   { "file",               opt_stringptr,
       (void *)(offsetof(aliasfile_director_options_block, file)) },
+  { "file_transport",     opt_transportptr,
+      (void *)(offsetof(aliasfile_director_options_block, file_transport)) },
   { "forbid_file",        opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, forbid_file)) },
   { "forbid_pipe",        opt_bool,
@@ -42,6 +50,8 @@ optionlist aliasfile_director_options[] = {
       (void *)(offsetof(aliasfile_director_options_block, gid)) },
   { "home_directory",     opt_stringptr,
       (void *)(offsetof(aliasfile_director_options_block, home_dir)) },
+  { "include_domain",     opt_bool,
+      (void *)(offsetof(aliasfile_director_options_block, include_domain)) },
   { "initgroups",         opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, initgroups)) },
   { "modemask",           opt_octint,
@@ -52,6 +62,12 @@ optionlist aliasfile_director_options[] = {
       (void *)(offsetof(aliasfile_director_options_block, owners)) },
   { "owngroups",          opt_gidlist,
       (void *)(offsetof(aliasfile_director_options_block, owngroups)) },
+  { "pipe_transport",     opt_transportptr,
+      (void *)(offsetof(aliasfile_director_options_block, pipe_transport)) },
+  { "qualify_preserve_domain", opt_bool,
+      (void *)(offsetof(aliasfile_director_options_block, qualify_preserve_domain)) },
+  { "queries",            opt_stringptr,
+      (void *)(offsetof(aliasfile_director_options_block, queries)) },
   { "query",              opt_stringptr,
       (void *)(offsetof(aliasfile_director_options_block, query)) },
   { "rewrite",            opt_bool,
@@ -60,6 +76,8 @@ optionlist aliasfile_director_options[] = {
       (void *)(offsetof(aliasfile_director_options_block, search_type)) },
   { "skip_syntax_errors", opt_bool,
       (void *)(offsetof(aliasfile_director_options_block, skip_syntax_errors)) },
+  { "syntax_errors_to",   opt_stringptr,
+      (void *)(offsetof(aliasfile_director_options_block, syntax_errors_to)) },
   { "user",               opt_expand_uid,
       (void *)(offsetof(aliasfile_director_options_block, uid)) }
 };
@@ -74,28 +92,37 @@ int aliasfile_director_options_count =
 
 aliasfile_director_options_block aliasfile_director_option_defaults = {
   NULL,     /* file */
+  NULL,     /* queries */
   NULL,     /* query */
   NULL,     /* home_dir */
   NULL,     /* current_dir */
-  NULL,     /* errors_to */
+  NULL,     /* syntax_errors_to */
   NULL,     /* expand_uid */
   NULL,     /* expand_gid */
   -1,       /* uid */
   -1,       /* gid */
   022,      /* modemask */
+  -1,       /* partial_match */
   -1,       /* search_type */
   NULL,     /* owners */
   NULL,     /* owngroups */
   FALSE,    /* uid_set */
   FALSE,    /* gid_set */
+  FALSE,    /* include_domain */
+  FALSE,    /* check_ancestor */
   FALSE,    /* initgroups */
   FALSE,    /* expand */
   FALSE,    /* optional */
+  FALSE,    /* qualify_preserve_domain */
   FALSE,    /* forbid_file */
   FALSE,    /* forbid_pipe */
   TRUE,     /* freeze_missing_include */
   TRUE,     /* rewrite */
-  FALSE     /* skip_syntax_errors */
+  FALSE,    /* skip_syntax_errors */
+  NULL,     /* directory_transport */
+  NULL,     /* directory2_transport */
+  NULL,     /* file_transport */
+  NULL      /* pipe_transport */
 };
 
 
@@ -136,7 +163,7 @@ if (ob->search_type < 0)
 a query is mandatory. An absolute file name is mandatory for lsearch and dbm;
 but can't check for absoluteness if the name is being looked up. */
 
-if (ob->search_type < stype_querystyle)
+if (!mac_islookup(ob->search_type, lookup_querystyle))
   {
   if (ob->query != NULL)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
@@ -146,13 +173,15 @@ if (ob->search_type < stype_querystyle)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
       "no file name specified", dblock->name);
 
-  if ((ob->search_type == stype_lsearch || ob->search_type == stype_dbm) &&
+  if (mac_islookup(ob->search_type, lookup_absfile) &&
        ob->file[0] != '/' && ob->file[0] != '$')
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
-      "an absolute file path name is required for lsearch or dbm", dblock->name);
+      "an absolute file path name is required for %s",
+        dblock->name, lookup_list[ob->search_type].name);
   }
 
-/* Non-single-key search type */
+/* Non-single-key search type; either a single query or a list of queries must
+be specified. */
 
 else
   {
@@ -160,9 +189,14 @@ else
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
       "\"file\" specified for a non-single-key search type", dblock->name);
 
-  if (ob->query == NULL)
+  if (ob->query == NULL && ob->queries == NULL)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
-      "no query specified", dblock->name);
+      "no query specified for %s search", dblock->name,
+      lookup_list[ob->search_type].name);
+
+  if (ob->query != NULL && ob->queries != NULL)
+    log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
+      "only one of \"query\" or \"queries\" may be given", dblock->name);
   }
 }
 
@@ -188,91 +222,87 @@ aliasfile_director_entry(
 aliasfile_director_options_block *ob =
   (aliasfile_director_options_block *)(dblock->options_block);
 address_item *generated = NULL;
-char *errors_to = addr->errors_address;
+char *save_qualify_domain_recipient = qualify_domain_recipient;
+char *aliastext = NULL;
+char *errors_to;
 char *filename;
 char *query;
-char *aliastext;
 char *error;
 void *handle;
+header_line *extra_headers;
+char *remove_headers;
 error_block *eblock = NULL;
 uid_t uid = ob->uid;
 gid_t gid = ob->gid;
 struct passwd *upw = NULL;
 BOOL uid_set = ob->uid_set;
 BOOL gid_set = ob->gid_set;
-int  extracted;
+int  extracted, rc;
 
-/* Do file existence tests */
+/* Perform file existence and sender verification checks. */
 
-switch (match_exists(dblock->require_files))
-  {
-  case FAIL:
-  DEBUG(9) debug_printf("%s director skipped: file existence failure\n",
-    dblock->name);
-  return FAIL;
-
-  case DEFER:
-  addr->message = string_sprintf("file existence defer in %s director: %s",
-    dblock->name, strerror(errno));
-  return DEFER;
-  }
+rc = direct_check_fsc(dblock, addr);
+if (rc != OK) return rc;
 
 /* For single-key+file search types, set the required file name and expand it.
 If the expansion fails, log the incident and indicate an internal error. The
 file name has already been checked for absoluteness, at initialization time,
 but only if it did not start with an expansion, so we double check here. */
 
-if (ob->search_type < stype_querystyle)
+if (!mac_islookup(ob->search_type, lookup_querystyle))
   {
-  query = addr->local_part;
+  query = (ob->include_domain)?
+    string_sprintf("%s@%s", addr->local_part, addr->domain) : addr->local_part;
   filename = expand_string(ob->file);
   if (filename == NULL)
     {
-    log_write(0, LOG_MAIN|LOG_PANIC, "%s director failed to expand %s: %s",
-      dblock->name, ob->file, expand_string_message);
-    addr->message = string_sprintf("%s director failed to expand %s: %s",
+    addr->message = string_sprintf("%s director failed to expand \"%s\": %s",
       dblock->name, ob->file, expand_string_message);
+    if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
-  else if ((ob->search_type == stype_lsearch || ob->search_type == stype_dbm) &&
+
+  else if (mac_islookup(ob->search_type, lookup_absfile) &&
            filename[0] != '/')
     {
     log_write(0, LOG_MAIN|LOG_PANIC, "%s director requires absolute file name "
-      "for lsearch or dbm: "
-      "%s generated from expanding %s", dblock->name, filename, ob->file);
+      "for %s: %s generated from expanding %s", dblock->name,
+      lookup_list[ob->search_type].name, filename, ob->file);
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
 
-  DEBUG(2) debug_printf("%s director: file = %s search type = %d\n",
-    dblock->name, filename, ob->search_type);
+  DEBUG(2) debug_printf("%s director: %s key=%s\n  file=\"%s\"\n",
+    dblock->name, lookup_list[ob->search_type].name, query, filename);
   }
 
-/* For query-style lookups, expand the query and set the filename NULL */
+/* For query-style lookups, expand the query and set the filename NULL.
+For historical reasons, there may be just one query or a list of queries, but
+not both. Initialization ensures that one and only one option is set. */
 
 else
   {
   filename = NULL;
-  query = expand_string(ob->query);
+  query = expand_string((ob->query == NULL)? ob->queries : ob->query);
   if (query == NULL)
     {
-    log_write(0, LOG_MAIN|LOG_PANIC, "%s director failed to expand %s: %s",
-      dblock->name, ob->query, expand_string_message);
-    addr->message = string_sprintf("%s director failed to expand %s: %s",
-      dblock->name, ob->query, expand_string_message);
+    addr->message = string_sprintf("%s director failed to expand \"%s\": %s",
+      dblock->name, (ob->query == NULL)? ob->queries : ob->query,
+      expand_string_message);
+    if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
+  DEBUG(2) debug_printf("%s director: %s query=%s\n", dblock->name,
+    lookup_list[ob->search_type].name, query);
   }
 
-/* Open the file (or whatever) for searching, according to the search type that
-is set. The search functions return a handle identifying the search. For files
-this is a FILE * or a DBM *; for other things is is < 0. If the optional flag
-is set, failure to open is not an error; we just fail to direct. */
-
-DEBUG(9) debug_printf("file=%s query=%s\n",
-  (filename == NULL)? "NULL" : filename, query);
+/* Open the file (or whatever) for searching, according to the search type
+that is set. The search functions return a handle identifying the search. For
+files this is a FILE * or a DBM *; for other things it is < 0. If the
+optional flag is set, failure to open is not an error; we just fail to
+direct. */
 
 handle = search_open(filename, ob->search_type, ob->modemask, ob->owners,
   ob->owngroups, &error);
@@ -292,13 +322,36 @@ if (handle == NULL)
   }
 
 /* Now search the file (or whatever) for the entry we are interested in.
-The text is returned in dynamic store. */
+The text is returned in dynamic store. If ob->queries is set, we have
+a list of queries to try. */
 
-aliastext = search_find(handle, filename, query, ob->search_type, 0, NULL,
-  &error);
+if (ob->queries != NULL)
+  {
+  char *onequery;
+  for (onequery = string_nextinlist(&query, ':', big_buffer, BIG_BUFFER_SIZE);
+       onequery != NULL;
+       onequery = string_nextinlist(&query, ':', big_buffer, BIG_BUFFER_SIZE))
+    {
+    aliastext = search_find(handle, filename, onequery, ob->search_type,
+      ob->partial_match, NULL, &error);
+    if (aliastext != NULL || search_find_defer) break;
+    }
+  }
+else aliastext = search_find(handle, filename, query, ob->search_type,
+  ob->partial_match, NULL, &error);
+
+/* The lookup did not succeed. Defer if it deferred; otherwise this director
+fails to handle the address. */
 
 if (aliastext == NULL)
   {
+  if (search_find_defer)
+    {
+    addr->message = string_sprintf("%s director: search_find deferred: %s",
+      dblock->name, error);
+    return DEFER;
+    }
+
   DEBUG(2) debug_printf("%s director failed for %s: %s\n", dblock->name,
     addr->local_part, error);
   return FAIL;
@@ -311,8 +364,10 @@ if (ob->expand)
   char *newtext = expand_string(aliastext);
   if (newtext == NULL)
     {
-    log_write(0, LOG_MAIN, "%s director failed to expand %s (generated from "
-      "local part %s)", dblock->name, aliastext, addr->local_part);
+    addr->message = string_sprintf("%s director failed to expand \"%s\" "
+      "(generated from local part %s): %s", dblock->name, aliastext,
+      addr->local_part, expand_string_message);
+    if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
@@ -321,21 +376,30 @@ if (ob->expand)
   }
 
 /* If there is no fixed uid set, see if there's a dynamic one that can
-be expanded and possibly looked up. Exim panics if such a lookup fails. */
+be expanded and possibly looked up. */
 
 if (!uid_set && ob->expand_uid != NULL)
   {
-  direct_find_expanded_user(ob->expand_uid, dblock->name, "director",
-    &upw, &uid);
-  uid_set = TRUE;
+  if (direct_find_expanded_user(ob->expand_uid, dblock->name, "director",
+    &upw, &uid, &(addr->message))) uid_set = TRUE;
+  else
+    {
+    addr->special_action = SPECIAL_FREEZE;
+    return ERROR;
+    }
   }
 
 /* Likewise for the gid */
 
 if (!gid_set && ob->expand_gid != NULL)
   {
-  direct_find_expanded_group(ob->expand_gid, dblock->name, "director", &gid);
-  gid_set = TRUE;
+  if (direct_find_expanded_group(ob->expand_gid, dblock->name, "director",
+    &gid, &(addr->message))) gid_set = TRUE;
+  else
+    {
+    addr->special_action = SPECIAL_FREEZE;
+    return ERROR;
+    }
   }
 
 /* If a uid is set, then a gid must also be available; use one from the passwd
@@ -353,29 +417,17 @@ if (uid_set && !gid_set)
       dblock->name);
   }
 
-/* If this director has a local errors_to setting for where to send error
-messages for its children, expand it, and then check that it is a valid
-address before using it, except when just verifying an address. Otherwise
-there could be directing loops if someone sets up a silly configuration. */
+/* Set up the errors address for the children of this address. */
 
-if (ob->errors_to != NULL)
-  {
-  char *s = expand_string(ob->errors_to);
-  if (s == NULL)
-    {
-    log_write(0, LOG_MAIN, "%s director failed to expand %s", dblock->name,
-      ob->errors_to);
-    addr->special_action = SPECIAL_FREEZE;
-    return ERROR;
-    }
+rc = direct_get_errors_address(addr, dblock, "director", verify, &errors_to);
+if (rc != OK) return rc;
 
-  if (verify) errors_to = s; else
-    {
-    char *snew;
-    if (verify_address(s, TRUE, TRUE, NULL, NULL, &snew, FALSE, FALSE) == OK)
-      errors_to = snew;
-    }
-  }
+/* Set up the additional and removeable headers for the children of this
+address. */
+
+rc = direct_get_munge_headers(addr, dblock, "director", &extra_headers,
+  &remove_headers);
+if (rc != OK) return rc;
 
 /* If there is a transport specified for the director, then set up this
 address to use that transport. Ignore the alias text. */
@@ -384,10 +436,13 @@ if (dblock->transport != NULL)
   {
   addr->transport = dblock->transport;
 
-  if (errors_to != NULL) addr->errors_address = errors_to;
+  addr->errors_address = errors_to;
   addr->director = dblock;
   addr->home_dir = ob->home_dir;
   addr->current_dir = ob->current_dir;
+  addr->extra_headers = extra_headers;
+  addr->remove_headers = remove_headers;
+
   if (uid_set)
     {
     addr->uid = uid;
@@ -424,43 +479,77 @@ if (dblock->transport != NULL)
   return OK;
   }
 
-/* There is a common function for use by aliasing and aliasing directors that
+/* There is a common function for use by forwarding and aliasing directors that
 extracts a list of addresses from a text string. Setting the fourth-last
 argument FALSE makes generating no addresses an error. However, setting the
-third-last TRUE allows an address that is ":blackhole:" in the alias file
-to generate nothing without causing an error. Setting the last argument NULL
-causes no checks to be made on :include: files. */
+third-last TRUE allows an address that is ":blackhole:" in the alias file to
+generate nothing without causing an error. It also permits :defer: and :fail:
+to occur. Setting the last argument NULL causes no checks to be made on
+:include: files.
+
+If qualify_preserve_domain is set, we temporarily reset the value of
+qualify_domain_recipient to the current domain, so that any unqualified
+addresses get qualified with the same domain as the incoming address. */
+
+if (ob->qualify_preserve_domain) qualify_domain_recipient = addr->domain;
+
+extracted = parse_extract_addresses(aliastext, addr->domain, &generated,
+  &error, FALSE, TRUE, ob->rewrite, NULL,
+  ob->skip_syntax_errors? &eblock : NULL);
 
-extracted = parse_extract_addresses(aliastext, &generated, &error, FALSE,
-  TRUE, ob->rewrite, NULL, ob->skip_syntax_errors? &eblock : NULL);
 store_free(aliastext);
+qualify_domain_recipient = save_qualify_domain_recipient;
 
 /* If extraction failed, return error and freeze, unless it was a missing
-include file and no_freeze_missing_include is set. */
+include file and no_freeze_missing_include is set, or unless it was an explicit
+:defer: or :fail: that was encountered. */
 
-if (extracted != 0)
+if (extracted != 0) switch (extracted)
   {
+  case EXTRACTED_DEFER:
+  addr->message = string_sprintf("forced defer: %s", error);
+  return DEFER;
+
+  case EXTRACTED_FAIL:
+  addr->message = string_sprintf("forced failure: %s", error);
+  return FORCEFAIL;
+
+  default:
   addr->basic_errno = ERRNO_BADALIAS;
-  addr->message =
-    string_sprintf("<%s> - error in alias file: %s", addr->orig, error);
+  addr->message = (filename != NULL)?
+    string_sprintf("<%s> - error in alias file %s: %s", addr->orig, filename,
+      error) :
+    string_sprintf("<%s> - error in alias lookup: %s", addr->orig, error);
   if (extracted > 0 && !ob->freeze_missing_include) return DEFER;
   addr->special_action = SPECIAL_FREEZE;
   return ERROR;
   }
 
 /* If skip_syntax_errors was set and there were syntax errors in the list,
-error messages will be present in eblock. Log them. */
+error messages will be present in eblock. Log them, and sent them off in
+a mail message if so configured. A common function is used by aliasfile
+and forwardfile. */
 
-while (eblock != NULL)
+if (eblock != NULL && !moan_skipped_syntax_errors(dblock->name, filename,
+    eblock, verify? NULL : ob->syntax_errors_to))
   {
-  log_write(0, LOG_MAIN, "%s director: syntax error in alias file %s: "
-    "%s in \"%s\"", dblock->name, filename, eblock->text1, eblock->text2);
-  eblock = eblock->next;
+  addr->special_action = SPECIAL_FREEZE;
+  return ERROR;
   }
 
 /* Add the new addresses to the list of new addresses, copying in the
 uid, gid and permission flags for use by pipes and files, setting
-the parent, and or-ing its ignore_error flag. */
+the parent, and or-ing its ignore_error flag. Also record the setting
+for any starting director.
+
+If the generated address is the same as one of its ancestors, and the
+check_ancestor flag is set, do not use this generated address, but replace it
+with a copy of the input address. This is to cope with cases where A is aliased
+to B and B has a .forward file pointing to A, though it is usually set on the
+forwardfile rather than the aliasfile. We can't just pass on the old
+address by returning FAIL, because it must act as a general parent for
+generated addresses, and only get marked "done" when all its children are
+delivered. */
 
 while (generated != NULL)
   {
@@ -468,12 +557,29 @@ while (generated != NULL)
   generated = next->next;
   next->parent = addr;
   next->ignore_error |= addr->ignore_error;
+  next->start_director = dblock->new;
   addr->child_count++;
 
   next->next = *addr_new;
   *addr_new = next;
 
-  if (errors_to != NULL) next->errors_address = errors_to;
+  if (ob->check_ancestor)
+    {
+    address_item *parent;
+    for (parent = addr; parent != NULL; parent = parent->parent)
+      {
+      if (strcmp(next->orig, parent->orig) == 0)
+        {
+        DEBUG(2) debug_printf("generated parent replaced by child\n");
+        next->orig = string_copy(addr->orig);
+        break;
+        }
+      }
+    }
+
+  next->errors_address = errors_to;
+  next->extra_headers = extra_headers;
+  next->remove_headers = remove_headers;
 
   if (next->pfr)
     {
@@ -493,22 +599,41 @@ while (generated != NULL)
     next->home_dir = ob->current_dir;
     next->allow_pipe = !ob->forbid_pipe;
     next->allow_file = !ob->forbid_file;
+
+    /* Aliasfile can produce only pipes or files; if the transport
+    setting is null, the global setting will get used later. */
+
+    if (next->orig[0] == '|') next->transport = ob->pipe_transport; else
+      {
+      int len = (int)strlen(next->orig);
+      if (next->orig[len-1] == '/')
+        {
+        next->transport = ob->directory_transport;
+        if (len > 1 && next->orig[len-2] == '/' &&
+            ob->directory2_transport != NULL)
+          next->transport = ob->directory2_transport;
+        }
+      else next->transport = ob->file_transport;
+      }
     }
 
   DEBUG(2)
     {
-    debug_printf("%s director generated %s%s%s%s%s\n",
+    debug_printf("%s director generated %s\n%s%s%s%s%s%s%s",
       dblock->name,
       next->orig,
-      next->pfr? " (pipe, file, or autoreply)" : "",
-      (errors_to != NULL)? " (errors to " : "",
+      next->pfr? "  pipe, file, or autoreply\n" : "",
+      (errors_to != NULL)? "  errors to " : "",
       (errors_to != NULL)? errors_to : "",
-      (errors_to != NULL)? ")" : "");
+      (errors_to != NULL)? "\n" : "",
+      (next->transport == NULL)? "" : "  transport=",
+      (next->transport == NULL)? "" : (next->transport)->name,
+      (next->transport == NULL)? "" : "\n");
 
     if (next->uid_set)
-      debug_printf("   uid=%d ", (int)(next->uid));
+      debug_printf("  uid=%d ", (int)(next->uid));
     else
-      debug_printf("   uid=unset ");
+      debug_printf("  uid=unset ");
 
     if (next->gid_set)
       debug_printf("gid=%d ", (int)(next->gid));
