@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -22,6 +22,8 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, gid_set)) },
   { "*set_user",          opt_bool | opt_hidden,
       (void *)(offsetof(forwardfile_director_options_block, uid_set)) },
+  { "allow_system_actions", opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, allow_system_actions)) },
   { "check_ancestor",     opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, check_ancestor)) },
   { "check_group",        opt_bool,
@@ -32,18 +34,24 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, current_dir)) },
   { "directory",          opt_stringptr | opt_hidden,
       (void *)(offsetof(forwardfile_director_options_block, file_dir)) },
-  { "errors_to",          opt_stringptr,
-      (void *)(offsetof(forwardfile_director_options_block, errors_to)) },
+  { "directory2_transport",opt_transportptr,
+      (void *)(offsetof(forwardfile_director_options_block, directory2_transport)) },
+  { "directory_transport",opt_transportptr,
+      (void *)(offsetof(forwardfile_director_options_block, directory_transport)) },
   { "file",               opt_stringptr,
       (void *)(offsetof(forwardfile_director_options_block, file)) },
   { "file_directory",     opt_stringptr,
       (void *)(offsetof(forwardfile_director_options_block, file_dir)) },
+  { "file_transport",     opt_transportptr,
+      (void *)(offsetof(forwardfile_director_options_block, file_transport)) },
   { "filter",             opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, filter)) },
   { "forbid_file",        opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_file)) },
-  { "forbid_filter_log",  opt_bool,
-      (void *)(offsetof(forwardfile_director_options_block, forbid_filter_log)) },
+  { "forbid_filter_log",  opt_bool|opt_hidden,
+      (void *)(offsetof(forwardfile_director_options_block, forbid_filter_logwrite)) },
+  { "forbid_filter_logwrite",opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, forbid_filter_logwrite)) },
   { "forbid_include",     opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_include)) },
   { "forbid_pipe",        opt_bool,
@@ -56,6 +64,10 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, gid)) },
   { "home_directory",     opt_stringptr,
       (void *)(offsetof(forwardfile_director_options_block, home_dir)) },
+  { "ignore_eacces",     opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, ignore_eacces)) },
+  { "ignore_enotdir",     opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, ignore_enotdir)) },
   { "initgroups",         opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, initgroups)) },
   { "modemask",           opt_octint,
@@ -64,12 +76,20 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, owners)) },
   { "owngroups",          opt_gidlist,
       (void *)(offsetof(forwardfile_director_options_block, owngroups)) },
+  { "pipe_transport",     opt_transportptr,
+      (void *)(offsetof(forwardfile_director_options_block, pipe_transport)) },
+  { "qualify_preserve_domain", opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, qualify_preserve_domain)) },
+  { "reply_transport",    opt_transportptr,
+      (void *)(offsetof(forwardfile_director_options_block, reply_transport)) },
   { "rewrite",            opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, rewrite)) },
   { "seteuid",            opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, use_seteuid)) },
   { "skip_syntax_errors", opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, skip_syntax_errors)) },
+  { "syntax_errors_to",   opt_stringptr,
+      (void *)(offsetof(forwardfile_director_options_block, syntax_errors_to)) },
   { "user",               opt_expand_uid,
       (void *)(offsetof(forwardfile_director_options_block, uid)) }
 };
@@ -87,7 +107,7 @@ forwardfile_director_options_block forwardfile_director_option_defaults = {
   NULL,     /* home_dir */
   NULL,     /* current_dir */
   NULL,     /* file */
-  NULL,     /* errors_to */
+  NULL,     /* syntax_errors_to */
   NULL,     /* expand_uid */
   NULL,     /* expand_gid */
   -1,       /* uid */
@@ -101,16 +121,25 @@ forwardfile_director_options_block forwardfile_director_option_defaults = {
   TRUE,     /* check_local_user */
   FALSE,    /* check_group */
   FALSE,    /* filter */
-  FALSE,    /* forbid_filter_log */
+  FALSE,    /* allow_system_actions */
+  FALSE,    /* forbid_filter_logwrite */
   FALSE,    /* forbid_file */
   FALSE,    /* forbid_include */
   FALSE,    /* forbid_pipe */
   FALSE,    /* forbid_reply */
+  FALSE,    /* ignore_eacces */
+  FALSE,    /* ignore_enotdir */
   FALSE,    /* initgroups */
+  FALSE,    /* qualify_preserve_domain */
   FALSE,    /* use_seteuid */
   TRUE,     /* freeze_missing_include */
   TRUE,     /* rewrite */
-  FALSE     /* skip_syntax_errors */
+  FALSE,    /* skip_syntax_errors */
+  NULL,     /* directory_transport */
+  NULL,     /* directory2_transport */
+  NULL,     /* file_transport */
+  NULL,     /* pipe_transport */
+  NULL,     /* reply_transport */
 };
 
 
@@ -172,7 +201,7 @@ if (ob->use_seteuid && !ob->check_local_user && !ob->uid_set &&
 
 /* A transport must *not* be specified */
 
-if (dblock->transport != NULL)
+if (dblock->transport != NULL || dblock->expand_transport != NULL)
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
     "a transport is not allowed for this director", dblock->name);
 }
@@ -197,12 +226,15 @@ int forwardfile_director_entry(
 forwardfile_director_options_block *ob =
   (forwardfile_director_options_block *)(dblock->options_block);
 address_item *generated = NULL;
+char *save_qualify_domain_recipient = qualify_domain_recipient;
 char *directory = NULL;
-char *errors_to = addr->errors_address;
+char *errors_to;
 char *filename;
 char *filebuf;
 char *error;
 char *s, *tag;
+char *remove_headers;
+header_line *extra_headers;
 struct stat statbuf;
 struct passwd *pw;
 struct passwd *upw = NULL;
@@ -211,9 +243,10 @@ FILE *fwd;
 uid_t uid = ob->uid;
 gid_t gid = ob->gid;
 int   yield = OK;
-int   saved_euid = -1;
-int   saved_egid = -1;
+uid_t saved_euid = 0;
+gid_t saved_egid = 0;
 int   extracted = 0;
+int   rc;
 BOOL  uid_set = ob->uid_set;
 BOOL  gid_set = ob->gid_set;
 BOOL  uid_ok = FALSE;
@@ -255,28 +288,20 @@ if (ob->file_dir != NULL)
   directory = expand_string(ob->file_dir);
   if (directory == NULL)
     {
-    log_write(0, LOG_MAIN, "%s director failed to expand %s: %s", dblock->name,
-      ob->file_dir, expand_string_message);
+    addr->message = string_sprintf("%s director failed to expand %s: %s",
+      dblock->name, ob->file_dir, expand_string_message);
+    if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
   deliver_home = directory;
   }
 
-/* Do file existence tests; these can include references to $home. */
+/* Perform file existence and sender verification checks now that we
+have $home available. */
 
-switch (match_exists(dblock->require_files))
-  {
-  case FAIL:
-  DEBUG(9) debug_printf("%s director skipped: file existence failure\n",
-    dblock->name);
-  return FAIL;
-
-  case DEFER:
-  addr->message = string_sprintf("file existence defer in %s director: %s",
-    dblock->name, strerror(errno));
-  return DEFER;
-  }
+yield = direct_check_fsc(dblock, addr);
+if (yield != OK) return yield;
 
 /* Get the required file name and expand it. If the expansion fails, log the
 incident and indicate an internal error. */
@@ -285,8 +310,9 @@ filename = expand_string(ob->file);
 
 if (filename == NULL)
   {
-  log_write(0, LOG_MAIN, "%s director failed to expand %s: %s", dblock->name,
-    ob->file, expand_string_message);
+  addr->message = string_sprintf("%s director failed to expand %s: %s",
+    dblock->name, ob->file, expand_string_message);
+  if (search_find_defer) return DEFER;
   addr->special_action = SPECIAL_FREEZE;
   return ERROR;
   }
@@ -322,17 +348,26 @@ password file. */
 
 if (!uid_set && ob->expand_uid != NULL)
   {
-  direct_find_expanded_user(ob->expand_uid, dblock->name, "director",
-    &upw, &uid);
-  uid_set = TRUE;
+  if (direct_find_expanded_user(ob->expand_uid, dblock->name, "director",
+    &upw, &uid, &(addr->message))) uid_set = TRUE;
+  else
+    {
+    addr->special_action = SPECIAL_FREEZE;
+    return ERROR;
+    }
   }
 
 /* Likewise, set up an expanded gid if necessary. */
 
 if (!gid_set && ob->expand_gid != NULL)
   {
-  direct_find_expanded_group(ob->expand_gid, dblock->name, "director", &gid);
-  gid_set = TRUE;
+  if (direct_find_expanded_group(ob->expand_gid, dblock->name, "director",
+    &gid, &(addr->message))) gid_set = TRUE;
+  else
+    {
+    addr->special_action = SPECIAL_FREEZE;
+    return ERROR;
+    }
   }
 
 /* If a uid was specified, a gid to go with it must also be available. Take it
@@ -400,11 +435,22 @@ the state of "NFS mount inaccessible" just by trying to open a file.
 The common case is specified with a relative path name (relative to the home
 directory or to a specified directory), and in that case we try to do a bit
 better by statting the directory first. If it cannot be statted, assume there
-is some mounting problem, and defer the delivery. */
+is some mounting problem, and defer the delivery.
+
+If an NFS host is taken offline, it is possible for the stat() to get stuck
+until it comes back. To guard against this, stick a timer round it. */
 
 if (directory != NULL && stat_directory)
   {
-  if (stat(directory, &statbuf) != 0)
+  int rc;
+  sigalrm_seen = FALSE;
+  signal(SIGALRM, sigalrm_handler);
+  alarm(30);
+  rc = stat(directory, &statbuf);
+  alarm(0);
+  signal(SIGALRM, SIG_DFL);
+
+  if (sigalrm_seen || rc != 0)
     {
     DEBUG(2) debug_printf("%s director failed to stat %s: deferred\n",
       dblock->name, directory);
@@ -413,29 +459,63 @@ if (directory != NULL && stat_directory)
     yield = DEFER;
     goto RESTORE_UID;             /* skip forward */
     }
+
   DEBUG(2) debug_printf("successful stat of %s\n", directory);
   }
 
 /* Now try to open the file for reading. If this fails with a non-existence
 error, we have no option but to believe that the file does not exist, so the
-director gives up on this address. */
+director gives up on this address. Some other cases are more dubious, and may
+indicate configuration errors. For this reason, their handling is controlled by
+options.
+
+ENOTDIR means that something along the line is not a directory: there are
+installations that set home directories to be /dev/null for non-login accounts
+but in normal circumstances this indicates some kind of configuration error.
+
+EACCES means there's a permissions failure. Some users turn off read
+permission on a .forward file to suspend forwarding, but this is probably an
+error in any kind of mailing list processing. */
 
 fwd = fopen(filename, "r");
 if (fwd == NULL)
   {
-  if (errno != ENOENT)
+  switch(errno)
     {
-    DEBUG(2) debug_printf("%s director failed to open %s: %s: deferred\n",
-      dblock->name, filename, strerror(errno));
-    addr->message = string_sprintf("%s director failed to open %s: %s",
-      dblock->name, filename, strerror(errno));
-    addr->special_action = SPECIAL_FREEZE;
+    case ENOENT:          /* File does not exist */
+    DEBUG(2) debug_printf("%s director: no file found\n", dblock->name);
+    yield = FAIL;
+    break;
+
+    case ENOTDIR:         /* Something on the path isn't a directory */
+    if (ob->ignore_enotdir)
+      {
+      DEBUG(2) debug_printf("%s director: non-directory on path: file assumed "
+        "not to exist\n", dblock->name);
+      yield = FAIL;
+      break;
+      }
+    goto DEFAULT_ERROR;
+
+    case EACCES:           /* Permission denied */
+    if (ob->ignore_eacces)
+      {
+      DEBUG(2) debug_printf("%s director: permission denied: file assumed not "
+        "to exist\n", dblock->name);
+      yield = FAIL;
+      break;
+      }
+    /* Else fall through */
+
+    DEFAULT_ERROR:
+    default:
+    addr->message = string_open_failed(errno, "%s (%s director)",
+      filename, dblock->name);
+    DEBUG(2) debug_printf("%s\n", addr->message);
     yield = ERROR;
-    goto RESTORE_UID;             /* skip forward */
+    break;
     }
 
-  DEBUG(2) debug_printf("%s director: no file found\n", dblock->name);
-  yield = FAIL;
   goto RESTORE_UID;               /* skip forward */
   }
 
@@ -517,6 +597,14 @@ if (fread(filebuf, 1, statbuf.st_size, fwd) != statbuf.st_size)
   }
 filebuf[statbuf.st_size] = 0;
 
+/* Don't pass statbuf.st_size directly to debug_printf. On some systems it is a
+long, which may not be the same as an int. */
+
+DEBUG(2)
+  {
+  int size = statbuf.st_size;
+  debug_printf("%d bytes read from %s\n", size, filename);
+  }
 
 /* If the filter option is set, the file is to be interpreted as a filter
 file instead of a straight list of addresses, if it starts with
@@ -538,17 +626,47 @@ for (; *tag != 0; s++, tag++)
   }
 if (*tag == 0) filter_header = TRUE;
 
+/* If qualify_preserve_domain is set, temporarily reset
+qualify_domain_recipient to the current domain so that any unqualified
+addresses will get qualified with the same domain as the incoming
+address. */
+
+if (ob->qualify_preserve_domain) qualify_domain_recipient = addr->domain;
+
 /* Filter interpretation is done by a general function that is also called from
-the filter testing option (-bf). The final argument specifies whether the log
-command can be used; this is OK if we have set the uid to a local user, but not
-otherwise. Set up the value of extracted to be the same as it is from
-parse_extract_addresses(). */
+the filter testing option (-bf). The second-last argument specifies whether the
+log command should be locked out or skipped; this is done when verifying, or if
+we have not set the uid to a local user, or when explicitly configured. Set up
+the value of extracted to be the same as it is from parse_extract_addresses().
+*/
 
 if (filter_header && ob->filter)
   {
+  int action = SPECIAL_NONE;
+  DEBUG(2) debug_printf("file is a filter file\n");
   is_filter = TRUE;
-  extracted = filter_interpret(filebuf, &generated, &delivered, NULL, &error,
-    restore_uid, ob->forbid_filter_log, ob->rewrite)? 0 : -1;
+  extracted = filter_interpret(filebuf, &generated, &delivered,
+    (ob->allow_system_actions)? &action : NULL,
+    &error,
+    verify? 1 :                        /* skip logging if verifying */
+      ob->forbid_filter_logwrite? 2 :  /* lock out if forbidden */
+      restore_uid? 0 : 3,              /* else permit only if uid set */
+    ob->rewrite)? 0 : -1;
+
+  /* Handle special action (will be SPECIAL_NONE unless permitted otherwise).
+  Like the system filter, freeze does not happen after a manual thaw. */
+
+  if (action == SPECIAL_FREEZE && !deliver_manual_thaw)
+    {
+    addr->message = (filter_fmsg == NULL)? "frozen by filter" : filter_fmsg;
+    addr->special_action = SPECIAL_FREEZE;
+    yield = ERROR;
+    }
+  else if (action == SPECIAL_FAIL)
+    {
+    addr->message = (filter_fmsg == NULL)? "rejected by filter" : filter_fmsg;
+    yield = FORCEFAIL;
+    }
   }
 
 /* Otherwise it's a vanilla .forward file; call parse_extract_addresses()
@@ -557,6 +675,8 @@ to get the values. The yield is 0=>OK, -1=>error, +1=> failed to open an
 
 else
   {
+  DEBUG(2) debug_printf("file is not a filter file\n");
+
   /* There is a common function for use by forwarding and aliasing
   directors that extracts a list of addresses from a text string.
   Setting the fourth argument TRUE says that generating no addresses
@@ -571,7 +691,10 @@ else
   seteuid has been used, just try to read anything; otherwise restrict
   to the directory or lock out if none. */
 
-  extracted = parse_extract_addresses(filebuf, &generated, &error,
+  extracted = parse_extract_addresses(filebuf,
+    addr->domain,                              /* to qualify \name */
+    &generated,                                /* where to hang them */
+    &error,                                    /* for errors */
     TRUE,                                      /* no addresses => no error */
     FALSE,                                     /* don't recognize :blackhole: */
     ob->rewrite,                               /* rewrite if configured */
@@ -582,9 +705,11 @@ else
     ob->skip_syntax_errors? &eblock : NULL);
   }
 
-/* The store for holding the forward file is now finished with. */
+/* The store for holding the forward file is now finished with, and we can
+reset the general qualify domain. */
 
 store_free(filebuf);
+qualify_domain_recipient = save_qualify_domain_recipient;
 
 /* At this point we are finished with the .forward file. Close it, and, if
 seteuid was used above, restore the previous effective uid and gid. The dreaded
@@ -616,7 +741,7 @@ if (yield != OK) return yield;
 
 if (extracted != 0)
   {
-  /* If extraction from a filter file failed, it is a "probably user error", to
+  /* If extraction from a filter file failed, it is a "probable user error", to
   use a good old IBM term. Just defer delivery and let the user clean things
   up. */
 
@@ -650,38 +775,27 @@ if (extracted != 0)
   }
 
 /* If skip_syntax_errors was set and there were syntax errors in the list,
-error messages will be present in eblock. Log them. */
+error messages will be present in eblock. Log them and send a message if
+so configured, using a function that is common to aliasfile and forwardfile. */
 
-while (eblock != NULL)
+if (eblock != NULL && !moan_skipped_syntax_errors(dblock->name, filename,
+    eblock, verify? NULL : ob->syntax_errors_to))
   {
-  log_write(0, LOG_MAIN, "%s director: syntax error in forward file %s: "
-    "%s in \"%s\"", dblock->name, filename, eblock->text1, eblock->text2);
-  eblock = eblock->next;
+  addr->special_action = SPECIAL_FREEZE;
+  return ERROR;
   }
 
-/* If this director has a local errors_to setting for where to send error
-messages for its children, expand it, and then check that it is a valid
-address before using it, except when verifying. Otherwise there could be
-directing loops if a silly config is set. */
+/* Set up the errors address for the children of this address. */
 
-if (ob->errors_to != NULL)
-  {
-  char *s = expand_string(ob->errors_to);
-  if (s == NULL)
-    {
-    log_write(0, LOG_MAIN, "%s director failed to expand %s: %s", dblock->name,
-      ob->errors_to, expand_string_message);
-    addr->special_action = SPECIAL_FREEZE;
-    return ERROR;
-    }
+rc = direct_get_errors_address(addr, dblock, "director", verify, &errors_to);
+if (rc != OK) return rc;
 
-  if (verify) errors_to = s; else
-    {
-    char *snew;
-    if (verify_address(s, TRUE, TRUE, NULL, NULL, &snew, FALSE, FALSE) == OK)
-      errors_to = snew;
-    }
-  }
+/* Set up the additional and removeable headers for the children of this
+address. */
+
+rc = direct_get_munge_headers(addr, dblock, "director", &extra_headers,
+  &remove_headers);
+if (rc != OK) return rc;
 
 /* Add the new addresses to the list of new addresses, copying in the
 uid, gid and permission flags for use by pipes and files and autoreplies,
@@ -691,7 +805,7 @@ If the generated address is the same as one of its ancestors, and the
 check_ancestor flag is set, do not use this generated address, but replace it
 with a copy of the input address. This is to cope with cases where A is aliased
 to B and B has a .forward file pointing to A. We can't just pass on the old
-address by returning FAIL, because is must act as a general parent for
+address by returning FAIL, because it must act as a general parent for
 generated addresses, and only get marked "done" when all its children are
 delivered. */
 
@@ -701,6 +815,7 @@ while (generated != NULL)
   generated = next->next;
   next->parent = addr;
   next->ignore_error |= addr->ignore_error;
+  next->start_director = dblock->new;
   addr->child_count++;
   next->next = *addr_new;
   *addr_new = next;
@@ -712,14 +827,16 @@ while (generated != NULL)
       {
       if (strcmp(next->orig, parent->orig) == 0)
         {
-        DEBUG(9) debug_printf("generated parent ignored\n");
+        DEBUG(2) debug_printf("generated parent replaced by child\n");
         next->orig = string_copy(addr->orig);
         break;
         }
       }
     }
 
-  if (errors_to != NULL) next->errors_address = errors_to;
+  next->errors_address = errors_to;
+  next->extra_headers = extra_headers;
+  next->remove_headers = remove_headers;
 
   if (next->pfr)
     {
@@ -751,15 +868,38 @@ while (generated != NULL)
     next->allow_pipe = !ob->forbid_pipe;
     next->allow_file = !ob->forbid_file;
     next->allow_reply = !ob->forbid_reply;
+
+    /* Forwardfile can produce opipes or files or autoreplies; if the transport
+    setting is null, the global setting will get used later. */
+
+    if (next->orig[0] == '|')
+      next->transport = ob->pipe_transport;
+    else
+      if (next->orig[0] == '>') next->transport = ob->reply_transport;
+    else
+      {
+      int len = (int)strlen(next->orig);
+      if (next->orig[len-1] == '/')
+        {
+        next->transport = ob->directory_transport;
+        if (len > 1 && next->orig[len-2] == '/' &&
+            ob->directory2_transport != NULL)
+          next->transport = ob->directory2_transport;
+        }
+      else next->transport = ob->file_transport;
+      }
     }
 
-  DEBUG(2) debug_printf("%s director generated %s%s%s%s%s\n",
+  DEBUG(2) debug_printf("%s director generated %s\n%s%s%s%s%s%s%s",
     dblock->name,
     next->orig,
-    next->pfr? " (pipe, file, or autoreply)" : "",
-    (errors_to != NULL)? " (errors to " : "",
+    next->pfr? "  pipe, file, or autoreply\n" : "",
+    (errors_to != NULL)? "  errors to " : "",
     (errors_to != NULL)? errors_to : "",
-    (errors_to != NULL)? ")" : "");
+    (errors_to != NULL)? "\n" : "",
+    (next->transport == NULL)? "" : "  transport=",
+    (next->transport == NULL)? "" : (next->transport)->name,
+    (next->transport == NULL)? "" : "\n");
   }
 
 /* If the filter interpreter returned "delivered" then we have succeeded
@@ -787,15 +927,17 @@ if (is_filter)
     next->next = *addr_new;
     *addr_new = next;
 
-    if (errors_to != NULL) next->errors_address = errors_to;
+    next->errors_address = errors_to;
+    next->extra_headers = extra_headers;
+    next->remove_headers = remove_headers;
 
-    DEBUG(2) debug_printf("%s director generated %s%s%s%s%s\n",
+    DEBUG(2) debug_printf("%s director generated %s\n%s%s%s%s",
       dblock->name,
       next->orig,
-      next->pfr? " (pipe, file, or autoreply)" : "",
-      (errors_to != NULL)? " (errors to " : "",
+      next->pfr? "  pipe, file, or autoreply\n" : "",
+      (errors_to != NULL)? "  errors to " : "",
       (errors_to != NULL)? errors_to : "",
-      (errors_to != NULL)? ")" : "");
+      (errors_to != NULL)? "\n" : "");
     }
 
   yield = OK;
