@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -11,6 +11,10 @@ that is needed. They don't all need everything, of course, but it's far too
 messy to have each one importing its own list, and anyway, most of them need
 most of these includes. */
 
+/* Assume most systems have statfs() unless os.h undefines this macro */
+
+#define HAVE_STATFS
+
 /* First of all include the os-specific header, which might set things that
 are needed by any of the other headers, including system headers. */
 
@@ -38,53 +42,119 @@ are needed by any of the other headers, including system headers. */
 #else
 #include <limits.h>
 #endif
+
+#include <sys/types.h>
 #include <dirent.h>
 #include <netdb.h>
 #include <pwd.h>
 #include <grp.h>
 #include <syslog.h>
+
 #ifndef NO_SYSEXITS      /* some OS don't have this */
 #include <sysexits.h>
 #endif
+
 #include <sys/time.h>
 #include <sys/param.h>
 #include <sys/resource.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
+
+/* The new standard is statvfs; some OS have statfs. For statvfs the block
+counts must be multiplied by the "fragment size" f_frsize to get the actual
+size. In other cases the value seems to be f_bsize (which is sometimes the only
+block size), so we use a macro to get that instead.
+
+Also arrange to be able to cut it out altogether for way-out OS that don't have
+anything. I've indented a bit here to try to make the mess a bit more
+intelligible. */
+
+#ifdef HAVE_STATFS
+  #ifdef HAVE_SYS_STATVFS_H
+    #include <sys/statvfs.h>
+  #else
+    #define statvfs statfs
+    #define f_frsize f_bsize
+    #ifdef HAVE_SYS_VFS_H
+      #include <sys/vfs.h>
+      #ifdef HAVE_SYS_STATFS_H
+      #include <sys/statfs.h>
+      #endif
+    #endif
+    #ifdef HAVE_SYS_MOUNT_H
+    #include <sys/mount.h>
+    #endif
+  #endif
+#endif
+
+
 #ifndef  SIOCGIFCONF	 /* HACK for SunOS 5 */
 #include <sys/sockio.h>
 #endif
+
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/utsname.h>
 #include <fcntl.h>
+
+/* There's a shambles in IRIX6 - it defines EX_OK in unistd.h which conflicts
+with the definition in sysexits.h. Arrange to preserve it, even though at
+present Exim doesn't actually use it. */
+
+#ifdef EX_OK
+#define SAVE_EX_OK EX_OK
+#undef EX_OK
+#endif
+
 #include <unistd.h>
+
+#ifdef SAVE_EX_OK
+#ifdef EX_OK
+#undef EX_OK
+#endif
+#define EX_OK SAVE_EX_OK
+#endif
+
 #include <utime.h>
 #include <net/if.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
+
+/* The resolv.h header defines __P(x) on some Solaris 2.5.1 systems (without
+checking that it is already defined, in fact). This conflicts with other
+headers that behave likewise (see below), leading to compiler warnings. Arrange
+to undefine it if resolv.h defines it. */
+
+#if defined(__P)
+#define __P_WAS_DEFINED_BEFORE_RESOLV
+#endif
+
 #include <resolv.h>
 
+#if defined(__P) && ! defined (__P_WAS_DEFINED_BEFORE_RESOLV)
+#undef __P
+#endif
+
 /* These three are to support the IP option logging code. Linux is
-different to everyone else. */
+different to everyone else and there are also other systems which don't
+have netinet/ip_var.h, so there's a general macro to control its inclusion. */
 
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 
-#ifndef LINUX_IP_OPTIONS
+#ifndef NO_IP_VAR_H
 #include <netinet/ip_var.h>
 #endif
 
 
 /* The ident.h header defines __P(x) if it is not already defined (for handling
 prototypes). Unfortunately, on some systems, db.h (which is included from
-Exim's dbhdr.h for native Berkeley DB use) also defines it, without checking
-for a previous definition. This leads to a warning message. However, on other
-systems, db.h doesn't define it, expecting it to be done by other headers. To
-avoid warnings and nevertheless keep things working in all cases, we arrange
-to undefine __P if and only if ident.h defines it. */
+Exim's dbfn.h for native Berkeley DB 1.85 use) also defines it, without
+checking for a previous definition. This leads to a warning message. However,
+on other systems, db.h doesn't define it, expecting it to be done by other
+headers. To avoid warnings and nevertheless keep things working in all cases,
+we arrange to undefine __P if and only if ident.h defines it. */
 
 #if defined(__P)
 #define __P_WAS_DEFINED_BEFORE_IDENT
@@ -131,22 +201,29 @@ use different values for specific OS if ever necessary. */
 #define S_ADDR_TYPE u_long
 #endif
 
-/* The header from the regexp package */
+/* The header from the PCRE regex package */
 
-#include "../regexp/regexp.h"
+#include "pcre/pcre.h"
 
 /* Exim includes are in several files */
 
 #include "macros.h"
 #include "config.h"
-#include "dbhdr.h"
+
+#include "dbfn.h"
 #include "structs.h"
 #include "globals.h"
 #include "functions.h"
 
-/* If LOG_FILE_PATH or PID_FILE_PATH have not been defined, set them to
-the null string. */
+/* If SPOOL_DIRECTORY, STDERR_FILE, LOG_FILE_PATH or PID_FILE_PATH have not
+been defined, set them to the null string. */
 
+#ifndef SPOOL_DIRECTORY
+  #define SPOOL_DIRECTORY ""
+#endif
+#ifndef STDERR_FILE
+  #define STDERR_FILE ""
+#endif
 #ifndef LOG_FILE_PATH
   #define LOG_FILE_PATH ""
 #endif
