@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -28,10 +28,6 @@ indented lines. */
 
 
 #include "exim.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <fcntl.h>
 
 
 #define maxsize 10000
@@ -42,6 +38,8 @@ int main(int argc, char **argv)
 int started;
 int count = 0;
 int yield = 0;
+int arg = 1;
+BOOL lowercase = TRUE;
 #ifndef USE_DB
 int is_db = 0;
 struct stat statbuf;
@@ -56,18 +54,29 @@ char  real_dbmname[256];
 char *buffer = malloc(maxsize);
 char *line = malloc(maxsize);
 
+while (argc > 1)
+  {
+  if (strcmp(argv[arg], "-nolc") == 0)
+    {
+    lowercase = FALSE;
+    arg++;
+    argc--;
+    }
+  else break;
+  }
+
 if (argc != 3)
   {
-  printf("usage: exim_dbmbuild <source file> <dbm base name>\n");
+  printf("usage: exim_dbmbuild [-nolc] <source file> <dbm base name>\n");
   exit(1);
   }
 
-if (strcmp(argv[1], "-") == 0) f = stdin; else
+if (strcmp(argv[arg], "-") == 0) f = stdin; else
   {
-  f = fopen(argv[1], "r");
+  f = fopen(argv[arg], "r");
   if (f == NULL)
     {
-    printf("exim_dbmbuild: unable to open %s: %s\n", argv[1], strerror(errno));
+    printf("exim_dbmbuild: unable to open %s: %s\n", argv[arg], strerror(errno));
     exit(1);
     }
   }
@@ -76,20 +85,20 @@ if (strcmp(argv[1], "-") == 0) f = stdin; else
 can be painful! */
 
 #ifdef USE_DB
-if (strcmp(argv[1], argv[2]) == 0)
+if (strcmp(argv[arg], argv[arg+1]) == 0)
   {
   printf("exim_dbmbuild: input and output filenames are the same\n");
   exit(1);
   }
 #endif
 
-strcpy(temp_dbmname, argv[2]);
+strcpy(temp_dbmname, argv[arg+1]);
 strcat(temp_dbmname, ".dbmbuild_temp");
 
 /* It is apparently necessary to open with O_RDWR for this to work
 with gdbm-1.7.3, though no reading is actually going to be done. */
 
-d = EXIM_DBOPEN(temp_dbmname, O_RDWR|O_CREAT|O_EXCL, 0444);
+EXIM_DBOPEN(temp_dbmname, O_RDWR|O_CREAT|O_EXCL, 0644, &d);
 
 if (d == NULL)
   {
@@ -140,7 +149,8 @@ while (fgets(line, maxsize, f) != NULL)
 
   /* A first line must have a name followed by a colon or whitespace or
   end of line, but first finish with a previous line. The key is lower
-  cased - this is what the newaliases program for sendmail does. */
+  cased by default - this is what the newaliases program for sendmail does.
+  However, there's an option not to do this. */
 
   else
     {
@@ -148,6 +158,9 @@ while (fgets(line, maxsize, f) != NULL)
     char *s = line;
     if (started)
       {
+      #ifdef EXIM_CLEAR_DATUMSTRUCT
+      memset(&content, 0, sizeof(content));
+      #endif
       EXIM_DATUM_DATA(content) = buffer;
       EXIM_DATUM_SIZE(content) = bptr - buffer + 1;
       EXIM_DBPUT(d, key, content);
@@ -155,6 +168,9 @@ while (fgets(line, maxsize, f) != NULL)
       bptr = buffer;
       }
     while (*s != 0 && *s != ':' && !isspace(*s)) s++;
+    #ifdef EXIM_CLEAR_DATUMSTRUCT
+    memset(&key, 0, sizeof(key));
+    #endif
     EXIM_DATUM_DATA(key) = keybuffer;
     EXIM_DATUM_SIZE(key) = s - line + 1;
 
@@ -166,8 +182,17 @@ while (fgets(line, maxsize, f) != NULL)
       break;
       }
 
-    for (i = 0; i < EXIM_DATUM_SIZE(key) - 1; i++)
-      keybuffer[i] = tolower(line[i]);
+    if (lowercase)
+      {
+      for (i = 0; i < EXIM_DATUM_SIZE(key) - 1; i++)
+        keybuffer[i] = tolower(line[i]);
+      }
+    else
+      {
+      for (i = 0; i < EXIM_DATUM_SIZE(key) - 1; i++)
+        keybuffer[i] = line[i];
+      }
+
     keybuffer[i] = 0;
     started = 1;
 
@@ -184,6 +209,9 @@ while (fgets(line, maxsize, f) != NULL)
 
 if (started)
   {
+  #ifdef EXIM_CLEAR_DATUMSTRUCT
+  memset(&content, 0, sizeof(content));
+  #endif
   EXIM_DATUM_DATA(content) = buffer;
   EXIM_DATUM_SIZE(content) = bptr - buffer + 1;
   EXIM_DBPUT(d, key, content);
@@ -202,7 +230,7 @@ if (yield == 0)
 
   #ifdef USE_DB
   strcpy(real_dbmname, temp_dbmname);
-  strcpy(buffer, argv[2]);
+  strcpy(buffer, argv[arg+1]);
   if (rename(real_dbmname, buffer) != 0)
     {
     printf("Unable to rename %s as %s\n", real_dbmname, buffer);
@@ -215,7 +243,7 @@ if (yield == 0)
   if (is_db)
     {
     sprintf(real_dbmname, "%s.db", temp_dbmname);
-    sprintf(buffer, "%s.db", argv[2]);
+    sprintf(buffer, "%s.db", argv[arg+1]);
     if (rename(real_dbmname, buffer) != 0)
       {
       printf("Unable to rename %s as %s\n", real_dbmname, buffer);
@@ -228,7 +256,7 @@ if (yield == 0)
   else
     {
     sprintf(real_dbmname, "%s.dir", temp_dbmname);
-    sprintf(buffer, "%s.dir", argv[2]);
+    sprintf(buffer, "%s.dir", argv[arg+1]);
     if (rename(real_dbmname, buffer) != 0)
       {
       printf("Unable to rename %s as %s\n", real_dbmname, buffer);
@@ -236,7 +264,7 @@ if (yield == 0)
       }
 
     sprintf(real_dbmname, "%s.pag", temp_dbmname);
-    sprintf(buffer, "%s.pag", argv[2]);
+    sprintf(buffer, "%s.pag", argv[arg+1]);
     if (rename(real_dbmname, buffer) != 0)
       {
       printf("Unable to rename %s as %s\n", real_dbmname, buffer);
