@@ -2,9 +2,10 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
+#include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
@@ -13,10 +14,11 @@
 #include <sys/stat.h>
 
 #include <pwd.h>                 /* Because used in includes below */
-#include "../regexp/regexp.h"    /* Ditto */
+#include "pcre/pcre.h"           /* Ditto */
 
 #include "macros.h"
-#include "dbhdr.h"
+#include "config.h"
+#include "dbfn.h"
 #include "structs.h"
 #include "functions.h"
 
@@ -42,9 +44,24 @@ formatting can be done by some other program subsequently. Don't use
 a pipe! */
 
 
-/* Need spool_directory from the globals module, but no other globals. */
+/* Need a few globals that the dbfn and string functions use. */
 
 extern char *spool_directory;
+char *spool_directory;
+
+extern volatile int sigalrm_seen;
+volatile int sigalrm_seen;
+
+extern int debug_level;
+int debug_level;
+
+extern BOOL debug_trace_memory;
+BOOL debug_trace_memory;
+
+extern BOOL print_topbitchars;
+BOOL print_topbitchars = FALSE;
+
+/* Functions for handling time */
 
 static char time_buffer[sizeof("09-xxx-1999 hh:mm:ss  ")];
 
@@ -56,13 +73,16 @@ strftime(time_buffer, sizeof(time_buffer), "%d-%b-%Y %H:%M:%S", tmstr);
 return time_buffer;
 }
 
+/* The real code */
 
 int
 main(int argc, char **argv)
 {
-int db_type = 0;
+int dbdata_type = 0;
 int yield = 0;
-EXIM_DB *dbm;
+open_db dbblock;
+open_db *dbm;
+EXIM_CURSOR *cursor;
 char *key;
 char keybuffer[1024];
 
@@ -76,12 +96,12 @@ if (argc != 3)
   exit(1);
   }
 
-if (strcmp(argv[2], "retry") == 0) db_type = type_retry;
-  else if (strcmp(argv[2], "reject") == 0) db_type = type_reject;
-  else if (strncmp(argv[2], "wait-", 5) == 0) db_type = type_wait;
-  else if (strncmp(argv[2], "serialize-", 10) == 0) db_type = type_serialize;
+if (strcmp(argv[2], "retry") == 0) dbdata_type = type_retry;
+  else if (strcmp(argv[2], "reject") == 0) dbdata_type = type_reject;
+  else if (strncmp(argv[2], "wait-", 5) == 0) dbdata_type = type_wait;
+  else if (strncmp(argv[2], "serialize-", 10) == 0) dbdata_type = type_serialize;
 
-if (db_type == 0)
+if (dbdata_type == 0)
   {
   printf("Usage: exim_dumpdb <spool-directory> <database-name>\n");
   printf("<database-name> = retry | wait-<transport> | reject | "
@@ -93,7 +113,7 @@ if (db_type == 0)
 gets a read lock as well. */
 
 spool_directory = argv[1];
-dbm = db_open(argv[2], O_RDONLY);
+dbm = dbfn_open(argv[2], O_RDONLY, &dbblock);
 
 if (dbm == NULL)
   {
@@ -106,13 +126,13 @@ if (dbm == NULL)
 that data is returned in a malloc'ed block, in order that it be
 correctly aligned. */
 
-key = db_scan(dbm, TRUE);
+key = dbfn_scan(dbm, TRUE, &cursor);
 while (key != NULL)
   {
-  db_retry *retry;
-  db_wait *wait;
-  db_reject *reject;
-  db_serialize *serialize;
+  dbdata_retry *retry;
+  dbdata_wait *wait;
+  dbdata_reject *reject;
+  dbdata_serialize *serialize;
   int count_bad = 0;
   int i, length;
   char *t;
@@ -128,22 +148,23 @@ while (key != NULL)
     return 1;
     }
   strcpy(keybuffer, key);
-  value = db_read_with_length(dbm, keybuffer, &length);
+  value = dbfn_read_with_length(dbm, keybuffer, &length);
 
   if (value == NULL)
-    fprintf(stderr, "**** Entry %s not found - shurely shome mistake!\n",
+    fprintf(stderr, "**** Entry \"%s\" was in the key scan, but the record "
+                    "was not found in the file - something is wrong!\n",
       keybuffer);
   else
     {
     /* Note: don't use print_time more than once in one statement, since
     it uses a single buffer. */
 
-    switch(db_type)
+    switch(dbdata_type)
       {
       case type_retry:
-      retry = (db_retry *)value;
+      retry = (dbdata_retry *)value;
       printf("  %s %d %d %s\n%s  ", keybuffer, retry->basic_errno,
-        retry->more_errno, string_printing(retry->text, FALSE),
+        retry->more_errno, retry->text,
         print_time(retry->first_failed));
       printf("%s  ", print_time(retry->last_try));
       printf("%s %s\n", print_time(retry->next_try),
@@ -151,7 +172,7 @@ while (key != NULL)
       break;
 
       case type_wait:
-      wait = (db_wait *)value;
+      wait = (dbdata_wait *)value;
       printf("%s ", keybuffer);
       t = wait->text;
       name[MESSAGE_ID_LENGTH] = 0;
@@ -190,8 +211,8 @@ while (key != NULL)
       break;
 
       case type_reject:
-      reject = (db_reject *)value;
-      if (length < sizeof(db_reject))
+      reject = (dbdata_reject *)value;
+      if (length < sizeof(dbdata_reject))
         printf("%s %s\n", print_time(reject->time_stamp), keybuffer);
       else
         printf("%s %c %s\n", print_time(reject->time_stamp),
@@ -199,16 +220,16 @@ while (key != NULL)
       break;
 
       case type_serialize:
-      serialize = (db_serialize *)value;
+      serialize = (dbdata_serialize *)value;
       printf("%s %s\n", print_time(serialize->time_stamp), keybuffer);
       break;
       }
     store_free(value);
     }
-  key = db_scan(dbm, FALSE);
+  key = dbfn_scan(dbm, FALSE, &cursor);
   }
 
-db_close(dbm);
+dbfn_close(dbm);
 return yield;
 }
 
