@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -22,21 +22,35 @@ static char *expand_string_internal(char *, BOOL, char **, BOOL);
 *            Local statics and tables            *
 *************************************************/
 
+/* Type for main variable table */
+
 typedef struct {
   char *name;
   int   type;
   void *value;
 } var_entry;
 
+/* Type for entries pointing to address/length pairs. Not currently
+in use. */
+
+typedef struct {
+  char **address;
+  int  *length;
+} alblock;
+
+/* Types of table entry */
+
 enum {
   vtype_int,            /* value is address of int */
+  vtype_filter_int,     /* ditto, but recognized only when filtering */
   vtype_string,         /* value is address of string */
   vtype_stringptr,      /* value is address of pointer to string */
   vtype_msgbody,        /* as stringptr, but read when first required */
+  vtype_msgheaders,     /* the message's headers */
   vtype_localpart,      /* extract local part from string */
   vtype_domain,         /* extract domain from string */
-  vtype_recipients,     /* extract recipients from chain of addresses */
-                        /* (not currently in use) */
+  vtype_recipients,     /* extract recipients from recipients list */
+                        /* (enabled only during system filtering */
   vtype_todbsdin,       /* value not used; generate BSD inbox tod */
   vtype_todf,           /* value not used; generate full tod */
   vtype_todl,           /* value not used; generate log tod */
@@ -56,46 +70,120 @@ static var_entry var_table[] = {
   { "compile_date",        vtype_stringptr,   &version_date },
   { "compile_number",      vtype_stringptr,   &version_cnumber },
   { "domain",              vtype_stringptr,   &deliver_domain },
+  { "domain_data",         vtype_stringptr,   &domain_data },
+  { "errmsg_recipient",    vtype_stringptr,   &errmsg_recipient },
   { "home",                vtype_stringptr,   &deliver_home },
   { "host",                vtype_stringptr,   &deliver_host },
+  { "host_address",        vtype_stringptr,   &deliver_host_address },
   { "key",                 vtype_stringptr,   &lookup_key },
   { "local_part",          vtype_stringptr,   &deliver_localpart },
+  { "local_part_data",     vtype_stringptr,   &local_part_data },
   { "local_part_prefix",   vtype_stringptr,   &deliver_localpart_prefix },
   { "local_part_suffix",   vtype_stringptr,   &deliver_localpart_suffix },
   { "message_body",        vtype_msgbody,     &message_body },
+  { "message_headers",     vtype_msgheaders,  NULL },
   { "message_id",          vtype_stringptr,   &message_id },
   { "message_precedence",  vtype_stringptr,   &message_precedence },
   { "message_size",        vtype_int,         &message_size },
+  { "n0",                  vtype_filter_int,  &filter_n[0] },
+  { "n1",                  vtype_filter_int,  &filter_n[1] },
+  { "n2",                  vtype_filter_int,  &filter_n[2] },
+  { "n3",                  vtype_filter_int,  &filter_n[3] },
+  { "n4",                  vtype_filter_int,  &filter_n[4] },
+  { "n5",                  vtype_filter_int,  &filter_n[5] },
+  { "n6",                  vtype_filter_int,  &filter_n[6] },
+  { "n7",                  vtype_filter_int,  &filter_n[7] },
+  { "n8",                  vtype_filter_int,  &filter_n[8] },
+  { "n9",                  vtype_filter_int,  &filter_n[9] },
   { "original_domain",     vtype_stringptr,   &deliver_domain_orig },
   { "original_local_part", vtype_stringptr,   &deliver_localpart_orig },
   { "primary_hostname",    vtype_stringptr,   &primary_hostname },
+  { "prohibition_reason",  vtype_stringptr,   &prohibition_reason },
+  { "qualify_domain",      vtype_stringptr,   &qualify_domain_sender },
+  { "qualify_recipient",   vtype_stringptr,   &qualify_domain_recipient },
+  { "received_for",        vtype_stringptr,   &received_for },
   { "received_protocol",   vtype_stringptr,   &received_protocol },
+  { "recipients",          vtype_recipients,  NULL },
   { "recipients_count",    vtype_int,         &recipients_count },
   { "reply_address",       vtype_reply,       NULL },
   { "return_path",         vtype_stringptr,   &return_path },
+  { "return_size_limit",   vtype_int,         &return_size_limit },
   { "route_option",        vtype_stringptr,   &route_option },
+  { "self_hostname",       vtype_stringptr,   &self_hostname },
   { "sender_address",      vtype_stringptr,   &sender_address },
   { "sender_address_domain", vtype_domain,    &sender_address },
   { "sender_address_local_part", vtype_localpart, &sender_address },
   { "sender_fullhost",     vtype_stringptr,   &sender_fullhost },
+  { "sender_helo_name",    vtype_stringptr,   &sender_helo_name },
   { "sender_host_address", vtype_stringptr,   &sender_host_address },
   { "sender_host_name",    vtype_stringptr,   &sender_host_name },
   { "sender_ident",        vtype_stringptr,   &sender_ident },
+  { "sender_rcvhost",      vtype_stringptr,   &sender_rcvhost },
+  { "sn0",                 vtype_filter_int,  &filter_sn[0] },
+  { "sn1",                 vtype_filter_int,  &filter_sn[1] },
+  { "sn2",                 vtype_filter_int,  &filter_sn[2] },
+  { "sn3",                 vtype_filter_int,  &filter_sn[3] },
+  { "sn4",                 vtype_filter_int,  &filter_sn[4] },
+  { "sn5",                 vtype_filter_int,  &filter_sn[5] },
+  { "sn6",                 vtype_filter_int,  &filter_sn[6] },
+  { "sn7",                 vtype_filter_int,  &filter_sn[7] },
+  { "sn8",                 vtype_filter_int,  &filter_sn[8] },
+  { "sn9",                 vtype_filter_int,  &filter_sn[9] },
   { "spool_directory",     vtype_stringptr,   &spool_directory },
+  { "thisaddress",         vtype_stringptr,   &filter_thisaddress },
   { "tod_bsdinbox",        vtype_todbsdin,    NULL },
   { "tod_full",            vtype_todf,        NULL },
   { "tod_log",             vtype_todl,        NULL },
   { "value",               vtype_stringptr,   &lookup_value },
-  { "version_number",      vtype_stringptr,   &version_string }
+  { "version_number",      vtype_stringptr,   &version_string },
+  { "warnmsg_delay",       vtype_stringptr,   &warnmsg_delay },
+  { "warnmsg_recipient",   vtype_stringptr,   &warnmsg_recipients },
+  { "warnmsg_recipients",  vtype_stringptr,   &warnmsg_recipients }
 };
 
 static int var_table_size = sizeof(var_table)/sizeof(var_entry);
 static char var_buffer[256];
 
+static char *hashcodes = "abcdefghijklmnopqrtsuvwxyz"
+                         "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                         "0123456789";
 
 
 
 
+
+/*************************************************
+*          Check a condition string              *
+*************************************************/
+
+/* This function is called to expand a string, and test the result for a "true"
+or "false" value. Failure of the expansion is panic-worthy.
+
+Arguments:
+  condition     the condition string
+  m1            text to be incorporated in panic error
+  m2            ditto
+
+Returns:        TRUE if condition is met, FALSE if not
+*/
+
+BOOL
+expand_check_condition(char *condition, char *m1, char *m2)
+{
+int rc;
+char *ss = expand_string(condition);
+if (ss == NULL && !expand_string_forcedfail)
+  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "failed to expand condition\"%s\" "
+    "for %s %s: %s", condition, m1, m2, expand_string_message);
+if (ss == NULL) return FALSE;
+rc = ss[0] != 0 && strcmp(ss, "0") != 0 && strcmpic(ss, "no") != 0 &&
+  strcmpic(ss, "false") != 0;
+store_free(ss);
+return rc;
+}
+
+
+
 /*************************************************
 *             Pick out a name from a string      *
 *************************************************/
@@ -279,6 +367,60 @@ return NULL;
 
 
 
+
+/*************************************************
+*   Extract numbered subfield from string        *
+*************************************************/
+
+/* Extracts a numbered field from a string that is divided by tokens - for
+example a line from /etc/passwd is divided by colon characters.  First field is
+numbered one.  Returns NULL if the field number is < 0, or if there are
+insufficient tokens in the string
+
+***WARNING***
+Modifies final argument - this is a dynamically generated string, so that's OK.
+
+Arguments:
+  field       number of field to be extracted,
+                first field = 1, whole string = 0
+  separators  characters that are used to break string into tokens
+  s           points to the string from which to extract the subfield
+
+Returns:      NULL if the field was not found,
+              a pointer to the field's data inside s (modified to add 0)
+*/
+
+static char *
+expand_gettokened (int field, char *separators, char *s)
+{
+int sep = 1;
+char *fieldtext;
+
+/* Invalid field number returns NULL; zero field number returns the whole
+subject string. */
+
+if (field < 0) return NULL;
+if (field == 0) return s;
+
+/* Search for the required field; create it in place. */
+
+while (field-- > 0)
+  {
+  size_t len;
+  if (sep == 0) return NULL;   /* Previous field was end of string */
+  fieldtext = s;
+  len = strcspn(s, separators);
+  sep = s[len];
+  s[len] = 0;
+  s += len + 1;
+  }
+
+return fieldtext;
+}
+
+
+
+
 /*************************************************
 *               Find value of a variable         *
 *************************************************/
@@ -290,18 +432,21 @@ cause chars at the end of the returned string to be disregarded.
 It should already be zero on entry.
 
 Arguments:
-  name      the name of the variable being sought
-  choplen   a pointer to an int which is set if characters at the end
-            of the returned data are to be ignored (typically '\n' at the
-            end of header lines); it should normally be set zero before
-            calling this function
+  name        the name of the variable being sought
+  choplen     a pointer to an int which is set if characters at the end
+              of the returned data are to be ignored (typically '\n' at the
+              end of header lines); it should normally be set zero before
+              calling this function
+  freestore   a pointer to a BOOL which is set TRUE if the value is built
+              in store which should ultimately be freed; it should be set
+              FALSE before calling this function
 
-Returns:    NULL if the variable does not exist, or
-            a pointer to the variable's contents
+Returns:      NULL if the variable does not exist, or
+              a pointer to the variable's contents
 */
 
 static char *
-find_variable(char *name, int *choplen)
+find_variable(char *name, int *choplen, BOOL *freestore)
 {
 int first = 0;
 int last = var_table_size;
@@ -316,6 +461,10 @@ while (last > first)
 
   if (c == 0) switch (var_table[middle].type)
     {
+    case vtype_filter_int:
+    if (!filter_running) return NULL;
+    /* Fall through */
+
     case vtype_int:
     sprintf(var_buffer, "%d", *(int *)(var_table[middle].value)); /* Integer */
     return var_buffer;
@@ -334,6 +483,9 @@ while (last > first)
     if (domain == NULL) return s;
     localpart = domain - 1;
     while (localpart > s && localpart[-1] != ':') localpart--;
+    if (domain - localpart > sizeof(var_buffer) - 1)
+      log_write(0, LOG_PANIC_DIE, "local part longer than %d in string "
+        "expansion", sizeof(var_buffer));
     strncpy(var_buffer, localpart, domain - localpart);
     var_buffer[domain - localpart] = 0;
     return var_buffer;
@@ -344,6 +496,22 @@ while (last > first)
     domain = strrchr(s, '@');
     return (domain == NULL)? "" : domain + 1;
 
+    case vtype_msgheaders:
+      {
+      int ptr = 0;
+      int size = 100;
+      header_line *h;
+      s = store_malloc(size);
+      for (h = header_list; h != NULL; h = h->next)
+        {
+        if (h->type == '*') continue;
+        s = string_cat(s, &size, &ptr, h->text, h->slen);
+        }
+      s[ptr] = 0;
+      }
+    *freestore = TRUE;
+    return s;
+
     case vtype_msgbody:                        /* Pointer to msgbody string */
     ss = (char **)(var_table[middle].value);
     if (*ss == NULL && deliver_datafile >= 0)  /* Read body when needed */
@@ -382,29 +550,33 @@ while (last > first)
       }
     if (s == NULL) return "";
     while (isspace(*s)) s++;
-    if (choplen != NULL) *choplen = 1;     /* Disregard final \n in header */
+
+    /* Disregard final \n in header (but note that isspace() might already
+    have skipped over it if the header is empty). */
+
+    if (choplen != NULL && *s != 0) *choplen = 1;
     return s;
 
-    /****** This code was for an idea that didn't quite make it. Keep it
-    around in case it gets revived one day. ******/
+    /* A recipients list is available only during system message filtering
+    or if there is only one recipient, but not elsewhere. */
 
-    #ifdef NEVER
-    case vtype_recipients:                     /* Get recipients list */
+    case vtype_recipients:
+    if (!enable_dollar_recipients) return NULL; else
       {
       address_item *addr;
       int size = 128;
       int ptr = 0;
+      int i;
       s = store_malloc(size);
-      for (addr = (address_item *)(var_table[middle].value); addr != NULL;
-           addr = addr->next)
+      for (i = 0; i < recipients_count; i++)
         {
-        if (ptr != 0) s = string_cat(s, &size, &ptr, " ", 1);
-        s = string_cat(s, &size, &ptr, addr->orig, (int)strlen(addr->orig));
+        if (i != 0) s = string_cat(s, &size, &ptr, ", ", 2);
+        s = string_cat(s, &size, &ptr, recipients_list[i],
+          (int)strlen(recipients_list[i]));
         }
       s[ptr] = 0;                             /* string_cat leaves room */
       }
     return s;
-    #endif    /* NEVER */
     }
 
   else if (c > 0) first = middle + 1;
@@ -453,7 +625,7 @@ for (h = header_list; h != NULL; h = h->next)
         }
       else
         {
-        char *newyield = store_malloc((int)strlen(yield) + h->slen - len);
+        char *newyield = store_malloc((int)strlen(yield) + h->slen - len + 1);
         strcpy(newyield, yield);
         strcat(newyield, h->text + len);
         if (*freestore) store_free(yield); else *freestore = TRUE;
@@ -498,20 +670,40 @@ for (;;)
 
 s = read_name(name, 256, s, "_");
 
-/* def: tests for a non-zero or non-NULL variable */
+/* def: tests for a non-zero or non-NULL variable, or for an existing
+header */
 
 if (strcmp(name, "def") == 0 && *s == ':')
   {
   char *value;
+  BOOL freestore = FALSE;
+
   s = read_name(name, 256, s+1, "_");
-  value = find_variable(name, NULL);
-  if (value == NULL)
+
+  /* Test for a header's existence */
+
+  if (strncmp(name, "header_", 7) == 0 || strncmp(name, "h_", 2) == 0)
     {
-    expand_string_message = string_sprintf("unknown variable: %s", name);
-    return NULL;
+    s = read_header_name(name, 256, s);
+    value = find_header(name, &freestore);
+    if (yield != NULL) *yield = (value != NULL) == testfor;
     }
-  if (yield != NULL)
-    *yield = (value[0] != 0 && strcmp(value, "0") != 0) == testfor;
+
+  /* Test for a variable's having a non-empty value. */
+
+  else
+    {
+    value = find_variable(name, NULL, &freestore);
+    if (value == NULL)
+      {
+      expand_string_message = string_sprintf("unknown variable: %s", name);
+      return NULL;
+      }
+    if (yield != NULL)
+      *yield = (value[0] != 0 && strcmp(value, "0") != 0) == testfor;
+    }
+
+  if (freestore) store_free(value);
   return s;
   }
 
@@ -540,8 +732,9 @@ variables if it succeeds */
 else if (strcmp(name, "eq") == 0 || strcmp(name, "match") == 0)
   {
   int i;
-  regexp *re;
-  char *rerror;
+  int roffset;
+  pcre *re;
+  const char *rerror;
   char *sub[2];
 
   for (i = 0; i < 2; i++)
@@ -574,33 +767,18 @@ else if (strcmp(name, "eq") == 0 || strcmp(name, "match") == 0)
 
   /* Regular expression match */
 
-  rerror = "";
-  regcomp_error_pointer = &rerror;
-  re = regcomp(sub[1]);
-  regcomp_error_pointer = NULL;
+  re = pcre_compile(sub[1], PCRE_COPT, &rerror, &roffset);
   if (re == NULL)
     {
     expand_string_message = string_sprintf("regular expression error in "
-      "\"%s\": %s", sub[1], rerror);
+      "\"%s\": %s at offset %d", sub[1], rerror, roffset);
     return NULL;
     }
 
-  *yield = regexec(re, sub[0]);
-  if (*yield)
-    {
-    expand_nmax = 0;
-    for (; expand_nmax < NSUBEXP; expand_nmax++)
-      {
-      expand_nstring[expand_nmax] = re->startp[expand_nmax];
-      expand_nlength[expand_nmax] = re->endp[expand_nmax] -
-        expand_nstring[expand_nmax];
-      }
-    expand_nmax--;
-    }
+  *yield = regex_match_and_setup(re, sub[0], 0, -1) == testfor;
 
-  free(re);   /* NB *not* store_free, as got by regcomp */
-
-  store_free(sub[1]);  /* Can't free sub[0]; pointers point to it */
+  store_free(re);
+  store_free(sub[1]);   /* Not sub[0] 'cause things point into it */
   return s;
   }
 
@@ -672,6 +850,9 @@ sequences are:
    ${op:string}          operates on the string value
    ${extract {key} {string}}
                          extracts keyed substring; null if not found
+   ${extract {field} {separator} {string}}
+                         extracts numbered field from separated string;
+                         null if not found; field 0 is the whole string
    ${if cond {s1} {s2}}  conditional; the yielded string is expanded
                          {s2} can be replaced by "fail" or be omitted
 
@@ -691,16 +872,29 @@ Database-query lookups:
                          omitted.
 
 Operators:
+   domain                  extracts domain from an address
+   expand                  expands the string one more time
+   hash_<n>_<m>            hash the string, making one that is of length n,
+                             using m (default 26) characters from hashcodes
    lc                      lowercase the string
    length_<n>              take first n chars only
    l_<n>                   ditto
+   local_part              extracts local part from an address
+   quote                   quote the argument if it contains anything other
+                             than letters, digits, underscores, dots, & hyphens;
+                             quoting means putting inside "" and \-quoting any
+                             \ or " in the string
+   rxquote                 regular expression quote: any non-alphameric is
+                             quoted with \
    substr_<m>_<n>          substring n chars from offset m
    s_<m>_<n>               ditto
-   expand                  expands the string one more time
+                             negative offset works from the rhs
+                             omitted length => rest (either to left or right)
 
 Conditions:
    !cond                   negates condition
    def:variable            variable is defined and not empty
+   def:$h_xxx              header xxx exists
    exists {string}         file exists
    match {string}{re}      regular expression match
    eq {string1}{string2}   strings are equal, case included
@@ -735,7 +929,7 @@ static char *
 expand_string_internal(char *string, BOOL ket_ends, char **left, BOOL skipping)
 {
 int ptr = 0;
-int size = sizeof(string) + 50;
+int size = (int)strlen(string) + 50;
 char *yield = (char *)store_malloc(size);
 char *s = string;
 
@@ -784,16 +978,16 @@ while (*s != 0)
     {
     BOOL freestore = FALSE;
     int choplen = 0;
-    s = read_name(name, 256, s, "_");
+    s = read_name(name, sizeof(name), s, "_");
     if (strncmp(name, "header_", 7) == 0 || strncmp(name, "h_", 2) == 0)
       {
-      s = read_header_name(name, 256, s);
+      s = read_header_name(name, sizeof(name), s);
       value = find_header(name, &freestore);
       choplen = 1;
       }
     else
       {
-      value = find_variable(name, &choplen);
+      value = find_variable(name, &choplen, &freestore);
       if (value == NULL)
         {
         expand_string_message = string_sprintf("unknown variable: %s", name);
@@ -851,7 +1045,12 @@ while (*s != 0)
     expand_string_message = "letter or digit expected after ${";
     goto EXPAND_FAILED;
     }
-  s = read_name(name, 256, s, "_");
+
+  /* Allow "-" in names to cater for substrings with negative
+  arguments. Since we are checking for known names after { this is
+  OK. */
+
+  s = read_name(name, sizeof(name), s, "_-");
 
   /* Handle conditionals - preserve the values of the numerical expansion
   variables in case they get changed by a regular expression match in the
@@ -896,7 +1095,7 @@ while (*s != 0)
       }
     else
       {
-      s = read_name(name, 256, s, "_");
+      s = read_name(name, sizeof(name), s, "_");
       if (strcmp(name, "fail") == 0)
         {
         if (!cond)
@@ -949,7 +1148,7 @@ while (*s != 0)
   if (strcmp(name, "lookup") == 0)
     {
     int stype;
-    int pv = 0;
+    int pv = -1;
     void *handle;
     char *ss, *key, *filename, *sub1, *sub2;
     char *lookup_errmsg = NULL;
@@ -989,14 +1188,20 @@ while (*s != 0)
       expand_string_message = "missing lookup type";
       goto EXPAND_FAILED;
       }
-    s = read_name(name, 256, s, "-_");
+    s = read_name(name, sizeof(name), s, "-_");
     ss = name;
+    while (isspace(*s)) s++;
+
+    /* Handle request for partial lookup */
 
     if (strncmp(ss, "partial", 7) == 0)
       {
       ss += 7;
       if (isdigit (*ss))
+        {
+        pv = 0;
         while (isdigit(*ss)) pv = pv*10 + *ss++ - '0';
+        }
       else pv = 2;
       if (*ss++ != '-')
         {
@@ -1006,44 +1211,34 @@ while (*s != 0)
         }
       }
 
-    if (strcmp(ss, "lsearch") == 0) stype = stype_lsearch;
-    else if (strcmp(ss, "dbm") == 0) stype = stype_dbm;
-    else if (strcmp(ss, "nis") == 0)
+    /* If a single-key lookup type is followed by "*" it requests looking
+    for a default "*" entry if all else fails. If it is followed by "*@" it
+    requests replacing everying before @ by * first. These are independent of
+    partial matching, but are encoded by adding 1024 and 2048 to the partial
+    match value. */
+
+    if (strncmp(s, "*@", 2) == 0)
       {
-      if (have_nis) stype = stype_nis; else
-        {
-        expand_string_message = "lookup type \"nis\" unavailable";
-        goto EXPAND_FAILED;
-        }
+      s += 2;
+      pv += 1024 + 2048;
       }
-    else if (strcmp(ss, "nis0") == 0)
+    else if (*s == '*')
       {
-      if (have_nis) stype = stype_nis0; else
-        {
-        expand_string_message = "lookup type \"nis0\" unavailable";
-        goto EXPAND_FAILED;
-        }
-      }
-    else if (strcmp(ss, "nisplus") == 0)
-      {
-      if (have_nisplus) stype = stype_nisplus; else
-        {
-        expand_string_message = "lookup type \"nisplus\" unavailable";
-        goto EXPAND_FAILED;
-        }
-      }
-    else
-      {
-      expand_string_message = string_sprintf("unknown lookup type \"%s\"",
-        name);
-      goto EXPAND_FAILED;
+      s++;
+      pv += 1024;
       }
 
+    /* Now check for the individual search type. Only those that are actually
+    in the binary are valid. */
+
+    stype = search_findtype(ss, &expand_string_message);
+    if (stype < 0) goto EXPAND_FAILED;
+
     /* Check that a key was provided for those lookup types that need it,
     and was not supplied for those that use the query style, and that
     "partial" was provided only for a non-query lookup. */
 
-    if (stype < stype_querystyle)
+    if (!mac_islookup(stype, lookup_querystyle))
       {
       if (key == NULL)
         {
@@ -1054,7 +1249,7 @@ while (*s != 0)
       }
     else
       {
-      if (pv > 0)
+      if (pv >= 0)
         {
         expand_string_message = string_sprintf("\"partial\" is not permitted "
           "for lookup type \"%s\"", ss);
@@ -1069,10 +1264,9 @@ while (*s != 0)
         }
       }
 
-    /* Get the next string in quotes and expand it. It is the file name for
+    /* Get the next string in brackets and expand it. It is the file name for
     single-key+file lookups, and the whole query otherwise. */
 
-    while (isspace(*s)) s++;
     if (*s != '{') goto EXPAND_FAILED_CURLY;
     filename = expand_string_internal(s+1, TRUE, &s, FALSE);
     if (filename == NULL) goto EXPAND_FAILED;
@@ -1107,6 +1301,13 @@ while (*s != 0)
       if (handle == NULL) goto EXPAND_FAILED;
       lookup_value = search_find(handle, filename, key, stype,
         pv, &expand_setup, &lookup_errmsg);
+
+      if (search_find_defer)
+        {
+        expand_string_message =
+          string_sprintf("lookup of \"%s\" gave DEFER: %s", key, lookup_errmsg);
+        goto EXPAND_FAILED;
+        }
       if (expand_setup > 0) expand_nmax = expand_setup;
       }
 
@@ -1146,7 +1347,7 @@ while (*s != 0)
 
     else
       {
-      s = read_name(name, 256, s, "_");
+      s = read_name(name, sizeof(name), s, "_");
       if (strcmp(name, "fail") == 0)
         {
         if (lookup_value == NULL)
@@ -1189,30 +1390,64 @@ while (*s != 0)
     continue;
     }
 
-  /* Handle keyed substring extraction */
+  /* Handle keyed and numbered substring extraction. They are distinguised
+  by different numbers of arguments. */
 
   if (strcmp(name, "extract") == 0)
     {
     int i;
-    char *sub[3];
-    for (i = 0; i < 2; i++)
+    char *sub[4];
+    for (i = 0; i < 3; i++)
       {
       while (isspace(*s)) s++;
-      if (*s != '{') goto EXPAND_FAILED_CURLY;
-      sub[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
-      if (sub[i] == NULL) goto EXPAND_FAILED;
-      if (*s++ != '}') goto EXPAND_FAILED_CURLY;
+      if (*s == '{')
+	{
+	sub[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
+	if (sub[i] == NULL) goto EXPAND_FAILED;
+	if (*s++ != '}') goto EXPAND_FAILED_CURLY;
+	}
+      else
+	{
+	/* Handle either 2 or 3 arguments for 2 versions of extract */
+	if (i < 2) goto EXPAND_FAILED_CURLY;
+	sub[i] = NULL;
+	}
       }
+
     while (isspace(*s)) s++;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-    sub[2] = expand_getkeyed(sub[0], sub[1]);
+
+    /* If there are three arguments, this is a numbered field extraction,
+    and the first argument must be numerical. If there are two arguments,
+    this is a keyed field extraction. Note that the result of
+    expand_gettokened() is a pointer into a modified sub[2], so it must not
+    itself be freed in that case. */
+
     if (sub[2] != NULL)
       {
-      yield = string_cat(yield, &size, &ptr, sub[2], (int)strlen(sub[2]));
-      store_free(sub[2]);
+      char *endptr;
+      int field_number = strtoul(sub[0], &endptr, 10);
+      if (*endptr != 0)
+        {
+        expand_string_message = string_sprintf("field number \"%s\" for "
+          "extract is not numeric", sub[0]);
+        goto EXPAND_FAILED;
+        }
+      sub[3] = expand_gettokened(field_number, sub[1], sub[2]);
       }
+    else sub[3] = expand_getkeyed(sub[0], sub[1]);
+
+    /* Insert yield if extraction succeeded; otherwise nothing. */
+
+    if (sub[3] != NULL)
+      yield = string_cat(yield, &size, &ptr, sub[3], (int)strlen(sub[3]));
+
+    /* Free temporary store */
+
     store_free(sub[0]);
     store_free(sub[1]);
+    if (sub[2] != NULL) store_free(sub[2]); else store_free(sub[3]);
+
     continue;
     }
 
@@ -1254,24 +1489,112 @@ while (*s != 0)
       continue;
       }
 
-    /* length_n or l_n takes just the first n characters or the whole
-    string, whichever is the shorter; substr_m_n or s_m_n take n characters
-    from offset m; l_n is synonymous with s_0_n. */
+    /* local_part and domain split up an address; parse fail yields null */
+
+    if (strcmp(name, "local_part") == 0 || strcmp(name, "domain") == 0)
+      {
+      char *error;
+      int start, end, domain;
+      char *t = parse_extract_address(sub, &error, &start, &end, &domain,
+        FALSE);
+      if (t != NULL)
+        {
+        if (name[0] == 'l')
+          {
+          if (domain != 0) end = start + domain - 1;
+          yield = string_cat(yield, &size, &ptr, sub+start, end-start);
+          }
+        else if (domain != 0)
+          {
+          domain += start;
+          yield = string_cat(yield, &size, &ptr, sub+domain, end-domain);
+          }
+        }
+      store_free(sub);
+      continue;
+      }
+
+    /* quote sticks it in quotes if it contains anything other than
+    alphamerics, underscore, dot, or hyphen. */
+
+    if (strcmp(name, "quote") == 0)
+      {
+      BOOL needs_quote = FALSE;
+      char *t = sub - 1;
+      while (*(++t) != 0)
+        {
+        if (*t != '_' && *t != '-' && *t != '.' && !isalnum(*t))
+          { needs_quote = TRUE; break; }
+        }
+
+      if (needs_quote)
+        {
+        yield = string_cat(yield, &size, &ptr, "\"", 1);
+        t = sub - 1;
+        while (*(++t) != 0)
+          {
+          if (*t == '\\' || *t == '"')
+            yield = string_cat(yield, &size, &ptr, "\\", 1);
+          yield = string_cat(yield, &size, &ptr, t, 1);
+          }
+        yield = string_cat(yield, &size, &ptr, "\"", 1);
+        }
+      else yield = string_cat(yield, &size, &ptr, sub, (int)strlen(sub));
+      store_free(sub);
+      continue;
+      }
+
+    /* rx quote sticks in \ before any non-alphameric character so that
+    the insertion works in a regular expression. */
+
+    if (strcmp(name, "rxquote") == 0)
+      {
+      char *t = sub -1;
+      while (*(++t) != 0)
+        {
+        if (!isalnum(*t)) yield = string_cat(yield, &size, &ptr, "\\", 1);
+        yield = string_cat(yield, &size, &ptr, t, 1);
+        }
+      store_free(sub);
+      continue;
+      }
+
+    /* length_n or l_n takes just the first n characters or the whole string,
+    whichever is the shorter; substr_m_n, and s_m_n take n characters from
+    offset m; negative m take from the end; l_n is synonymous with s_0_n. If
+    n is omitted in substr it takes the rest, either to the right or to the
+    left. hash_n or h_n makes a hash of length n from the string, yielding
+    n characters from the set a-z; hash_n_m makes a hash of length n, but
+    uses m characters from the set a-zA-Z0-9. */
 
     if (strncmp(name, "length_", 7) == 0 || strncmp(name, "l_", 2) == 0 ||
-        strncmp(name, "substr_", 7) == 0 || strncmp(name, "s_", 2) == 0)
+        strncmp(name, "substr_", 7) == 0 || strncmp(name, "s_", 2) == 0 ||
+        strncmp(name, "hash_",   5) == 0 || strncmp(name, "h_", 2) == 0)
       {
-      int len = 0;
+      int sign = 1;
+      int len = -1;
       int offset = 0;
-      int *pn = (name[0] == 's')? &offset : &len;
+      int hashcount;
+      int *pn;
       int sublen = (int)strlen(sub);
       char *num = strchr(name, '_') + 1;
 
+      /* First argument of substr may be negative; len and hash have only
+      one argument. */
+
+      if (name[0] != 'l')
+        {
+        pn = &offset;
+        if (name[0] == 's' && *num == '-') { sign = -1; num++; }
+        }
+      else { pn = &len; len = 0; }
+
       while (*num != 0)
         {
         if (*num == '_' && pn == &offset)
           {
           pn = &len;
+          len = 0;
           num++;
           }
         else if (!isdigit(*num))
@@ -1282,10 +1605,77 @@ while (*s != 0)
           }
         else *pn = (*pn)*10 + *num++ - '0';
         }
+      offset *= sign;
+
+      /* For a hash, the first argument is in fact the length, and the
+      second is the number of characters to use. */
+
+      if (name[0] == 'h')
+        {
+        hashcount = (len < 0)? 26 : len;
+        if (len > (int)strlen(hashcodes))
+          {
+          expand_string_message =
+            string_sprintf("hash char count too big in \"%s\"", name);
+          goto EXPAND_FAILED;
+          }
+        len = offset;
+        offset = 0;
+        }
+
+      /* Otherwise, a negative offset positions from the rhs (happens only
+      for substr) */
+
+      else if (offset < 0)
+        {
+        offset += sublen;
+
+        /* If the position is before the start, skip to the start, unless
+        length is unset, in which case the substring is null. */
+
+        if (offset < 0)
+          {
+          if (len < 0) len = 0; else len += offset;
+          offset = 0;
+          }
+
+        /* Otherwise an unset length => characters before the offset */
+
+        else if (len < 0)
+          {
+          len = offset;
+          offset = 0;
+          }
+        }
+
+      /* For a non-negative offset, no length means "rest"; just set
+      it to the maximum. */
+
+      else if (len < 0) len = sublen;
+
+      /* Cut length down to maximum possible and if non-zero, get the
+      required characters or compute the hash, free the temporary string,
+      and continue. For a hash, offset is always zero. */
 
       if (sublen < offset + len) len = sublen - offset;
       if (len > 0)
+        {
+        if (name[0] == 'h' && len < sublen)
+          {
+          int c;
+          int i = 0;
+          int j = len;
+          while ((c = (uschar)(sub[j])) != 0)
+            {
+            int shift = (c + j++) & 7;
+            sub[i] ^= (c << shift) | (c >> (8-shift));
+            if (++i >= len) i = 0;
+            }
+          for (i = 0; i < len; i++)
+            sub[i] = hashcodes[(uschar)(sub[i]) % hashcount];
+          }
         yield = string_cat(yield, &size, &ptr, sub + offset, len);
+        }
       store_free(sub);
       continue;
       }
@@ -1311,7 +1701,7 @@ while (*s != 0)
       }
     else
       {
-      value = find_variable(name, &choplen);
+      value = find_variable(name, &choplen, &freestore);
       if (value == NULL)
         {
         expand_string_message = string_sprintf("unknown variable: %s", name);
@@ -1329,6 +1719,8 @@ while (*s != 0)
 
   /* Else there's something wrong */
 
+  expand_string_message = string_sprintf("\"${%s%.3s\" unrecognized", name,
+    (*(--s) == 0)? "" : s);
   goto EXPAND_FAILED;
   }
 
@@ -1353,13 +1745,15 @@ return NULL;
 /* This is the external function call.
 
 Argument: the string to be expanded
-Returns:  the expanded string, or NULL if expansion failed
+Returns:  the expanded string, or NULL if expansion failed; if failure was
+          due to a lookup deferring, search_find_defer will be TRUE
 */
 
 char *
 expand_string(char *string)
 {
 lookup_value = NULL;
+search_find_defer = FALSE;
 return expand_string_internal(string, FALSE, NULL, FALSE);
 }
 
@@ -1373,25 +1767,40 @@ return expand_string_internal(string, FALSE, NULL, FALSE);
 
 #ifdef STAND_ALONE
 
-void
-regerror(char *s)
+
+BOOL
+regex_match_and_setup(pcre *re, char *subject, int options, int setup)
 {
-if (regcomp_error_pointer == NULL)
-  log_write(0, LOG_PANIC_DIE, "regular expression error: %s%s%s", s,
-    (regexp_compiling == NULL)? "" : " while compiling ",
-    (regexp_compiling == NULL)? "" : regexp_compiling);
-else *regcomp_error_pointer = string_copy(s);
+int ovector[2*(EXPAND_MAXN+1)];
+int n = pcre_exec(re, NULL, subject, (int)strlen(subject), PCRE_EOPT|options,
+  ovector, sizeof(ovector)/sizeof(int));
+BOOL yield = n >= 0;
+if (n == 0) n = EXPAND_MAXN + 1;
+if (yield)
+  {
+  int nn;
+  expand_nmax = (setup < 0)? 0 : setup + 1;
+  for (nn = (setup < 0)? 0 : 2; nn < n*2; nn += 2)
+    {
+    expand_nstring[expand_nmax] = subject + ovector[nn];
+    expand_nlength[expand_nmax++] = ovector[nn+1] - ovector[nn];
+    }
+  expand_nmax--;
+  }
+return yield;
 }
 
 
-int main(void)
+int main(int argc, char **argv)
 {
 char buffer[256];
 debug_level = 1;
 debug_file = stderr;
 big_buffer = malloc(big_buffer_size);
 
-printf("Testing string expansion\n");
+if (argc > 1) debug_level = atoi(argv[1]);
+
+printf("Testing string expansion: debug_level = %d\n\n", debug_level);
 
 expand_nstring[1] = "string 1....";
 expand_nlength[1] = 8;
@@ -1402,12 +1811,18 @@ while (fgets(buffer, 256, stdin) != NULL)
   char *yield = expand_string(buffer);
   if (yield != NULL)
     {
-    printf("%s", yield);
+    printf("%s\n", yield);
     store_free(yield);
     }
-  else printf("Failed: %s\n", expand_string_message);
+  else
+    {
+    if (search_find_defer) printf("search_find deferred\n");
+    printf("Failed: %s\n\n", expand_string_message);
+    }
   }
 
+search_tidyup();
+
 return 0;
 }
 
