@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* All the global variables are defined together in this one module, so that
@@ -29,12 +29,18 @@ static void dummy(void) { dummy(); }
 duplicate them here... */
 
 BOOL   accept_8bitmime        = FALSE;
+BOOL   accept_call_bombout    = FALSE;
 int    accept_timeout         = 0;
+address_item *addr_duplicate  = NULL;
+char  *address_directory_transport = "address_directory";
+char  *address_directory2_transport = "address_directory2";
 char  *address_file_transport = "address_file";
 char  *address_pipe_transport = "address_pipe";
 char  *address_reply_transport = "address_reply";
 BOOL   address_test_mode      = FALSE;
 BOOL   admin_user             = FALSE;
+BOOL   allow_unqualified_recipient = TRUE;    /* For local messages */
+BOOL   allow_unqualified_sender = TRUE;       /* Reset for SMTP */
 BOOL   always_bcc             = FALSE;
 int    auto_thaw              = 0;
 
@@ -42,8 +48,15 @@ int    batch_max              = -1;
 char  *bi_command             = NULL;
 char  *big_buffer             = NULL;
 int    big_buffer_size        = BIG_BUFFER_SIZE;
+int    body_linecount         = 0;
 
 BOOL   check_relay            = FALSE;
+int    check_log_inodes       = 0;
+int    check_log_space        = 0;
+int    check_spool_inodes     = 0;
+int    check_spool_space      = 0;
+int    clmacro_count          = 0;
+char  *clmacros[MAX_CLMACROS];
 BOOL   collapse_source_routes = FALSE;
 BOOL   config_changed         = FALSE;
 FILE  *config_file            = NULL;
@@ -51,18 +64,20 @@ char  *config_filename        = CONFIGURE_FILE
 			"\0<-----------Space to patch configure_filename->";
 int    config_lineno          = 0;
 char  *continue_hostname      = NULL;
+BOOL   continue_more          = FALSE;
 int    continue_sequence      = 1;
 char  *continue_transport     = NULL;
 
 BOOL   daemon_listen          = FALSE;
 off_t  data_start_offset      = MESSAGE_ID_LENGTH + 3;
 FILE  *debug_file             = NULL;
-int    debug_level            = 0;
+int    debug_level            = -1;
 pid_t  debug_pid              = 0;
 BOOL   debug_trace_memory     = FALSE;
 transport_info *debug_transport = NULL;
 char  *debug_transport_file   = NULL;
-int    delay_warning          = 24*60*60;
+int    delay_warning[DELAY_WARNING_SIZE] = { DELAY_WARNING_SIZE, 1, 24*60*60 };
+char  *delay_warning_condition= NULL;
 BOOL   delivery_date_remove   = TRUE;
 char  *deliver_in_buffer      = NULL;
 char  *deliver_out_buffer     = NULL;
@@ -74,12 +89,14 @@ BOOL   deliver_freeze         = FALSE;
 int    deliver_frozen_at      = 0;
 char  *deliver_home           = NULL;
 char  *deliver_host           = NULL;
+char  *deliver_host_address   = NULL;
 int    deliver_load_max       = -1;
 char  *deliver_localpart      = NULL;
 char  *deliver_localpart_orig = NULL;
 char  *deliver_localpart_prefix = NULL;
 char  *deliver_localpart_suffix = NULL;
 BOOL   deliver_manual_thaw    = FALSE;
+int    deliver_queue_load_max = -1;
 address_item *deliver_recipients = NULL;
 char  *deliver_selectstring   = NULL;
 director_instance *directors  = NULL;
@@ -90,40 +107,60 @@ director_instance director_defaults = {
    NULL,                      /* info */
    NULL,                      /* private options block pointer */
    NULL,                      /* driver name */
-   NULL,                      /* prefix */
-   NULL,                      /* suffix */
    NULL,                      /* domains */
    NULL,                      /* except_domains */
    NULL,                      /* local_parts */
    NULL,                      /* except_local_parts */
-   NULL,                      /* require_files */
+   NULL,                      /* errors_to */
+   NULL,                      /* extra_headers */
+   NULL,                      /* remove_headers */
    NULL,                      /* re_domains */
    NULL,                      /* re_except_domains */
    NULL,                      /* re_local_parts */
    NULL,                      /* re_except_local_parts */
-   NULL,                      /* transport */
    TRUE,                      /* more */
+   FALSE,                     /* verify_only */
+   TRUE,                      /* verify_sender */
+   TRUE,                      /* verify_recipient */
+   NULL,                      /* transport */
+   NULL,                      /* expanded_transport */
+   NULL,                      /* condition */
+   NULL,                      /* prefix */
+   NULL,                      /* suffix */
+   NULL,                      /* senders */
+   NULL,                      /* except_senders */
+   NULL,                      /* require_files */
+   NULL,                      /* new_director */
+   NULL,                      /* re_senders */
+   NULL,                      /* re_except_senders */
+   NULL,                      /* new (director) */
    FALSE,                     /* unseen */
    FALSE,                     /* prefix_optional */
    FALSE,                     /* suffix_optional */
-   TRUE,                      /* verify_sender */
-   TRUE,                      /* verify_recipient */
    FALSE,                     /* fail_verify_sender */
    FALSE,                     /* fail_verify_recipient */
-   FALSE                      /* verify_only */
+   TRUE                       /* expn */
 };
 
+char  *dns_again_means_nonexist = NULL;
 int    dns_retrans            = 0;
 int    dns_retry              = 0;
+char  *domain_data            = NULL;
 BOOL   dont_deliver           = FALSE;
 BOOL   dot_ends               = TRUE;
+BOOL   drop_cr                = FALSE;
 
+BOOL   enable_dollar_recipients = FALSE;
 BOOL   envelope_to_remove     = TRUE;
+char  *errmsg_file            = NULL;
+char  *errmsg_text            = NULL;
+char  *errmsg_recipient       = NULL;
 int    errno_quota            = ERRNO_QUOTA;
 char  *errors_copy            = NULL;
 int    error_handling         = ERRORS_SENDER;
 char  *errors_address         = "postmaster";
 char  *errors_reply_to        = NULL;
+int    errors_sender_rc       = EXIT_FAILURE;
 
 #ifdef EXIM_GID
 gid_t  exim_gid               = EXIM_GID;
@@ -150,7 +187,12 @@ char  *expand_nstring[EXPAND_MAXN+1];
 BOOL   expand_string_forcedfail = FALSE;
 char  *expand_string_message;
 
+char  *filter_fmsg            = NULL;
+int    filter_n[FILTER_VARIABLE_COUNT];
+BOOL   filter_running         = FALSE;
+int    filter_sn[FILTER_VARIABLE_COUNT];
 char  *filter_test            = NULL;
+char  *filter_thisaddress     = NULL;
 int    finduser_retries       = 0;
 BOOL   forbid_domain_literals = FALSE;
 BOOL   freeze_tell_mailmaster = FALSE;
@@ -158,8 +200,6 @@ BOOL   freeze_tell_mailmaster = FALSE;
 char  *gecos_name             = NULL;
 char  *gecos_pattern          = NULL;
 
-BOOL   have_nis               = HAVE_NIS;
-BOOL   have_nisplus           = HAVE_NISPLUS;
 BOOL   have_seteuid           = HAVE_SETEUID || HAVE_SETRESUID;
 BOOL   header_changed         = FALSE;
 header_line *header_last      = NULL;
@@ -190,11 +230,25 @@ header_name header_names_resent[] = {
   { "Resent-Subject",    14 }
 };
 
-char  *helo_verify_nets       = NULL;
-ip_net_item *helo_verify_netlist = NULL;
-char  *hex_digits             = "0123456789abcdef";
+BOOL   headers_check_syntax   = FALSE;
+BOOL   headers_checks_fail    = TRUE;
+BOOL   headers_sender_verify  = FALSE;
+BOOL   headers_sender_verify_errmsg = FALSE;
+BOOL   helo_strict_syntax     = FALSE;
+BOOL   helo_verify            = FALSE;
+char  *helo_verify_except_hosts = NULL;
+host_item *helo_verify_except_hostlist = NULL;
+char  *helo_verify_except_nets = NULL;
+ip_net_item *helo_verify_except_netlist = NULL;
+char  *hold_domains           = NULL;
+char  *hold_domains_except    = NULL;
+BOOL   host_checking          = FALSE;
+char  *host_findbyaddr_msg    = "";
+char  *host_lookup_nets       = NULL;
+ip_net_item *host_lookup_netlist = NULL;
 
 BOOL   ignore_errmsg_errors   = FALSE;
+int    ignore_errmsg_errors_after = 0;
 int    journal_fd             = -1;
 
 int    keep_malformed         = 4*24*60*60;    /* 4 days */
@@ -207,31 +261,36 @@ BOOL   local_domains_include_host_literals = FALSE;
 BOOL   local_error_message    = FALSE;
 char  *local_interfaces       = NULL;
 ip_address_item *local_interface_data = NULL;
+char  *local_part_data        = NULL;
 BOOL   locally_caseless       = TRUE;
+BOOL   log_all_parents        = FALSE;
+BOOL   log_arguments          = FALSE;
 char  *log_buffer             = NULL;
 char  *log_file_path          = LOG_FILE_PATH
 			  "\0<--------------Space to patch log_file_path->";
 BOOL   log_ip_options         = TRUE;
 int    log_level              = 5;
+BOOL   log_received_sender    = FALSE;
 BOOL   log_received_recipients = FALSE;
-BOOL   log_smtp_confirmation = FALSE;
-BOOL   log_subject = FALSE;
-char  *lookup_key = NULL;
+BOOL   log_refused_recipients = FALSE;
+BOOL   log_rewrites           = FALSE;
+BOOL   log_smtp_confirmation  = FALSE;
+FILE  *log_stderr;
+BOOL   log_subject            = FALSE;
+char  *lookup_key             = NULL;
 
-/* The first argument in this vector is set to exim_path after the
-configuration file has been read and exim_path is fixed. This argument
-list is used for generating error messages. */
-
-char  *mailer_argv[]          = { NULL, "-t", "-oem", "-oi", "-f", "<>",
-                                  (char *)0,    /* -E<id> is added here */
-                                  (char *)0,    /* -d<n> may be added */
-                                  (char *)0,    /* -N may be added */
-                                  (char *)0,    /* -C and its argument may */
-                                  (char *)0,    /*    be added */
-                                  (char *)0 };  /* the terminating zero */
+macro_item *macros            = NULL;
+char  **mailer_argv;
 char  *message_body           = NULL;
 int    message_body_visible   = 500;
 char  *message_filter         = NULL;
+
+char  *message_filter_directory_transport = NULL;
+char  *message_filter_directory2_transport = NULL;
+char  *message_filter_file_transport = NULL;
+char  *message_filter_pipe_transport = NULL;
+char  *message_filter_reply_transport = NULL;
+
 gid_t  message_filter_gid     = 0;
 BOOL   message_filter_gid_set = FALSE;
 uid_t  message_filter_uid     = 0;
@@ -240,10 +299,12 @@ char  *message_id;
 char  *message_id_text        = NULL;
 char   message_id_option[MESSAGE_ID_LENGTH + 3];
 char  *message_id_external;
+int    message_linecount      = 0;
 FILE  *message_log            = NULL;
 char  *message_precedence     = NULL;
 int    message_size           = 0;
 int    message_size_limit     = 0;
+char   message_subdir[2]      = "\0";
 char  *message_reference      = NULL;
 
 uid_t *never_users            = NULL;
@@ -252,61 +313,92 @@ BOOL   nobody_gid_set         = FALSE;
 uid_t  nobody_uid             = 0;
 BOOL   nobody_uid_set         = FALSE;
 
+uid_t  original_euid;
 gid_t  originator_gid;
 char  *originator_login;
 uid_t  originator_uid;
 
 BOOL   parse_allow_group      = FALSE;
 BOOL   parse_found_group      = FALSE;
+BOOL   passed_with_pending    = FALSE;
 char  *percent_hack_domains   = NULL;
 char  *pid_file_path          = PID_FILE_PATH
 			  "\0<--------------Space to patch pid_file_path->";
 BOOL   preserve_message_logs  = FALSE;
 char  *primary_hostname       = NULL;
-char   process_info[256];
+char  *primary_hostname_lc    = NULL;
+BOOL   print_topbitchars      = FALSE;
+char   process_info[PROCESS_INFO_SIZE];
+BOOL   prod_requires_admin    = TRUE;
+char  *prohibition_message    = NULL;
+char  *prohibition_reason     = NULL;
 
 char  *qualify_domain_recipient = NULL;
 char  *qualify_domain_sender  = NULL;
+BOOL   queue_2stage           = FALSE;
 BOOL   queue_run_force        = FALSE;
+BOOL   queue_run_in_order     = FALSE;
 BOOL   queue_run_local        = FALSE;
-BOOL   queue_running          = TRUE;
+BOOL   queue_running          = FALSE;
 int    queue_interval         = -1;
 BOOL   queue_only             = FALSE;
 int    queue_only_load        = -1;
 BOOL   queue_remote           = FALSE;
+char  *queue_remote_except    = NULL;
 int    queue_run_max          = 5;
 BOOL   queue_smtp             = FALSE;
+char  *queue_smtp_except      = NULL;
 
+char  *rbl_domains            = NULL;
+char  *rbl_except_nets        = NULL;
+ip_net_item *rbl_except_netlist = NULL;
+header_line *rbl_header       = NULL;
+BOOL   rbl_reject_recipients  = TRUE;
+BOOL   rbl_warn_header        = TRUE;
+char  *raw_sender             = NULL;
 char **raw_recipients         = NULL;
 
+re_block *re_dns_again_means_nonexist = NULL;
+re_block *re_hold_domains     = NULL;
+re_block *re_hold_domains_except = NULL;
 re_block *re_local_domains    = NULL;
 re_block *re_percent_hack_domains = NULL;
+re_block *re_queue_remote_except = NULL;
+re_block *re_queue_smtp_except = NULL;
+re_block *re_receiver_verify_addresses = NULL;
+re_block *re_receiver_verify_senders = NULL;
+re_block *re_receiver_verify_senders_except = NULL;
+re_block *re_recipients_reject_except = NULL;
 re_block *re_relay_domains    = NULL;
 re_block *re_remote_sort      = NULL;
 re_block *re_sender_accept    = NULL;
 re_block *re_sender_accept_recipients = NULL;
 re_block *re_sender_address_relay = NULL;
 re_block *re_sender_reject    = NULL;
+re_block *re_sender_reject_except = NULL;
 re_block *re_sender_reject_recipients = NULL;
 
 gid_t  real_gid;
 uid_t  real_uid;
-BOOL   really_exim            = FALSE;
+char  *real_sender_address;
+BOOL   really_exim            = TRUE;
 int    received_count         = 0;
+char  *received_for           = NULL;
 
 /* This is the default text for Received headers generated by Exim. The
 date will be automatically added on the end. */
 
 char  *received_header_text   =
-             "Received: "
-             "${if def:sender_fullhost {from ${sender_fullhost} "
-             "${if def:sender_ident {(${sender_ident})}}\n\t}"
-             "{${if def:sender_ident {from ${sender_ident} }}}}"
-             "by ${primary_hostname} "
-             "${if def:received_protocol {with ${received_protocol}}} "
-             "(Exim ${version_number} #${compile_number})\n\t"
-             "id ${message_id}"
-	     "\0<---------------Space to patch received_header_text->";
+    "Received: "
+    "${if def:sender_rcvhost {from ${sender_rcvhost}\n\t}"
+    "{${if def:sender_ident {from ${sender_ident} }}"
+    "${if def:sender_helo_name {(helo=${sender_helo_name})\n\t}}}}"
+    "by ${primary_hostname} "
+    "${if def:received_protocol {with ${received_protocol}}} "
+    "(Exim ${version_number} #${compile_number})\n\t"
+    "${if def:received_for {for $received_for\n\t}}"
+    "id ${message_id}"
+    "\0<---------------Space to patch received_header_text->";
 
 int    received_headers_max   = 30;
 char  *received_protocol      = NULL;
@@ -317,21 +409,27 @@ host_item *receiver_unqualified_hostlist = NULL;
 char  *receiver_unqualified_nets = NULL;
 ip_net_item *receiver_unqualified_netlist = NULL;
 BOOL   receiver_verify        = FALSE;
+char  *receiver_verify_addresses = NULL;
 char  *receiver_verify_except_hosts = NULL;
 host_item *receiver_verify_except_hostlist = NULL;
 char  *receiver_verify_except_nets = NULL;
 ip_net_item *receiver_verify_except_netlist = NULL;
+char  *receiver_verify_senders = NULL;
+char  *receiver_verify_senders_except = NULL;
 int    recipients_count       = 0;
 char **recipients_list        = NULL;
 int    recipients_list_max    = 0;
 int    recipients_max         = 0;
+BOOL   recipients_max_reject  = FALSE;
+char  *recipients_reject_except = NULL;
 BOOL   refuse_all_rcpts       = FALSE;
 BOOL   refuse_ip_options      = TRUE;
 char **regcomp_error_pointer  = NULL;
-char  *regexp_compiling       = NULL;
-regexp *regexp_From           = NULL;
-regexp *regexp_spoolfile;
+pcre  *regex_From             = NULL;
+pcre  *regex_SIZE             = NULL;
+pcre  *regex_ismsgid          = NULL;
 char   *relay_domains         = NULL;
+BOOL   relay_domains_include_local_mx = FALSE;
 BOOL   relay_need_either      = FALSE;
 int    remote_max_parallel    = 1;
 char  *remote_sort            = NULL;
@@ -347,7 +445,7 @@ char  *rfc1413_except_hosts   = NULL;
 host_item *rfc1413_except_hostlist = NULL;
 char  *rfc1413_except_nets    = NULL;
 ip_net_item *rfc1413_except_netlist = NULL;
-int    rfc1413_query_timeout  = 60;
+int    rfc1413_query_timeout  = 30;
 uid_t  root_uid               = ROOT_UID;
 char  *route_option           = NULL;
 
@@ -358,37 +456,49 @@ router_instance router_defaults = {
    NULL,                      /* info */
    NULL,                      /* private options block pointer */
    NULL,                      /* driver name */
-   NULL,                      /* transport instance */
    NULL,                      /* domains */
    NULL,                      /* except_domains */
    NULL,                      /* local_parts */
    NULL,                      /* except_local_parts */
-   NULL,                      /* require_files */
+   NULL,                      /* errors_to */
+   NULL,                      /* extra_headers */
+   NULL,                      /* remove_headers */
    NULL,                      /* re_domains */
    NULL,                      /* re_except_domains */
    NULL,                      /* re_local_parts */
    NULL,                      /* re_except_local_parts */
    TRUE,                      /* more */
-   FALSE,                     /* unseen */
+   FALSE,                     /* verify_only */
    TRUE,                      /* verify_sender */
    TRUE,                      /* verify_recipient */
+   NULL,                      /* transport instance */
+   NULL,                      /* expanded_transport */
+   NULL,                      /* condition */
+   NULL,                      /* senders */
+   NULL,                      /* except_senders */
+   NULL,                      /* require_files */
+   NULL,                      /* re_senders */
+   NULL,                      /* re_except_senders */
+   FALSE,                     /* unseen */
    FALSE,                     /* fail_verify_sender */
    FALSE,                     /* fail_verify_recipient */
-   FALSE,                     /* verify_only */
    FALSE,                     /* pass_on_timeout */
-   NULL,                      /* self */
+   "freeze",                  /* self */
    self_freeze,               /* self_code */
    FALSE                      /* self_rewrite */
 };
 
+BOOL   search_find_defer      = FALSE;
 int    security_level         = 0;
 char  *security_type          = NULL;
+char  *self_hostname          = NULL;
 char  *sender_accept          = NULL;
 char  *sender_accept_recipients = NULL;
 char  *sender_address         = NULL;
 BOOL   sender_address_forced  = FALSE;
 char  *sender_address_relay   = NULL;
 char  *sender_fullhost        = NULL;
+char  *sender_helo_name       = NULL;
 char  *sender_host_accept     = NULL;
 host_item *sender_host_accept_hosts = NULL;
 char  *sender_host_accept_relay = NULL;
@@ -396,7 +506,9 @@ host_item *sender_host_accept_relay_hosts = NULL;
 char  *sender_host_address    = NULL;
 char  *sender_host_name       = NULL;
 char  *sender_host_reject     = NULL;
+char  *sender_host_reject_except = NULL;
 host_item *sender_host_reject_hosts = NULL;
+host_item *sender_host_reject_except_hosts = NULL;
 char  *sender_host_reject_recipients = NULL;
 host_item *sender_host_reject_recipients_hosts = NULL;
 char  *sender_host_reject_relay = NULL;
@@ -409,12 +521,16 @@ ip_net_item *sender_net_accept_nets = NULL;
 char  *sender_net_accept_relay = NULL;
 ip_net_item *sender_net_accept_relay_nets = NULL;
 char  *sender_net_reject      = NULL;
+char  *sender_net_reject_except = NULL;
 ip_net_item *sender_net_reject_nets = NULL;
+ip_net_item *sender_net_reject_except_nets = NULL;
 char  *sender_net_reject_recipients = NULL;
 ip_net_item *sender_net_reject_recipients_nets = NULL;
 char  *sender_net_reject_relay = NULL;
 ip_net_item *sender_net_reject_relay_nets = NULL;
+char  *sender_rcvhost         = NULL;
 char  *sender_reject          = NULL;
+char  *sender_reject_except   = NULL;
 char  *sender_reject_recipients = NULL;
 BOOL   sender_try_verify      = FALSE;
 char  *sender_unqualified_hosts = NULL;
@@ -422,6 +538,7 @@ host_item *sender_unqualified_hostlist = NULL;
 char  *sender_unqualified_nets = NULL;
 ip_net_item *sender_unqualified_netlist = NULL;
 BOOL   sender_verify          = FALSE;
+BOOL   sender_verify_batch    = TRUE;
 char  *sender_verify_except_hosts = NULL;
 host_item *sender_verify_except_hostlist = NULL;
 char  *sender_verify_except_nets = NULL;
@@ -429,46 +546,62 @@ ip_net_item *sender_verify_except_netlist = NULL;
 BOOL   sender_verify_fixup    = FALSE;
 BOOL   sender_verify_log_details = FALSE;
 BOOL   sender_verify_reject   = TRUE;
+volatile BOOL sigalrm_seen    = FALSE;
 char **sighup_argv            = NULL;
 int    smtp_accept_count      = 0;
 int    smtp_accept_max        = 20;
 int    smtp_accept_queue      = 0;
 int    smtp_accept_reserve    = 0;
-char  *smtp_banner            = "${primary_hostname} Exim ${version_number} "
-                            "#${compile_number} ready at ${tod_full}"
+char  *smtp_banner            = "${primary_hostname} ESMTP "
+                            "Exim ${version_number} #${compile_number} "
+                            "${tod_full}"
 			    "\0<---------------Space to patch smtp_banner->";
 BOOL   smtp_batched_input     = FALSE;
 int    smtp_connect_backlog   = 5;
+FILE  *smtp_in                = NULL;
 BOOL   smtp_input             = FALSE;
 int    smtp_load_reserve      = -1;
+FILE  *smtp_out               = NULL;
 int    smtp_port              = -1;
 int    smtp_receive_timeout   = 5*60;
 host_item *smtp_etrn_hostlist = NULL;
 char  *smtp_etrn_hosts        = NULL;
 ip_net_item *smtp_etrn_netlist = NULL;
 char  *smtp_etrn_nets         = NULL;
+BOOL   smtp_etrn_serialize    = TRUE;
+host_item *smtp_expn_hostlist = NULL;
+char  *smtp_expn_hosts        = NULL;
+ip_net_item *smtp_expn_netlist = NULL;
+char  *smtp_expn_nets         = NULL;
 host_item *smtp_reserve_hostlist = NULL;
 char  *smtp_reserve_hosts     = NULL;
 ip_net_item *smtp_reserve_netlist = NULL;
 char  *smtp_reserve_nets      = NULL;
+BOOL   smtp_use_size          = FALSE;
 BOOL   smtp_verify            = FALSE;
+BOOL   split_spool_directory  = FALSE;
 char  *spool_directory        = SPOOL_DIRECTORY
 			  "\0<--------------Space to patch spool_directory->";
-char  *stderr_filename        =
-#ifdef STDERR_FILE
-  STDERR_FILE             "\0<--------------Space to patch stderr_filename->";
-#else
-  NULL;
-#endif
+char  *stderr_filename        = STDERR_FILE
+                          "\0<--------------Space to patch stderr_filename->";
 
 BOOL   strip_excess_angle_brackets = FALSE;
 BOOL   strip_trailing_dot     = FALSE;
 
+transport_instance *transport_address_directory = NULL;
+transport_instance *transport_address_directory2 = NULL;
 transport_instance *transport_address_file = NULL;
 transport_instance *transport_address_pipe = NULL;
 transport_instance *transport_address_reply = NULL;
 transport_instance *transports = NULL;
 
+transport_instance *transport_message_filter_directory = NULL;
+transport_instance *transport_message_filter_directory2 = NULL;
+transport_instance *transport_message_filter_file = NULL;
+transport_instance *transport_message_filter_pipe = NULL;
+transport_instance *transport_message_filter_reply = NULL;
+
+
 transport_instance transport_defaults = {
    NULL,                     /* chain pointer */
    NULL,                     /* name */
@@ -489,17 +622,25 @@ transport_instance transport_defaults = {
    NULL,                     /* current_dir */
    FALSE,                    /* deliver_as_creator */
    FALSE,                    /* initgroups */
+   FALSE,                    /* bsmtp_helo */
    TRUE,                     /* multi-domain */
+   NULL,                     /* shadow */
+   NULL,                     /* shadow_condition */
+   NULL,                     /* filter_command */
+   NULL,                     /* add_headers */
+   NULL,                     /* remove_headers */
    FALSE,                    /* return_output */
    FALSE,                    /* return_fail_output */
    FALSE,                    /* log_output */
    FALSE,                    /* log_fail_output */
+   FALSE,                    /* log_defer_output */
    2,                        /* BOOL, but set neither 1 nor 0 so can detect */
    100                       /* max_addresses */
 };
 
+int    transport_count;
 int    transport_chunk_timeout= 0;
-volatile BOOL transport_sigalrm_seen = FALSE;
+char **transport_filter_argv  = NULL;
 
 tree_node *tree_duplicates    = NULL;
 tree_node *tree_nonrecipients = NULL;
@@ -515,8 +656,37 @@ char  *user_login             = NULL;
 char  *user_name              = NULL;
 BOOL   user_null_sender       = FALSE;
 
+/* A regex for matching a "From_" line in an incoming message, in the form
+
+   From ph10 Fri Jan  5 12:35 GMT 1996
+
+which the "mail" commands send to the MTA (undocumented, of course), or in
+the form
+
+   From ph10 Fri, 7 Jan 97 14:00:00 GMT
+
+which is apparently used by some UUCPs, despite it not being in RFC 976.
+Because of variations in time formats, just match up to the minutes. That
+should be sufficient. Examples have been seen of time fields like 12:1:03,
+so just require one digit for hours and minutes. The weekday is also absent
+in some forms. */
+
+char  *uucp_from_pattern      =
+  "^From\\s+(\\S+)\\s+(?:[a-zA-Z]{3},?\\s+)?"    /* Common start */
+  "(?:"                                          /* Non-extracting bracket */
+  "[a-zA-Z]{3}\\s+\\d?\\d|"                      /* First form */
+  "\\d?\\d\\s+[a-zA-Z]{3}\\s+\\d\\d(?:\\d\\d)?"  /* Second form */
+  ")"                                            /* End alternation */
+  "\\s+\\d\\d?:\\d\\d?";                         /* Start of time */
+
+char  *uucp_from_sender       = "$1";
+
+char  *warnmsg_delay          = NULL;
+char  *warnmsg_file           = NULL;
+char  *warnmsg_recipients     = NULL;
+
 BOOL   verify_only            = FALSE;
-char  *version_copyright      = "Copyright (c) University of Cambridge 1997";
+char  *version_copyright      = "Copyright (c) University of Cambridge 1998";
 char  *version_date           = "?";
 char  *version_cnumber        = "????";
 char  *version_string         = "?";
