@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -36,7 +36,8 @@ va_list ap;
 if (header_last == NULL) return;
 
 va_start(ap, format);
-vsprintf(big_buffer, format, ap);
+if (!string_vformat(big_buffer, BIG_BUFFER_SIZE, format, ap))
+  log_write(0, LOG_PANIC_DIE, "string too long in header_add");
 va_end(ap);
 slen = (int)strlen(big_buffer);
 
