@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for finding hosts, either by gethostbyname, gethostbyaddr, or via
@@ -15,8 +15,8 @@ concerned with hosts and addresses. */
 
 #include "exim.h"
 
-/* Pro tem, while experimenting with IPv6: the definition of T_AAAA may not be
-included in arpa/nameser.h. Fudge it here. */
+/* While IPv6 is yet young: the definition of T_AAAA may not be included in
+arpa/nameser.h. Fudge it here. */
 
 #if HAVE_IPV6
 #ifndef T_AAAA
@@ -24,34 +24,13 @@ included in arpa/nameser.h. Fudge it here. */
 #endif
 #endif
 
+/* Ancient systems (e.g. SunOS4) don't appear to have T_TXT defined in their
+header files. */
 
-#if PACKETSZ < 1024
-  #define MAXPACKET 1024
-#else
-  #define MAXPACKET PACKETSZ
+#ifndef T_TXT
+#define T_TXT 16
 #endif
 
-static unsigned char *dns_answer = NULL;
-static int dns_answerlen = 0;
-
-enum { RESET_NEXT, RESET_ANSWERS, RESET_ADDITIONAL };
-
-enum { DNS_SUCCEED, DNS_NOMATCH, DNS_AGAIN, DNS_FAIL };
-
-
-
-
-/* Structure for holding a partially decoded DNS record; the name has been
-uncompressed, but the data pointer is into the raw data. */
-
-typedef struct {
-  char  name[DNS_MAXNAME];        /* domain name */
-  int   type;                     /* record type */
-  int   size;                     /* size of data */
-  u_char *data;                   /* pointer to data */
-} dns_record;
-
-
 
 
 
@@ -84,6 +63,96 @@ return (unsigned int)(seed >> 16) % limit;
 
 
 
+#ifndef STAND_ALONE    /* Omit when standalone testing */
+
+/*************************************************
+*     Build sender_fullhost and sender_rcvhost   *
+*************************************************/
+
+/* This function is called when sender_host_name and/or sender_helo_name
+have been set. Or might have been set - for a local message read off the spool
+they won't be. In that case, do nothing. Otherwise, set up the fullhost string
+as follows:
+
+(a) No sender_host_name or sender_helo_name: "[ip address]"
+(b) Just sender_host_name: "host_name [ip address]"
+(c) Just sender_helo_name: "(helo_name) [ip address]"
+(d) The two are identical: "host_name [ip address]"
+(e) The two are different: "host_name (helo_name) [ip address]"
+
+This function also builds sender_rcvhost for use in Received: lines, whose
+syntax is a bit different. This value also includes the RFC 1413 identity.
+There wouldn't be two different variables if I had got all this right in the
+first place.
+
+Arguments:  none
+Returns:    nothing
+*/
+
+void
+host_build_sender_fullhost(void)
+{
+if (sender_host_address == NULL) return;
+
+if (sender_fullhost != NULL) store_free(sender_fullhost);
+if (sender_rcvhost != NULL) store_free(sender_rcvhost);
+
+if (sender_host_name == NULL)
+  {
+  if (sender_helo_name == NULL)
+    {
+    sender_fullhost = string_sprintf("[%s]", sender_host_address);
+    sender_rcvhost = (sender_ident == NULL)?
+      string_sprintf("[%s]", sender_host_address) :
+      string_sprintf("[%s] (ident=%s)", sender_host_address, sender_ident);
+    }
+  else
+    {
+    sender_fullhost = string_sprintf("(%s) [%s]", sender_helo_name,
+      sender_host_address);
+
+    sender_rcvhost = (sender_ident == NULL)?
+      string_sprintf("[%s] (helo=%s)", sender_host_address, sender_helo_name) :
+      string_sprintf("[%s] (helo=%s ident=%s)", sender_host_address,
+        sender_helo_name, sender_ident);
+    }
+  }
+else
+  {
+  int len;
+  if (sender_helo_name == NULL ||
+      strcmpic(sender_host_name, sender_helo_name) == 0 ||
+        (sender_helo_name[0] == '[' &&
+         sender_helo_name[(len=(int)strlen(sender_helo_name))-1] == ']' &&
+         strncmpic(sender_helo_name+1, sender_host_address, len - 2) == 0))
+    {
+    sender_fullhost = string_sprintf("%s [%s]", sender_host_name,
+      sender_host_address);
+    sender_rcvhost = (sender_ident == NULL)?
+      string_sprintf("%s ([%s])", sender_host_name, sender_host_address) :
+      string_sprintf("%s ([%s] ident=%s)", sender_host_name,
+        sender_host_address, sender_ident);
+    }
+  else
+    {
+    sender_fullhost = string_sprintf("%s (%s) [%s]", sender_host_name,
+      sender_helo_name, sender_host_address);
+    sender_rcvhost = (sender_ident == NULL)?
+      string_sprintf("%s ([%s] helo=%s)", sender_host_name,
+        sender_host_address, sender_helo_name) :
+      string_sprintf("%s\n\t([%s] helo=%s ident=%s)", sender_host_name,
+        sender_host_address, sender_helo_name, sender_ident);
+    }
+  }
+
+DEBUG(5) debug_printf("sender_fullhost = %s\n", sender_fullhost);
+DEBUG(5) debug_printf("sender_rcvhost = %s\n", sender_rcvhost);
+}
+
+#endif   /* STAND_ALONE */
+
+
+
 
 /*************************************************
 *        Decode actions for self reference       *
@@ -102,17 +171,32 @@ two cases:
      generic configuration option called "self" on each router, which
      can be one of:
 
-     .  freeze: Log the incident, freeze, and return ERROR
-     .  defer:  Log the incident and return ERROR
-     .  send:   Carry on with the delivery regardless - this makes sense only
-                if the SMTP listener on this machine is a differently
-                configured MTA
-     .  <new-domain>: Change the domain to the given domain and return
-        ISLOCAL so it gets passed back to the directors or routers
+     . freeze:                       Log the incident, freeze, and return ERROR
+
+     . defer:                        Log the incident and return DEFER
+
+     . send:                         Carry on with the delivery regardless -
+                                     this makes sense only if the SMTP
+                                     listener on this machine is a differently
+                                     configured MTA
+
+     . fail_soft                     The router just fails, and the address
+                                     gets passed to the next router.
+
+     . reroute:<new-domain>          Change the domain to the given domain
+                                     and return ISLOCAL so it gets passed back
+                                     to the directors or routers.
+
+     . reroute:rewrite:<new-domain>  The same, but headers containing the
+                                     old domain get rewritten.
+
+     . local                         Set addr->forced_local and return ISLOCAL,
+                                     so the address gets passed to the
+                                     directors.
 
 Arguments:
   addr     the address being routed
-  mx       the MX value, or -1 if MX records not used
+  host     the host that is local, with MX set (or -1 if MX not used)
   code     the action to be taken (one of the self_xxx enums) for non-
              source-routed addresses
   rewrite  TRUE if rewriting headers required for ISLOCAL
@@ -120,11 +204,12 @@ Arguments:
 
 Returns:   (1) ISLOCAL for a source-routed address, or
            (2) ERROR, DEFER, ISLOCAL, FAIL, FORCEFAIL, or OK, according
-               to the value of code
+               to the value of code.
 */
 
 int
-host_self_action(address_item *addr, int mx, int code, BOOL rewrite, char *new)
+host_self_action(address_item *addr, host_item *host, int code, BOOL rewrite,
+  char *new)
 {
 char *format, *msg;
 
@@ -157,7 +242,7 @@ if (addr->local_part[0] == ',' || addr->local_part[0] == ':')
 
 /* Handle a non-source-routed address */
 
-if (mx >= 0)
+if (host->mx >= 0)
   {
   format = "lowest MX record for %s points to local host";
   msg = "lowest numbered MX record points to local host";
@@ -181,6 +266,17 @@ switch (code)
   return DEFER;
 
   case self_local:
+  DEBUG(2)
+    {
+    debug_printf(format, addr->route_domain);
+    debug_printf("\n  address passed to directors: self_host = %s\n",
+      host->name);
+    }
+  addr->forced_local = TRUE;
+  addr->self_hostname = string_copy(host->name);
+  return ISLOCAL;
+
+  case self_reroute:
   DEBUG(2)
     {
     debug_printf(format, addr->route_domain);
@@ -292,7 +388,7 @@ for (cp = buf; cp < buf + ifc.ifc_len; cp +=
   /* If not an IP interface, skip */
 
   if (ifr->ifr_addr.sa_family != AF_INET
-  #if HAVE_IPv6
+  #if HAVE_IPV6
     && ifr->ifr_addr.sa_family != AF_INET6
   #endif
     ) continue;
@@ -403,7 +499,7 @@ either be given directly, or passed over in a sockaddr structure.
 Arguments:
   type       if < 0 then arg points to a sockaddr, else
              either AF_INET or AF_INET6
-  arg        points to a sockaddr if type is < 0,
+  arg        points to a sockaddr if type is < 0, or
              points to an IPv4 address (32 bits), or
              points to an IPv6 address (128 bits)
   buffer     if NULL, the result is returned in gotten store;
@@ -418,7 +514,10 @@ host_ntoa(int type, const void *arg, char *buffer)
 char *yield;
 
 /* The new world. It is annoying that we have to fish out the address from
-different places in the block, depending on what kind of address it is. */
+different places in the block, depending on what kind of address it is. It
+is also a pain that inet_ntop() returns a const char *, whereas the IPv4
+function inet_ntoa() returns just char *, and some picky compilers insist
+on warning if one assigns a const char * to a char *. Hence the casts. */
 
 #if HAVE_IPV6
 char addr_buffer[46];
@@ -428,18 +527,18 @@ if (type < 0)
   if (family == AF_INET6)
     {
     struct sockaddr_in6 *sk = (struct sockaddr_in6 *)arg;
-    yield = inet_ntop(family, &(sk->sin6_addr), addr_buffer,
+    yield = (char *)inet_ntop(family, &(sk->sin6_addr), addr_buffer,
       sizeof(addr_buffer));
     }
   else
     {
     struct sockaddr_in *sk = (struct sockaddr_in *)arg;
-    yield = inet_ntop(family, &(sk->sin_addr), addr_buffer,
+    yield = (char *)inet_ntop(family, &(sk->sin_addr), addr_buffer,
       sizeof(addr_buffer));
     }
   }
 else
-  yield = inet_ntop(type, arg, addr_buffer, sizeof(addr_buffer));
+  yield = (char *)inet_ntop(type, arg, addr_buffer, sizeof(addr_buffer));
 #else
 
 /* The old world */
@@ -554,6 +653,81 @@ return v4offset+1;
 
 
 
+/*************************************************
+*       Convert address + mask to binary         *
+*************************************************/
+
+/* This function is given the textual form of an address and a mask, separated
+by a slash. It converts both to binary and returns them in vectors of ints.
+The number of ints used is returned. Both IPv4 and IPv6 addressing modes are
+supported.
+
+Arguments:
+  s         the input string
+  address   points to vector of int for address
+  mask      points to vector of int for mask
+  error     where to put a message on error
+
+Returns:    number of vector words used, or 0 if there was a problem
+*/
+
+int
+host_amton(char *s, int *address, int *mask, char **error)
+{
+int size;
+char *slash = strchr(s, '/');
+
+if (slash == NULL)
+  {
+  *error = "\"/\" missing";
+  return 0;
+  }
+
+/* Temporarily terminate the string at the slash, and turn the initial
+address into binary. */
+
+*slash = 0;
+size = host_aton(s, address);
+*slash = '/';
+
+/* For IPv4 addresses, the mask may be another IPv4 address, containing dots,
+or an integer. The syntax has been checked. */
+
+if (size == 1 && strchr(slash+1, '.') != NULL) host_aton(slash+1, mask);
+
+/* For IPv6 addresses, and optionally for IPv4 addresses, the mask is just
+an integer, and we have to make up the bit pattern. */
+
+else
+  {
+  int i;
+  int mlen = atoi(slash+1);
+  for (i = 0; i < 4; i++)
+    {
+    if (mlen >= 32)
+      {
+      mask[i] = -1;
+      mlen -= 32;
+      }
+    else if (mlen == 0) mask[i] = 0;
+    else
+      {
+      mask[i] = (-1) << (32 - mlen);
+      mlen = 0;
+      }
+    }
+  }
+
+/* Return the number of ints used */
+
+return size;
+}
+
+
+
+
+
+
 /*************************************************
 *       Scan host list for local hosts           *
 *************************************************/
@@ -574,6 +748,8 @@ Otherwise, give a success code if at least one host address has been found.
 Arguments:
   host        the first host in the chain
   last        the last host in the chain
+  removed     if not NULL, set TRUE if some local addresses were removed
+                from the list
 
 Returns:
   HOST_FOUND       if there is at least one host with an IP address on the chain
@@ -586,12 +762,14 @@ Returns:
 */
 
 int
-host_scan_for_local_hosts(host_item *host, host_item *last)
+host_scan_for_local_hosts(host_item *host, host_item *last, BOOL *removed)
 {
 int yield = HOST_FIND_FAILED;
 host_item *prev = NULL;
 host_item *h;
 
+if (removed != NULL) *removed = FALSE;
+
 if (local_interface_data == NULL) local_interface_data = host_find_interfaces();
 
 for (h = host; h != last->next; h = h->next)
@@ -618,6 +796,7 @@ for (h = host; h != last->next; h = h->next)
             debug_printf("  %s %s %d\n", h->name, h->address, h->mx);
           }
 
+        if (removed != NULL) *removed = TRUE;
         prev->next = NULL;
         return yield;
         }
@@ -650,40 +829,35 @@ single string. Lower case it.
 Argument: an IPv4 address in dotted-quad form OR
           an IPv6 address in colon-separated form
 Returns:  the name of the host or NULL if not found
-*/
+
+In addition, the variable host_findbyaddr_msg is set to an empty string on
+sucess, or to a reason for the failure otherwise, in a form suitable for
+tagging onto an error message. */
 
 char *
 host_find_byaddr(char *address)
 {
 char *s, *t, *yield;
 struct hostent *hosts;
+struct in_addr addr;
+
+host_findbyaddr_msg = "";
+
+HDEBUG(2) debug_printf("looking up host name for %s\n", address);
 
 /* Lookup on IPv6 system */
 
 #if HAVE_IPV6
-
-/* Sun's release 5 software hasn't got the new gethostbyaddr; instead it
-has the older addr2hostname function. We can't just macro-define gethostbyaddr
-because that would screw up the IPv4 case. As a fudge, just test for the
-definition of inet_pton as a macro - because that is also used for release
-5. Once the macros are taken out of os.h-SunOS5 (expected soon, with release
-6 of the software) this fudge can be removed. */
-
-#ifdef inet_pton
-#define gethostbyaddr(a,b,c) addr2hostname(a, b, c)
-#endif
-
 if (strchr(address, ':') != NULL)
   {
   struct in6_addr addr6;
-  if (!inet_pton(AF_INET6, address, &addr6))
+  if (inet_pton(AF_INET6, address, &addr6) != 1)
     log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", address);
   hosts = gethostbyaddr((char *)(&addr6), sizeof(addr6), AF_INET6);
   }
 else
   {
-  struct in_addr addr;
-  if (!inet_pton(AF_INET, address, &addr))
+  if (inet_pton(AF_INET, address, &addr) != 1)
     log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", address);
   hosts = gethostbyaddr((char *)(&addr), sizeof(addr), AF_INET);
   }
@@ -691,7 +865,6 @@ else
 /* Do lookup on IPv4 system */
 
 #else
-struct in_addr addr;
 addr.s_addr = (S_ADDR_TYPE)inet_addr(address);
 hosts = gethostbyaddr((char *)(&addr), sizeof(addr), AF_INET);
 #endif
@@ -700,7 +873,8 @@ hosts = gethostbyaddr((char *)(&addr), sizeof(addr), AF_INET);
 
 if (hosts == NULL)
   {
-  DEBUG(2) debug_printf("host_find_by_addr failed for %s\n", address);
+  HDEBUG(2) debug_printf("host lookup failed\n");
+  host_findbyaddr_msg = " (failed to find sending host name from IP address)";
   return NULL;
   }
 
@@ -710,7 +884,7 @@ s = (char *)hosts->h_name;
 t = yield = store_malloc((int)strlen(s) + 1);
 while (*s != 0) *t++ = tolower(*s++);
 *t = 0;
-DEBUG(2) debug_printf("host_find_by_addr yields %s\n", yield);
+HDEBUG(2) debug_printf("host lookup yielded %s\n", yield);
 return yield;
 }
 
@@ -782,7 +956,7 @@ for (i = 1; i <= times; af = AF_INET, i++)
     host->name = string_copy((char *)hostdata->h_name);
   if (fully_qualified_name != NULL) *fully_qualified_name = host->name;
 
-  /* Get the list of addresses. Ipv4 and IPv6 addresses can be distinguished
+  /* Get the list of addresses. IPv4 and IPv6 addresses can be distinguished
   by their different lengths. */
 
   addrlist = hostdata->h_addr_list;
@@ -826,7 +1000,7 @@ if (host->address == NULL) return HOST_FIND_FAILED;
 
 /* Now check to see if this is the local host. */
 
-yield = host_scan_for_local_hosts(host, last);
+yield = host_scan_for_local_hosts(host, last, NULL);
 
 DEBUG(8)
   {
@@ -846,360 +1020,6 @@ return yield;
 
 
 
-/*************************************************
-*       Get next DNS record from answer block    *
-*************************************************/
-
-/* Call this with reset == RESET_ANSWERS to scan the answer block, reset ==
-RESET_ADDITIONAL to scan the additional records, and reset == RESET_NEXT to get
-the next record. The result is in static storage which must be copied if it is
-to be preserved.
-
-Argument:   option specifing what portion to scan, as described above
-Returns:    next dns record, or NULL when no more
-*/
-
-static dns_record *
-next_rr(int reset)
-{
-static int rrcount;
-static dns_record rr;
-static unsigned char *ptr;
-
-int namelen;
-HEADER *h = (HEADER *)dns_answer;
-
-
-/* Reset the saved data when requested to, and skip to the first required RR */
-
-if (reset != RESET_NEXT)
-  {
-  rrcount = ntohs(h->qdcount);
-  ptr = dns_answer + sizeof(HEADER);
-
-  /* Skip over questions; failure to expand the name just gives up */
-
-  while (rrcount-- > 0)
-    {
-    namelen = dn_expand(dns_answer, dns_answer + dns_answerlen, ptr,
-      (DN_EXPAND_ARG4_TYPE) &(rr.name), DNS_MAXNAME);
-    if (namelen < 0) { rrcount = 0; return NULL; }
-    ptr += namelen + 4;    /* skip name & type & class */
-    }
-
-  /* Get the number of answer records. */
-
-  rrcount = ntohs(h->ancount);
-
-  /* Skip over answers and NS records if wanting to look at the additional
-  records. */
-
-  if (reset == RESET_ADDITIONAL)
-    {
-    rrcount += ntohs(h->nscount);
-    while (rrcount-- > 0)
-      {
-      namelen = dn_expand(dns_answer, dns_answer + dns_answerlen, ptr,
-        (DN_EXPAND_ARG4_TYPE) &(rr.name), DNS_MAXNAME);
-      if (namelen < 0) { rrcount = 0; return NULL; }
-      ptr += namelen + 8;     /* skip name & type & class & TTL */
-      GETSHORT(rr.size, ptr); /* size of data portion */
-      ptr += rr.size;         /* skip over it */
-      }
-    rrcount = ntohs(h->arcount);
-    }
-  }
-
-
-/* The variable "ptr" is now pointing at the next RR, and rrcount contains the
-number of RR records left. */
-
-if (rrcount-- <= 0) return NULL;
-
-/* Expand the RR domain name into the slot in the static rr structure; if this
-fails, behave as if no more records (something safe). */
-
-namelen = dn_expand(dns_answer, dns_answer + dns_answerlen, ptr,
-  (DN_EXPAND_ARG4_TYPE) &(rr.name), DNS_MAXNAME);
-if (namelen < 0) { rrcount = 0; return NULL; }
-
-/* Move the pointer past the name and fill in the rest of the data structure
-from the following bytes. */
-
-ptr += namelen;
-GETSHORT(rr.type, ptr);        /* Record type */
-ptr += 6;                      /* Don't want class or TTL */
-GETSHORT(rr.size, ptr);        /* Size of data portion */
-rr.data = ptr;                 /* The record's data follows */
-ptr += rr.size;                /* Advance to next RR */
-
-/* Return a pointer to the static structure. */
-
-return &rr;
-}
-
-
-
-
-/*************************************************
-*        Initializer and configure resolver      *
-*************************************************/
-
-/* Initialize the resolver and the storage for holding DNS answers if this is
-the first time we have been here, and set the resolver options.
-
-Arguments:
-  qualify_single    TRUE to set the RES_DEFNAMES option
-  search_parents    TRUE to set the RES_DNSRCH option
-
-Returns:            nothing
-*/
-
-static void
-dns_init(BOOL qualify_single, BOOL search_parents)
-{
-if (dns_answer == NULL)
-  {
-  dns_answer = store_malloc(MAXPACKET);
-  res_init();
-  DEBUG(11) _res.options |= RES_DEBUG;
-  }
-_res.options &= ~(RES_DNSRCH | RES_DEFNAMES);
-_res.options |= (qualify_single? RES_DEFNAMES : 0) |
-                (search_parents? RES_DNSRCH : 0);
-
-if (dns_retrans > 0) _res.retrans = dns_retrans;
-if (dns_retry > 0) _res.retry = dns_retry;
-}
-
-
-
-
-/*************************************************
-*            Turn DNS type into text             *
-*************************************************/
-
-/* Turn the coded record type into a string for printing.
-
-Argument:   record type
-Returns:    pointer to string
-*/
-
-static char *
-dns_text_type(int t)
-{
-switch(t)
-  {
-  case T_A:    return "A";
-  case T_MX:   return "MX";
-  #if HAVE_IPV6
-  case T_AAAA: return "AAAA";
-  #endif
-  default:     return "?";
-  }
-}
-
-
-
-/*************************************************
-*              Do basic DNS lookup               *
-*************************************************/
-
-/* Call the resolver to look up the given domain name, using the given type,
-and check the result.
-
-Arguments:
-  name      name to look up
-  type      type of DNS record required (T_A, T_MX, etc)
-
-Returns:    DNS_SUCCEED   successful lookup
-            DNS_NOMATCH   name not found, or no data found for the given type
-            DNS_AGAIN     soft failure, try again later
-            DNS_FAIL      DNS failure
-*/
-
-static int
-dns_basic_lookup(char *name, int type)
-{
-dns_answerlen = res_search(name, C_IN, type, dns_answer, MAXPACKET);
-
-if (dns_answerlen < 0) switch (h_errno)
-  {
-  case HOST_NOT_FOUND:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave HOST_NOT_FOUND\n",
-    name, dns_text_type(type));
-  return DNS_NOMATCH;
-
-  case TRY_AGAIN:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave TRY_AGAIN\n",
-    name, dns_text_type(type));
-  return DNS_AGAIN;
-
-  case NO_RECOVERY:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave NO_RECOVERY\n",
-    name, dns_text_type(type));
-  return DNS_FAIL;
-
-  case NO_DATA:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave NO_DATA\n",
-    name, dns_text_type(type));
-  return DNS_NOMATCH;
-
-  default:
-  DEBUG(8) debug_printf("DNS lookup of %s (%s) gave unknown DNS error %d\n",
-    name, dns_text_type(type), h_errno);
-  return DNS_FAIL;
-  }
-
-DEBUG(8) debug_printf("DNS lookup of %s (%s) succeeded\n",
-  name, dns_text_type(type));
-
-return DNS_SUCCEED;
-}
-
-
-
-
-/************************************************
-*        Do a DNS lookup and handle CNAMES      *
-************************************************/
-
-/* Look up the given domain name, using the given type. Follow CNAMEs if
-necessary, but only so many times. There aren't supposed to be CNAME chains in
-the DNS, but you are supposed to cope with them if you find them.
-
-The assumption is made that if the resolver gives back records of the requested
-type *and* a CNAME, we don't need to make another call to look up the CNAME. I
-can't see how it could return only some of the right records. If it's done a
-CNAME lookup in the past, it will have all of them; if not, it won't return
-any.
-
-If fully_qualified_name is not NULL, set it to point to the full name returned
-by the resolver, if this is different to what it is given, unless the returned
-name starts with "*" as some nameservers seem to be returning wildcards in this
-form.
-
-Arguments:
-  name                  domain name to look up
-  type                  DNS record type (T_A, T_MX, etc)
-  fully_qualified_name  if not NULL, return the returned name here
-
-Returns:                DNS_SUCCEED   successful lookup
-                        DNS_NOMATCH   name not found, or no data found
-                        DNS_AGAIN     soft failure, try again later
-                        DNS_FAIL      DNS failure
-*/
-
-static int
-dns_lookup(char *name, int type, char **fully_qualified_name)
-{
-int i;
-char *orig_name = name;
-
-/* Loop to follow CNAME chains so far, but no further... */
-
-for (i = 0; i < 10; i++)
-  {
-  char data[256];
-  dns_record *rr, cname_rr, type_rr;
-  int datalen, rc;
-
-  /* DNS lookup failures get passed straight back. */
-
-  if ((rc = dns_basic_lookup(name, type)) != DNS_SUCCEED) return rc;
-
-  /* We should have either records of the required type, or a CNAME record,
-  or both. We need to know whether both exist for getting the fully qualified
-  name, but avoid scanning more than necessary. Note that we must copy the
-  contents of any rr blocks returned by next_rr() as they use the same static
-  area. */
-
-  cname_rr.data = type_rr.data = NULL;
-  for (rr = next_rr(RESET_ANSWERS); rr != NULL; rr = next_rr(RESET_NEXT))
-    {
-    if (rr->type == type)
-      {
-      if (type_rr.data == NULL) type_rr = *rr;
-      if (cname_rr.data != NULL) break;
-      }
-    else if (rr->type == T_CNAME) cname_rr = *rr;
-    }
-
-  /* If a CNAME was found, take the fully qualified name from it; otherwise
-  from the first data record, if present. */
-
-  if (fully_qualified_name != NULL)
-    {
-    if (cname_rr.data != NULL)
-      {
-      if (strcmp(cname_rr.name, *fully_qualified_name) != 0 &&
-          cname_rr.name[0] != '*')
-        *fully_qualified_name = string_copy(cname_rr.name);
-      }
-    else if (type_rr.data != NULL)
-      {
-      if (strcmp(type_rr.name, *fully_qualified_name) != 0 &&
-          type_rr.name[0] != '*')
-        *fully_qualified_name = string_copy(type_rr.name);
-      }
-    }
-
-  /* If any data records of the correct type were found, we are done. */
-
-  if (type_rr.data != NULL) return DNS_SUCCEED;
-
-  /* If there are no data records, we need to re-scan the DNS using the
-  domain given in the CNAME record, which should exist (otherwise we should
-  have had a failure from dns_lookup). However code against the possibility of
-  its not existing. */
-
-  if (cname_rr.data == NULL) return DNS_FAIL;
-  datalen = dn_expand(dns_answer, dns_answer + dns_answerlen,
-    cname_rr.data, (DN_EXPAND_ARG4_TYPE)data, 256);
-  if (datalen < 0) return DNS_FAIL;
-  name = data;
-  }
-
-/*Control reaches here after 10 times round the CNAME loop. Something isn't
-right... */
-
-log_write(0, LOG_MAIN, "CNAME loop for %s encountered", orig_name);
-return DNS_FAIL;
-}
-
-
-
-/*************************************************
-*          Get address from DNS record           *
-*************************************************/
-
-/* The record type is either T_A for an IPv4 address or T_AAAA for an IPv6
-address.
-
-Argument:    the DNS record
-Returns:     pointer to textual representation of address, in new store
-*/
-
-static char *
-address_from_rr(dns_record *rr)
-{
-#if HAVE_IPV6
-if (rr->type == T_AAAA)
-  {
-  char buffer[50];
-  inet_ntop(AF_INET6, rr->data, buffer, 50);
-  return string_copy(buffer);
-  }
-else
-#endif
-
-return string_sprintf("%d.%d.%d.%d",
-  rr->data[0], rr->data[1], rr->data[2], rr->data[3]);
-}
-
-
-
-
 /*************************************************
 *        Fill in a host address from the DNS     *
 *************************************************/
@@ -1223,7 +1043,9 @@ records.
 
 Arguments:
   host                  points to initial host item
-  fully_qualified_name  if not NULL, return fully qualified name here
+  fully_qualified_name  if not NULL, return fully qualified name here if
+                          the contents are different (i.e. it must be preset
+                          to something)
 
 Returns:       HOST_FIND_FAILED     couldn't find A record
                HOST_FIND_AGAIN      try again later
@@ -1231,7 +1053,7 @@ Returns:       HOST_FIND_FAILED     couldn't find A record
 */
 
 static int
-dns_set_address(host_item *host, char **fully_qualified_name)
+set_address_from_dns(host_item *host, char **fully_qualified_name)
 {
 dns_record *rr;
 host_item *last = NULL;
@@ -1262,11 +1084,12 @@ for (; i >= 0; type = T_A, i--)
   /* Fill in the given host item with the first address found; create
   additional items for any others. */
 
-  for (rr = next_rr(RESET_ANSWERS); rr != NULL; rr = next_rr(RESET_NEXT))
+  for (rr = dns_next_rr(RESET_ANSWERS); rr != NULL;
+       rr = dns_next_rr(RESET_NEXT))
     {
     if (rr->type == type)
       {
-      char *address = address_from_rr(rr);
+      char *address = dns_address_from_rr(rr);
 
       if (last == NULL)
         {
@@ -1345,6 +1168,7 @@ Arguments:
   qualify_single        if TRUE, ask DNS to qualify single-component names
   search_parents        if TRUE, asd DNS to search parent domains
   fully_qualified_name  if not NULL, return fully-qualified name
+  removed               set TRUE if local host was removed from the list
 
 Returns:                HOST_FIND_FAILED  Failed to find the host or domain
                         HOST_FIND_AGAIN   Could not resolve at this time
@@ -1357,7 +1181,8 @@ Returns:                HOST_FIND_FAILED  Failed to find the host or domain
 
 int
 host_find_bydns(host_item *host, BOOL mx_only, BOOL a_only,
-  BOOL qualify_single, BOOL search_parents, char **fully_qualified_name)
+  BOOL qualify_single, BOOL search_parents, char **fully_qualified_name,
+  BOOL *removed)
 {
 host_item *h, *last;
 dns_record *rr;
@@ -1375,7 +1200,11 @@ dns_init(qualify_single, search_parents);
 if (!a_only)
   {
   rc = dns_lookup(host->name, T_MX, fully_qualified_name);
-  if (rc == DNS_FAIL) return HOST_FIND_FAILED;
+
+/* Changed to be the same as in set_address_from_dns above */
+/*  if (rc == DNS_FAIL) return HOST_FIND_FAILED; */
+
+  if (rc == DNS_FAIL) return HOST_FIND_AGAIN;
   if (rc == DNS_AGAIN) return HOST_FIND_AGAIN;
   }
 
@@ -1390,7 +1219,7 @@ if (rc != DNS_SUCCEED)
     host_item *next = host->next;
     last = host;
     host->mx = -1;
-    rc = dns_set_address(host, fully_qualified_name);
+    rc = set_address_from_dns(host, fully_qualified_name);
 
     /* If one or more A records have been found, find the last one and
     check that none of them are local. Since we know the host items all
@@ -1400,7 +1229,7 @@ if (rc != DNS_SUCCEED)
     if (rc == HOST_FOUND)
       {
       while (last->next != next) last = last->next;
-      rc = host_scan_for_local_hosts(host, last);
+      rc = host_scan_for_local_hosts(host, last, NULL);
       }
 
     DEBUG(8)
@@ -1420,7 +1249,7 @@ if (rc != DNS_SUCCEED)
     return rc;
     }
 
-  DEBUG(8) debug_printf("No MX records, and mx_only is FALSE\n");
+  DEBUG(8) debug_printf("No MX records, and mx_only is TRUE\n");
   return HOST_FIND_FAILED;
   }
 
@@ -1437,7 +1266,7 @@ which is not the primary hostname. */
 last = NULL;
 mx_count = mx_addressed = 0;
 
-for (rr = next_rr(RESET_ANSWERS); rr != NULL; rr = next_rr(RESET_NEXT))
+for (rr = dns_next_rr(RESET_ANSWERS); rr != NULL; rr = dns_next_rr(RESET_NEXT))
   {
   int precedence, datalen;
   unsigned char *s;             /* MUST be unsigned for GETSHORT */
@@ -1445,12 +1274,11 @@ for (rr = next_rr(RESET_ANSWERS); rr != NULL; rr = next_rr(RESET_NEXT))
 
   if (rr->type != T_MX) continue;
   s = rr->data;
-  GETSHORT(precedence, s);
+  GETSHORT(precedence, s);      /* Pointer s is advanced */
 
   /* Get the name of the host pointed to. */
 
-  datalen = dn_expand(dns_answer, dns_answer + dns_answerlen, s,
-    (DN_EXPAND_ARG4_TYPE)data, 256);
+  datalen = dns_expand(s, data, 256);
 
   /* Check that we haven't already got this host on the chain; if we have,
   keep only the lower precedence. This situation shouldn't occur, but you
@@ -1550,7 +1378,8 @@ have to expand the names) we do a single scan over them, and multiple scans
 of the chain of host items (which is typically only 3 or 4 long anyway.)
 Add extra host items for multi-homed hosts. */
 
-for (rr = next_rr(RESET_ADDITIONAL); rr != NULL; rr = next_rr(RESET_NEXT))
+for (rr = dns_next_rr(RESET_ADDITIONAL); rr != NULL;
+     rr = dns_next_rr(RESET_NEXT))
   {
   char *address;
 
@@ -1570,7 +1399,7 @@ for (rr = next_rr(RESET_ADDITIONAL); rr != NULL; rr = next_rr(RESET_NEXT))
   /* Set up the textual address for this RR; the function puts it into
   new store. */
 
-  address = address_from_rr(rr);
+  address = dns_address_from_rr(rr);
 
   /* If the address is already set for this host, it may be that
   we just have a duplicate A-record. Alternatively, this may be
@@ -1589,7 +1418,7 @@ for (rr = next_rr(RESET_ADDITIONAL); rr != NULL; rr = next_rr(RESET_NEXT))
       if (hh->address != NULL && strcmp(address, hh->address) == 0)
         {
         store_free(address);
-        goto NEXT_RR;         /* Need goto to escape from inner loop */
+        goto dns_next_rr;         /* Need goto to escape from inner loop */
         }
       thishostlast = hh;
       hh = hh->next;
@@ -1649,7 +1478,7 @@ for (rr = next_rr(RESET_ADDITIONAL); rr != NULL; rr = next_rr(RESET_NEXT))
   mx_addressed >= mx_count, but we can't be sure there won't be an additional
   address for a multi-homed host further down the list. */
 
-  NEXT_RR: continue;
+  dns_next_rr: continue;
   }
 
 /* Set the default yield to failure */
@@ -1666,7 +1495,7 @@ if (mx_addressed < mx_count)
   for (h = host; h != last->next; h = h->next)
     {
     if (h->address != NULL) continue;
-    rc = dns_set_address(h, NULL);
+    rc = set_address_from_dns(h, NULL);
 
     /* If the DNS lookup failed, mark this host item unusable and carry
     on. If no addresses are found, we'll fail below. */
@@ -1693,14 +1522,19 @@ HOST_FOUND is returned. Otherwise, HOST_FIND_FAILED is returned, but in this
 case do not update the yield, as it might have been set to HOST_FIND_AGAIN
 just above here. If not, it will already be HOST_FIND_FAILED. */
 
-rc = host_scan_for_local_hosts(host, last);
+rc = host_scan_for_local_hosts(host, last, removed);
 if (rc != HOST_FIND_FAILED) yield = rc;
 
 DEBUG(8)
   {
   if (fully_qualified_name != NULL)
     debug_printf("fully qualified name = %s\n", *fully_qualified_name);
-  debug_printf("host_find_bydns yield = %d returned hosts:\n", yield);
+  debug_printf("host_find_bydns yield = %s (%d); returned hosts:\n",
+    (yield == HOST_FOUND)? "HOST_FOUND" :
+    (yield == HOST_FOUND_LOCAL)? "HOST_FOUND_LOCAL" :
+    (yield == HOST_FIND_AGAIN)? "HOST_FIND_AGAIN" :
+    (yield == HOST_FIND_FAILED)? "HOST_FIND_FAILED" : "?",
+    yield);
   for (h = host; h != last->next; h = h->next)
     debug_printf("  %s %s %d %d %s\n", h->name,
       (h->address == NULL)? "<null>" : h->address, h->mx, h->sort_key,
@@ -1712,6 +1546,113 @@ return yield;
 
 
 
+/*************************************************
+*        Check an IP address against an RBL      *
+*************************************************/
+
+/* Anti-spamming "realtime blocking lists" are appearing on the DNS. This
+function, given an IP address and the domain of one such list, looks up the
+address in the blocking domain.
+
+Arguments:
+  address      the string version of the address
+  domain       the rbl domain to use
+  buffer       somewhere to put an error message if not NULL;
+               if no error, set to the null string
+  buflen       size of buffer
+
+Returns:    DNS_SUCCEED   successful lookup (i.e. the address is blocked)
+            DNS_NOMATCH   name not found, or no data found for the given type
+            DNS_AGAIN     soft failure, try again later
+            DNS_FAIL      DNS failure
+*/
+
+int
+host_check_rbl(char *address, char *domain, char *buffer, int buflen)
+{
+int bin[4];
+int rc;
+int type;
+char name[256];
+char *t = name;
+
+/* In case this is the first time the DNS resolver is being used. */
+
+dns_init(FALSE, FALSE);
+
+/* Handle IPv4 address: if HAVE_IPV6 is false, the result of
+host_aton() is always 1. */
+
+if (host_aton(address, bin) == 1)
+  {
+  int i;
+  int x = bin[0];
+  for (i = 0; i < 4; i++)
+    {
+    sprintf(t, "%d.", x & 255);
+    while (*t) t++;
+    x >>= 8;
+    }
+  type = T_A;
+  }
+
+/* Handle IPv6 address */
+
+#if HAVE_IPV6
+else
+  {
+  int i, j;
+  for (j = 3; j >= 0; j--)
+    {
+    int x = bin[j];
+    for (i = 0; i < 8; i++)
+      {
+      sprintf(t, "%x.", x & 15);
+      while (*t) t++;
+      x >>= 4;
+      }
+    }
+  type = T_AAAA;
+  }
+#endif
+
+
+strcpy(t, domain);
+HDEBUG(8) debug_printf("RBL lookup: %s\n", name);
+rc = dns_basic_lookup(name, type);
+
+/* If a buffer exists, set its content to the null string, or to a TXT record's
+data if present after success. */
+
+if (buffer != NULL)
+  {
+  buffer[0] = 0;
+  if (rc == DNS_SUCCEED)
+    {
+    if (dns_basic_lookup(name, T_TXT) == DNS_SUCCEED)
+      {
+      dns_record *rr;
+
+      for (rr = dns_next_rr(RESET_ANSWERS); rr != NULL; rr = dns_next_rr(RESET_NEXT))
+        if (rr->type == T_TXT) break;
+
+      if (rr != NULL)
+        {
+        int len = (rr->data)[0];
+        if (len > buflen - 1) len = buflen - 1;
+        strncpy(buffer, (const char *)(rr->data + 1), len);
+        buffer[len] = 0;
+        }
+
+      DEBUG(8) debug_printf("RBL TXT: %s\n", buffer);
+      }
+    }
+  }
+
+return rc;
+}
+
+
 
 
 /*************************************************
@@ -1729,7 +1670,7 @@ BOOL byname = FALSE;
 BOOL mx_only = FALSE;
 BOOL a_only = FALSE;
 BOOL qualify_single = TRUE;
-BOOL search_parents = TRUE;
+BOOL search_parents = FALSE;
 char buffer[256];
 
 primary_hostname = "";
@@ -1793,7 +1734,7 @@ while (fgets(buffer, 256, stdin) != NULL)
 
     rc = byname? host_find_byname(&h, &fully_qualified_name) :
       host_find_bydns(&h, mx_only, a_only, qualify_single, search_parents,
-        &fully_qualified_name);
+        &fully_qualified_name, NULL);
 
     if (rc == HOST_FIND_FAILED) printf("Failed\n");
       else if (rc == HOST_FIND_AGAIN) printf("Again\n");
@@ -1841,6 +1782,21 @@ while (fgets(buffer, 256, stdin) != NULL)
   printf("\n> ");
   }
 
+printf("\n");
+
+printf("Testing host_check_rbl\n");
+printf("> ");
+while (fgets(buffer, 256, stdin) != NULL)
+  {
+  int yield;
+  int len = (int)strlen(buffer);
+  while (len > 0 && isspace(buffer[len-1])) len--;
+  buffer[len] = 0;
+  if (strcmp(buffer, "q") == 0) break;
+  yield = host_check_rbl(buffer, "rbl.maps.vix.com", buffer, sizeof(buffer));
+  printf("\n> ");
+  }
+
 printf("\n");
 return 0;
 }
