@@ -26,6 +26,12 @@
 #  include <sys/select.h>
 #endif
 
+#ifdef _AIX
+#  include <sys/select.h>
+#endif
+#ifdef VMS
+#  include <sys/socket.h>     /* for fd_set */
+#endif
 #define IN_LIBIDENT_SRC
 #include "ident.h"
 
@@ -43,20 +49,21 @@ int id_query __P4(ident_t *, id,
     int res;
     char buf[80];
     fd_set ws;
-    
+
     sprintf(buf, "%d , %d\r\n", lport, fport);
-    
+
     if (timeout)
     {
 	FD_ZERO(&ws);
 	FD_SET(id->fd, &ws);
-	
-	if ((res = select(FD_SETSIZE,
-			  (fd_set *)0, &ws,
-			  (fd_set *)0,
-			  timeout)) < 0)
+
+#ifdef __hpux
+	if ((res = select(FD_SETSIZE, (int *)0, (int *)&ws, (int *)0, timeout)) < 0)
+#else
+	if ((res = select(FD_SETSIZE, (fd_set *)0, &ws, (fd_set *)0, timeout)) < 0)
+#endif
 	    return -1;
-	
+
 	if (res == 0)
 	{
 	    errno = ETIMEDOUT;
@@ -65,10 +72,10 @@ int id_query __P4(ident_t *, id,
     }
 
     old_sig = signal(SIGPIPE, SIG_IGN);
-    
+
     res = write(id->fd, buf, strlen(buf));
-    
+
     signal(SIGPIPE, old_sig);
-    
+
     return res;
 }
