@@ -2,14 +2,24 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
-/* This isn't a macro, but it's the sort of general definition
-that fits in this file and we need to have it defined early. */
+/* These aren't macros, but they're the sort of general definition
+that fits in this file and we need to have them defined early. Some
+operating systems (naughtily, imo) include a definition for "uchar"
+in the standard header files, so we use "uschar". This is just a typing
+convenience, of course. */
 
 typedef int BOOL;
+typedef unsigned char uschar;
+
+/* These two macros make it possible to obtain the result of macro-expanding
+a string as a text string. This is sometimes useful for debugging output. */
+
+#define mac_string(s) # s
+#define mac_expanded_string(s) mac_string(s)
 
 /* Define which ends of pipes are for reading and writing, as some systems
 don't make both fd's two-way. */
@@ -48,13 +58,43 @@ Most have sysconf(), but a few don't. */
   #define mac_maxfd 255;    /* just in case */
 #endif
 
+/* A macro to simplify testing bits in lookup types */
+
+#define mac_islookup(a,b) ((lookup_list[a].type & (b)) != 0)
+
+/* A macro to test for a printing character. We don't use isprint() because
+although it can be affected by the locale, that often isn't set the way people
+actually want it. */
+
+#define mac_isprint(c) ((c >= 32 && c <= 126) || (c >= 128 && print_topbitchars))
+
 /* Some generalities */
 
 #define DEBUG(x)      if ((x) <= debug_level)
+#define HDEBUG(x)     if (host_checking || (x) <= debug_level)
 
 #define FALSE         0
 #define TRUE          1
 
+/* The maximum permitted number of command-line (-D) macro definitions. We
+need a limit only to make it easier to generate argument vectors for re-exec
+of Exim. */
+
+#define MAX_CLMACROS 10
+
+/* The number of integer variables available in filter files. If this is
+changed, then the tables in expand.c for accessing them must be changed too. */
+
+#define FILTER_VARIABLE_COUNT 10
+
+/* The size of the vector holding delay warning times */
+
+#define DELAY_WARNING_SIZE 12
+
+/* The size of the buffer holding the processing information string. */
+
+#define PROCESS_INFO_SIZE 256
+
 /* The size of buffer to get for constructing log entries. Make it big
 enough to hold all the headers from a normal kind of message. */
 
@@ -85,9 +125,15 @@ record. */
 
 #define WAIT_NAME_MAX 50
 
+/* Fixed option values for all PCRE functions */
+
+#define PCRE_COPT 0   /* compile */
+#define PCRE_EOPT 0   /* exec */
+
 /* Macros for trivial functions */
 
-#define mac_ismsgid(s) regexec(regexp_spoolfile, (s))
+#define mac_ismsgid(s) \
+  (pcre_exec(regex_ismsgid, NULL, s, (int)strlen(s), PCRE_EOPT, NULL, 0) >= 0)
 
 /* Macros for calling the memory allocation routines with
 tracing information for debugging. */
@@ -95,6 +141,10 @@ tracing information for debugging. */
 #define store_free(addr)   store_free_3(addr, __FILE__, __LINE__)
 #define store_malloc(size) store_malloc_3(size, __FILE__, __LINE__)
 
+/* Options for dns_next_rr */
+
+enum { RESET_NEXT, RESET_ANSWERS, RESET_ADDITIONAL };
+
 /* Argument values for the time-of-day function */
 
 enum { tod_log, tod_full, tod_bsdin };
@@ -102,6 +152,7 @@ enum { tod_log, tod_full, tod_bsdin };
 /* For identifying which kind of driver deferred */
 
 enum {
+  DTYPE_NONE,
   DTYPE_DIRECTOR,
   DTYPE_ROUTER,
   DTYPE_TRANSPORT
@@ -134,8 +185,11 @@ apply to all of them) */
 
 enum {
   OK,                       /* Success */
+  OK_CONTINUE,              /* Success; if verifying, always continue with
+                               generated address(es) - directors only */
   DEFER,                    /* Failed soft */
   FAIL,                     /* Failed hard */
+  FAIL_CONDITION,           /* Failed a condition test in a director */
   FORCEFAIL,                /* Failed, and don't pass on */
   ISLOCAL,                  /* Remote address turned out to be local */
   PASS,                     /* Pass to next - routers only */
@@ -143,40 +197,52 @@ enum {
   PANIC                     /* Hard failed with internal error */
 };
 
+/* Returns from DNS lookup functions. */
+
+enum { DNS_SUCCEED, DNS_NOMATCH, DNS_AGAIN, DNS_FAIL };
+
 /* Private error numbers for delivery failures, set negative so as not
 to conflict with system errno values. */
 
-#define ERRNO_UNKNOWNERROR  (-1)
-#define ERRNO_USERSLASH     (-2)
-#define ERRNO_EXISTRACE     (-3)
-#define ERRNO_NOTREGULAR    (-4)
-#define ERRNO_BADUGID       (-5)
-#define ERRNO_BADMODE       (-6)
-#define ERRNO_INODECHANGED  (-7)
-#define ERRNO_LOCKFAILED    (-8)
-#define ERRNO_BADADDRESS2   (-9)
-#define ERRNO_BADFORWARD   (-10)
-#define ERRNO_FORBIDPIPE   (-11)
-#define ERRNO_FORBIDFILE   (-12)
-#define ERRNO_FORBIDREPLY  (-13)
-#define ERRNO_MISSINGPIPE  (-14)
-#define ERRNO_MISSINGFILE  (-15)
-#define ERRNO_MISSINGREPLY (-16)
-#define ERRNO_BADALIAS     (-17)
-#define ERRNO_SMTPCLOSED   (-18)
-#define ERRNO_SMTPFORMAT   (-19)
-#define ERRNO_SPOOLFORMAT  (-20)
-#define ERRNO_NOTABSOLUTE  (-21)
-#define ERRNO_EXIMQUOTA    (-22)   /* Exim-imposed quota */
+#define ERRNO_UNKNOWNERROR    (-1)
+#define ERRNO_USERSLASH       (-2)
+#define ERRNO_EXISTRACE       (-3)
+#define ERRNO_NOTREGULAR      (-4)
+#define ERRNO_NOTDIRECTORY    (-5)
+#define ERRNO_BADUGID         (-6)
+#define ERRNO_BADMODE         (-7)
+#define ERRNO_INODECHANGED    (-8)
+#define ERRNO_LOCKFAILED      (-9)
+#define ERRNO_BADADDRESS2    (-10)
+#define ERRNO_BADFORWARD     (-11)
+#define ERRNO_FORBIDPIPE     (-12)
+#define ERRNO_FORBIDFILE     (-13)
+#define ERRNO_FORBIDREPLY    (-14)
+#define ERRNO_MISSINGPIPE    (-15)
+#define ERRNO_MISSINGFILE    (-16)
+#define ERRNO_MISSINGREPLY   (-17)
+#define ERRNO_BADALIAS       (-18)
+#define ERRNO_SMTPCLOSED     (-19)
+#define ERRNO_SMTPFORMAT     (-20)
+#define ERRNO_SPOOLFORMAT    (-21)
+#define ERRNO_NOTABSOLUTE    (-22)
+#define ERRNO_EXIMQUOTA      (-23)   /* Exim-imposed quota */
+#define ERRNO_HELD           (-24)
+#define ERRNO_FILTER_FAIL    (-25)   /* Delivery filter process failure */
+#define ERRNO_ADDHEADER_FAIL (-26)   /* Delivery add header failure */
+#define ERRNO_WRITEINCOMPLETE (-27)  /* Delivery write incomplete error */
+#define ERRNO_EXPANDFAIL     (-28)   /* Some expansion failed */
+#define ERRNO_GIDFAIL        (-29)   /* Failed to get gid */
+#define ERRNO_UIDFAIL        (-30)   /* Failed to get uid */
 
 /* These must be last, so all retry deferments can easily be identified */
 
-#define ERRNO_RETRY_BASE   (-22)   /* Base to test against */
-#define ERRNO_DRETRY       (-22)   /* Not time for directing */
-#define ERRNO_RRETRY       (-23)   /* Not time for routing */
-#define ERRNO_LRETRY       (-24)   /* Not time for local delivery */
-#define ERRNO_HRETRY       (-25)   /* Not time for any remote host */
-#define ERRNO_LOCAL_ONLY   (-26)   /* Local-only delivery */
+#define ERRNO_RETRY_BASE     (-31)   /* Base to test against */
+#define ERRNO_DRETRY         (-31)   /* Not time for directing */
+#define ERRNO_RRETRY         (-32)   /* Not time for routing */
+#define ERRNO_LRETRY         (-33)   /* Not time for local delivery */
+#define ERRNO_HRETRY         (-34)   /* Not time for any remote host */
+#define ERRNO_LOCAL_ONLY     (-35)   /* Local-only delivery */
 
 /* Special actions to take after failure or deferment. */
 
@@ -208,33 +274,40 @@ enum { hn_bcc, hn_cc, hn_date, hn_from, hn_msgid, hn_sender, hn_to,
 
 /* Types of item in options lists */
 
-enum { opt_stringptr, opt_lcstringptr, opt_transportptr, opt_int,
-  opt_octint, opt_mkint, opt_fixed, opt_time, opt_bool, opt_bool_verify,
-  opt_uid, opt_gid, opt_uidlist, opt_gidlist, opt_expand_uid, opt_expand_gid,
-  opt_searchtype, opt_local_batch };
+enum { opt_stringptr, opt_domainlist, opt_transportptr, opt_int,
+  opt_octint, opt_mkint, opt_fixed, opt_time, opt_timelist, opt_bool,
+  opt_bool_verify, opt_uid, opt_gid, opt_uidlist, opt_gidlist, opt_expand_uid,
+  opt_expand_gid, opt_searchtype, opt_local_batch };
 
 /* There's a high-ish bit which is used to flag duplicate options, kept
 for compatibility, which shouldn't be output. Also used for hidden options
 that are automatically maintained from others. Another high bit is used to
 flag driver options that although private (so as to be settable only on some
 drivers), are stored in the instance block so as to be accessible from outside.
-*/
+A third high bit is set when an option is read, so as to be able to give an
+error if any option is set twice. */
 
 #define opt_hidden  0x100
 #define opt_public  0x200
+#define opt_set     0x400
 #define opt_mask    0x0ff
 
 /* Verify types when directing and routing */
 
-enum { v_none, v_sender, v_recipient };
+enum { v_none, v_sender, v_recipient, v_expn };
 
-/* Search types for alias files etc. The single-key+file types must
-come first, with all the "query" types last. Start the values at a printing
-characters, as they are concatenated with file names in the caching tree. */
+/* Option flags for verify_address() */
 
-enum {
-  stype_lsearch='0', stype_dbm, stype_nis, stype_nis0,   /* key+file types */
-  stype_querystyle, stype_nisplus };                     /* query types */
+#define vopt_is_recipient   0x01
+#define vopt_local          0x02
+#define vopt_log_details    0x04
+#define vopt_address_test   0x08
+#define vopt_expn           0x10
+
+/* Options for lookup functions */
+
+#define lookup_querystyle      1    /* query-style lookup */
+#define lookup_absfile         2    /* requires absolute file name */
 
 /* Status values for host_item blocks. Require hstatus_unusable and
 hstatus_unusable_expired to be last. */
@@ -252,26 +325,40 @@ enum { lk_pass, lk_byname, lk_bydns, lk_bydns_a, lk_bydns_mx };
 
 /* Values for the self_code fields */
 
-enum { self_freeze, self_defer, self_send, self_local,
+enum { self_freeze, self_defer, self_send, self_reroute, self_local,
   self_fail, self_forcefail };
 
 /* Flags for rewrite rules */
 
-#define rewrite_sender       0x001
-#define rewrite_from         0x002
-#define rewrite_to           0x004
-#define rewrite_cc           0x008
-#define rewrite_bcc          0x010
-#define rewrite_replyto      0x020
-#define rewrite_all_headers  0x03F  /* all header flags */
+#define rewrite_sender       0x0001
+#define rewrite_from         0x0002
+#define rewrite_to           0x0004
+#define rewrite_cc           0x0008
+#define rewrite_bcc          0x0010
+#define rewrite_replyto      0x0020
+#define rewrite_all_headers  0x003F  /* all header flags */
 
-#define rewrite_envfrom      0x040
-#define rewrite_envto        0x080
-#define rewrite_all_envelope 0x0C0  /* all envelope flags */
+#define rewrite_envfrom      0x0040
+#define rewrite_envto        0x0080
+#define rewrite_all_envelope 0x00C0  /* all envelope flags */
 
 #define rewrite_all      (rewrite_all_headers | rewrite_all_envelope)
 
-#define rewrite_whole   0x100   /* option bit for headers */
+#define rewrite_smtp         0x0100  /* rewrite at SMTP time */
+#define rewrite_qualify      0x0200  /* qualify if necessary */
+#define rewrite_repeat       0x0400  /* repeat rewrite rule */
+#define rewrite_X            0x0800  /* extra for relay checking */
+
+#define rewrite_whole        0x1000  /* option bit for headers */
+#define rewrite_quit         0x2000  /* "no more" option */
+
+/* Special return values from parse_extract_addresses */
+
+#define EXTRACTED_INCLUDEFAIL  (+1)    /* :include: failed */
+#define EXTRACTED_OK            0
+#define EXTRACTED_DEFER        (-1)    /* :defer: */
+#define EXTRACTED_FAIL         (-2)    /* :fail: */
+#define EXTRACTED_ERROR        (-3)    /* some problem */
 
 /* Flags for log_write() */
 
@@ -280,9 +367,10 @@ enum { self_freeze, self_defer, self_send, self_local,
 #define LOG_PANIC_DIE   6    /* Write to the panic log and then die */
 #define LOG_PROCESS     8    /* Write to the process log */
 #define LOG_REJECT     16    /* Write to the reject log, with headers */
-#define LOG_RECIPIENTS 32    /* Add recipients to the message */
-#define LOG_CONFIG     64    /* Add "Exim configuration error:\n" */
-#define LOG_CONFIG2   128    /* Add "Exim configuration error for" */
+#define LOG_SENDER     32    /* Add raw sender to the message */
+#define LOG_RECIPIENTS 64    /* Add raw recipients to the message */
+#define LOG_CONFIG    128    /* Add "Exim configuration error:\n" */
+#define LOG_CONFIG2   256    /* Add "Exim configuration error for" */
 
 /* Returns from host_find_by{name,dns}() */
 
@@ -336,5 +424,6 @@ enum {
 #define topt_from_hack          0x08
 #define topt_use_crlf           0x10
 #define topt_smtp_dots          0x20
+#define topt_writing_file       0x40
 
 /* End of macros.h */
