@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for parsing addresses */
@@ -11,18 +11,17 @@
 #include "exim.h"
 
 
-static char  fixed_phrase[256];
-static char *last_comment_position;
+static uschar *last_comment_position;
 
 
 
 /* In stand-alone mode, provide a replacement for deliver_make_addr()
-and rewrite_address so as to avoid having to drag in too much redundant
-apparatus. */
+and rewrite_address[_qualify]() so as to avoid having to drag in too much
+redundant apparatus. */
 
 #ifdef STAND_ALONE
 
-static address_item *deliver_make_addr(char *address)
+address_item *deliver_make_addr(char *address)
 {
 address_item *addr = store_malloc(sizeof(address_item));
 addr->next = NULL;
@@ -31,7 +30,12 @@ addr->orig = address;
 return addr;
 }
 
-static char *rewrite_address(char *recipient, BOOL dummy1, BOOL dummy2)
+char *rewrite_address(char *recipient, BOOL dummy1, BOOL dummy2)
+{
+return recipient;
+}
+
+char *rewrite_address_qualify(char *recipient, BOOL dummy1)
 {
 return recipient;
 }
@@ -64,10 +68,16 @@ int no_term = source_routing? 1 : 0;
 
 while (*s != 0 && (*s != ',' || no_term > 0) && (*s != '\n' || !nl_ends))
   {
+  /* Skip single quoted characters. Strictly these should not occur outside
+  quoted strings in RFC 822 addresses, but they can in RFC 821 addresses. Pity
+  about the lack of consistency, isn't it? */
+
+  if (*s == '\\' && s[1] != 0) s += 2;
+
   /* Skip quoted items that are not inside brackets. Note that
   quoted pairs are allowed inside quoted strings. */
 
-  if (*s == '\"')
+  else if (*s == '\"')
     {
     while (*(++s) != 0)
       {
@@ -119,7 +129,9 @@ return s;
 /* We have to be a bit more clever than just a plain search, in order to handle
 addresses like "thing@thong"@a.b.c correctly. Since quotes may not legally
 be part of a domain name, we can give up on hitting the first quote when
-searching from the right.
+searching from the right. Now that the parsing also permits the RFC 821 form of
+address, where quoted-pairs are allowed in unquoted local parts, we must take
+care to handle that too.
 
 Argument:  pointer to an address
 Returns:   pointer to the last @ in an address, or NULL if none
@@ -131,8 +143,14 @@ parse_find_at(char *s)
 char *t = s + (int)strlen(s);
 while (--t >= s)
   {
-  if (*t == '@') return t;
-  if (*t == '\"') return NULL;
+  if (*t == '@')
+    {
+    int backslash_count = 0;
+    char *tt = t - 1;
+    while (tt > s && *tt-- == '\\') backslash_count++;
+    if ((backslash_count & 1) == 0) return t;
+    }
+  else if (*t == '\"') return NULL;
   }
 return NULL;
 }
@@ -167,8 +185,8 @@ Argument: current character pointer
 Regurns:  new character pointer
 */
 
-static char *
-skip_comment(char *s)
+static uschar *
+skip_comment(uschar *s)
 {
 last_comment_position = s;
 while (*s)
@@ -189,67 +207,6 @@ return s;
 
 
 
-/*************************************************
-*                 Read an atom                   *
-*************************************************/
-
-/* An atom is a sequence of any characters except specials, space,
-and controls. The specials are ( ) < > @ , ; : \ " . [ and ].
-
-Arguments:
-  s        current character pointer
-  t        where to put the atom
-
-Returns:   new character pointer
-*/
-
-static char *
-read_atom(char *s, char *t)
-{
-s = skip_comment(s);
-while (!iscntrl(*s) && strchr(" ()<>@,;:\\\".[]", *s) == NULL) *t++ = *s++;
-*t = 0;
-return skip_comment(s);
-}
-
-
-
-/*************************************************
-*                 Read a word                    *
-*************************************************/
-
-/* A <word> is either a quoted string, or an <atom>.
-
-Arguments:
-  s        current character pointer
-  t        where to put the word
-
-Returns:   new character pointer
-*/
-
-static char *
-read_word (char *s, char *t)
-{
-s = skip_comment(s);
-if (*s == '\"')
-  {
-  int c;
-  *t++ = '\"';
-  while ((c = *(++s)) > 0 && c != '\"')
-    {
-    *t++ = c;
-    if (c == '\\' && s[1] != 0) *t++ = *(++s);
-    }
-  *t++ = '\"';
-  *t = 0;
-  return skip_comment(s + ((c == '\"')? 1:0));
-  }
-else return read_atom(s, t);
-}
-
-
-
-
 /*************************************************
 *             Read a domain                      *
 *************************************************/
@@ -268,10 +225,10 @@ Arguments:
 Returns:     new character pointer
 */
 
-static char *
-read_domain(char *s, char *t, char **errorptr)
+static uschar *
+read_domain(uschar *s, uschar *t, char **errorptr)
 {
-char *tt = t;
+uschar *tt = t;
 s = skip_comment(s);
 
 /* Handle domain literals if permitted. An RFC 822 domain literal may contain
@@ -307,24 +264,32 @@ if (*s == '[')
   return skip_comment(s);
   }
 
-/* Handle proper domains */
+/* Handle a proper domain, which is a sequence of dot-separated atoms.
+An atom is a sequence of any characters except specials, space, and controls.
+The specials are ( ) < > @ , ; : \ " . [ and ]. */
 
 for (;;)
   {
-  s = read_atom(s, t);
-  if (*t == 0)
+  uschar *tsave = t;
+  while (!iscntrl(*s) && strchr(" ()<>@,;:\\\".[]", *s) == NULL) *t++ = *s++;
+  s = skip_comment(s);
+  *t = 0;
+
+  if (t == tsave)   /* empty component */
     {
-    if (strip_trailing_dot && t > tt) t[-1] = 0; else
+    if (strip_trailing_dot && t > tt && *s != '.') t[-1] = 0; else
       {
       *errorptr = "domain missing or malformed";
       *tt = 0;
       }
     return s;
     }
-  t += (int)strlen(t);
+
   if (*s != '.') break;
   *t++ = *s++;
+  s = skip_comment(s);
   }
+
 return s;
 }
 
@@ -338,6 +303,14 @@ return s;
 between dots is not strictly allowed but apparently many mailers permit it,
 so, sigh, better be compatible. Even accept a trailing dot...
 
+A <word> is either a quoted string, or an <atom>, which is a sequence
+of any characters except specials, space, and controls. The specials are
+( ) < > @ , ; : \ " . [ and ]. In RFC 822, a single quoted character, (a
+quoted-pair) is not allowed in a word. However, in RFC 821, it is permitted in
+the local part of an address. Rather than have separate parsing functions for
+the different cases, take the liberal attitude always. At least one MUA is
+happy to recognize this case; I don't know how many other programs do.
+
 Arguments:
   s           current character pointer
   t           where to put the local part
@@ -347,25 +320,62 @@ Arguments:
 Returns:   new character pointer
 */
 
-static char *
-read_local_part(char *s, char *t, char **error, BOOL allow_null)
+static uschar *
+read_local_part(uschar *s, uschar *t, char **error, BOOL allow_null)
 {
-char *tt = t;
+uschar *tt = t;
 *error = NULL;
 for (;;)
   {
+  int c;
+  uschar *tsave = t;
   s = skip_comment(s);
-  s = read_word(s, t);
-  if (*t == 0)
+
+  /* Handle a quoted string */
+
+  if (*s == '\"')
+    {
+    *t++ = '\"';
+    while ((c = *(++s)) != 0 && c != '\"')
+      {
+      *t++ = c;
+      if (c == '\\' && s[1] != 0) *t++ = *(++s);
+      }
+    *t++ = '\"';
+    if (c == '\"') s++;
+    }
+
+  /* Handle an atom, but allow quoted pairs within it. */
+
+  else while (!iscntrl((c = *s)) && strchr(" ()<>@,;:\".[]", c) == NULL)
+    {
+    *t++ = *s++;
+    if (c == '\\' && *s != 0) *t++ = *s++;
+    }
+
+  /* Terminate the word and skip subsequent comment */
+
+  *t = 0;
+  s = skip_comment(s);
+
+  /* If we have read a null component at this point, give an error unless it is
+  the first component of the local part and an empty local part is permitted,
+  in which case just return normally. */
+
+  if (t == tsave)
     {
     if (t == tt && !allow_null)
       *error = "missing or malformed local part";
     return s;
     }
-  t += (int)strlen(t);
+
+  /* Anything other than a dot terminates the local part. Treat multiple dots
+  as a single dot, as this seems to be a common extension. */
+
   if (*s != '.') break;
   do { *t++ = *s++; } while (*s == '.');
   }
+
 return s;
 }
 
@@ -386,10 +396,10 @@ Arguments:
 Returns:     new character pointer
 */
 
-static char *
-read_route(char *s, char *t, char **errorptr)
+static uschar *
+read_route(uschar *s, uschar *t, char **errorptr)
 {
-char *t_init = t;
+uschar *t_init = t;
 *errorptr = NULL;
 
 while (*s == '@')
@@ -397,7 +407,7 @@ while (*s == '@')
   *t++ = '@';
   s = read_domain(s+1, t, errorptr);
   if (*t == 0) return s;
-  t += (int)strlen(t);
+  t += (int)strlen((const char *)t);
   if (*s != ',') break;
   *t++ = *s++;
   s = skip_comment(s);
@@ -431,22 +441,23 @@ Arguments:
 Returns:     new character pointer
 */
 
-static char *
-read_addr_spec(char *s, char *t, int term, char **errorptr, char **domainptr)
+static uschar *
+read_addr_spec(uschar *s, uschar *t, int term, char **errorptr,
+  uschar **domainptr)
 {
 s = read_local_part(s, t, errorptr, FALSE);
 if (*errorptr != NULL)
-  *errorptr = string_sprintf("%s in route address", *errorptr);
+  *errorptr = string_sprintf("%s in address", *errorptr);
 else
   {
   if (*s != term)
     {
     if (*s != '@')
       *errorptr = string_sprintf(
-        "\"@\" or \".\" expected in route address after \"%s\"", t);
+        "\"@\" or \".\" expected in address after \"%s\"", t);
     else
       {
-      t += (int)strlen(t);
+      t += (int)strlen((const char *)t);
       *t++ = *s++;
       *domainptr = t;
       s = read_domain(s, t, errorptr);
@@ -459,19 +470,20 @@ return s;
 
 
 /*************************************************
-*           Extract route address                *
+*         Extract operative address              *
 *************************************************/
 
-/* This function extracts an address from a full RFC822 mailbox and returns it
-in a piece of malloc'ed store. We take the easy way and get a piece of store
-the same size as the input, and then copy into it whatever is necessary. If we
-cannot find a valid address (syntax error), return NULL, and point the error
-pointer to the reason. The arguments "start" and "end" are used to return the
-offsets of the first and one past the last characters in the original mailbox
-of the address that has been extracted, to aid in re-writing. The argument
-"domain" is set to point to the first character after "@" in the final part of
-the returned address, or zero if there is no @. Note: if the address is of the
-form @domain,...:route-addr "domain" refers to the final route-addr.
+/* This function extracts an operative address from a full RFC822 mailbox and
+returns it in a piece of malloc'ed store. We take the easy way and get a piece
+of store the same size as the input, and then copy into it whatever is
+necessary. If we cannot find a valid address (syntax error), return NULL, and
+point the error pointer to the reason. The arguments "start" and "end" are used
+to return the offsets of the first and one past the last characters in the
+original mailbox of the address that has been extracted, to aid in re-writing.
+The argument "domain" is set to point to the first character after "@" in the
+final part of the returned address, or zero if there is no @. Note: if the
+address is of the form @domain,...:route-addr "domain" refers to the final
+route-addr.
 
 There are some special fudges here for handling RFC 822 group address notation
 which may appear in certain headers. If the flag parse_allow_group is set
@@ -500,10 +512,10 @@ char *
 parse_extract_address(char *mailbox, char **errorptr, int *start, int *end,
   int *domain, BOOL allow_null)
 {
-char *yield = store_malloc((int)strlen(mailbox) + 1);
-char *startptr, *endptr;
-char *s = mailbox;
-char *t = yield;
+uschar *yield = store_malloc((int)strlen(mailbox) + 1);
+uschar *startptr, *endptr;
+uschar *s = (uschar *)mailbox;
+uschar *t = (uschar *)yield;
 
 *domain = 0;
 
@@ -561,19 +573,19 @@ if (*s != '@' && *s != '<')
   /* Assert *s == '<' */
   }
 
-/* At this point the next character is either '@' or '<'. If it is '@',
-only a single local-part has previously been read. An angle bracket
-signifies the start of a <route-addr>. Throw away anything we have saved so
-far before processing it. Note that this is "if" rather than "else if"
-because it's also used after reading a preceding phrase.
+/* At this point the next character is either '@' or '<'. If it is '@', only a
+single local-part has previously been read. An angle bracket signifies the
+start of an <addr-spec>. Throw away anything we have saved so far before
+processing it. Note that this is "if" rather than "else if" because it's also
+used after reading a preceding phrase.
 
-There are a lot of broken sendmails out there that put additional pairs
-of <> round <route-addr>s. If strip_excess_angle_brackets is set, allow
-any number of them, as long as they match. */
+There are a lot of broken sendmails out there that put additional pairs of <>
+round <route-addr>s. If strip_excess_angle_brackets is set, allow any number of
+them, as long as they match. */
 
 if (*s == '<')
   {
-  char *domainptr = yield;
+  uschar *domainptr = yield;
   BOOL source_routed = FALSE;
   int bracket_count = 1;
 
@@ -593,7 +605,7 @@ if (*s == '<')
     {
     s = read_route(s, t, errorptr);
     if (*errorptr != NULL) goto PARSE_FAILED;
-    t += (int)strlen(t);
+    t += (int)strlen((const char *)t);
     source_routed = TRUE;
     }
 
@@ -618,8 +630,14 @@ if (*s == '<')
 
   endptr = s;
   if (*errorptr != NULL) goto PARSE_FAILED;
-  while (bracket_count-- > 0)
-    if (*s++ != '>') FAILED("> missing at end of route address");
+  while (bracket_count-- > 0) if (*s++ != '>')
+    {
+    *errorptr = (s[-1] == 0)? "'>' missing at end of address" :
+      string_sprintf("malformed address: %s may not follow %.*s",
+        s-1, s - (uschar *)mailbox - 1, mailbox);
+    goto PARSE_FAILED;
+    }
+
   s = skip_comment(s);
   }
 
@@ -631,10 +649,10 @@ part preceding '@'. */
 
 else if (*t == 0)
   {
-  char *domainptr = yield;
+  uschar *domainptr = yield;
   s = read_route(s, t, errorptr);
   if (*errorptr != NULL) goto PARSE_FAILED;
-  t += (int)strlen(t);
+  t += (int)strlen((const char *)t);
   s = read_addr_spec(s, t, 0, errorptr, &domainptr);
   if (*errorptr != NULL) goto PARSE_FAILED;
   *domain = domainptr - yield;
@@ -646,7 +664,7 @@ else if (*t == 0)
 
 else
   {
-  t += (int)strlen(t);
+  t += (int)strlen((const char *)t);
   *t++ = *s++;
   *domain = t - yield;
   s = read_domain(s, t, errorptr);
@@ -666,16 +684,28 @@ if (*s != 0)
     parse_found_group = FALSE;
     parse_allow_group = TRUE;
     }
-  else FAILED("extraneous characters at end of address");
+  else
+    {
+    *errorptr = string_sprintf("malformed address: %s may not follow %.*s",
+      s, s - (uschar *)mailbox, mailbox);
+    goto PARSE_FAILED;
+    }
   }
-*start = startptr - mailbox;      /* Return offsets */
+*start = startptr - (uschar *)mailbox;      /* Return offsets */
 while (isspace(endptr[-1])) endptr--;
-*end = endptr - mailbox;
-return yield;
+*end = endptr - (uschar *)mailbox;
+return (char *)yield;
 
-/* Use goto (via the macro FAILED) to get to here from a variety of places. */
+/* Use goto (via the macro FAILED) to get to here from a variety of places.
+We might have an empty address in a group - the caller can choose to ignore
+this. We must, however, keep the flags correct. */
 
 PARSE_FAILED:
+if (parse_found_group && *s == ';')
+  {
+  parse_found_group = FALSE;
+  parse_allow_group = TRUE;
+  }
 store_free(yield);
 return NULL;
 }
@@ -690,7 +720,8 @@ return NULL;
 
 /* This function is called to repair any syntactic defects in the "phrase" part
 of an RFC822 address. In particular, it is applied to the user's name as read
-from the passwd file when accepting a local message.
+from the passwd file when accepting a local message, and to the data from the
+-F option.
 
 If the string contains existing quoted strings or comments containing
 freestanding quotes, then we just quote those bits that need quoting -
@@ -709,9 +740,13 @@ Sheesh! This is tedious code. It is a great pity that the syntax of RFC822 is
 the way it is...
 
 Non-printing characters are turned into question marks, which do not need to be
-quoted. The result is passed back in a static buffer, and must therefore be
+quoted. The result is passed back in the big buffer, and must therefore be
 copied - this makes sense, since it is usually going to be added to some other
-string.
+string. In order to be sure there is going to be no overflow, restrict the
+length of the input to 1/3 of the big buffer size - this allows for every
+single character to be quoted without overflowing, and that wouldn't happen
+because of amalgamation. If the phrase is too long, return a fixed string -
+heck, this is just a paranoia check, after all.
 
 Argument: an RFC822 phrase
 Returns:  the fixed RFC822 phrase
@@ -722,24 +757,25 @@ parse_fix_phrase(char *s)
 {
 int ch;
 BOOL quoted = FALSE;
-char *t = fixed_phrase + 1;
+char *t = big_buffer + 1;
 char *yield = t;
 
-while (isspace(*s)) s++;
+while (isspace((uschar)*s)) s++;
+if ((int)strlen(s) > big_buffer_size/3) return "Name too long";
 
-while ((ch = (*s++) & 127) != 0)
+while ((ch = (uschar)*s++) != 0)
   {
-  if (!isprint(ch)) ch = '?';
+  if (!mac_isprint(ch)) ch = '?';
 
   /* Copy over quoted strings, remembering we encountered one */
 
   if (ch == '\"')
     {
     *t++ = '\"';
-    while ((ch = *s++ & 127) > 0 && ch != '\"')
+    while ((ch = *s++) != 0 && ch != '\"')
       {
       *t++ = ch;
-      if (ch == '\\' && *s != 0) *t++ = *s++ & 127;
+      if (ch == '\\' && *s != 0) *t++ = *s++;
       }
     *t++ = '\"';
     if (ch == 0) break;
@@ -753,7 +789,7 @@ while ((ch = (*s++) & 127) != 0)
     {
     int level = 1;
     *t++ = '(';
-    while((ch = *s++ & 127) != 0)
+    while((ch = *s++) != 0)
       {
       *t++ = ch;
       if (ch == '(') level++;
@@ -780,7 +816,7 @@ while ((ch = (*s++) & 127) != 0)
       while (*(--tt) != ' ' && *tt != '\"' && *tt != ')') tt[1] = *tt;
       tt[1] = '\"';
       *t++ = ch;
-      while((ch = *s++ & 127) != 0)
+      while((ch = *s++) != 0)
         if (ch == ' ' || ch == '\"') break; else *t++ = ch;
       *t++ = '\"';
       }
@@ -790,11 +826,10 @@ while ((ch = (*s++) & 127) != 0)
 
     else
       {
-      fixed_phrase[0] = '\"';
-      yield--;
+      *(--yield) = '\"';
       *t++ = ch;
 
-      while ((ch = *s++ & 127) != 0)
+      while ((ch = *s++) != 0)
         {
         /* If hit subsequent quotes, insert our quote before any
         trailing spaces and go into quoted mode. */
@@ -816,7 +851,7 @@ while ((ch = (*s++) & 127) != 0)
           {
           char *ss = s;     /* Char after '(' */
           int level = 1;
-          while((ch = *ss++ & 127) != 0)
+          while((ch = *ss++) != 0)
             {
             if (ch == '(') level++;
             else if (ch == ')') { if (--level <= 0) break; }
@@ -911,7 +946,8 @@ Yield 0 if the string was successfully parsed; non-zero otherwise, with
 a message pointed to by the error variable. A positive value is given if
 the failure is simply a failure to open or subsequently stat an :include: file
 - some callers treat this as a soft error - and a negative value is given
-otherwise.
+otherwise. The error numbers are now numbered, with special values for :fail:
+and :defer:.
 
 If optional is TRUE, then failure to generate any addresses, without a syntax
 error, is not an error.
@@ -920,33 +956,34 @@ If directory is not NULL, then require any :include: files to begin with the
 directory string AND be regular files.
 
 Arguments:
-  s           pointer to the list of addresses (typically a complete .forward
-                file or a list of entries in an alias file)
-  anchor      where to hang the chain of newly-created addresses
-  error       where to return an error text
-  optional    if TRUE and no addresses generated, treat as success; otherwise
-                no addresses generated is an error
-  allow_blackhole
-              if TRUE, allow :blackhole: to generate no addresses without
-                an error, even if optional is FALSE
-  rewrite     TRUE if generated addresses are to be passed through the
-                rewriting functions
-  directory   if NULL, no checks are done on :include: files
-              if "*", no included files are allowed
-              otherwise, included file names must start with the given
-                directory
-  syntax_errors
-              if not NULL, it carries on after syntax errors in addresses,
-                building up a list of errors as error blocks chained on
-                here
+  s                Pointer to the list of addresses (typically a complete
+                     .forward file or a list of entries in an alias file)
+  incoming domain  Domain of the incoming address; used to qualify unqualified
+                     local parts preceded by \
+  anchor           Where to hang the chain of newly-created addresses
+  error            Where to return an error text
+  optional         If TRUE and no addresses generated, treat as success;
+                     otherwise no addresses generated is an error
+  allow_specials   If TRUE, allow :blackhole: to generate no addresses without
+                     an error, even if optional is FALSE, and also recognize
+                     :defer: and :fail:
+  rewrite          TRUE if generated addresses are to be passed through the
+                     rewriting functions
+  directory        If NULL, no checks are done on :include: files
+                   If "*", no included files are allowed
+                   Otherwise, included file names must start with the given
+                     directory
+  syntax_errors    If not NULL, it carries on after syntax errors in addresses,
+                     building up a list of errors as error blocks chained on
+                     here.
 
 Returns:      0 on success, non-zero otherwise: >0 if failed to stat or open
               an :include: file; <0 for other errors.
 */
 
 int
-parse_extract_addresses(char *s, address_item **anchor, char **error,
-  BOOL optional, BOOL allow_blackhole, BOOL rewrite,
+parse_extract_addresses(char *s, char *incoming_domain, address_item **anchor,
+  char **error, BOOL optional, BOOL allow_specials, BOOL rewrite,
   char *directory, error_block **syntax_errors)
 {
 int count = 0;
@@ -962,15 +999,15 @@ for (;;)
 
   for (;;)
     {
-    while (isspace(*s) || *s == ',') s++;
+    while (isspace((uschar)*s) || *s == ',') s++;
     if (*s == '#') { while (*s != 0 && *s != '\n') s++; } else break;
     }
 
   if (*s == 0)
     {
-    if (count > 0 || optional) return 0;
+    if (count > 0 || optional) return EXTRACTED_OK;
     *error = "no addresses generated";
-    return -1;
+    return EXTRACTED_ERROR;
     }
 
   /* Find the end of the next address. Quoted strings in addresses may contain
@@ -987,7 +1024,7 @@ for (;;)
 
   /* Remove any trailing spaces; we know there's at least one non-space. */
 
-  while (isspace(ss[-1])) ss--;
+  while (isspace((uschar)(ss[-1]))) ss--;
 
   /* We now have s->start and ss->end of the next address. Remove quotes
   if they completely enclose, remembering the address started with a quote
@@ -999,8 +1036,8 @@ for (;;)
     s++;
     ss--;
     inquote = TRUE;
-    while (s < ss && isspace(*s)) s++;
-    while (ss > s && isspace(ss[-1])) ss--;
+    while (s < ss && isspace((uschar)*s)) s++;
+    while (ss > s && isspace((uschar)(ss[-1]))) ss--;
     }
 
   /* Set up the length of the address. */
@@ -1011,15 +1048,38 @@ for (;;)
     {
     int save = s[len];
     s[len] = 0;
-    debug_printf("extracted: %s\n", s);
+    debug_printf("extract item: %s\n", s);
     s[len] = save;
     }
 
-  /* If the address is :blackhole: and black holes are permitted, then ignore
-  it, and set the "optional" flag so that generating no addresses at all in
-  this expansion is no longer an error. */
+  /* Handle special addresses if permitted. If the address is :blackhole:
+  then ignore it, and set the "optional" flag so that generating no addresses
+  at all in this expansion is no longer an error. For :defer: or :fail: we have
+  to set up the error message and give up right away. */
 
-  if (allow_blackhole && strncmp(s, ":blackhole:", len) == 0) optional = TRUE;
+  if (allow_specials)
+    {
+    int special = 0;
+    if (strncmp(s, ":blackhole:", len) == 0)
+      {
+      optional = TRUE;
+      s = nexts;
+      continue;
+      }
+
+    if      (strncmp(s, ":defer:", 7) == 0) special = EXTRACTED_DEFER;
+    else if (strncmp(s, ":fail:", 6) == 0) special = EXTRACTED_FAIL;
+
+    if (special != 0)
+      {
+      char *ss = strchr(s+1, ':') + 1;
+      while (*ss != 0 && isspace(*ss)) ss++;
+      while (s[len] != 0 && s[len] != '\n') len++;
+      s[len] = 0;
+      *error = string_copy(ss);
+      return special;
+      }
+    }
 
   /* If the address is of the form :include:pathname, read the file, and call
   this function recursively to extract the addresses from it. There are
@@ -1028,25 +1088,41 @@ for (;;)
   that the file name starts with the given directory and is a regular file.
   If the directory is "*", no included files are allowed. */
 
-  else if (strncmp(s, ":include:", 9) == 0)
+  if (strncmp(s, ":include:", 9) == 0)
     {
     char *filebuf;
     char filename[256];
+    char *t = s+9;
+    int flen = len - 9;
+    int extracted;
     struct stat statbuf;
     address_item *last;
-    int extracted;
     FILE *f;
 
-    strncpy(filename, s+9, len-9);
-    filename[len-9] = 0;
+    while (flen > 0 && isspace((uschar)*t)) { t++; flen--; }
+
+    if (flen <= 0)
+      {
+      *error = string_sprintf("file name missing after :include:");
+      return EXTRACTED_ERROR;
+      }
+
+    if (flen > 255)
+      {
+      *error = string_sprintf("included file name \"%s\" is too long", t);
+      return EXTRACTED_ERROR;
+      }
+
+    strncpy(filename, t, flen);
+    filename[flen] = 0;
 
     /* Insist on absolute path */
 
     if (filename[0]!= '/')
       {
-      *error = string_sprintf("included file %s is not an absolute path",
+      *error = string_sprintf("included file \"%s\" is not an absolute path",
         filename);
-      return -1;
+      return EXTRACTED_ERROR;
       }
 
     /* Check file name if required - this is requested only when not running
@@ -1060,13 +1136,13 @@ for (;;)
       if (directory[0] == '*')
         {
         *error = "included files not permitted";
-        return -1;
+        return EXTRACTED_ERROR;
         }
       else if (strncmp(filename, directory, len) != 0 || *p != '/')
         {
         *error = string_sprintf("included file %s is not in directory %s",
           filename, directory);
-        return -1;
+        return EXTRACTED_ERROR;
         }
 
       /* It is necessary to check that every component inside the directory
@@ -1086,7 +1162,7 @@ for (;;)
           *error = string_sprintf("failed to stat %s (component of included "
             "file)", filename);
           *p = temp;
-          return -1;
+          return EXTRACTED_ERROR;
           }
 
         *p = temp;
@@ -1095,7 +1171,7 @@ for (;;)
           {
           *error = string_sprintf("included file %s in the %s directory "
             "involves a symbolic link", filename, directory);
-          return -1;
+          return EXTRACTED_ERROR;
           }
         }
       }
@@ -1104,9 +1180,8 @@ for (;;)
 
     if ((f = fopen(filename, "r")) == NULL)
       {
-      *error = string_sprintf("failed to open included file %s: %s",
-        filename, strerror(errno));
-      return +1;
+      *error = string_open_failed(errno, "included file %s", filename);
+      return EXTRACTED_INCLUDEFAIL;
       }
 
     if (fstat(fileno(f), &statbuf) != 0)
@@ -1114,7 +1189,7 @@ for (;;)
       *error = string_sprintf("failed to stat included file %s: %s",
         filename, strerror(errno));
       fclose(f);
-      return +1;
+      return EXTRACTED_INCLUDEFAIL;
       }
 
     /* If directory was checked, double check that we opened a regular file */
@@ -1123,7 +1198,7 @@ for (;;)
       {
       *error = string_sprintf("included file %s is not a regular file in "
         "the %s directory", filename, directory);
-      return -1;
+      return EXTRACTED_ERROR;
       }
 
     /* Get a buffer and read the contents */
@@ -1134,22 +1209,28 @@ for (;;)
       *error = string_sprintf("error while reading included file %s: %s",
         filename, strerror(errno));
       fclose(f);
-      return -1;
+      return EXTRACTED_ERROR;
       }
     filebuf[statbuf.st_size] = 0;
     fclose(f);
 
     addr = NULL;
-    extracted = parse_extract_addresses(filebuf, &addr, error, optional,
-      allow_blackhole, rewrite, directory, syntax_errors);
+    extracted = parse_extract_addresses(filebuf, incoming_domain, &addr,
+      error, optional, allow_specials, rewrite, directory, syntax_errors);
     store_free(filebuf);
     if (extracted != 0) return extracted;    /* error already set */
 
-    last = addr;
-    while (last->next != NULL) { count++; last = last->next; }
-    last->next = *anchor;
-    *anchor = addr;
-    count++;
+    /* If addr == NULL the included file contained a permitted :blackhole: */
+
+    if (addr != NULL)
+      {
+      last = addr;
+      while (last->next != NULL) { count++; last = last->next; }
+      last->next = *anchor;
+      *anchor = addr;
+      count++;
+      }
+    else optional = TRUE;
     }
 
   /* Else (not :include:) ensure address is syntactically correct and fully
@@ -1179,17 +1260,20 @@ for (;;)
     s[len] = 0;
 
     /* If it starts with \ and the rest of it parses as a valid mail address
-    without a domain, carry on with that address. Otherwise arrange for it
-    to fall through, causing an error message on the re-parse. */
+    without a domain, carry on with that address, but qualify it with the
+    incoming domain. Otherwise arrange for the address to fall through,
+    causing an error message on the re-parse. */
 
     if (*s == '\\')
       {
       recipient =
         parse_extract_address(s+1, error, &start, &end, &domain, FALSE);
-      if (recipient != NULL && domain != 0)
+      if (recipient != NULL)
         {
+        char *new = (domain != 0)? NULL :
+          string_sprintf("%s@%s", recipient, incoming_domain);
         store_free(recipient);
-        recipient = NULL;
+        recipient = new;
         }
       }
 
@@ -1259,7 +1343,7 @@ for (;;)
           {
           *error = string_sprintf("%s in \"%s\"", *error, s);
           s[len] = save;   /* _after_ using it for *error */
-          return -1;
+          return EXTRACTED_ERROR;
           }
         }
 
@@ -1294,15 +1378,19 @@ for (;;)
 **************************************************
 *************************************************/
 
-#ifdef STAND_ALONE
+#if defined STAND_ALONE && !defined TEST_EXPAND
 int main(void)
 {
 int start, end, domain;
-char buffer[256];
+char buffer[1024];
+
+big_buffer = store_malloc(big_buffer_size);
+
+/* strip_trailing_dot = TRUE; */
 
 printf("Testing parse_extract_address without group syntax\n");
 
-while (fgets(buffer, 256, stdin) != NULL)
+while (fgets(buffer, sizeof(buffer), stdin) != NULL)
   {
   char *out;
   char *errmess;
@@ -1310,7 +1398,7 @@ while (fgets(buffer, 256, stdin) != NULL)
   out = parse_extract_address(buffer, &errmess, &start, &end, &domain, FALSE);
   if (out == NULL) printf("*** bad address: %s\n", errmess); else
     {
-    char extract[100];
+    char extract[1024];
     strncpy(extract, buffer+start, end-start);
     extract[end-start] = 0;
     printf("%s %d %d %d \"%s\"\n", out, start, end, domain, extract);
@@ -1321,41 +1409,64 @@ while (fgets(buffer, 256, stdin) != NULL)
 printf("Testing parse_extract_address with group syntax\n");
 
 parse_allow_group = TRUE;
-while (fgets(buffer, 256, stdin) != NULL)
+while (fgets(buffer, sizeof(buffer), stdin) != NULL)
   {
   char *out;
   char *errmess;
+  char *s;
   buffer[(int)strlen(buffer) - 1] = 0;
-  out = parse_extract_address(buffer, &errmess, &start, &end, &domain, FALSE);
-  if (out == NULL) printf("*** bad address: %s\n", errmess); else
+  s = buffer;
+  while (*s != 0)
     {
-    char extract[100];
-    strncpy(extract, buffer+start, end-start);
-    extract[end-start] = 0;
-    printf("%s %d %d %d \"%s\"\n", out, start, end, domain, extract);
-    store_free(out);
+    char *ss = parse_find_address_end(s, FALSE);
+    int terminator = *ss;
+    *ss = 0;
+    out = parse_extract_address(buffer, &errmess, &start, &end, &domain, FALSE);
+    *ss = terminator;
+
+    if (out == NULL) printf("*** bad address: %s\n", errmess); else
+      {
+      char extract[1024];
+      strncpy(extract, buffer+start, end-start);
+      extract[end-start] = 0;
+      printf("%s %d %d %d \"%s\"\n", out, start, end, domain, extract);
+      store_free(out);
+      }
+
+    s = ss + (terminator? 1:0);
+    while (isspace(*s)) s++;
     }
   }
 
 printf("Testing parse_fix_phrase\n");
 
-while (fgets(buffer, 256, stdin) != NULL)
+while (fgets(buffer, sizeof(buffer), stdin) != NULL)
   {
   buffer[(int)strlen(buffer)-1] = 0;
   printf("%s\n", parse_fix_phrase(buffer));
   }
 
+printf("Testing parse_find_at\n");
+
+while (fgets(buffer, sizeof(buffer), stdin) != NULL)
+  {
+  char *s;
+  buffer[(int)strlen(buffer)-1] = 0;
+  s = parse_find_at(buffer);
+  if (s == NULL) printf("no @ found\n");
+    else printf("offset = %d\n", s - buffer);
+  }
+
 printf("Testing parse_extract_addresses\n");
 
-while (fgets(buffer, 256, stdin) != NULL)
+while (fgets(buffer, sizeof(buffer), stdin) != NULL)
   {
-  char *out;
   char *errmess;
   int extracted;
   address_item *anchor = NULL;
   buffer[(int)strlen(buffer) - 1] = 0;
-  if ((extracted = parse_extract_addresses(buffer, &anchor, &errmess,
-      FALSE, TRUE, TRUE, NULL)) == 0)
+  if ((extracted = parse_extract_addresses(buffer, "incoming.domain", &anchor,
+      &errmess, FALSE, TRUE, TRUE, NULL, NULL)) == 0)
     {
     while (anchor != NULL)
       {
