@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions that operate on the input queue. */
@@ -26,31 +26,40 @@ been delivered in the meantime by some other means.
 The global variables queue_force and queue_local may be set to cause forced
 deliveries or local-only deliveries, respectively.
 
-If deliver_selectstring is not NULL, the deliver_message() function skips
-messages whose recipients do not contain the string. As this option is used
-when a machine comes back online, we want to ensure that at least one
-delivery attempt takes place, so force the first one.
+If deliver_selectstring is not NULL, skip messages whose recipients do not
+contain the string. As this option is typically used when a machine comes back
+online, we want to ensure that at least one delivery attempt takes place, so
+force the first one.
+
+If queue_2stage is set, the queue is scanned twice. The first time, queue_smtp
+is set so that routing is done for all messages. Thus in the second run those
+that are routed to the same host should go down the same SMTP connection.
 
 Arguments:
   start_id   message id to start at, or NULL for all
   stop_id    message id to end at, or NULL for all
+  recurse    TRUE if recursing for 2-stage run
 
 Returns:     nothing
 */
 
 void
-queue_run(char *start_id, char *stop_id)
+queue_run(char *start_id, char *stop_id, BOOL recurse)
 {
 BOOL force_delivery = queue_run_force || deliver_selectstring != NULL;
 queue_filename *f;
-log_write(0, LOG_MAIN, "Start queue run: pid=%d%s%s", getpid(),
-  (deliver_selectstring == NULL)? "" : " -R ",
-  (deliver_selectstring == NULL)? "" : deliver_selectstring);
+
+if (!recurse)
+  log_write(0, LOG_MAIN, "Start queue run: pid=%d%s%s", getpid(),
+    (deliver_selectstring == NULL)? "" : " -R ",
+    (deliver_selectstring == NULL)? "" : deliver_selectstring);
 
 /* Turn off the flags that cause SMTP deliveries and remote processing not to
-happen, and set the flag that indicates a queue run is in progress. */
+happen, unless doing a 2-stage queue run, when the SMTP flag gets set. Set
+the flag that indicates a queue run is in progress. */
 
-queue_smtp = queue_remote = FALSE;
+queue_remote = FALSE;
+queue_smtp = queue_2stage;
 queue_running = TRUE;
 
 /* Run the queue. If a start or finish id is given, we must take the queue in
@@ -60,7 +69,8 @@ that matches the string. This does mean we have to read the header file, and
 then again when actually delivering, but it's cheaper than forking a delivery
 process for each file. */
 
-for (f = queue_get_spool_list(start_id == NULL && stop_id == NULL);
+for (f = queue_get_spool_list(!queue_run_in_order &&
+       start_id == NULL && stop_id == NULL);
      f != NULL; f = f->next)
   {
   if (stop_id != NULL && strcmp(f->text, stop_id) > 0) break;
@@ -71,7 +81,8 @@ for (f = queue_get_spool_list(start_id == NULL && stop_id == NULL);
     struct stat statbuf;
     char buffer[256];
 
-    sprintf(buffer, "%s/input/%s", spool_directory, f->text);
+    message_subdir[0] = f->dir_char;
+    sprintf(buffer, "%s/input/%s/%s", spool_directory, message_subdir, f->text);
     if (stat(buffer, &statbuf) < 0) continue;
 
     /* Check for a matching address if deliver_selectstring is set. If so,
@@ -81,7 +92,7 @@ for (f = queue_get_spool_list(start_id == NULL && stop_id == NULL);
     if (deliver_selectstring != NULL)
       {
       BOOL found = FALSE;
-      if (spool_read_header(f->text, FALSE) != spool_read_OK) continue;
+      if (spool_read_header(f->text, FALSE, TRUE) != spool_read_OK) continue;
 
       if (recipients_list != NULL)
         {
@@ -136,9 +147,36 @@ for (f = queue_get_spool_list(start_id == NULL && stop_id == NULL);
     has already been written to the log by the delivery process. */
 
     if ((status & 0x00ff) == 0 && (status & 0xff00) == (DEFER << 8)) break;
+
+    /* If the process crashed, tell somebody */
+
+    if ((status & 0x00ff) != 0)
+      log_write(0, LOG_MAIN|LOG_PANIC,
+        "queue run: process %d crashed with signal %d while delivering %s",
+        (int)pid, status & 0x00ff, f->text);
     }
   }
-log_write(0, LOG_MAIN, "End queue run: pid=%d", getpid());
+
+/* If queue_2stage is true, we do it all again, with the 2stage flag
+turned off. */
+
+if (queue_2stage)
+  {
+  queue_2stage = FALSE;
+  queue_run(start_id, stop_id, TRUE);
+  }
+
+/* At top level, if smtp_etrn_serialize is set, remove the serialization
+block for this -R text, if any. Then log the end of the run. */
+
+if (!recurse)
+  {
+  if (smtp_etrn_serialize && deliver_selectstring != NULL)
+    transport_end_serialized("etrn-runs", deliver_selectstring);
+  log_write(0, LOG_MAIN, "End queue run: pid=%d%s%s", getpid(),
+    (deliver_selectstring == NULL)? "" : " -R ",
+    (deliver_selectstring == NULL)? "" : deliver_selectstring);
+  }
 }
 
 
@@ -157,6 +195,10 @@ according to the file name. This should give the order in which the messages
 arrived. It is used only for presentation to humans, so the insertion sort
 that it does is not part of the normal operational code.
 
+If any single-character sub-directories are encountered in the initial
+scan, they are subsequently scanned for message files too. The returned
+data blocks contain the identifying character of the subdirectory, if any.
+
 Argument:   TRUE if the order of the list is to be unpredictable
 Returns:    pointer to a chain of queue name items
 */
@@ -164,117 +206,228 @@ Returns:    pointer to a chain of queue name items
 queue_filename *
 queue_get_spool_list(BOOL randomize)
 {
+int i;
+int subdir_max = 1;
 int flags = 0;
 int resetflags = -1;
+int subptr;
 queue_filename *yield = NULL;
 queue_filename *last = NULL;
 struct dirent *ent;
 DIR *dd;
+char subdirs[64];
 char buffer[256];
 
-sprintf(buffer, "%s/input", spool_directory);
-dd = opendir(buffer);
-if (dd == NULL) return NULL;
-
 /* The file names are added onto the start or end of the list according to the
 bits of the flags variable. When randomizing, get a collection of bits from the
 current time. Use the bottom 16 and just keep re-using them if necessary. */
 
 if (randomize) resetflags = time(NULL) & 0xFFFF;
 
-/* Now scan the directory. */
+/* This loop runs at least once, for the main directory, and then as many
+times as necessary to scan any subdirectories encountered in the main
+directory. */
 
-while ((ent = readdir(dd)) != NULL)
+subdirs[0] = 0;
+sprintf(buffer, "%s/input", spool_directory);
+subptr = (int)strlen(buffer);
+buffer[subptr+2] = 0;               /* terminator for lengthened name */
+
+for (i = 0; i < subdir_max; i++)
   {
-  char *name = ent->d_name;
-  if ((int)strlen(name) == SPOOL_NAME_LENGTH &&
-      strcmp(name + SPOOL_NAME_LENGTH - 2, "-H") == 0)
+  int count = 0;
+  int subdirchar = subdirs[i];      /* 0 for main directory */
+  if (subdirchar != 0)
     {
-    queue_filename *next =
-      store_malloc(sizeof(queue_filename) + (int)strlen(name));
-    strcpy(next->text, name);
+    buffer[subptr] = '/';
+    buffer[subptr+1] = subdirchar;
+    }
+  dd = opendir(buffer);
+  if (dd == NULL) continue;
 
-    /* First item becomes the top and bottom of the list. */
+  /* Now scan the directory. */
 
-    if (yield == NULL)
+  while ((ent = readdir(dd)) != NULL)
+    {
+    char *name = ent->d_name;
+    int len = (int)strlen(name);
+
+    /* Count entries */
+
+    count++;
+
+    /* If we find a single alphameric sub-directory on the first
+    pass, add it to the list for subsequent scans. */
+
+    if (i == 0 && len == 1 && isalnum(*name))
       {
-      next->next = NULL;
-      yield = last = next;
+      subdirs[subdir_max++] = *name;
+      continue;
       }
 
-    /* If randomizing, insert at either top or bottom, randomly. This is, I
-    argue, faster than doing a sort by allocating a random number to each item,
-    and it also saves having to store the number with each item. */
+    /* Otherwise, if it is a header spool file, add it to the list */
 
-    else if (randomize)
+    if (len == SPOOL_NAME_LENGTH &&
+        strcmp(name + SPOOL_NAME_LENGTH - 2, "-H") == 0)
       {
-      if (flags == 0) flags = resetflags;
-      if ((flags & 1) == 0)
+      queue_filename *next =
+        store_malloc(sizeof(queue_filename) + (int)strlen(name));
+      strcpy(next->text, name);
+      next->dir_char = subdirchar;
+
+      /* First item becomes the top and bottom of the list. */
+
+      if (yield == NULL)
+        {
+        next->next = NULL;
+        yield = last = next;
+        }
+
+      /* If randomizing, insert at either top or bottom, randomly. This is, I
+      argue, faster than doing a sort by allocating a random number to each item,
+      and it also saves having to store the number with each item. */
+
+      else if (randomize)
+        {
+        if (flags == 0) flags = resetflags;
+        if ((flags & 1) == 0)
+          {
+          next->next = yield;
+          yield = next;
+          }
+        else
+          {
+          next->next = NULL;
+          last->next = next;
+          last = next;
+          }
+        flags = flags >> 1;
+        }
+
+      /* Otherwise do an insertion sort based on the name. First see if
+      it should go before the first item. */
+
+      else if (strcmp(next->text, yield->text) < 0)
         {
         next->next = yield;
         yield = next;
         }
+
+      /* Otherwise find the item it should go after; check the last one
+      first, because that will often be the case. */
+
       else
         {
-        next->next = NULL;
-        last->next = next;
-        last = next;
+        queue_filename *this;
+        if (strcmp(next->text, last->text) < 0)
+          {
+          for (this = yield; this != last; this = this->next)
+            if (strcmp(next->text, this->next->text) < 0) break;
+          }
+        else this = last;
+        next->next = this->next;
+        this->next = next;
+        if (this == last) last = next;
         }
-      flags = flags >> 1;
       }
+    }
 
-    /* Otherwise do an insertion sort based on the name. First see if
-    it should go before the first item. */
+  /* Close the directory and, it was empty (count == 2 implies just "." and
+  ".." entries) and it was a subdirectory, and Exim is no longer configured
+  to use subdirectories, attempt to get rid of it. At the same time, try
+  to get rid of any corresponding msglog subdirectory. These are just
+  cosmetic tidying actions, so just ignore failures. */
 
-    else if (strcmp(next->text, yield->text) < 0)
-      {
-      next->next = yield;
-      yield = next;
-      }
-
-    /* Otherwise find the item it should go after; check the last one
-    first, because that will often be the case. */
-
-    else
-      {
-      queue_filename *this;
-      if (strcmp(next->text, last->text) < 0)
-        {
-        for (this = yield; this != last; this = this->next)
-          if (strcmp(next->text, this->next->text) < 0) break;
-        }
-      else this = last;
-      next->next = this->next;
-      this->next = next;
-      if (this == last) last = next;
-      }
+  closedir(dd);
+  if (i != 0 && !split_spool_directory && count <= 2)
+    {
+    rmdir(buffer);
+    sprintf(big_buffer, "%s/msglog/%c", spool_directory, subdirchar);
+    rmdir(big_buffer);
     }
   }
 
-closedir(dd);
 return yield;
 }
 
 
 
 
+/************************************************
+*          List extra deliveries                *
+************************************************/
+
+/* This is called from queue_list below to print out all addresses that
+have received a message but which were not primary addresses. That is, all
+the addresses in the tree of non-recipients that are not primary addresses.
+The tree has been scanned and the data field filled in for those that are
+primary addresses.
+
+Argument:    points to the tree node
+Returns:     nothing
+*/
+
+static void queue_list_extras(tree_node *p)
+{
+if (p->left != NULL) queue_list_extras(p->left);
+if (!p->data.val) printf("       +D %s\n", p->name);
+if (p->right != NULL) queue_list_extras(p->right);
+}
+
+
+
 /************************************************
 *          List messages on the queue           *
 ************************************************/
 
-/* We get a list of file names as quickly as possible, then scan each one for
-information to output. If any disappear while we are processing, just leave
-them out. This function is a top-level function that is obeyed as a result of
-the -bp argument. There is no point wasting resources in freeing the store used
-for the list of names at the end. However, as there may be a lot of messages on
-the queue, we must tidy up the store after reading the headers for each one. */
+/* Or a given list of messages. In the "all" case, we get a list of file names
+as quickly as possible, then scan each one for information to output. If any
+disappear while we are processing, just leave them out, but give an error if an
+explicit list was given. This function is a top-level function that is obeyed
+as a result of the -bp argument. There is no point wasting resources in freeing
+the store used for the list of names at the end. However, as there may be a lot
+of messages on the queue, we must tidy up the store after reading the headers
+for each one.
+
+Arguments:
+   option     0 => list top-level recipients, with "D" for those delivered
+              1 => list only undelivered top-level recipients
+              2 => as 0, plus any generated delivered recipients
+   list       => first of any message ids to list
+   count      count of message ids; 0 => all
+
+Returns:      nothing
+*/
 
 void
-queue_list(void)
+queue_list(int option, char **list, int count)
 {
 int i;
 int now = (int)time(NULL);
-queue_filename *f = queue_get_spool_list(FALSE);
+queue_filename *f;
+
+/* If given a list of messages, build a chain containing their ids. */
+
+if (count > 0)
+  {
+  queue_filename *last;
+  for (i = 0; i < count; i++)
+    {
+    queue_filename *next =
+      store_malloc(sizeof(queue_filename) + (int)strlen(list[i]) + 2);
+    sprintf(next->text, "%s-H", list[i]);
+    next->dir_char = '*';
+    next->next = NULL;
+    if (i == 0) f = next; else last->next = next;
+    last = next;
+    }
+  }
+
+/* Otherwise get a list of the entire queue. */
+
+else f = queue_get_spool_list(FALSE);
+
+/* Now scan the chain and print information. */
 
 for (; f != NULL; f = f->next)
   {
@@ -283,14 +436,17 @@ for (; f != NULL; f = f->next)
   BOOL env_read;
 
   message_size = 0;
-  rc = spool_read_header(f->text, FALSE);
+  message_subdir[0] = f->dir_char;
+  rc = spool_read_header(f->text, FALSE, count <= 0);
+  if (rc == spool_read_notopen && errno == ENOENT && count <= 0) continue;
   save_errno = errno;
   env_read = (rc == spool_read_OK || rc == spool_read_hdrerror);
 
   if (env_read)
     {
     struct stat statbuf;
-    sprintf(big_buffer, "%s/input/%s", spool_directory, f->text);
+    sprintf(big_buffer, "%s/input/%s/%s", spool_directory, message_subdir,
+      f->text);
     big_buffer[(int)strlen(big_buffer)-1] = 'D';
     if (stat(big_buffer, &statbuf) == 0) size = message_size + statbuf.st_size;
     i = (now - received_time)/60;  /* minutes on queue */
@@ -302,12 +458,14 @@ for (; f != NULL; f = f->next)
     else printf("%2dm ", i);
     }
 
-  fprintf(stdout, "%s ", string_formatsize(size, big_buffer));
+  fprintf(stdout, "%s ", string_format_size(size, big_buffer));
   for (i = 0; i < 16; i++) fputc(f->text[i], stdout);
 
   if (env_read && sender_address != NULL)
     {
-    printf(" <%s>", sender_address);
+    if (user_null_sender && sender_address[0] != 0)
+      printf(" <> (%s)", sender_address);
+    else printf(" <%s>", sender_address);
     store_free(sender_address);
     }
 
@@ -317,15 +475,19 @@ for (; f != NULL; f = f->next)
     if (save_errno == ERRNO_SPOOLFORMAT)
       {
       struct stat statbuf;
-      sprintf(big_buffer, "%s/input/%s", spool_directory, f->text);
+      sprintf(big_buffer, "%s/input/%s/%s", spool_directory, message_subdir,
+        f->text);
       if (stat(big_buffer, &statbuf) == 0)
-        printf("*** spool format error: size=%d ***", (int)statbuf.st_size);
+        {
+        int size = statbuf.st_size;    /* Because might be a long */
+        printf("*** spool format error: size=%d ***", size);
+        }
       else printf("*** spool format error ***");
       }
     else printf("*** spool read error: %s ***", strerror(save_errno));
     if (rc != spool_read_hdrerror)
       {
-      printf("\n");
+      printf("\n\n");
       continue;
       }
     }
@@ -338,13 +500,18 @@ for (; f != NULL; f = f->next)
     {
     for (i = 0; i < recipients_count; i++)
       {
-      printf("        %s %s\n",
-        (tree_search_addr(tree_nonrecipients,recipients_list[i],FALSE) == NULL)?
-          " ":"D", recipients_list[i]);
+      tree_node *delivered =
+        tree_search_addr(tree_nonrecipients, recipients_list[i], FALSE);
+      if (!delivered || option != 1)
+        printf("        %s %s\n", (delivered != NULL)? "D":" ",
+          recipients_list[i]);
+      if (delivered != NULL) delivered->data.val = TRUE;
       store_free(recipients_list[i]);
       }
-    printf("\n");
     store_free(recipients_list);
+    if (option == 2 && tree_nonrecipients != NULL)
+      queue_list_extras(tree_nonrecipients);
+    printf("\n");
     }
 
   tree_free(tree_nonrecipients);
@@ -376,7 +543,7 @@ BOOL
 queue_action(char *id, int action, FILE *f, char **argv, int argc,
   int recipients_arg)
 {
-int i;
+int i, j;
 BOOL yield = TRUE;
 struct passwd *pw;
 char *doing = NULL;
@@ -399,20 +566,28 @@ if (!spool_open_datafile(id))
     yield = FALSE;
     if (f != NULL) fprintf(f, "Spool data file for %s does not exist\n", id);
     if (action != MSG_REMOVE || !admin_user) return FALSE;
-    if (f != NULL) fprintf(f, "Continuing to ensure all files removed\n");
+    if (f != NULL) fprintf(f, "Continuing, to ensure all files removed\n");
     }
   else
     {
-    if (f != NULL) fprintf(f, "Message %s is locked\n", id);
+    if (f != NULL)
+      {
+      if (errno == 0) fprintf(f, "Message %s is locked\n", id);
+        else fprintf(f, "Couldn't open spool file for %s: %s\n", id,
+          strerror(errno));
+      }
     return FALSE;
     }
   }
 
 /* Read the spool header file for the message. Again, continue after an
-error only in the case of deleting by an administrator. */
+error only in the case of deleting by an administrator. Setting the third
+argument false causes it to look both in the main spool directory and in
+the appropriate subdirectory, and set message_subdir according to where it
+found the message. */
 
 sprintf(spoolname, "%s-H", id);
-if (spool_read_header(spoolname, TRUE) != spool_read_OK)
+if (spool_read_header(spoolname, TRUE, FALSE) != spool_read_OK)
   {
   yield = FALSE;
   if (f != NULL)
@@ -465,7 +640,7 @@ switch(action)
     deliver_freeze = TRUE;
     deliver_manual_thaw = FALSE;
     deliver_frozen_at = time(NULL);
-    if (spool_write_header(id) > 0)
+    if (spool_write_header(id) >= 0)
       {
       if (f != NULL) fprintf(f, "is now frozen\n");
       log_write(0, LOG_MAIN, "frozen by %s", username);
@@ -491,7 +666,7 @@ switch(action)
     {
     deliver_freeze = FALSE;
     deliver_manual_thaw = TRUE;
-    if (spool_write_header(id) > 0)
+    if (spool_write_header(id) >= 0)
       {
       if (f != NULL) fprintf(f, "is no longer frozen\n");
       log_write(0, LOG_MAIN, "unfrozen by %s", username);
@@ -507,22 +682,17 @@ switch(action)
   break;
 
 
-  case MSG_REMOVE:
-  sprintf(spoolname, "%s/msglog/%s", spool_directory, id);
-  if (unlink(spoolname) < 0)
-    {
-    if (errno != ENOENT)
-      {
-      yield = FALSE;
-      if (f != NULL)
-        fprintf(f, "Error while removing %s: %s\n", spoolname,
-          strerror(errno));
-      }
-    }
+  /* We must ensure all files are removed from both the input directory
+  and the appropriate subdirectory, to clean up cases when there are odd
+  files left lying around in odd places. In the normal case message_subdir
+  will have been set correctly by spool_read_header, but as this is a rare
+  operation, just run everything twice. */
 
-  for (i = 0; i < 3; i++)
+  case MSG_REMOVE:
+  message_subdir[0] = id[5];
+  for (j = 0; j < 2; message_subdir[0] = 0, j++)
     {
-    sprintf(spoolname, "%s/input/%s-%c", spool_directory, id, "DHJ"[i]);
+    sprintf(spoolname, "%s/msglog/%s/%s", spool_directory, message_subdir, id);
     if (unlink(spoolname) < 0)
       {
       if (errno != ENOENT)
@@ -533,6 +703,22 @@ switch(action)
             strerror(errno));
         }
       }
+
+    for (i = 0; i < 3; i++)
+      {
+      sprintf(spoolname, "%s/input/%s/%s-%c", spool_directory, message_subdir,
+        id, "DHJ"[i]);
+      if (unlink(spoolname) < 0)
+        {
+        if (errno != ENOENT)
+          {
+          yield = FALSE;
+          if (f != NULL)
+            fprintf(f, "Error while removing %s: %s\n", spoolname,
+              strerror(errno));
+          }
+        }
+      }
     }
 
   /* In the common case, the datafile is open (and locked), so give the
@@ -553,7 +739,7 @@ switch(action)
     {
     tree_add_nonrecipient(recipients_list[i], FALSE);
     }
-  if (spool_write_header(id) > 0)
+  if (spool_write_header(id) >= 0)
     {
     if (f != NULL) fprintf(f, "has been modified\n");
     for (i = 0; i < recipients_count; i++)
@@ -650,6 +836,7 @@ switch(action)
         else  /* MSG_EDIT_SENDER */
           {
           sender_address = receiver;
+          user_null_sender = FALSE;
           log_write(0, LOG_MAIN, "sender address changed to <%s> by %s",
             receiver, username);
           }
@@ -659,7 +846,7 @@ switch(action)
 
     if (yield)
       {
-      if (spool_write_header(id) > 0)
+      if (spool_write_header(id) >= 0)
         {
         if (f != NULL) fprintf(f, "has been modified\n");
         }
@@ -700,7 +887,8 @@ switch(action)
 
     /* Make a temporary file to copy to */
 
-    sprintf(copyname, "%s/input/%s-D-%d", spool_directory, id, (int)getpid());
+    sprintf(copyname, "%s/input/%s/%s-D-%d", spool_directory, message_subdir,
+      id, (int)getpid());
     copy_fd = open(copyname, O_WRONLY|O_CREAT, SPOOL_MODE);
     if (copy_fd < 0)
       {
@@ -744,11 +932,13 @@ switch(action)
 
     if (yield)
       {
-      sprintf(spoolname, "/bin/sh -c \"${VISUAL:-${EDITOR:-vi}} %s\"", copyname);
+      sprintf(spoolname, "/bin/sh -c \"${VISUAL:-${EDITOR:-vi}} %s\"",
+        copyname);
 
       if ((system(spoolname) & 0xffff) == 0)
         {
-        sprintf(spoolname, "%s/input/%s-D", spool_directory, id);
+        sprintf(spoolname, "%s/input/%s/%s-D", spool_directory, message_subdir,
+          id);
         if ((rc = rename(copyname, spoolname)) == 0)
           {
           if (f != NULL) fprintf(f, "has been modified\n");
