@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with retrying unsuccessful deliveries. */
@@ -43,9 +43,12 @@ char *
 retry_check_address(host_item *host)
 {
 char *key;
-EXIM_DB *dbm_file;
+open_db dbblock;
+open_db *dbm_file;
 tree_node *node;
-db_retry *retry_record;
+dbdata_retry *retry_record;
+
+DEBUG(9) debug_printf("checking status of %s\n", host->name);
 
 /* Do nothing if status already set; otherwise initialize status as usable. */
 
@@ -65,6 +68,7 @@ the retry database when it is updated). */
 node = tree_search(tree_unusable, key);
 if (node != NULL)
   {
+  DEBUG(9) debug_printf("found in tree of unusables\n");
   host->status = hstatus_unusable;
   host->why = node->data.val;
   return NULL;
@@ -73,14 +77,15 @@ if (node != NULL)
 /* Open the retry database, giving up if there isn't one. Otherwise, search for
 the retry record, and then close the database again. */
 
-if ((dbm_file = db_open("retry", O_RDONLY)) == NULL) return NULL;
-retry_record = db_read(dbm_file, key);
-db_close(dbm_file);
+if ((dbm_file = dbfn_open("retry", O_RDONLY, &dbblock)) == NULL) return NULL;
+retry_record = dbfn_read(dbm_file, key);
+dbfn_close(dbm_file);
 
 /* If there's no retry record, free the key and return. */
 
 if (retry_record == NULL)
   {
+  DEBUG(9) debug_printf("no retry record\n");
   store_free(key);
   return NULL;
   }
@@ -267,7 +272,8 @@ void
 retry_update(address_item **addr_defer, address_item **addr_failed,
   address_item **addr_succeed)
 {
-EXIM_DB *dbm_file = NULL;
+open_db dbblock;
+open_db *dbm_file = NULL;
 time_t now = time(NULL);
 int i;
 
@@ -315,7 +321,7 @@ for (i = 0; i < 3; i++)
         int message_length, message_space, failing_interval, next_try;
         retry_rule *rule, *final_rule;
         retry_config *retry;
-        db_retry *retry_record;
+        dbdata_retry *retry_record;
 
         /* Open the retry database if it is not already open; failure to open
         the file is logged, but otherwise ignored - deferred addresses will
@@ -323,7 +329,8 @@ for (i = 0; i < 3; i++)
         opening if no addresses have retry items - common when none have yet
         reached their retry next try time. */
 
-        if (dbm_file == NULL) dbm_file = db_open("retry", O_RDWR|O_CREAT);
+        if (dbm_file == NULL)
+          dbm_file = dbfn_open("retry", O_RDWR|O_CREAT, &dbblock);
         if (dbm_file == NULL)
           {
           log_write(0, LOG_MAIN, "Failed to open retry database");
@@ -335,7 +342,7 @@ for (i = 0; i < 3; i++)
 
         if (rti->delete)
           {
-          (void)db_delete(dbm_file, rti->destination);
+          (void)dbfn_delete(dbm_file, rti->destination);
           DEBUG(5) debug_printf("deleted retry information for %s\n",
             rti->destination);
           if (addr == endaddr) timed_out = FALSE;
@@ -377,10 +384,10 @@ for (i = 0; i < 3; i++)
 
         /* Read a retry record from the database or construct a new one */
 
-        retry_record = db_read(dbm_file, rti->destination);
+        retry_record = dbfn_read(dbm_file, rti->destination);
         if (retry_record == NULL)
           {
-          retry_record = store_malloc(sizeof(db_retry) + message_length);
+          retry_record = store_malloc(sizeof(dbdata_retry) + message_length);
           message_space = message_length;
           retry_record->first_failed = now;
           retry_record->last_try = now;
@@ -490,8 +497,8 @@ for (i = 0; i < 3; i++)
 
         if (message_length > message_space)
           {
-          db_retry *newr = store_malloc(sizeof(db_retry) + message_length);
-          memcpy(newr, retry_record, sizeof(db_retry));
+          dbdata_retry *newr = store_malloc(sizeof(dbdata_retry) + message_length);
+          memcpy(newr, retry_record, sizeof(dbdata_retry));
           retry_record = newr;
           }
 
@@ -513,8 +520,8 @@ for (i = 0; i < 3; i++)
             retry_record->more_errno, retry_record->text);
           }
 
-        db_write(dbm_file, rti->destination, retry_record,
-          sizeof(db_retry) + message_length);
+        dbfn_write(dbm_file, rti->destination, retry_record,
+          sizeof(dbdata_retry) + message_length);
         }                            /* Loop for each retry item */
       }                              /* Loop for address & parents */
 
@@ -533,7 +540,11 @@ for (i = 0; i < 3; i++)
       *addr_failed = endaddr;
       endaddr->message = (endaddr->message == NULL)? "retry timeout exceeded" :
         string_sprintf("%s: retry timeout exceeded", endaddr->message);
-      log_write(0, LOG_MAIN, "** %s: retry timeout exceeded", endaddr->orig);
+      log_write(0, LOG_MAIN, "** %s%s%s%s: retry timeout exceeded",
+        endaddr->orig,
+       (endaddr->parent == NULL)? "" : " <",
+       (endaddr->parent == NULL)? "" : endaddr->parent->orig,
+       (endaddr->parent == NULL)? "" : ">");
       }
 
     else paddr = &(endaddr->next);    /* Advance to next address */
@@ -542,7 +553,7 @@ for (i = 0; i < 3; i++)
 
 /* Close and unlock the database */
 
-if (dbm_file != NULL) db_close(dbm_file);
+if (dbm_file != NULL) dbfn_close(dbm_file);
 
 DEBUG(2) debug_printf("end of retry processing\n");
 }
