@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with rewriting headers */
@@ -59,14 +59,17 @@ return s;
 
 /* The incoming address is NOT freed. The yield is the input address if there
 is no rewriting to be done. For source-routed addresses, it is only the final
-route-addr that gets considered. Assume the input is a valid address. This
-function is called only
+route-addr that gets considered. Assume the input is a valid address, except in
+the case of SMTP-time rewriting, which is handled specially. When this function
+is called while processing filter and forward files, the uid may be that of the
+user. Ensure it is reset while expanding a replacement, in case that involves
+file lookups.
 
 Arguments:
   s           address to rewrite
   flag        indicates which header this address comes from; it must
                 match the flags in the rewriting rule
-  whole       if not NULL, set TRUE if the rewriting rule contained the
+  whole       if not NULL, set TRUE if any rewriting rule contained the
                 "whole" bit and it is a header that is being rewritten
   add_header  if TRUE and rewriting occurs, add an "X-rewrote-xxx" header
                 if headers are in existence
@@ -77,7 +80,7 @@ Returns:      new address if rewritten; the input address if no change;
               rewritten address is returned, not just the active bit.
 */
 
-static char *
+char *
 rewrite_one(char *s, int flag, BOOL *whole, BOOL add_header, char *name)
 {
 rewrite_rule *rule;
@@ -85,85 +88,156 @@ char *yield = s;
 char *subject = s;
 char *domain = NULL;
 char *user = NULL;
+BOOL uid_checked = FALSE;
+BOOL uid_set = FALSE;
+BOOL done = FALSE;
+uid_t saved_euid = 0;
+gid_t saved_egid = 0;
 int pre_length = 0;
+int rule_number = 1;
+int yield_start, yield_end;
+
+if (whole != NULL) *whole = FALSE;
 
 /* Scan the rewriting rules */
 
-for (rule = rewrite_rules; rule != NULL; rule = rule->next)
+for (rule = rewrite_rules;
+     rule != NULL && !done;
+     rule_number++, rule = rule->next)
   {
   int start, end, pdomain;
+  int count = 0;
   char *save_localpart, *save_domain;
   char *error, *new, *newparsed;
   char buffer[256];
 
+  /* Ensure that the flag matches the flags in the rule. */
+
   if ((rule->flags & flag) == 0) continue;
 
-  /* If this is the first rule we are trying on this address, set domain
-  to point past the last '@' in the address and user to the start of the
-  address or following the last ':' if source-routed. If a source routed
-  address does not have an @ after the : then rewrite it using the
-  immediately preceding domain. */
+  /* Come back here for a repeat after a successful rewrite. We do this
+  only so many times. */
 
-  if (domain == NULL)
+  REPEAT_RULE:
+
+  /* If this is an SMTP-time rewrite, the pattern must be a regex and
+  the subject may have any structure. No local part or domain variables
+  can be set for the expansion. */
+
+  if ((flag & rewrite_smtp) != 0)
     {
-    domain = strrchr(subject, '@') + 1;
-    if (subject[0] != '@')
-      {
-      user = subject;
-      pre_length = 0;
-      }
-    else
-      {
-      user = strchr(subject, ':') + 1;
-      if (domain < user)
-        {
-        if (domain == subject+1)
-          user = subject = string_sprintf("%s@%.*s", user, user - domain - 1,
-            domain);
-        else
-          {
-          subject = string_sprintf("%.*s:%s@%.*s", domain - subject - 2,
-            subject, user, user - domain - 1, domain);
-          user = strchr(subject, ':') + 1;
-          }
-        domain = strrchr(subject, '@') + 1;
-        DEBUG(9)
-          debug_printf("re-wrote source routed address as %s\n", subject);
-        }
-      pre_length = user - subject;
-      }
+    if (!match_check_string(subject, rule->key, &(rule->re), 0, TRUE, NULL))
+      continue;
+    new = expand_string(rule->replacement);
     }
 
-  /* Use the general function for matching an address against a list (here
-  just one item, so set the separator to 0). */
+  /* All other rewrites expect the input to be a valid address, so local part
+  and domain variables can be set for expansion. */
 
-  if (!match_address_list(user, domain - user, rule->key, &(rule->re), 0, 0))
-    continue;
+  else
+    {
+    /* If this is the first rule we are trying on this address, set domain
+    to point past the last '@' in the address and user to the start of the
+    address or following the last ':' if source-routed. If a source routed
+    address does not have an @ after the : then rewrite it using the
+    immediately preceding domain. */
 
-  /* The source address matches, and numerical variables have been
-  set up. Expand the replacement string. While doing so, set $local_part
-  and $domain to the appropriate values, restoring whatever value they
-  previously had afterwards. */
+    if (domain == NULL)
+      {
+      domain = strrchr(subject, '@') + 1;
+      if (subject[0] != '@')
+        {
+        user = subject;
+        pre_length = 0;
+        }
+      else
+        {
+        user = strchr(subject, ':') + 1;
 
-  save_localpart = deliver_localpart;
-  save_domain = deliver_domain;
+        /* The rewriting to ensure there is an @ after the : always produces
+        a string that is one character shorter than it was before. Hence we
+        can malloc a piece of store of length strlen(subject). */
 
-  sprintf(buffer, "%.*s", domain-user-1, user);
-  deliver_localpart = buffer;
-  deliver_domain = domain;
+        if (domain < user)
+          {
+          char *new = store_malloc((int)strlen(subject));
+          if (domain == subject+1)
+            {
+            sprintf(new, "%s@%.*s", user, user-domain-1, domain);
+            user = subject = new;
+            }
+          else
+            {
+            sprintf(new, "%.*s:%s@%.*s", domain - subject - 2, subject,
+              user, user - domain - 1, domain);
+            subject = new;
+            user = strchr(subject, ':') + 1;
+            }
+          domain = strrchr(subject, '@') + 1;
+          DEBUG(9)
+            debug_printf("re-wrote source routed address as %s\n", subject);
+          }
+        pre_length = user - subject;
+        }
+      }
 
-  new = expand_string(rule->replacement);
+    /* Use the general function for matching an address against a list (here
+    just one item, so set the separator to 0). */
 
-  deliver_localpart = save_localpart;
-  deliver_domain = save_domain;
+    if (!match_address_list(user, domain - user, rule->key, &(rule->re), 0, 0))
+      continue;
+
+    /* The source address matches, and numerical variables have been
+    set up. Expand the replacement string, changing the uid/gid first if
+    necessary. While doing the expansion, set $local_part and $domain to the
+    appropriate values, restoring whatever value they previously had afterwards.
+    */
+
+    if (!uid_checked)
+      {
+      uid_t uid = exim_uid_set? exim_uid : root_uid;
+      uid_checked = TRUE;
+      saved_euid = geteuid();
+      saved_egid = getegid();
+      if (saved_euid != uid)
+        {
+        if (saved_euid != root_uid) mac_seteuid(root_uid);
+        if (exim_gid_set) mac_setegid(exim_gid);
+        mac_seteuid(uid);
+        uid_set = TRUE;
+        DEBUG(2) debug_printf("rewrite_one set euid=%d egid=%d\n",
+          (int)geteuid(), (int)getegid());
+        }
+      }
+
+    save_localpart = deliver_localpart;
+    save_domain = deliver_domain;
+
+    /* Paranoia */
+    if (domain - user > sizeof(buffer))
+      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "Attempt to re-write a local "
+        "part that is longer than %d characters (process %s): <%s>",
+        sizeof(buffer) - 1, process_info, s);
+
+    sprintf(buffer, "%.*s", domain - user - 1, user);
+    deliver_localpart = buffer;
+    deliver_domain = domain;
+
+    new = expand_string(rule->replacement);
+
+    deliver_localpart = save_localpart;
+    deliver_domain = save_domain;
+    }
 
   /* If the expansion failed with the "forcedfail" flag, don't generate
-  an error - just give up on this rewriting rule. Otherwise we have a
-  configuration error. */
+  an error - just give up on this rewriting rule. If the "q" flag is set,
+  give up altogether. For other expansion failures we have a configuration
+  error. */
 
   if (new == NULL)
     {
-    if (expand_string_forcedfail) continue;
+    if (expand_string_forcedfail)
+      { if ((rule->flags & rewrite_quit) != 0) break; else continue; }
     log_write(0, LOG_MAIN|LOG_PANIC, "Expansion of %s failed while rewriting: "
       "%s", rule->replacement, expand_string_message);
     break;
@@ -174,12 +248,16 @@ for (rule = rewrite_rules; rule != NULL; rule = rule->next)
 
   if (pre_length != 0)
     {
-    char *fullnew = string_sprintf("%.*s%s", pre_length, subject, new);
+    char *fullnew = store_malloc(pre_length + (int)strlen(new) + 1);
+    sprintf(fullnew, "%.*s%s", pre_length, subject, new);
     store_free(new);
     new = fullnew;
     }
 
-  /* Validate what has been generated. */
+  /* Validate what has been generated, qualifying if permitted to do so. */
+
+  if ((rule->flags & rewrite_qualify) != 0)
+    new = rewrite_address_qualify(new, TRUE);
 
   newparsed = parse_extract_address(new, &error, &start, &end, &pdomain,
     FALSE);
@@ -191,44 +269,105 @@ for (rule = rewrite_rules; rule != NULL; rule = rule->next)
     }
   else
     {
-    DEBUG(2) debug_printf("%s rewritten as %s\n", subject, newparsed);
+    DEBUG(2) debug_printf("\"%s\" rewritten as ", yield);
+
+    if (log_rewrites)
+      log_write(0, LOG_MAIN, "\"%s\" rewritten as \"%s\" by rule %d",
+        yield, new, rule_number);
 
     /* A header will only actually be added if header_last is non-NULL,
-    i.e. during message rectption or delivery. */
+    i.e. during message reception or delivery. */
 
     if (add_header)
       header_add(htype_old, "X-rewrote-%s: %s\n", name, subject);
 
-    /* If we are on the second or subsequent re-write, free the previous
-    yield and subject, if different. */
+    /* Handle the case when replacement of the whole address is possible.
+    This happens only when whole is not NULL and we are rewriting a header.
+    If *whole is already TRUE it means that a previous rule had the w
+    flag set and so we must preserve the non-active portion of the current
+    subject unless the current rule also has the w flag set. */
 
-    if (yield != s) store_free(yield);
-    if (subject != yield) store_free(subject);
-
-    /* If whole is not NULL and this rewrite rule has the "whole" flag set,
-    and we are rewriting a header, yield the entire new address, not just the
-    active part, and tell the caller so. */
-
-    if (whole != NULL && (rule->flags & rewrite_whole) != 0 &&
-        (flag & rewrite_all_headers) != 0)
+    if (whole != NULL && (flag & rewrite_all_headers) != 0)
       {
-      *whole = TRUE;
-      yield = new;
-      subject = newparsed;
+      /* Current rule has the w flag set; free previous yield and subject
+      if we are on 2nd or subsequent rewrite, and set up new values. */
+
+      if ((rule->flags & rewrite_whole) != 0)
+        {
+        if (yield != s) store_free(yield);
+        if (subject != yield) store_free(subject);
+        yield = new;
+        yield_start = start;
+        yield_end = end;
+        subject = newparsed;
+        *whole = TRUE;
+        }
+
+      /* Current rule does not have the w flag set; if not previously
+      done any whole rewriting, behave in non-whole manner. */
+
+      else if (!*whole) goto NEVER_WHOLE;
+
+      /* Current rule does not have the w flag set, but a previous
+      rule did rewrite the whole address. Thus yield and subject will be
+      different. Preserve the previous non-active part of the address. */
+
+      else
+        {
+        store_free(subject);
+        subject = newparsed;
+        store_free(new);
+        new = string_sprintf("%.*s%s%n%s",
+           yield_start, yield, subject, &end, yield + yield_end);
+        yield_end = end;
+        store_free(yield);
+        yield = new;
+        }
       }
 
-    /* Rule just rewrites active part, or handling an envelope */
+    /* Rule just rewrites active part, or handling an envelope. This
+    code is obeyed only when all rules so far have not done "whole"
+    replacement. */
 
     else
       {
-      if (whole != NULL) *whole = FALSE;
+      NEVER_WHOLE:
+      if (yield != s) store_free(yield);
       subject = yield = newparsed;
       store_free(new);
       }
+
+    DEBUG(2) debug_printf("\"%s\"\n", yield);
     domain = NULL;    /* Reset for next rule */
     }
+
+  /* If no further rewrites are to be done, set the done flag. This does
+  allow repeats of the current rule if configured. */
+
+  if ((rule->flags & rewrite_quit) != 0) done = TRUE;
+
+  /* Allow the current rule to be applied up to 10 times if
+  requested. */
+
+  if ((rule->flags & rewrite_repeat) != 0)
+    {
+    if (count++ < 10) goto REPEAT_RULE;
+    log_write(0, LOG_MAIN|LOG_PANIC, "rewrite rule repeat ignored after 10 "
+      "times");
+    }
   }
 
+/* Reset the uid/gid if necessary; free the original subject if it
+has been rewritten; unset expansion numeric variables, and that's it. */
+
+if (uid_set)
+  {
+  mac_seteuid(root_uid);
+  mac_setegid(saved_egid);
+  mac_seteuid(saved_euid);
+  DEBUG(2) debug_printf("restored euid=%d egid=%d\n", (int)geteuid(),
+    (int)getegid());
+  }
 if (subject != yield) store_free(subject);
 expand_nmax = -1;
 return yield;
@@ -236,35 +375,6 @@ return yield;
 
 
 
-/*************************************************
-*        Rewrite address for filter              *
-*************************************************/
-
-/* When running the mail filtering code, $local_part is taken from the
-message's envelope, but tests are done on the contents of headers. It may be
-that the addresses in the headers have been subject to rewriting, while the
-addresses in the envelope have not. The particular case in point is the
-"personal" test. This function applies the given rewriting rules (it is called
-with rewrite_from or rewrite_to); because the fourth argument given to
-rewrite_one() is FALSE, the fifth is not relevant.
-
-Arguments:
-  s           address to rewrite
-  flags       either rewrite_from or rewrite_to
-
-Returns:      result of call to rewrite_one()
-*/
-
-char *
-rewrite_address_for_filter(char *s, int flag)
-{
-return rewrite_one(s, flag, NULL, FALSE, "");
-}
-
-
-
-
-
 /*************************************************
 *         Ensure qualification and rewrite       *
 *************************************************/
@@ -311,6 +421,16 @@ domain. The first rewriting rule in this case is "change routed_old into
 routed_new", and it applies to all header lines that contain addresses. Then
 header-specific rewriting rules are applied.
 
+Before rewriting can be done, addresses without domains have to be qualified.
+This should only be done for messages from "local" senders. This is a difficult
+concept to pin down, what with the use of SMTP both as a submission and as a
+transmission protocol. Exim normally requires incoming SMTP to contain fully-
+qualified addresses, but there are options to permit unqualified ones from
+certain hosts. For those hosts only, addresses in headers can also be
+qualified. For other hosts, unqualified addresses in headers do not get touched
+in any way. For locally sourced messages, unqualified addresses always get
+qualified.
+
 Arguments:
   h           pointer to header line block
   flag        indicates which header this is
@@ -344,9 +464,9 @@ while (*s != 0)
   int terminator = *ss;
   int start, end, domain;
 
-  /* Temporarily terminate the string at this point, and extract
-  the route address within. Then put back the terminator and prepare for the
-  next address, saving the start of the old one. */
+  /* Temporarily terminate the string at this point, and extract the
+  operative address within. Then put back the terminator and prepare for
+  the next address, saving the start of the old one. */
 
   *ss = 0;
   recipient = parse_extract_address(s,&errmess,&start,&end,&domain,FALSE);
@@ -422,23 +542,35 @@ while (*s != 0)
 
 
   /* This is not a router-inspired rewrite. Ensure the address is fully
-  qualified. (Note: rewrite_address-qualify frees the old store.) From
-  or Reply-to headers are treated as senders, the rest as recipients. This
-  matters only when there are different qualify strings. */
+  qualified if that is permitted. (Note: rewrite_address-qualify frees the
+  old store.) If an unqualified address was received from a host that isn't
+  listed, do not continue rewriting this address. Sender, From or Reply-to
+  headers are treated as senders, the rest as recipients. This matters only
+  when there are different qualify strings. */
 
   else
     {
-    new = rewrite_address_qualify(recipient,
-      (flag & (rewrite_from | rewrite_replyto)) == 0);
+    BOOL is_recipient =
+      (flag & (rewrite_sender | rewrite_from | rewrite_replyto)) == 0;
+    new = rewrite_address_qualify(recipient, is_recipient);
     changed = (new != recipient);
     recipient = new;
+
+    /* Can only qualify if permitted; if not, no rewrite. */
+
+    if (changed && ((is_recipient && !allow_unqualified_recipient) ||
+                    (!is_recipient && !allow_unqualified_sender)))
+      {
+      store_free(recipient);
+      continue;
+      }
     }
 
   /* If there are rewrite rules for this type of header, apply
   them. This test is just for efficiency, to save scanning the rules
-  in cases when nothing is going to change. If the final rewrite rule
-  had the "whole" flag set, adjust the pointers so that the whole address
-  gets replaced, except possibly a final \n. */
+  in cases when nothing is going to change. If any rewrite rule had the
+  "whole" flag set, adjust the pointers so that the whole address gets
+  replaced, except possibly a final \n. */
 
   if ((rewrite_existflags & flag) != 0)
     {
@@ -572,27 +704,21 @@ switch (h->type)
   {
   case htype_sender:
   return rewrite_one_header(h, rewrite_sender, routed_old, routed_new);
-  break;
 
   case htype_from:
   return rewrite_one_header(h, rewrite_from, routed_old, routed_new);
-  break;
 
   case htype_to:
   return rewrite_one_header(h, rewrite_to, routed_old, routed_new);
-  break;
 
   case htype_cc:
   return rewrite_one_header(h, rewrite_cc, routed_old, routed_new);
-  break;
 
   case htype_bcc:
   return rewrite_one_header(h, rewrite_bcc, routed_old, routed_new);
-  break;
 
   case htype_replyto:
   return rewrite_one_header(h, rewrite_replyto, routed_old, routed_new);
-  break;
   }
 
 return NULL;
@@ -613,8 +739,14 @@ Returns:  nothing
 
 void rewrite_test(char *s)
 {
-int i, start, end, domain;
 char *recipient, *error;
+int i, start, end, domain;
+
+if ((rewrite_existflags & rewrite_smtp) != 0)
+  {
+  char *new = rewrite_one(s, rewrite_smtp, NULL, FALSE, "");
+  if (new != s) printf("    SMTP: %s\n", new);
+  }
 
 if (parse_find_at(s) == NULL)
   s = string_sprintf("%s@%s", s, qualify_domain_recipient);
@@ -636,6 +768,14 @@ for (i = 0; i < 8; i++)
   if (whole || (flag & rewrite_all_headers) == 0) printf("%s\n", new);
     else printf("%.*s%s%s\n", start, s, new, s+end);
   }
+
+/* Only show the X case if there is a rewrite rule for it. */
+
+if ((rewrite_existflags & rewrite_X) != 0)
+  {
+  char *new = rewrite_one(recipient, rewrite_X, NULL, FALSE, "");
+  if (new != recipient) printf("       X: %s\n", new);
+  }
 }
 
 /* End of rewrite.c */
