@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with routing, and the list of generic router options. */
@@ -16,20 +16,32 @@
 data blocks and which therefore have the opt_public flag set. */
 
 optionlist optionlist_routers[] = {
+  { "*expand_transport",  opt_stringptr|opt_public|opt_hidden,
+                 (void *)offsetof(router_instance, expand_transport) },
+  { "condition",          opt_stringptr|opt_public,
+                 (void *)offsetof(router_instance, condition) },
   { "domains",            opt_stringptr|opt_public,
                  (void *)offsetof(router_instance, domains) },
   { "driver",             opt_stringptr|opt_public,
                  (void *)offsetof(router_instance, driver_name) },
+  { "errors_to",          opt_stringptr|opt_public,
+                 (void *)(offsetof(router_instance, errors_to)) },
   { "except_domains",     opt_stringptr|opt_public,
                  (void *)offsetof(router_instance, except_domains) },
   { "except_local_parts", opt_stringptr|opt_public,
                  (void *)offsetof(router_instance, except_local_parts) },
+  { "except_senders",     opt_stringptr|opt_public,
+                 (void *)offsetof(router_instance, except_senders) },
   { "fail_verify",        opt_bool_verify|opt_hidden|opt_public,
                  (void *)offsetof(router_instance, fail_verify_sender) },
   { "fail_verify_recipient", opt_bool|opt_public,
                  (void *)offsetof(router_instance, fail_verify_recipient) },
   { "fail_verify_sender", opt_bool|opt_public,
                  (void *)offsetof(router_instance, fail_verify_sender) },
+  { "headers_add",        opt_stringptr|opt_public,
+                 (void *)offsetof(router_instance, extra_headers) },
+  { "headers_remove",     opt_stringptr|opt_public,
+                 (void *)offsetof(router_instance, remove_headers) },
   { "local_parts",        opt_stringptr|opt_public,
                  (void *)offsetof(router_instance, local_parts) },
   { "more",               opt_bool|opt_public,
@@ -40,6 +52,8 @@ optionlist optionlist_routers[] = {
                  (void *)offsetof(router_instance, require_files) },
   { "self",               opt_stringptr|opt_public,
                  (void *)(offsetof(router_instance, self)) },
+  { "senders",            opt_stringptr|opt_public,
+                 (void *)offsetof(router_instance, senders) },
   { "transport",          opt_transportptr|opt_public,
                  (void *)offsetof(router_instance, transport) },
   { "unseen",             opt_bool|opt_public,
@@ -87,25 +101,19 @@ and a rewrite boolean. */
 
 for (r = routers; r != NULL; r = r->next)
   {
-  char *s;
+  char *s = r->self;
 
   if (r->unseen && !r->more )
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
       "the combination of \"unseen\" and \"no_more\" is not permitted on "
       "a router,\n  but was set for the %s router", r->name);
 
-  /* If the setting is NULL, convert it to "freeze" so that it prints
-  correctly. Can't yet set this as the data default because we want to
-  detect "unset" for overriding by the obsolete self_mx option in lookuphost. */
-
-  if (r->self == NULL) r->self = "freeze";
-  s = r->self;
-
   if      (strcmp(s, "freeze") == 0)    r->self_code = self_freeze;
   else if (strcmp(s, "defer") == 0)     r->self_code = self_defer;
   else if (strcmp(s, "send") == 0)      r->self_code = self_send;
   else if (strcmp(s, "fail_soft") == 0) r->self_code = self_fail;
   else if (strcmp(s, "fail_hard") == 0) r->self_code = self_forcefail;
+  else if (strcmp(s, "local") == 0)     r->self_code = self_local;
   else if (strncmp(s, "reroute:", 8) == 0)
     {
     s += 8;
@@ -117,7 +125,7 @@ for (r = routers; r != NULL; r = r->next)
       while (isspace(*s)) s++;
       }
     r->self = s;
-    r->self_code = self_local;
+    r->self_code = self_reroute;
     }
   else log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
       "%s is not valid for the self option", r->name, s);
@@ -146,6 +154,180 @@ for (r = routers; r != NULL; r = r->next)
 
 
 
+
+/*************************************************
+*             Set transport                      *
+*************************************************/
+
+/* If expand_transport is set on a driver, it must be expanded each time and
+used as a transport name. This action is identical for both routers and
+directors, and so the relevant data is in the same place in the options blocks
+for both. This function is therefore also called from direct_address().
+
+Argument:
+  dr            points to the router (director) instance block
+  driver_type   "router" or "transport"
+  errmsg        where to put error message
+
+Returns:        TRUE if successful; false if something went wrong
+*/
+
+BOOL
+route_set_transport(router_instance *dr, char *driver_type, char **errmsg)
+{
+char *ss;
+char *s = dr->expand_transport;
+transport_instance *tp;
+
+if (s == NULL) return TRUE;    /* Fixed transport may be set */
+
+ss = expand_string(s);
+if (ss == NULL)
+  {
+  *errmsg = string_sprintf("failed to expand dynamic transport "
+    "name \"%s\" for %s %s: %s", s, dr->name, driver_type,
+      expand_string_message);
+  return FALSE;
+  }
+
+for (tp = transports; tp != NULL; tp = tp->next)
+  {
+  if (strcmp(tp->name, ss) == 0)
+    {
+    DEBUG(9) debug_printf("dynamic transport \"%s\" set\n", ss);
+    dr->transport = tp;
+    store_free(ss);
+    return TRUE;
+    }
+  }
+
+*errmsg = string_sprintf("dynamic transport \"%s\" not found "
+  "for %s %s", ss, dr->name, driver_type);
+return FALSE;
+}
+
+
+
+/*************************************************
+*             Check for driver skipping          *
+*************************************************/
+
+/* This function performs various checks to see whether a driver should be
+skipped. Many of the checks are identical for both routers and directors. The
+relevant data is arranged to be in the same place in the options blocks for
+both types of driver so that this function can be used for both types of
+driver for the common cases. Hence it is globally accessible.
+
+Arguments:
+  dr           pointer to router or director instance block
+  local_part   local part of the address being handled
+  domain       ditto domain
+  verify       the verification type
+  pmore        address of the "more" flag
+  driver_type  either "router" or "director"
+
+Returns:       TRUE if this driver is to be skipped
+*/
+
+
+BOOL
+route_skip_driver(router_instance *dr, char *local_part, char *domain,
+  int verify, BOOL *pmore, char *driver_type)
+{
+
+/* Variables to hold data from lookup of a domain or local part. Free
+previous value and set empty by default. */
+
+if (domain_data != NULL)
+  {
+  store_free(domain_data);
+  domain_data = NULL;
+  }
+
+if (local_part_data != NULL)
+  {
+  store_free(local_part_data);
+  local_part_data = NULL;
+  }
+
+/* Skip this driver if not verifying and it has verify_only set */
+
+if ((verify == v_none || verify == v_expn) && dr->verify_only)
+  {
+  DEBUG(9) debug_printf("%s %s skipped: verify_only set\n",
+    dr->name, driver_type);
+  return TRUE;
+  }
+
+/* Skip this driver if domain mismatch */
+
+if (dr->domains != NULL &&
+     !match_isinlist_get(domain, dr->domains, &(dr->re_domains),
+       TRUE, &domain_data))
+  {
+  DEBUG(9) debug_printf("%s %s skipped: domain mismatch\n",
+    dr->name, driver_type);
+  return TRUE;
+  }
+
+if (dr->except_domains != NULL &&
+     match_isinlist(domain, dr->except_domains, &(dr->re_except_domains), TRUE))
+  {
+  DEBUG(9) debug_printf("%s %s skipped: except domain mismatch\n",
+    dr->name, driver_type);
+  return TRUE;
+  }
+
+/* Skip this driver if local part mismatch */
+
+if (dr->local_parts != NULL &&
+     !match_isinlist_get(local_part, dr->local_parts,
+       &(dr->re_local_parts), FALSE, &local_part_data))
+  {
+  DEBUG(9) debug_printf("%s %s skipped: local part mismatch\n",
+    dr->name, driver_type);
+  return TRUE;
+  }
+
+if (dr->except_local_parts != NULL &&
+     match_isinlist(local_part, dr->except_local_parts,
+       &(dr->re_except_local_parts), FALSE))
+  {
+  DEBUG(9) debug_printf("%s %s skipped: except local part mismatch\n",
+    dr->name, driver_type);
+  return TRUE;
+  }
+
+/* If this driver's "more" flag is FALSE, arrange that no subsequent
+drivers are called. */
+
+*pmore = dr->more;
+DEBUG(9)
+  {
+  if (!dr->more) debug_printf("%s %s has more set FALSE\n", dr->name,
+    driver_type);
+  }
+
+/* Skip this driver if verifying and it hasn't got the appropriate verify flag
+set. */
+
+if ((verify == v_sender && !dr->verify_sender) ||
+    (verify == v_recipient && !dr->verify_recipient))
+  {
+  DEBUG(9) debug_printf("%s %s skipped: verify %d %d %d\n",
+    dr->name, driver_type, verify, dr->verify_sender, dr->verify_recipient);
+  return TRUE;
+  }
+
+/* Nothing in this function calls for skipping the driver; specific code for
+routers and directors may yet do so later. */
+
+return FALSE;
+}
+
+
+
+
 /*************************************************
 *                 Route one address              *
 *************************************************/
@@ -187,7 +369,8 @@ route_address(address_item *addr, address_item **addr_local,
 BOOL more = TRUE;
 router_instance *r;
 
-DEBUG(9) debug_printf("routing %s, domain %s\n", addr->orig, addr->domain);
+DEBUG(9) debug_printf(">>>>>>>>>>>>>>>>>>>>>>>>\n"
+  "routing %s, domain %s\n", addr->orig, addr->domain);
 
 /* Set the domain used by the routers; a router is permitted to change this and
 yield PASS, causing subsequent routers to use the new name, without changing
@@ -196,7 +379,6 @@ permitted to change the envelope address. */
 
 addr->route_domain = addr->domain;
 
-
 /* Loop through all router instances. */
 
 for (r = routers; more && r != NULL; r = r->next)
@@ -205,73 +387,12 @@ for (r = routers; more && r != NULL; r = r->next)
   address_item *parent;
   BOOL loop_detected = FALSE;
   int yield = FALSE;
+  int rc;
 
-  /* Skip this router if not verifying and it has verify_only set */
+  /* Check the conditions that are common to both routers and directors. */
 
-  if (verify == v_none && r->verify_only)
-    {
-    DEBUG(9) debug_printf("%s router skipped: verify_only set\n",
-      r->name);
-    continue;
-    }
-
-  /* Skip this router if domain mismatch */
-
-  if (r->domains != NULL &&
-       !match_isinlist(addr->route_domain, r->domains, &(r->re_domains), TRUE))
-    {
-    DEBUG(9) debug_printf("%s router skipped: domain mismatch\n",
-      r->name);
-    continue;
-    }
-
-  if (r->except_domains != NULL &&
-       match_isinlist(addr->route_domain, r->except_domains,
-         &(r->re_except_domains), TRUE))
-    {
-    DEBUG(9) debug_printf("%s router skipped: except domain mismatch\n",
-      r->name);
-    continue;
-    }
-
-  /* Skip this router if local part mismatch */
-
-  if (r->local_parts != NULL &&
-       !match_isinlist(addr->local_part, r->local_parts,
-         &(r->re_local_parts), FALSE))
-    {
-    DEBUG(9) debug_printf("%s router skipped: local part mismatch\n",
-      r->name);
-    continue;
-    }
-
-  if (r->except_local_parts != NULL &&
-       match_isinlist(addr->local_part, r->except_local_parts,
-         &(r->re_except_local_parts), FALSE))
-    {
-    DEBUG(9) debug_printf("%s router skipped: except local part mismatch\n",
-      r->name);
-    continue;
-    }
-
-  /* If this router's "more" flag is FALSE, arrange that no subsequent
-  routers are called. */
-
-  more = r->more;
-  DEBUG(9)
-    {
-    if (!more) debug_printf("%s router has more set FALSE\n", r->name);
-    }
-
-  /* Skip this router if verifying and it hasn't got the verify flag */
-
-  if ((verify == v_sender && !r->verify_sender) ||
-      (verify == v_recipient && !r->verify_recipient))
-    {
-    DEBUG(9) debug_printf("%s router skipped: verify %d %d %d\n",
-      r->name, verify, r->verify_sender, r->verify_recipient);
-    continue;
-    }
+  if (route_skip_driver(r, addr->local_part, addr->route_domain,
+    verify, &more, "router")) continue;
 
   /* Loop protection: If this address has a parent with the same address that
   was routed by this router, we skip this router. This prevents a looping
@@ -304,8 +425,8 @@ for (r = routers; more && r != NULL; r = r->next)
   addr->rewrite_headers = FALSE;
 
   /* Do file existence tests - must do after setting expansion values
-  as the string is expanded - and if they succeed, run the router. Otherwise
-  set an appropriate yield value. */
+  as the string is expanded - and if they succeed, do any necessary sender
+  checks, and run the router. Otherwise set an appropriate yield value. */
 
   switch(match_exists(r->require_files))
     {
@@ -321,7 +442,41 @@ for (r = routers; more && r != NULL; r = r->next)
     break;
 
     case OK:
-    yield = (r->info->code)(r, addr, addr_local, addr_remote);
+    if ((r->except_senders != NULL &&
+         (rc = match_sender(r->except_senders, &(r->re_except_senders)))
+            == OK) ||
+        (r->senders != NULL &&
+         (rc = match_sender(r->senders, &(r->re_senders))) != OK))
+      {
+      if (rc == DEFER)
+        {
+        addr->message = string_sprintf("sender check defer in %s router",
+          r->name);
+        yield = DEFER;
+        }
+      DEBUG(9) debug_printf("%s router: sender match failure\n",
+        r->name);
+      yield = FAIL;
+      }
+    else if (r->condition != NULL &&
+          !expand_check_condition(r->condition, r->name, "router"))
+      {
+      DEBUG(9) debug_printf("%s router: condition failure; more set TRUE\n",
+        r->name);
+      yield = FAIL;
+      more = TRUE;      /* Condition tests are logically before "more" */
+      }
+    else
+      {
+      addr->local_host_removed = FALSE;
+      if (!route_set_transport(r, "router", &(addr->message)))
+        {
+        yield = DEFER;
+        DEBUG(9) debug_printf("%s", addr->message);
+        }
+      else yield = (r->info->code)(r, addr, addr_local, addr_remote,
+        verify != v_none);
+      }
     break;
     }
 
@@ -345,11 +500,7 @@ for (r = routers; more && r != NULL; r = r->next)
 
   if (yield == OK && r->unseen)
     {
-    char *new_address =
-      (addr->local_part[0] == ',' || addr->local_part[0] == ':')?
-        string_sprintf("@%s%s", addr->domain, addr->local_part) :
-        string_sprintf("%s@%s", addr->local_part, addr->domain);
-    address_item *new = deliver_make_addr(new_address);
+    address_item *new = deliver_make_addr(string_copy(addr->orig));
     new->parent = addr;
     new->ignore_error |= addr->ignore_error;
     new->errors_address = addr->errors_address;
@@ -421,6 +572,8 @@ for (r = routers; more && r != NULL; r = r->next)
   if (addr->rewrite_headers &&
       strcmp(old_route_domain, addr->route_domain) != 0)
     {
+    DEBUG(9) debug_printf("rewriting after %s router\n", r->name);
+
     /* First the domain in the envelope, if it was the same as the old routing
     domain. */
 
@@ -460,16 +613,20 @@ for (r = routers; more && r != NULL; r = r->next)
 
 
   /* If the yield was ISLOCAL, an apparently remote address was really that
-  of a local domain (e.g. foo.bar got looked up in the DNS and turned into
-  foo.bar.co.uk, which is one of the local domains). We return ISLOCAL so that
-  the address can be passed to the directors instead. If rewrite_headers
-  is not set we won't have processed the domain above, so do it here. This
-  doesn't happen for domain expansion, but it can happen when MX->self causes
-  re-routing. */
+  of a local domain (e.g. foo got looked up in the DNS and turned into
+  foo.co.uk, which is one of the local domains). Alternatively, a remote
+  domain had its name changed as a result of routing (to another non-local
+  domain), but its lowest numbered MX record points to this host. Again,
+  this could be foo turning into foo.co.uk.
+
+  We return ISLOCAL so that the expanded address can be re-processed by the
+  directors or routers, as appropriate. If rewrite_headers is not set we
+  won't have processed the domain above, so do it here. This doesn't happen
+  for domain expansion, but it can happen when MX->self causes re-routing. */
 
   if (yield == ISLOCAL)
     {
-    DEBUG(2) debug_printf("%s router found %s to be local\n",
+    DEBUG(2) debug_printf("%s router found %s to be local, or locally MX'd\n",
       r->name, old_route_domain);
     if (!addr->rewrite_headers && strcmp(addr->domain, old_route_domain) == 0)
       addr->domain = addr->route_domain;
@@ -516,9 +673,26 @@ for (r = routers; more && r != NULL; r = r->next)
   return OK;
   }
 
+/* Ensure store for domain and local part data is freed */
+
+if (domain_data != NULL)
+  {
+  store_free(domain_data);
+  domain_data = NULL;
+  }
+
+if (local_part_data != NULL)
+  {
+  store_free(local_part_data);
+  local_part_data = NULL;
+  }
+
 /* No routers accepted this address; fail it. */
 
-if (addr->message == NULL) addr->message = "unknown mail domain";
+if (addr->message == NULL)
+  addr->message = string_sprintf("unrouteable mail domain \"%s\"",
+    addr->route_domain);
+
 return FAIL;
 }
 
