@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -10,6 +10,14 @@
 #include "domainlist.h"
 
 
+/* Codes for the host_find_failed option. */
+
+#define hff_freeze     0
+#define hff_defer      1
+#define hff_fail_soft  2
+#define hff_fail_hard  3
+
+
 
 /* Options specific to the domainlist router. The partial_match option is
 not set explicitly, but as a side-effect of search_type. An entry in this
@@ -18,6 +26,8 @@ table allows the option-reading function to find where to put the value. */
 optionlist domainlist_router_options[] = {
   { "*partial_match", opt_int | opt_hidden,
       (void *)(offsetof(domainlist_router_options_block, partial_match)) },
+  { "host_find_failed", opt_stringptr,
+      (void *)(offsetof(domainlist_router_options_block, host_find_failed)) },
   { "modemask",       opt_octint,
       (void *)(offsetof(domainlist_router_options_block, modemask)) },
   { "owners",         opt_uidlist,
@@ -28,6 +38,8 @@ optionlist domainlist_router_options[] = {
       (void *)(offsetof(domainlist_router_options_block, route_file)) },
   { "route_list",     opt_stringptr,
       (void *)(offsetof(domainlist_router_options_block, route_list)) },
+  { "route_queries",  opt_stringptr,
+      (void *)(offsetof(domainlist_router_options_block, route_queries)) },
   { "route_query",    opt_stringptr,
       (void *)(offsetof(domainlist_router_options_block, route_query)) },
   { "search_type",    opt_searchtype,
@@ -44,11 +56,14 @@ int domainlist_router_options_count =
 
 domainlist_router_options_block domainlist_router_option_defaults = {
   -1,           /* search_type */
-  0,            /* partial_match */
+  -1,           /* partial_match */
   022,          /* modemask */
+  hff_freeze,   /* host_find_failed code */
   NULL,         /* owners */
   NULL,         /* owngroups */
+  "freeze",     /* host_find_failed */
   NULL,         /* route_file */
+  NULL,         /* route_queries */
   NULL,         /* route_query */
   NULL,         /* route_list */
   NULL          /* re_list */
@@ -72,6 +87,20 @@ domainlist_router_init(router_instance *rblock)
 domainlist_router_options_block *ob =
   (domainlist_router_options_block *)(rblock->options_block);
 
+/* Host_find_failed must be a recognized word */
+
+if      (strcmp(ob->host_find_failed, "freeze") == 0)
+  ob->hff_code = hff_freeze;
+else if (strcmp(ob->host_find_failed, "defer") == 0)
+  ob->hff_code = hff_defer;
+else if (strcmp(ob->host_find_failed, "fail_soft") == 0)
+  ob->hff_code = hff_fail_soft;
+else if (strcmp(ob->host_find_failed, "fail_hard") == 0)
+  ob->hff_code = hff_fail_hard;
+else
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
+    "unrecognized setting for host_find_failed option", rblock->name);
+
 /* One of route_list or route_file or route_query must be specified */
 
 if (ob->route_file == NULL && ob->route_list == NULL && ob->route_query == NULL)
@@ -92,7 +121,7 @@ if (ob->search_type < 0) return;
 a query is mandatory. An absolute file name is mandatory for lsearch and dbm;
 but can't check for absoluteness if the name is being looked up. */
 
-if (ob->search_type < stype_querystyle)
+if (!mac_islookup(ob->search_type, lookup_querystyle))
   {
   if (ob->route_query != NULL)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
@@ -102,14 +131,15 @@ if (ob->search_type < stype_querystyle)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
       "no file name specified but search type set", rblock->name);
 
-  if ((ob->search_type == stype_lsearch || ob->search_type == stype_dbm) &&
+  if (mac_islookup(ob->search_type, lookup_absfile) &&
        ob->route_file[0] != '/' && ob->route_file[0] != '$')
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
-      "an absolute file path name is required for lsearch or dbm",
-      rblock->name);
+      "an absolute file path name is required for %s",
+      rblock->name, lookup_list[ob->search_type].name);
   }
 
-/* Non-single-key search type */
+/* Non-single-key search type; either a single query or a list of queries must
+be specified. */
 
 else
   {
@@ -117,9 +147,14 @@ else
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
       "\"file\" specified for a non-single-key search type", rblock->name);
 
-  if (ob->route_query == NULL)
+  if (ob->route_query == NULL && ob->route_queries == NULL)
     log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
-      "no query specified but non-single-key search type set", rblock->name);
+      "no query specified for %s search", rblock->name,
+      lookup_list[ob->search_type].name);
+
+  if (ob->route_query != NULL && ob->route_queries != NULL)
+    log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
+      "only one of \"query\" or \"queries\" may be given", rblock->name);
   }
 }
 
@@ -209,13 +244,14 @@ domainlist_router_entry(
   router_instance *rblock,        /* data for this instantiation */
   address_item *addr,             /* address we are working on */
   address_item **addr_local,      /* add it to this if successful & local */
-  address_item **addr_remote)     /* add it to this if successful & remote */
+  address_item **addr_remote,     /* add it to this if successful & remote */
+  BOOL verify)                    /* TRUE if verifying */
 {
 BOOL matched = FALSE;
 BOOL free_routelist_item = FALSE;
 BOOL individual_transport_set = FALSE;
-int lookup_type;
-host_item *h, *prev, **hchain;
+int lookup_type, rc;
+host_item *h, *next_h, *prev, **hchain;
 char *routelist_item = NULL;
 char *domain, *hostlist, *newhostlist, *options, *name, *listptr;
 domainlist_router_options_block *ob =
@@ -223,7 +259,7 @@ domainlist_router_options_block *ob =
 transport_instance *transport = rblock->transport;
 char route_buffer[1024];
 
-DEBUG(2) debug_printf("%s router called for %s: route_domain = %s\n",
+DEBUG(2) debug_printf("%s router called for %s\n  route_domain = %s\n",
   rblock->name, addr->orig, addr->route_domain);
 
 /* The initialization check ensures that either route_file or route_list or
@@ -245,17 +281,25 @@ if (ob->route_list != NULL)
       continue;     /* Ignore blank items */
 
     /* Check the current route_domain; if it matches, set flag and break
-    the loop. The final argument of zero requests $0 be set to the whole
+    the loop. The fourth argument of zero requests $0 be set to the whole
     thing and $1, $2, etc. as appropriate. If no match, advance the regular
     expression compiled pointer if we have just passed a regular expression. */
 
-    if (match_check_string(addr->route_domain, domain, rechain, 0, TRUE))
+    if (match_check_string(addr->route_domain, domain, rechain, 0, TRUE, NULL))
       {
       matched = TRUE;
       break;
       }
-    else if (domain[0] == '^' && *rechain != NULL)
-      rechain = &((*rechain)->next);
+    else
+      {
+      if (search_find_defer)
+        {
+        addr->message = "lookup defer in route_list";
+        return DEFER;
+        }
+      if (domain[0] == '^' && *rechain != NULL)
+        rechain = &((*rechain)->next);
+      }
     }
   }
 
@@ -268,7 +312,8 @@ the search. For files this is a FILE * or a DBM *; for other things is is < 0.
 If this router has been called earlier for this message, the search database
 may already be open. */
 
-if (!matched && (ob->route_file != NULL || ob->route_query != NULL))
+if (!matched && (ob->route_file != NULL || ob->route_query != NULL ||
+    ob->route_queries != NULL))
   {
   int expand_setup = 0;
   char *error;
@@ -276,31 +321,58 @@ if (!matched && (ob->route_file != NULL || ob->route_query != NULL))
   char *query;
   void *handle;
 
-  if (ob->search_type < stype_querystyle)
+  /* For single-key+file search types, set the required file name and expand it.
+  If the expansion fails, log the incident and indicate an internal error. The
+  file name has already been checked for absoluteness, at initialization time,
+  but only if it did not start with an expansion, so we double check here. */
+
+  if (!mac_islookup(ob->search_type, lookup_querystyle))
     {
     query = addr->route_domain;
     filename = expand_string(ob->route_file);
     if (filename == NULL)
       {
-      addr->message = string_sprintf("%s router: failed to expand %s: %s",
+      addr->message = string_sprintf("%s router: failed to expand \"%s\": %s",
         rblock->name, ob->route_file, expand_string_message);
-      log_write(0, LOG_MAIN|LOG_PANIC, "%s", addr->message);
+      if (search_find_defer) return DEFER;
       addr->special_action = SPECIAL_FREEZE;
       return ERROR;
       }
+    else if (mac_islookup(ob->search_type, lookup_absfile) &&
+             filename[0] != '/')
+      {
+      log_write(0, LOG_MAIN|LOG_PANIC, "%s router requires absolute file name "
+        "for %s: %s generated from expanding %s", rblock->name,
+        lookup_list[ob->search_type].name, filename, ob->route_file);
+      addr->special_action = SPECIAL_FREEZE;
+      return ERROR;
+      }
+
+    DEBUG(2) debug_printf("%s router: %s key=%s\n  file=\"%s\"\n",
+      rblock->name, lookup_list[ob->search_type].name, query, filename);
     }
+
+  /* For query-style lookups, expand the query and set the filename NULL.
+  For historical reasons, there may be just one query or a list of queries, but
+  not both. Initialization ensures that one and only one option is set. */
+
   else
     {
     filename = NULL;
-    query = expand_string(ob->route_query);
+    query = expand_string((ob->route_query == NULL)?
+      ob->route_queries : ob->route_query);
     if (query == NULL)
       {
-      addr->message = string_sprintf("%s router: failed to expand %s: %s",
-        rblock->name, ob->route_query, expand_string_message);
-      log_write(0, LOG_MAIN|LOG_PANIC, "%s", addr->message);
+      addr->message = string_sprintf("%s router: failed to expand \"%s\": %s",
+        rblock->name,
+        (ob->route_query == NULL)? ob->route_queries : ob->route_query,
+        expand_string_message);
+      if (search_find_defer) return DEFER;
       addr->special_action = SPECIAL_FREEZE;
       return ERROR;
       }
+    DEBUG(2) debug_printf("%s router: %s query=%s\n", rblock->name,
+      lookup_list[ob->search_type].name, query);
     }
 
   handle = search_open(filename, ob->search_type, ob->modemask,
@@ -321,9 +393,22 @@ if (!matched && (ob->route_file != NULL || ob->route_query != NULL))
 
   /* Now search the file (or whatever) for the entry we are interested in.
   The text is returned in dynamic store. Partial matching is handled within
-  the search_find() function. */
+  the search_find() function. If ob->route_queries is set, we have a list of
+  queries to try. */
 
-  routelist_item = search_find(handle, filename, query, ob->search_type,
+  if (ob->route_queries != NULL)
+    {
+    char *onequery;
+    for (onequery = string_nextinlist(&query, ':', big_buffer, BIG_BUFFER_SIZE);
+         onequery != NULL;
+         onequery = string_nextinlist(&query, ':', big_buffer, BIG_BUFFER_SIZE))
+      {
+      routelist_item = search_find(handle, filename, onequery, ob->search_type,
+        ob->partial_match, &expand_setup, &error);
+      if (routelist_item != NULL || search_find_defer) break;
+      }
+    }
+  else routelist_item = search_find(handle, filename, query, ob->search_type,
     ob->partial_match, &expand_setup, &error);
 
   /* If we found the domain, parse the resulting data; in this case,
@@ -338,7 +423,16 @@ if (!matched && (ob->route_file != NULL || ob->route_query != NULL))
     expand_nmax = expand_setup;
     }
   else
-    DEBUG(9) debug_printf("search_find failed: %s\n", error);
+    {
+    DEBUG(9) debug_printf("search_find %s: %s\n",
+      search_find_defer? "deferred" : "failed", error);
+    if (search_find_defer)
+      {
+      addr->message = string_sprintf("%s router: search_find deferred: %s",
+        rblock->name, error);
+      return DEFER;
+      }
+    }
   }
 
 
@@ -364,9 +458,9 @@ there is a configuration problem. */
 if (newhostlist == NULL)
   {
   if (expand_string_forcedfail) return FAIL;
-  addr->message = string_sprintf("%s router: failed to expand %s: %s",
+  addr->message = string_sprintf("%s router: failed to expand \"%s\": %s",
     rblock->name, hostlist , expand_string_message);
-  log_write(0, LOG_MAIN|LOG_PANIC, "%s", addr->message);
+  if (search_find_defer) return DEFER;
   addr->special_action = SPECIAL_FREEZE;
   return ERROR;
   }
@@ -422,6 +516,20 @@ while (*options != 0)
     }
   }
 
+/* Set up the errors address, if any. The function uses only fields that
+are in the same place in director and router blocks. */
+
+rc = direct_get_errors_address(addr, (director_instance *)rblock, "router",
+  verify, &(addr->errors_address));
+if (rc != OK) return rc;
+
+/* Set up the additional and removeable headers for this address. The function
+uses only fields that are in the same place in director and router blocks. */
+
+rc = direct_get_munge_headers(addr, (director_instance *)rblock, "router",
+  &(addr->extra_headers), &(addr->remove_headers));
+if (rc != OK) return rc;
+
 /* Deal with the case of a local transport. A single host is allowed (can
 be used, for example, as an expansion in a file name). */
 
@@ -540,24 +648,29 @@ store_free(hostlist);
 
 
 /* Look up each host address. A lookup may add additional items into the chain
-if there are multiple addresses. If any host is identified as being the local
+if there are multiple addresses. Hence the use of next_h to start each cycle of
+the loop at the next original host. If any host is identified as being the local
 host, omit it and any subsequent hosts - i.e. treat the list like an ordered
 list of MX hosts. If the first host is the local host, act according to the
 "self" option in the configuration. */
 
 prev = NULL;
-for (h = addr->host_list; h != NULL; prev = h, h = h->next)
+for (h = addr->host_list; h != NULL; prev = h, h = next_h)
   {
   char *canonical_name;
   int rc;
 
+  next_h = h->next;
   if (h->address != NULL) continue;
 
   if (lookup_type == lk_byname)
     rc = host_find_byname(h, &canonical_name);
   else
     rc = host_find_bydns(h, (lookup_type == lk_bydns_mx),
-      (lookup_type == lk_bydns_a), FALSE, FALSE, &canonical_name);
+      (lookup_type == lk_bydns_a), FALSE, FALSE, &canonical_name,
+      &(addr->local_host_removed));
+
+  /* Temporary failure defers, unless pass_on_timeout is set */
 
   if (rc == HOST_FIND_AGAIN)
     {
@@ -570,15 +683,25 @@ for (h = addr->host_list; h != NULL; prev = h, h = h->next)
     return DEFER;
     }
 
+  /* Permanent failure is controlled by host_find_failed */
+
   if (rc == HOST_FIND_FAILED)
     {
-    log_write(0, LOG_MAIN, "Error in %s router: lookup of host %s failed",
-      rblock->name, h->name);
-    addr->message = "error in router: host lookup failed";
+    if (ob->hff_code == hff_fail_soft) return FAIL;
+
+    addr->message = string_sprintf("lookup of host \"%s\" failed in %s router",
+      h->name, rblock->name);
+
+    if (ob->hff_code == hff_defer) return DEFER;
+    if (ob->hff_code == hff_fail_hard) return FORCEFAIL;
+
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
 
+  /* A local host gets chopped, with its successors, if there are previous
+  hosts. Otherwise the self option is used. */
+
   if (rc == HOST_FOUND_LOCAL)
     {
     if (prev != NULL)
@@ -591,7 +714,8 @@ for (h = addr->host_list; h != NULL; prev = h, h = h->next)
       prev->next = NULL;
       break;
       }
-    rc = host_self_action(addr, h->mx, rblock->self_code, rblock->self_rewrite,
+    addr->local_host_removed = TRUE;
+    rc = host_self_action(addr, h, rblock->self_code, rblock->self_rewrite,
       rblock->self);
     if (rc != OK) return rc;
     }
