@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the domainlist router */
@@ -13,9 +13,12 @@ typedef struct {
   int   search_type;
   int   partial_match;
   int   modemask;
+  int   hff_code;
   int  *owners;
   int  *owngroups;
+  char *host_find_failed;
   char *route_file;
+  char *route_queries;
   char *route_query;
   char *route_list;
   re_block *re_list;
@@ -33,7 +36,7 @@ extern domainlist_router_options_block domainlist_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int domainlist_router_entry(router_instance *, address_item *,
-  address_item **, address_item **);
+  address_item **, address_item **, BOOL);
 extern void domainlist_router_init(router_instance *);
 
 /* End of routers/domainlist.h */
