@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -52,7 +52,7 @@ iplookup_router_options_block iplookup_router_option_defaults = {
   NULL,     /* protocol_name */
   NULL,     /* hosts */
   NULL,     /* query; NULL => local_part@domain */
-  NULL,     /* response_pattern; NULL => don't apply regexp */
+  NULL,     /* response_pattern; NULL => don't apply regex */
   NULL,     /* reroute; NULL => just used returned data */
   NULL,     /* re_response_pattern; compiled pattern */
   FALSE     /* optional */
@@ -86,7 +86,7 @@ if (ob->hosts == NULL)
 
 /* A transport must NOT be given. */
 
-if (rblock->transport != NULL)
+if (rblock->transport != NULL || rblock->expand_transport != NULL)
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s router:\n  "
     "a transport must not be given for the %s driver",
       rblock->name, rblock->info->driver_name);
@@ -104,11 +104,7 @@ if (ob->protocol_name != NULL)
 /* If a response pattern is given, compile it now to get the error early. */
 
 if (ob->response_pattern != NULL)
-  {
-  regexp_compiling = ob->response_pattern;
-  ob->re_response_pattern = regcomp(ob->response_pattern);
-  regexp_compiling = NULL;
-  }
+  ob->re_response_pattern = regex_must_compile(ob->response_pattern);
 }
 
 
@@ -124,7 +120,8 @@ iplookup_router_entry(
   router_instance *rblock,        /* data for this instantiation */
   address_item *addr,             /* address we are working on */
   address_item **addr_local,      /* add it to this if successful & local */
-  address_item **addr_remote)     /* add it to this if successful & remote */
+  address_item **addr_remote,     /* add it to this if successful & remote */
+  BOOL verify)                    /* TRUE if verifying */
 {
 u_short net_port;
 char *query = NULL;
@@ -134,8 +131,8 @@ char host_buffer[256];
 host_item *host = store_malloc(sizeof(host_item));
 iplookup_router_options_block *ob =
   (iplookup_router_options_block *)(rblock->options_block);
-regexp *re = ob->re_response_pattern;
-int count, query_socket, query_len;
+pcre *re = ob->re_response_pattern;
+int count, query_len, rc;
 int error_yield = (ob->optional)? PASS : DEFER;
 int yield = PASS;
 
@@ -153,29 +150,20 @@ else
   query = expand_string(ob->query);
   if (query == NULL)
     {
-    log_write(0, LOG_MAIN, "%s router: failed to expand %s", rblock->name,
-      ob->query);
-    addr->message = string_sprintf("%s router: failed to expand %s",
-      rblock->name, ob->query);
+    addr->message = string_sprintf("%s router: failed to expand %s: %s",
+      rblock->name, ob->query, expand_string_message);
+    if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
   }
+
 query_len = (int)strlen(query);
 DEBUG(9) debug_printf("%s router query is %s\n", rblock->name, query);
 
-/* Create a socket, for UDP or TCP, as configured. */
-
-query_socket = socket(AF_INET,
-  (ob->protocol == ip_udp)? SOCK_DGRAM:SOCK_STREAM, 0);
-if (query_socket < 0)
-  {
-  log_write(0, LOG_MAIN, "socket creation failed in %s router", rblock->name);
-  return error_yield;
-  }
-
-/* Now bind the socket to the required port for each of the hosts in turn.
-Initialization insists on the port being set and there being a host list. */
+/* Now connect to the required port for each of the hosts in turn, until a
+response it received. Initialization insists on the port being set and there
+being a host list. */
 
 net_port = htons(ob->service);
 listptr = ob->hosts;
@@ -186,58 +174,108 @@ for (hostname = string_nextinlist(&listptr, ':', host_buffer,
      hostname = string_nextinlist(&listptr, ':', host_buffer,
         sizeof(host_buffer)))
   {
-  struct sockaddr_in sin;
   host_item *h;
 
+  DEBUG(9) debug_printf("calling host %s\n", hostname);
+
   host->name = hostname;
   host->address = NULL;
   host->next = NULL;
 
-  if (host_find_byname(host, NULL) == HOST_FIND_FAILED) continue;
+  if (string_is_ip_address(host->name)) host->address = host->name;
+    else if (host_find_byname(host, NULL) == HOST_FIND_FAILED) continue;
 
   /* Loop for possible multiple IP addresses for the given name. */
 
   for (h = host; h != NULL; h = h->next)
     {
-    int rc, save_errno;
+    int rc, save_errno, host_af, query_socket;
+
+    #if HAVE_IPV6
+    struct sockaddr_in6 sin;
+    #else
+    struct sockaddr_in sin;
+    #endif
+
+    /* Skip any hosts for which we have no address */
+
     if (h->address == NULL) continue;
 
+    /* Create a socket, for UDP or TCP, as configured. IPv6 addresses are
+    detected by checking for a colon in the address. */
+
+    host_af = (strchr(h->address, ':') != NULL)? AF_INET6 : AF_INET;
+    query_socket = socket(host_af,
+      (ob->protocol == ip_udp)? SOCK_DGRAM:SOCK_STREAM, 0);
+    if (query_socket < 0)
+      {
+      log_write(0, LOG_MAIN, "socket creation failed in %s router: %s",
+        rblock->name, strerror(errno));
+      return error_yield;
+      }
+
+    /* Clear the socket block, whichever type it is. */
+
     memset(&sin, 0, sizeof(sin));
+
+    /* Set up the address and port on an IPv6 system. You wouldn't have thought
+    it would take much to put the IPv6 address in the same place as the IPv4
+    one, but no... */
+
+    #if HAVE_IPV6
+    sin.sin6_family = host_af;
+    sin.sin6_port = net_port;
+
+    if (host_af == AF_INET6)
+      {
+      if (inet_pton(host_af, h->address, &sin.sin6_addr) != 1)
+        log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", h->address);
+      }
+    else
+      {
+      if (inet_pton(host_af, h->address, &sin.sin6_flowinfo) != 1)
+        log_write(0, LOG_PANIC_DIE, "unable to parse \"%s\"", h->address);
+      }
+
+    /* Set up the address and port on an IPv4 system */
+
+    #else
     sin.sin_family = AF_INET;
     sin.sin_port = net_port;
     sin.sin_addr.s_addr = (S_ADDR_TYPE)inet_addr(h->address);
+    #endif
 
     /* Just try the next one if connection or sending fails; use the
-    transport timeout handler code for timing out the connection - this
-    isn't a transport, but it's not worth making up something else. In fact,
+    common timeout handler code for timing out the connection. In fact,
     timeouts can occur only for TCP calls; for a UDP socket, "connect" always
     works (the router will timeout later on the read call). */
 
-    transport_sigalrm_seen = FALSE;
-    signal(SIGALRM, transport_timeout_handler);
+    sigalrm_seen = FALSE;
+    signal(SIGALRM, sigalrm_handler);
     alarm(ob->timeout);
 
     rc = connect(query_socket, (struct sockaddr *)&sin, sizeof(sin));
     save_errno = errno;
 
     alarm(0);
-    signal(SIGALRM, SIG_DFL);
+    signal(SIGALRM, SIG_IGN);
 
     if (rc < 0)
       {
       DEBUG(9)
         {
-        if (save_errno == EINTR && transport_sigalrm_seen)
-          save_errno = ETIMEDOUT;
-        debug_printf("connection to %s failed: %s\n", host->address,
+        if (save_errno == EINTR && sigalrm_seen) save_errno = ETIMEDOUT;
+        debug_printf("connection to %s failed: %s\n", h->address,
           strerror(save_errno));
         }
+      close(query_socket);
       continue;
       }
 
     if (send(query_socket, query, query_len, 0) < 0)
       {
-      DEBUG(9) debug_printf("send to %s failed\n", host->address);
+      DEBUG(9) debug_printf("send to %s failed\n", h->address);
+      close(query_socket);
       continue;
       }
 
@@ -284,16 +322,24 @@ for (hostname = string_nextinlist(&listptr, ':', host_buffer,
         }
       }
 
+    /* Finished with the socket */
+
+    close(query_socket);
+
+    /* recv() failed; try next IP address */
+
     if (count < 0)
       {
       DEBUG(9) debug_printf("%s from %s\n", (errno == ETIMEDOUT)?
-        "timed out" : "recv failed", host->address);
+        "timed out" : "recv failed", h->address);
       continue;
       }
 
+    /* Success; break the loop */
+
     reply[count] = 0;
     DEBUG(9) debug_printf("%s router received %s from %s\n",
-      rblock->name, reply, host->address);
+      rblock->name, reply, h->address);
     break;
     }
 
@@ -303,10 +349,9 @@ for (hostname = string_nextinlist(&listptr, ':', host_buffer,
   if (h != NULL) break;
   }
 
-/* Free the query and close the socket */
+/* Free the query. */
 
 if (query != NULL) store_free(query);
-close(query_socket);
 
 /* If hostname is NULL, we have failed to find any host, or failed to
 connect to any of the IP addresses, or timed out while reading or writing to
@@ -328,25 +373,17 @@ addr->routed_by_domain = FALSE;
 
 
 /* If a response pattern was supplied, match the returned string against it. A
-failure to match causes the router to fail. After a successful match, set
-up the numerical variables for expanding the rerouted address. */
+failure to match causes the router to fail. After a successful match, the
+numerical variables for expanding the rerouted address are set up. */
 
 if (re != NULL)
   {
-  if (!regexec(re, reply))
+  if (!regex_match_and_setup(re, reply, 0, -1))
     {
     DEBUG(9) debug_printf("%s router: %s failed to match response %s\n",
       rblock->name, ob->response_pattern, reply);
     return FAIL;
     }
-  expand_nmax = 0;
-  for (; expand_nmax < NSUBEXP; expand_nmax++)
-    {
-    expand_nstring[expand_nmax] = re->startp[expand_nmax];
-    expand_nlength[expand_nmax] = re->endp[expand_nmax] -
-      expand_nstring[expand_nmax];
-    }
-  expand_nmax--;
   }
 
 /* If no response pattern was supplied, set up ${0} as the response up to the
@@ -384,10 +421,9 @@ if (ob->reroute != NULL)
   reroute = expand_string(ob->reroute);
   if (reroute == NULL)
     {
-    log_write(0, LOG_MAIN, "%s router: failed to expand %s", rblock->name,
-      ob->reroute);
-    addr->message = string_sprintf("%s router: failed to expand %s",
-      rblock->name, ob->reroute);
+    addr->message = string_sprintf("%s router: failed to expand %s: %s",
+      rblock->name, ob->reroute, expand_string_message);
+    if (search_find_defer) return DEFER;
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
@@ -409,10 +445,27 @@ if (domain == NULL)
   return ERROR;
   }
 
-/* Update the domain and local part. */
+/* Update the domain and local part. If verifying, we update only the
+route_domain, which is the domain being worked on, since we do not want
+to change the original in this case. */
 
 addr->local_part = string_copyn(reroute, domain - reroute);
-addr->domain = addr->route_domain = string_copy(++domain);
+addr->route_domain = string_copy(++domain);
+if (!verify) addr->domain = addr->route_domain;
+
+/* Set up the errors address, if any. The function uses only fields that
+are in the same place in director and router blocks. */
+
+rc = direct_get_errors_address(addr, (director_instance *)rblock, "router",
+  verify, &(addr->errors_address));
+if (rc != OK) return rc;
+
+/* Set up the additional and removeable headers for this address. The function
+uses only fields that are in the same place in director and router blocks. */
+
+rc = direct_get_munge_headers(addr, (director_instance *)rblock, "router",
+  &(addr->extra_headers), &(addr->remove_headers));
+if (rc != OK) return rc;
 
 /* Check whether the new domain is actually a local domain. If so, we have to
 pass back this address to the directors. */
