@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -17,7 +17,7 @@ typedef struct {
   char *query;
   char *response_pattern;
   char *reroute;
-  regexp *re_response_pattern;
+  pcre *re_response_pattern;
   BOOL  optional;
 } iplookup_router_options_block;
 
@@ -33,7 +33,7 @@ extern iplookup_router_options_block iplookup_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int iplookup_router_entry(router_instance *, address_item *,
-  address_item **, address_item **);
+  address_item **, address_item **, BOOL);
 
 extern void iplookup_router_init(router_instance *);
 
