@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -12,7 +12,6 @@ typedef struct {
   BOOL qualify_single;
   BOOL search_parents;
   BOOL rewrite_headers;
-  char *self_mx;
   char *widen_domains;
   char *mx_domains;
   char *non_mx_domains;
@@ -32,7 +31,7 @@ extern lookuphost_router_options_block lookuphost_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int lookuphost_router_entry(router_instance *, address_item *,
-  address_item **, address_item **);
+  address_item **, address_item **, BOOL);
 
 extern void lookuphost_router_init(router_instance *);
 
