@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -11,6 +11,11 @@
 typedef struct {
   char *command;
   int timeout;
+  uid_t uid;
+  gid_t gid;
+  BOOL uid_set;
+  BOOL gid_set;
+  char *current_directory;
 } queryprogram_router_options_block;
 
 /* Data for reading the private options. */
@@ -25,7 +30,7 @@ extern queryprogram_router_options_block queryprogram_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int queryprogram_router_entry(router_instance *, address_item *,
-  address_item **, address_item **);
+  address_item **, address_item **, BOOL);
 
 extern void queryprogram_router_init(router_instance *);
 
