@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for reading spool files. */
@@ -32,24 +32,39 @@ Side effect: deliver_datafile is set to the fd of the open file.
 BOOL
 spool_open_datafile(char *id)
 {
+int i;
 struct stat statbuf;
 flock_t lock_data;
 char spoolname[256];
 
-sprintf(spoolname, "%s/input/%s-D", spool_directory, id);
-deliver_datafile = open(spoolname, O_RDWR | O_APPEND);
+/* If split_spool_directory is set, first look for the file in the appropriate
+sub-directory of the input directory. If it is not found there, try the input
+directory itself, to pick up leftovers from before the splitting. If split_
+spool_directory is not set, first look in the main input directory. If it is
+not found there, try the split sub-directory, in case it is left over from a
+splitting state. */
 
-if (deliver_datafile < 0)
+for (i = 0; i < 2; i++)
   {
+  int save_errno;
+  message_subdir[0] = (split_spool_directory == (i == 0))? id[5] : 0;
+  sprintf(spoolname, "%s/input/%s/%s-D", spool_directory, message_subdir, id);
+  deliver_datafile = open(spoolname, O_RDWR | O_APPEND);
+  if (deliver_datafile >= 0) break;
+  save_errno = errno;
   if (errno == ENOENT)
     {
+    if (i == 0) continue;
     log_write(0, LOG_MAIN, "Spool file %s-D not found", id);
-    return FALSE;
     }
-  log_write(0, LOG_MAIN, "Spool error for %s: %s", spoolname, strerror(errno));
+  else log_write(0, LOG_MAIN, "Spool error for %s: %s", spoolname,
+    strerror(errno));
+  errno = save_errno;
   return FALSE;
   }
 
+/* File is open and message_subdir is set. Lock the file. */
+
 lock_data.l_type = F_WRLCK;
 lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
 
@@ -61,10 +76,16 @@ if (fcntl(deliver_datafile, F_SETLK, &lock_data) < 0)
     log_write(5, LOG_MAIN, "Spool file is locked");
   close(deliver_datafile);
   deliver_datafile = -1;
+  errno = 0;
   return FALSE;
   }
 
-if (fstat(deliver_datafile, &statbuf) == 0) message_size = statbuf.st_size;
+/* Get the size of the data; don't include the leading filename line
+in the count, but add one for the newline before the data. */
+
+if (fstat(deliver_datafile, &statbuf) == 0)
+  message_size = statbuf.st_size - MESSAGE_ID_LENGTH - 3 + 1;
+
 return TRUE;
 }
 
@@ -170,6 +191,7 @@ malformed.
 Arguments:
   name          name of the header file, including the -H
   read_headers  TRUE if in-store header structures are to be built
+  subdir_set    TRUE is message_subdir is already set
 
 Returns:        spool_read_OK        success
                 spool_read_notopen   open failed
@@ -178,26 +200,40 @@ Returns:        spool_read_OK        success
 */
 
 int
-spool_read_header(char *name, BOOL read_headers)
+spool_read_header(char *name, BOOL read_headers, BOOL subdir_set)
 {
 FILE *f;
-int n;
+int n, uid, gid;
 BOOL inheader = FALSE;
 char originator[64];
 
-/* Ensure all pointers to store that will be obtained are NULL. This
-means that we can free partially got store if this function fails */
+/* Ensure all pointers to store that will be obtained are NULL, just
+in case. */
 
 header_list = header_last = NULL;
 sender_address = NULL;
+sender_host_address = NULL;
+sender_host_name = NULL;
+sender_helo_name = NULL;
+sender_fullhost = NULL;
+sender_ident = NULL;
 recipients_list = NULL;
 tree_nonrecipients = NULL;
 
-/* Generate the full name and open the file */
+/* Generate the full name and open the file. If message_subdir is already
+set, just look in the given directory. Otherwise, look in both the split
+and unsplit directories, as for the data file above. */
+
+for (n = 0; n < 2; n++)
+  {
+  if (!subdir_set)
+    message_subdir[0] = (split_spool_directory == (n == 0))? name[5] : 0;
+  sprintf(big_buffer, "%s/input/%s/%s", spool_directory, message_subdir, name);
+  f = fopen(big_buffer, "r");
+  if (f != NULL) break;
+  if (n != 0 || subdir_set || errno != ENOENT) return spool_read_notopen;
+  }
 
-sprintf(big_buffer, "%s/input/%s", spool_directory, name);
-f = fopen(big_buffer, "r");
-if (f == NULL) return spool_read_notopen;
 errno = 0;
 
 DEBUG(5) debug_printf("Opened spool file %s\n", name);
@@ -230,14 +266,17 @@ the mail address of the message's sender, enclosed in <>. The third contains
 the time the message was received, and the number of warning messages for
 delivery delays that have been sent. */
 
-if (sscanf(big_buffer, "%s %d %d", originator, (int *)(&originator_uid),
-  (int *)(&originator_gid)) != 3) goto SPOOL_FORMAT_ERROR;
+if (sscanf(big_buffer, "%s %d %d", originator, &uid, &gid) != 3)
+  goto SPOOL_FORMAT_ERROR;
 originator_login = string_copy(originator);
+originator_uid = (uid_t)uid;
+originator_gid = (gid_t)gid;
 
 if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 n = (int)strlen(big_buffer);
 if (n < 3 || big_buffer[0] != '<' || big_buffer[n-2] != '>')
   goto SPOOL_FORMAT_ERROR;
+
 sender_address = store_malloc(n-2);
 strncpy(sender_address, big_buffer+1, n-3);
 sender_address[n-3] = 0;
@@ -249,11 +288,13 @@ if (sscanf(big_buffer, "%d %d", &received_time, &warning_count) != 2)
 DEBUG(5) debug_printf("user=%s uid=%d gid=%d sender=%s\n", originator_login,
   (int)originator_uid, (int)originator_gid, sender_address);
 
-/* Now there may be a number of options lines, each starting with "-".
+/* Now there may be a number of optional lines, each starting with "-".
 First set default values for the options. */
 
 sender_local = FALSE;
 header_names = header_names_normal;
+body_linecount = -1;
+message_linecount = 0;
 deliver_freeze = FALSE;
 deliver_frozen_at = 0;
 local_error_message = FALSE;
@@ -263,21 +304,36 @@ for (;;)
   {
   if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
   if (big_buffer[0] != '-') break;
-  if (strcmp(big_buffer, "-local\n") == 0) sender_local = TRUE;
-    else if (strcmp(big_buffer, "-localerror\n") == 0)
+
+  big_buffer[(int)strlen(big_buffer) - 1] = 0;
+  if (strcmp(big_buffer, "-local") == 0) sender_local = TRUE;
+    else if (strcmp(big_buffer, "-localerror") == 0)
       local_error_message = TRUE;
-    else if (strcmp(big_buffer, "-resent\n") == 0)
+    else if (strcmp(big_buffer, "-resent") == 0)
       header_names = header_names_resent;
+    else if (strncmp(big_buffer, "-body_linecount", 15) == 0)
+      {
+      body_linecount = atoi(big_buffer + 15);
+      }
     else if (strncmp(big_buffer, "-frozen", 7) == 0)
       {
       deliver_freeze = TRUE;
       deliver_frozen_at = atoi(big_buffer + 7);
       }
-    else if (strcmp(big_buffer, "-manual_thaw\n") == 0)
+    else if (strcmp(big_buffer, "-manual_thaw") == 0)
       deliver_manual_thaw = TRUE;
-    else if (strcmp(big_buffer, "-user_null_sender\n") == 0)
+    else if (strcmp(big_buffer, "-user_null_sender") == 0)
       user_null_sender = TRUE;
 
+    else if (strncmp(big_buffer, "-host_address", 13) == 0)
+      sender_host_address = string_copy(big_buffer + 14);
+    else if (strncmp(big_buffer, "-host_name", 10) == 0)
+      sender_host_name = string_copy(big_buffer + 11);
+    else if (strncmp(big_buffer, "-helo_name", 10) == 0)
+      sender_helo_name = string_copy(big_buffer + 11);
+    else if (strncmp(big_buffer, "-ident", 6) == 0)
+      sender_ident = string_copy(big_buffer + 7);
+
   /* To allow new versions of Exim that add additional flags to interwork
   with older versions that do not understand them, just ignore any flagged
   lines that we don't recognize. Otherwise it wouldn't be possible to back
@@ -287,9 +343,14 @@ for (;;)
     /* else goto SPOOL_FORMAT_ERROR; */
   }
 
+/* Build sender_fullhost if required */
+
+host_build_sender_fullhost();
+
 DEBUG(5)
-  debug_printf("sender_local=%d resent=%s\n", sender_local,
-    (header_names == header_names_normal)? "no" : "yes");
+  debug_printf("sender_local=%d resent=%s ident=%s\n", sender_local,
+    (header_names == header_names_normal)? "no" : "yes",
+    (sender_ident == NULL)? "unset" : sender_ident);
 
 /* We now have the tree of addresses NOT to deliver to, or a line
 containing "XX", indicating no tree. */
@@ -343,15 +404,15 @@ if (big_buffer[0] != '\n') goto SPOOL_FORMAT_ERROR;
 while ((n = fgetc(f)) != EOF)
   {
   header_line *h;
-  char flag[4];
+  uschar flag[4];
   int i;
 
   if (!isdigit(n)) goto SPOOL_FORMAT_ERROR;
-
   ungetc(n, f);
   fscanf(f, "%d%c ", &n, flag);
-  if (!isprint(flag[0])) goto SPOOL_FORMAT_ERROR;
-  message_size += n;
+  if (!mac_isprint(flag[0])) goto SPOOL_FORMAT_ERROR;
+
+  if (flag[0] != '*') message_size += n;  /* Omit non-transmitted headers */
 
   if (read_headers)
     {
@@ -378,6 +439,7 @@ while ((n = fgetc(f)) != EOF)
       {
       int c = fgetc(f);
       if (c == 0 || c == EOF) goto SPOOL_FORMAT_ERROR;
+      if (c == '\n' && h->type != htype_old) message_linecount++;
       h->text[i] = c;
       }
     h->text[i] = 0;
@@ -404,9 +466,18 @@ while ((n = fgetc(f)) != EOF)
     }
   }
 
-/* We have successfully read the data in the header file. Close it and
+/* We have successfully read the data in the header file. If the value of
+body_linecount is set (it will be except for spool files written by older
+Exims - the test was written on 15Apr98 and can be removed in a couple of
+years time) set up message_linecount as the total count. Close the file and
 give a positive response. */
 
+message_linecount = (body_linecount >= 0)?
+  message_linecount + body_linecount : -1;
+
+DEBUG(5) debug_printf("body_linecount=%d message_linecount=%d\n",
+  body_linecount, message_linecount);
+
 fclose(f);
 return spool_read_OK;
 
