@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -16,6 +16,14 @@ struct director_info;
 struct transport_info;
 struct router_info;
 
+/* Structure for remembering macros for the configuration file */
+
+typedef struct macro_item {
+  struct macro_item *next;
+  BOOL  command_line;
+  char *replacement;
+  char name[1];
+} macro_item;
 
 /* Entries in lists of main and driver options are in this form. */
 
@@ -30,7 +38,7 @@ typedef struct {
 
 typedef struct re_block {
   struct re_block *next;
-  regexp *re;
+  pcre *re;
 } re_block;
 
 
@@ -77,8 +85,10 @@ typedef struct transport_instance {
   struct transport_info *info;   /* Info for this driver */
   void *options_block;           /* Pointer to private options */
   char *driver_name;             /* Must be first */
-  char *(*setup)(                /* Setup entry point for local transports */
-    struct transport_instance *);
+  int  (*setup)(                 /* Setup entry point for local transports */
+    struct transport_instance *,
+    char **);
+                                 /**************************************/
   int   local_batch;             /* )                                  */
   int   local_smtp;              /* )                                  */
   int   batch_max;               /* )                                  */
@@ -92,11 +102,20 @@ typedef struct transport_instance {
   char *current_dir;             /* )                                  */
   BOOL  deliver_as_creator;      /* )                                  */
   BOOL  initgroups;              /* )                                  */
-  BOOL  multi_domain;            /* Applies to remote transports only */
+  BOOL  bsmtp_helo;              /* )                                  */
+                                 /**************************************/
+  BOOL  multi_domain;            /* Applies to remote transports only  */
+                                 /**************************************/
+  struct transport_instance *shadow;  /* Shadow, to use on success. */
+  char *shadow_condition;        /* Condition for running it */
+  char *filter_command;          /* For on-the-fly-filtering */
+  char *add_headers;             /* Add these headers */
+  char *remove_headers;          /* Remove these headers */
   BOOL  return_output;           /* TRUE if output should always be returned */
   BOOL  return_fail_output;      /* ditto, but only on failure */
   BOOL  log_output;              /* Similarly for logging */
   BOOL  log_fail_output;
+  BOOL  log_defer_output;
   BOOL  retry_use_local_part;    /* Defaults true for local, false for remote */
   int   max_addresses;           /* Max amalgamation for remote delivery */
 } transport_instance;
@@ -131,27 +150,48 @@ typedef struct director_instance {
   struct director_info *info;
   void *options_block;           /* Pointer to private options */
   char *driver_name;             /* Must be first */
-  char *prefix;                  /* Address prefix */
-  char *suffix;                  /* Address suffix */
+
+  /********************************************************************/
+  /* This set of variables must be exactly the same for directors and */
+  /* routers, so that common code can be used. Some of the others are */
+  /* similarly named and have the same function, but don't share code */
+  /* so they aren't in this block.                                    */
+  /********************************************************************/
   char *domains;                 /* Specific domains */
   char *except_domains;          /* Specific not domains */
   char *local_parts;             /* Specific local parts */
   char *except_local_parts;      /* Specific not local parts */
-  char *require_files;           /* Required (or not) for director to run */
+  char *errors_to;               /* Errors address */
+  char *extra_headers;           /* Additional headers */
+  char *remove_headers;          /* Those to remove */
   re_block *re_domains;          /* Used when domains are wildcarded */
   re_block *re_except_domains;
   re_block *re_local_parts;
   re_block *re_except_local_parts;
-  transport_instance *transport; /* Assigned transport */
   BOOL  more;                    /* If FALSE, do no more if this one fails */
+  BOOL  verify_only;             /* Skip this director if not verifying */
+  BOOL  verify_sender;           /* Use this director when verifying a sender */
+  BOOL  verify_recipient;        /* Use this director when verifying a recipient*/
+  transport_instance *transport; /* Assigned transport */
+  char *expand_transport;        /* Dynamic transport string */
+  /********************************************************************/
+
+  char *condition;               /* General condition */
+  char *prefix;                  /* Address prefix */
+  char *suffix;                  /* Address suffix */
+  char *senders;                 /* Specific senders */
+  char *except_senders;          /* Specific not senders */
+  char *require_files;           /* Required (or not) for director to run */
+  char *new_director;            /* Name of director for generated address */
+  re_block *re_senders;
+  re_block *re_except_senders;
+  struct director_instance *new; /* Actual director for generated address */
   BOOL  unseen;                  /* If TRUE, carry on even after success */
   BOOL  prefix_optional;         /* Just what it says */
   BOOL  suffix_optional;         /* Ditto */
-  BOOL  verify_sender;           /* Use this director when verifying a sender */
-  BOOL  verify_recipient;        /* Use this director when verifying a recipient*/
   BOOL  fail_verify_sender;      /* Fail verify if sender match this director */
   BOOL  fail_verify_recipient;   /* Fail verify if recipient match this director */
-  BOOL  verify_only;             /* Skip this director if not verifying */
+  BOOL  expn;                    /* Use this director when processing EXPN */
 } director_instance;
 
 
@@ -186,23 +226,42 @@ typedef struct router_instance {
   struct router_info *info;
   void *options_block;           /* Pointer to private options */
   char *driver_name;             /* Must be first */
-  transport_instance *transport; /* Assigned transport */
-  char *domains;                 /* Domains to match for this router to run */
+
+  /********************************************************************/
+  /* This set of variables must be exactly the same for directors and */
+  /* routers, so that common code can be used. Some of the others are */
+  /* similarly named and have the same function, but don't share code */
+  /* so they aren't in this block.                                    */
+  /********************************************************************/
+  char *domains;                 /* Specific domains */
   char *except_domains;          /* Specific not domains */
   char *local_parts;             /* Specific local parts */
   char *except_local_parts;      /* Specific not local parts */
-  char *require_files;           /* Files required (or not) for router to run */
+  char *errors_to;               /* Errors address */
+  char *extra_headers;           /* Additional headers */
+  char *remove_headers;          /* Removed headers */
   re_block *re_domains;          /* Used when domains are wildcarded */
   re_block *re_except_domains;
   re_block *re_local_parts;
   re_block *re_except_local_parts;
+
   BOOL  more;                    /* If FALSE, do no more if this one fails */
-  BOOL  unseen;                  /* If TRUE carry on, even after success */
+  BOOL  verify_only;             /* Skip this router if not verifying */
   BOOL  verify_sender;           /* Use this router when verifying a sender */
-  BOOL  verify_recipient;        /* Use this router when verifying a recipient */
+  BOOL  verify_recipient;        /* Use this router when verifying a recipient*/
+  transport_instance *transport; /* Assigned transport */
+  char *expand_transport;        /* Dynamic transport string */
+  /********************************************************************/
+
+  char *condition;               /* General condition */
+  char *senders;                 /* Specific senders */
+  char *except_senders;          /* Specific not senders */
+  char *require_files;           /* Files required (or not) for router to run */
+  re_block *re_senders;
+  re_block *re_except_senders;
+  BOOL  unseen;                  /* If TRUE carry on, even after success */
   BOOL  fail_verify_sender;      /* Fail verify if sender match this router */
   BOOL  fail_verify_recipient;   /* Fail verify if recipient match this router */
-  BOOL  verify_only;             /* Skip this router if not verifying */
   BOOL  pass_on_timeout;         /* Treat timeout DEFERs as fails */
   char *self;                    /* Text option for handling self reference */
   int   self_code;               /* Encoded version of same */
@@ -218,7 +277,8 @@ typedef struct router_info {
     router_instance *,
     struct address_item *,
     struct address_item **,
-    struct address_item **);
+    struct address_item **,
+    BOOL);
   void (*init)(
     struct router_instance *);
   void (*tidyup)(
@@ -230,6 +290,34 @@ typedef struct router_info {
 } router_info;
 
 
+/* Structure for holding information about a lookup type. */
+
+typedef struct lookup_info {
+  char *name;                    /* e.g. "lsearch" */
+  int type;                      /* query/singlekey/abs-file */
+  void *(*open)(                 /* open function */
+    char *,                      /* file name for those that have one */
+    char **);                    /* for error message */
+  BOOL (*check)(                 /* file checking function */
+    void *,                      /* handle */
+    char *,                      /* file name */
+    int,                         /* modemask for file checking */
+    int *,                       /* owners for file checking */
+    int *,                       /* owngroups for file checking */
+    char **);                    /* for error messages */
+  int (*find)(                   /* find function */
+    void *,                      /* handle */
+    char *,                      /* file name or NULL */
+    char *,                      /* key or query */
+    int,                         /* length of key or query */
+    char **,                     /* for returning answer */
+    char **);                    /* for error message */
+  void (*close)(                 /* close function */
+    void *);                     /* handle */
+  void (*tidy)(void);            /* tidy function */
+} lookup_info;
+
+
 /* Structure for holding information about a host for use mainly by
 routers, but also used when checking lists of hosts. Looking up host
 addresses is done in both cases, using this structure. */
@@ -261,13 +349,17 @@ typedef struct ip_address_item {
 /* Structure for holding information about an IP net; used for controlling
 incoming and outgoing SMTP calls. The size field gives the number of words
 used in the address and mask fields. It's a bit untidy screwing in 4 here, but
-it makes things less messy. When we get to IPv8 this may have to change... */
+it makes things less messy. When we get to IPv8 this may have to change...
+The ability to do file lookups for networks is also available (at a cost, of
+course). If filename != NULL, is is scanned instead of using the other
+fields. */
 
 typedef struct ip_net_item {
   struct ip_net_item *next;
   int size;
   int address[4];
   int mask[4];
+  char *filename;
 } ip_net_item;
 
 
@@ -301,6 +393,7 @@ typedef struct reply_item {
   BOOL  file_expand;             /* expand the body */
   char *logfile;                 /* file to keep a log in */
   char *oncelog;                 /* file to keep records in for once only */
+  BOOL  return_message;          /* send back the original message */
 } reply_item;
 
 
@@ -330,13 +423,19 @@ typedef struct address_item {
   char *domain;                  /* domain of envelope address */
   char *route_domain;            /* domain to be used by routers */
   char *errors_address;          /* where to send errors (NULL => sender) */
+  char *self_hostname;           /* after routing->directors */
   BOOL pfr;                      /* pipe or file or reply delivery */
   BOOL rewrite_headers;          /* set TRUE by routers that want it done */
   BOOL local;                    /* local/remote flag */
+  BOOL local_part_quoted;        /* local part was quoted */
+  BOOL forced_local;             /* by a self option on a router */
   BOOL delivered;                /* already delivered */
   BOOL routed_by_domain;         /* routing did not depend on local part */
   BOOL ignore_error;             /* ignore delivery error */
+  BOOL local_host_removed;       /* local host was backup */
   header_line *extra_headers;    /* additional headers */
+  char *remove_headers;          /* list of those to remove */
+  director_instance *start_director; /* generated address starts here */
   director_instance *director;   /* the director that directed */
   router_instance *router;       /* OR the router that routed */
   transport_instance *transport; /* transport to use */
@@ -350,7 +449,8 @@ typedef struct address_item {
   int  mode;                     /* mode for local transporting to a file */
   char *home_dir;                /* home directory for local transporting */
   char *current_dir;             /* current directory for local transporting */
-  char *route_option;            /* undefined routing option string */
+  char *route_option;            /* routing option string */
+  char **pipe_expandn;           /* numeric expansions for pipe from filter */
   BOOL initgroups;               /* use initgroups() for local transporting */
   BOOL expand_pipe;              /* expand pipe arguments */
   BOOL allow_pipe;               /* allow pipe in generated address */
@@ -360,6 +460,7 @@ typedef struct address_item {
   int  basic_errno;              /* status after failure */
   int  more_errno;               /* additional error information */
   char *message;                 /* local error message */
+  char *shadow_message;          /* info about shadow transporting */
   int  special_action;           /* used when when deferred or failed; also */
                                  /* contains = or - when successful SMTP delivered */
   BOOL dr_retry_exists;          /* director or router retry record exists */
@@ -391,6 +492,7 @@ typedef struct error_block {
 
 typedef struct queue_filename {
   struct queue_filename *next;
+  char dir_char;
   char text[1];
 } queue_filename;
 
@@ -437,4 +539,14 @@ typedef struct tree_node {
   char name[1];                   /* node name - variable length */
 } tree_node;
 
+/* Structure for holding a partially decoded DNS record; the name has been
+uncompressed, but the data pointer is into the raw data. */
+
+typedef struct {
+  char  name[DNS_MAXNAME];        /* domain name */
+  int   type;                     /* record type */
+  int   size;                     /* size of data */
+  u_char *data;                   /* pointer to data */
+} dns_record;
+
 /* End of structs.h */
