@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -33,6 +33,8 @@ optionlist appendfile_transport_options[] = {
       (void *)(offsetof(transport_instance, batch_max)) },
   { "bsmtp",             opt_local_batch | opt_public,
       (void *)(offsetof(transport_instance, local_smtp)) },
+  { "bsmtp_helo",        opt_bool | opt_public,
+      (void *)(offsetof(transport_instance, bsmtp_helo)) },
   { "check_group",       opt_bool,
       (void *)(offsetof(appendfile_transport_options_block, check_group)) },
   { "create_directory",  opt_bool,
@@ -65,8 +67,14 @@ optionlist appendfile_transport_options[] = {
       (void *)(offsetof(appendfile_transport_options_block, lockfile_mode)) },
   { "lockfile_timeout",  opt_time,
       (void *)(offsetof(appendfile_transport_options_block, lockfile_timeout)) },
+  { "maildir_format",    opt_bool,
+      (void *)(offsetof(appendfile_transport_options_block, maildir_format )) } ,
+  { "maildir_retries",   opt_int,
+      (void *)(offsetof(appendfile_transport_options_block, maildir_retries)) },
   { "mode",              opt_octint,
       (void *)(offsetof(appendfile_transport_options_block, mode)) },
+  { "mode_fail_narrower",opt_bool,
+      (void *)(offsetof(appendfile_transport_options_block, mode_fail_narrower)) },
   { "notify_comsat",     opt_bool,
       (void *)(offsetof(appendfile_transport_options_block, notify_comsat)) },
   { "prefix",            opt_stringptr,
@@ -81,6 +89,10 @@ optionlist appendfile_transport_options[] = {
       (void *)(offsetof(appendfile_transport_options_block, return_path_add)) },
   { "suffix",            opt_stringptr,
       (void *)(offsetof(appendfile_transport_options_block, suffix)) },
+  { "use_crlf",          opt_bool,
+      (void *)(offsetof(appendfile_transport_options_block, use_crlf)) },
+  { "use_fcntl_lock",    opt_bool,
+      (void *)(offsetof(appendfile_transport_options_block, use_fcntl)) },
   { "use_lockfile",      opt_bool,
       (void *)(offsetof(appendfile_transport_options_block, use_lockfile)) },
   { "user",              opt_expand_uid | opt_public,
@@ -110,6 +122,7 @@ appendfile_transport_options_block appendfile_transport_option_defaults = {
   30*60,          /* lockfile_timeout */
   10,             /* lock_retries */
    3,             /* lock_interval */
+  10,             /* maildir_retries */
   create_anywhere,/* create_file */
   FALSE,          /* allow_symlink */
   FALSE,          /* check_group */
@@ -117,11 +130,15 @@ appendfile_transport_options_block appendfile_transport_option_defaults = {
   FALSE,          /* notify_comsat */
   TRUE,           /* require_lockfile */
   TRUE,           /* use_lockfile */
+  TRUE,           /* use_fcntl */
+  FALSE,          /* use_crlf */
   TRUE,           /* from_hack */
   TRUE,           /* return_path_add */
   TRUE,           /* delivery_date_add */
   TRUE,           /* envelope_to_add */
-  FALSE           /* file_must_exist */
+  FALSE,          /* file_must_exist */
+  TRUE,           /* mode_fail_narrower */
+  FALSE           /* maildir format */
 };
 
 
@@ -135,12 +152,15 @@ are changed and the main entry point is called. We use this function to
 expand any quota setting, so that it can access files that may not be readable
 by the user.
 
-Argument:   tblock   points to the transport instance
-Returns:             NULL if OK, pointer to error string if not
+Arguments:
+  tblock     points to the transport instance
+  errmsg     where to put an error message
+
+Returns:     OK, FAIL, or DEFER
 */
 
-char *
-appendfile_transport_setup(transport_instance *tblock)
+static int
+appendfile_transport_setup(transport_instance *tblock, char **errmsg)
 {
 appendfile_transport_options_block *ob =
   (appendfile_transport_options_block *)(tblock->options_block);
@@ -150,14 +170,17 @@ char *s, *rest;
 if (ob->quota == NULL)
   {
   ob->quota_value = 0;
-  return NULL;
+  return OK;
   }
 
 s = expand_string(ob->quota);
 if (s == NULL)
-  return string_sprintf("Expansion of \"%s\" (quota setting "
+  {
+  *errmsg = string_sprintf("Expansion of \"%s\" (quota setting "
     "for %s transport) failed: %s", ob->quota, tblock->name,
     expand_string_message);
+  return search_find_defer? DEFER : FAIL;
+  }
 
 d = strtod(s, &rest);
 if (tolower(*rest) == 'k') { d *= 1024.0; rest++; }
@@ -165,11 +188,14 @@ if (tolower(*rest) == 'm') { d *= 1024.0*1024.0; rest++; }
 while (isspace(*rest)) rest++;
 
 if (*rest != 0)
-  return string_sprintf("Malformed quota setting \"%s\" for "
+  {
+  *errmsg = string_sprintf("Malformed quota setting \"%s\" for "
     "%s transport", s, tblock->name);
+  return FAIL;
+  }
 
 ob->quota_value = (int)d;
-return NULL;
+return OK;
 }
 
 
@@ -207,11 +233,19 @@ if (ob->filename != NULL && ob->dirname != NULL)
   "only one of \"file\" or \"directory\" can be specified", tblock->name);
 
 /* If a file name was specified, it must be an absolute path. Can check here
-only if there are no expansions. */
+only if there are no expansions. Also, at least one locking mechanism must
+be specified. */
 
-if (ob->filename != NULL && ob->filename[0] != '/' && ob->filename[0] != '$')
-  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s transport:\n  "
-  "the file option must specify an absolute path", tblock->name);
+if (ob->filename != NULL)
+  {
+  if (ob->filename[0] != '/' && ob->filename[0] != '$')
+    log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s transport:\n  "
+    "the file option must specify an absolute path", tblock->name);
+  if (!ob->use_fcntl && (!ob->use_lockfile || !ob->require_lockfile))
+    log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s transport:\n  "
+    "either lockfile or fcntl locking (or both) must be specified",
+    tblock->name);
+  }
 
 /* If a directory name was specified, it must be an absolute path. Can check
 here only if there are no expansions. */
@@ -222,7 +256,7 @@ if (ob->dirname != NULL && ob->dirname[0] != '/' && ob->dirname[0] != '$')
 
 /* If a fixed uid field is set, then a gid field must also be set. */
 
-if (tblock->uid_set && !tblock->gid_set)
+if (tblock->uid_set && !tblock->gid_set && tblock->expand_gid == NULL)
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "user set without group for the %s transport", tblock->name);
 
@@ -311,7 +345,7 @@ if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
   return;
   }
 
-sprintf(buffer, "%s@%d\n", user, offset);
+sprintf(buffer, "%.200s@%d\n", user, offset);
 if (sendto(fd, buffer, (int)strlen(buffer) + 1, 0, (struct sockaddr *)(&sa),
     sizeof(sa)) < 0)
   {
@@ -368,7 +402,8 @@ write_string(int fd, char *format, ...)
 {
 va_list ap;
 va_start(ap, format);
-vsprintf(big_buffer, format, ap);
+if (!string_vformat(big_buffer, BIG_BUFFER_SIZE, format, ap))
+  log_write(0, LOG_PANIC_DIE, "overlong formatted string in appendfile");
 va_end(ap);
 return write_block(fd, big_buffer, strlen(big_buffer));
 }
@@ -471,10 +506,8 @@ directory name) is given:
 The following items apply in the case when a directory name is given:
 
 (2d) Create a new file in the directory using a temporary name, by opening for
-     writing and with O_CREAT.
-
-(3d) Generate a unique name for the file from the time and the inode number,
-     and rename file file.
+     writing and with O_CREAT. If maildir format is being used, the file
+     is created in a temporary subdirectory with a prescribed name.
 
 The following items apply in all cases:
 
@@ -492,12 +525,20 @@ The following items apply in all cases:
 
      For any other error (most commonly disc full), do the same.
 
-(12) Restore the atime; notify_comsat if required; close the file (which
+The following applies after appending to a file:
+
+(12f)Restore the atime; notify_comsat if required; close the file (which
      unlocks it if it was locked).
 
-This yields FAIL only when a file name is generated by an alias or forwarding
-operation and attempting to open it gives EPERM, ENOENT, or EACCES. All other
-failures return DEFER (in addr->transport_return). */
+The following applies after writing a unique file in a directory:
+
+(12d)For non-maildir-format: Generate a unique name for the file from the time
+     and the inode number, and rename the file after writing it. For maildir
+     format, rename the file into the new directory.
+
+This transport yields FAIL only when a file name is generated by an alias or
+forwarding operation and attempting to open it gives EPERM, ENOENT, or EACCES.
+All other failures return DEFER (in addr->transport_return). */
 
 
 void
@@ -508,10 +549,12 @@ appendfile_transport_entry(
 appendfile_transport_options_block *ob =
   (appendfile_transport_options_block *)(tblock->options_block);
 struct stat statbuf;
+char *path;
 char *filename;
-char *lockname = NULL;     /* keep picky compiler happy */
 char *hitchname;
-char tempname[256];
+char *lockname = NULL;     /* keep picky compiler happy */
+char *newname = NULL;      /* ditto */
+char *cr = "";
 time_t now = time(NULL);
 struct utimbuf times;
 BOOL smtp_dots = FALSE;
@@ -530,10 +573,16 @@ int i, len;
 
 DEBUG(9) debug_printf("appendfile transport entered\n");
 
-/* See if this is the address_file transport, used to deliver to files
-specified via .forward or an alias file. */
+/* See if this is the address_file or address_directory transport, used to
+deliver to files specified via .forward or an alias file. If it is, assume
+a directory-style delivery if the last character is '/'. (For address_file
+this shouldn't ever be the case.) */
 
-if (addr->pfr && addr->local_part[0] == '/') filename = addr->local_part;
+if (addr->pfr && addr->local_part[0] == '/')
+  {
+  path = addr->local_part;
+  if (path[(int)strlen(path)-1] == '/') isdirectory = TRUE;
+  }
 
 /* Handle a non-address file delivery. One of the file or directory options is
 mandatory; it will have already been checked to be an absolute path. */
@@ -555,9 +604,9 @@ else
     return;
     }
 
-  filename = expand_string(fdname);
+  path = expand_string(fdname);
 
-  if (filename == NULL)
+  if (path == NULL)
     {
     addr->transport_return = PANIC;
     addr->message = string_sprintf("Expansion of \"%s\" (file or directory "
@@ -566,7 +615,7 @@ else
     return;
     }
 
-  /* If the file name contained a reference to $local_part, check that it
+  /* If the path contained a reference to $local_part, check that it
   doesn't attempt to change directory. Note that deliver_localpart may be
   unset during some deliveries (e.g. batched SMTP). */
 
@@ -585,10 +634,10 @@ else
   it started with an expansion item. If not, the check is done at
   initialization time. */
 
-  if (filename[0] != '/')
+  if (path[0] != '/')
     {
     addr->message = string_sprintf("appendfile: file or directory name "
-      "\"%s\" is not absolute", filename);
+      "\"%s\" is not absolute", path);
     addr->basic_errno = ERRNO_NOTABSOLUTE;
     addr->special_action = SPECIAL_FREEZE;
     return;
@@ -601,9 +650,9 @@ DEBUG(9) debug_printf("appendfile: mode=%o notify_comsat=%d quota=%d\n"
   "  %s=%s\n  prefix=%s\n  suffix=%s\n",
   mode, ob->notify_comsat, ob->quota_value,
   isdirectory? "directory" : "file",
-  filename,
-  (ob->prefix == NULL)? "null":ob->prefix,
-  (ob->suffix==NULL)? "null":ob->suffix);
+  path,
+  (ob->prefix == NULL)? "null" : string_printing(ob->prefix, FALSE),
+  (ob->suffix == NULL)? "null" : string_printing(ob->suffix, FALSE));
 
 
 /* If the -N option is set, can't do any more. */
@@ -624,12 +673,16 @@ if (!isdirectory)
   BOOL use_lstat = TRUE;
   BOOL file_opened = FALSE;
 
-  if (strcmp(filename, "/dev/null") == 0)
+  if (strcmp(path, "/dev/null") == 0)
     {
     addr->transport_return = OK;
     return;
     }
 
+  /* The name of the file to be opened */
+
+  filename = path;
+
   /* The locking of mailbox files is worse than the naming of cats, which is
   known to be "a difficult matter" (T.S. Eliot) and just as cats must have
   three different names, so several different styles of locking are used.
@@ -656,7 +709,7 @@ if (!isdirectory)
   have reproduced this problem, but I do not know any way of forcing a host to
   update its attribute cache for an open NFS file. It would be nice if it did
   so when a lock was taken out, but this does not seem to happen. Anyway, to
-  reduce the risk of this problem happening, we now created the lock file
+  reduce the risk of this problem happening, we now create the lock file
   *before* opening the mailbox. That will prevent two different Exims opening
   the file simultaneously. It may not prevent clashes with MUAs, however, but
   Pine seems to operate in the same way.
@@ -727,6 +780,7 @@ if (!isdirectory)
       {
       int rc;
       hd = open(hitchname, O_WRONLY | O_CREAT | O_EXCL, ob->lockfile_mode);
+
       if (hd < 0)
         {
         if (errno != EACCES || ob->require_lockfile)
@@ -953,8 +1007,15 @@ if (!isdirectory)
             addr->special_action = SPECIAL_FREEZE;
             goto RETURN;
             }
+          oldmode = mode;
           }
-        else
+
+        /* Mode not from address, and newly-created permissions are greater
+        than existing permissions. Default is to complain, but it can be
+        configured to go ahead and try to deliver anyway if that's what
+        the administration wants. */
+
+        else if (ob->mode_fail_narrower)
           {
           addr->basic_errno = ERRNO_BADMODE;
           addr->message = string_sprintf("mailbox %s has wrong mode %o",
@@ -1039,7 +1100,7 @@ if (!isdirectory)
         goto RETURN;
         }
 
-      if ((statbuf.st_mode & 07777) != mode)
+      if ((statbuf.st_mode & 07777) != oldmode)
         {
         addr->basic_errno = ERRNO_BADMODE;
         addr->message = string_sprintf("opened mailbox %s has wrong mode %o "
@@ -1051,12 +1112,15 @@ if (!isdirectory)
       /* The file is OK. Carry on to do the locking. */
       }
 
-    /* We now have an open file, and must lock it using fcntl. If a lock file
-    is also required, it was created above and hd was left >= 0. If fcntl()
-    fails, close the file and go round the loop all over again, after waiting
-    for a bit. */
+    /* We now have an open file, and must lock it using fcntl if configured to
+    do so. If a lock file is also required, it was created above and hd was
+    left >= 0. At least one form of locking is required by the initialization
+    function. If fcntl() fails, close the file and go round the loop all over
+    again, after waiting for a bit. */
 
     file_opened = TRUE;
+    if (!ob->use_fcntl) break;
+
     lock_data.l_type = F_WRLCK;
     lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
     if (fcntl(fd, F_SETLK, &lock_data) >= 0) break;
@@ -1112,22 +1176,184 @@ name is used to create the file. Later, when it is written, the name is changed
 to a unique one. There is no need to lock the file. An attempt is made to
 create the directory if it does not exist. */
 
-else
+else if (!ob->maildir_format)
   {
-  sprintf(tempname, "%s/temp.%d", filename, (int)getpid());
-  fd = open(tempname, O_WRONLY|O_CREAT, mode);
+  DEBUG(9) debug_printf("delivering to unique file (non-maildir)\n");
+  filename = string_sprintf("%s/temp.%d.%s", path, (int)getpid(),
+    primary_hostname);
+  fd = open(filename, O_WRONLY|O_CREAT, mode);
   if (fd < 0 &&                                    /* failed to open, and */
-      (errno != ENOENT ||                          /* either not not exists */
+      (errno != ENOENT ||                          /* either not exists */
        !ob->create_directory ||                    /* or not allowed to make */
-       mkdir(filename, ob->dirmode) < 0 ||         /* or failed to create dir */
-       (fd = open(tempname, O_WRONLY|O_CREAT, mode)) < 0)) /* then open */
+       mkdir(path, ob->dirmode) < 0 ||             /* or failed to create dir */
+       (fd = open(filename, O_WRONLY|O_CREAT, mode)) < 0)) /* or then failed to open */
     {
     addr->basic_errno = errno;
-    addr->message = string_sprintf("while creating file %s", tempname);
+    addr->message = string_sprintf("while creating file %s", filename);
     return;
     }
-  chown(tempname, uid, gid);
-  chmod(tempname, mode);
+
+  /* Why are these here? I should have stuck in a comment. Perhaps it's to
+  do with soft links? */
+
+  chown(filename, uid, gid);
+  chmod(filename, mode);
+
+  /* Build the new unique name. It starts with 'q' for smail compatibility.
+  The fiddling around is because string_base62 returns its result in a fixed
+  buffer always (exactly 6 chars plus zero). */
+
+  if (stat(filename, &statbuf) < 0)
+    {
+    addr->basic_errno = errno;
+    addr->message = string_sprintf("after stat() for %s", filename);
+    goto RETURN;
+    }
+
+  strcpy(big_buffer, string_base62(time(NULL)));
+  newname = string_sprintf("%s/q%s-%s", path, big_buffer,
+    string_base62(statbuf.st_ino));
+  }
+
+/* Handle the case of a unique file in maildir format. The file is written to
+the tmp subdirectory, with a prescribed form of name. */
+
+else
+  {
+  char *subdirs[] = { "/tmp", "/new", "/cur" };
+
+  DEBUG(9) debug_printf("delivering in maildir format in %s\n", path);
+
+  /* First ensure that the path we have is a directory; if it does not exist,
+  create it. Then make sure the tmp, new & cur subdirs of the maildir are
+  there. If not we abort the delivery (even though the cur subdir is not
+  actually needed for delivery). Handle all 4 directory tests/creates in a
+  loop so that code can be shared. */
+
+  for (i = 0; i < 4; i++)
+    {
+    int j;
+    char *dir, *mdir;
+
+    if (i == 0)
+      {
+      mdir = "";
+      dir = path;
+      }
+    else
+      {
+      mdir = subdirs[i-1];
+      dir = mdir + 1;
+      }
+
+    /* Check an existing path is a directory. This is inside a loop because
+    there is a potential race condition when creating the directory - some
+    other process may get there first. Give up after trying several times,
+    though. */
+
+    for (j = 0; j < 10; j++)
+      {
+      if (stat(dir, &statbuf) == 0)
+        {
+        if (S_ISDIR(statbuf.st_mode)) break;   /* out of the race loop */
+        addr->message = string_sprintf("%s%s is not a directory", path,
+          mdir);
+        addr->basic_errno = ERRNO_NOTDIRECTORY;
+        addr->special_action = SPECIAL_FREEZE;
+        return;
+        }
+
+      /* Try to make if non-existent and configured to do so */
+
+      if (errno == ENOENT && ob->create_directory)
+        {
+        if (mkdir(dir, ob->dirmode) != 0)
+          {
+          if (errno == EEXIST) continue;     /* repeat the race loop */
+          addr->message = string_sprintf ("cannot mkdir %s%s", path, mdir);
+	  addr->basic_errno = errno;
+	  addr->special_action = SPECIAL_FREEZE;
+	  return;
+          }
+        DEBUG(9) debug_printf("created directory %s%s\n", path, mdir);
+        break;   /* out of the race loop */
+        }
+
+      /* stat() error other than ENOENT, or ENOENT and not creatable */
+
+      addr->message = string_sprintf("stat() error for %s%s", path, mdir);
+      addr->basic_errno = errno;
+      addr->special_action = SPECIAL_FREEZE;
+      return;
+      }
+
+    /* If we went round the loop 10 times, the directory was flickering in and
+    out of existence like someone in a malfunctioning Star Trek transporter. */
+
+    if (j >= 10)
+      {
+      addr->message = string_sprintf("existence of %s%s unclear\n");
+      addr->basic_errno = errno;
+      addr->special_action = SPECIAL_FREEZE;
+      return;
+      }
+
+    /* First time through the directories loop, cd to the main directory */
+
+    if (i == 0 && chdir(path) != 0)
+      {
+      addr->message = string_sprintf ("cannot chdir to %s", path);
+      addr->basic_errno = errno;
+      addr->special_action = SPECIAL_FREEZE;
+      return;
+      }
+    }
+
+  /* All relevant directories now exist. Attempt to open the temporary
+  file a limited number of times. I think this rather scary-looking for
+  statement is actually OK. If open succeeds, the loop is broken; if not,
+  there is a test on the value of i. */
+
+  for (i = 1;; i++)
+    {
+    char *basename = string_sprintf("%lu.%lu.%s", time(NULL), getpid(),
+      primary_hostname);
+
+    filename = string_sprintf("tmp/%s", basename);
+    newname = string_sprintf("new/%s", basename);
+
+    if (stat(filename, &statbuf) == 0)
+      errno = EEXIST;
+    else if (errno == ENOENT)
+      {
+      fd = open(filename, O_WRONLY | O_CREAT | O_EXCL, mode);
+      if (fd >= 0) break;
+      DEBUG (9) debug_printf ("open failed for %s: %s\n",
+        filename, strerror(errno));
+      }
+
+    if (i >= ob->maildir_retries)
+      {
+      addr->message = string_sprintf ("failed to open %s (%d tr%s)",
+        filename, i, (i == 1)? "y" : "ies");
+      addr->basic_errno = errno;
+      addr->transport_return = DEFER;
+      return;
+      }
+
+    /* Open or stat failed but we haven't tried too many times yet. */
+
+    store_free(basename);
+    store_free(filename);
+    store_free(newname);
+    sleep(2);
+    }
+
+  /* Why are these here? Put in because they are present in the non-maildir
+  directory case above. */
+
+  chown(filename, uid, gid);
+  chmod(filename, mode);
   }
 
 
@@ -1135,11 +1361,14 @@ else
 prefix line, and followed by any configured suffix line. If there are any
 writing errors, we must defer. */
 
+DEBUG(9) debug_printf("writing to file %s\n", filename);
+
 yield = OK;
 errno = 0;
 
 /* If there is a local quota setting, check that we are not going to exceed it
-with this message. */
+with this message. This of course applies only file appending, not when writing
+in a directory. */
 
 if (ob->quota_value > 0)
   {
@@ -1152,29 +1381,34 @@ if (ob->quota_value > 0)
 
 /* If the local_smtp option is not unset, we need to write SMTP prefix
 information. The various different values for batching are handled outside; if
-there is more than one address available here, all must be included. Force
-SMTP dot-handling. */
+there is more than one address available here, all must be included. If any
+address is a file, use it's parent in the RCPT TO. Force SMTP dot-handling. */
 
 if (tblock->local_smtp != local_smtp_off && yield == OK)
   {
+  if (ob->use_crlf) cr = "\r";
   smtp_dots = TRUE;
   return_path_add = delivery_date_add = envelope_to_add = FALSE;
-  if (!write_string(fd, "MAIL FROM: <%s>\n",
-    (addr->errors_address != NULL)? addr->errors_address : sender_address))
-      yield = DEFER;
+  if (tblock->bsmtp_helo &&
+      !write_string(fd, "HELO %s%s\n", primary_hostname, cr))
+    yield = DEFER;
+  else if (!write_string(fd, "MAIL FROM: <%s>%s\n",
+    ((addr->errors_address != NULL)? addr->errors_address : sender_address),
+      cr)) yield = DEFER;
   else
     {
     address_item *a;
     for (a = addr; a != NULL; a = a->next)
       {
-      if ((a->local_part[0] == ',' || a->local_part[0] == ':')?
+      address_item *b = (a->pfr)? a->parent: a;
+      if ((b->local_part[0] == ',' || b->local_part[0] == ':')?
         !write_string(fd,
-          "RCPT TO: <@%s%s>\n", a->domain, a->local_part) :
+          "RCPT TO: <@%s%s>%s\n", b->domain, b->local_part, cr) :
         !write_string(fd,
-          "RCPT TO: <%s@%s>\n", a->local_part, a->domain))
+          "RCPT TO: <%s@%s>%s\n", b->local_part, b->domain, cr))
         { yield = DEFER; break; }
       }
-    if (yield == OK && !write_string(fd, "DATA\n")) yield = DEFER;
+    if (yield == OK && !write_string(fd, "DATA%s\n", cr)) yield = DEFER;
     }
   }
 
@@ -1201,12 +1435,14 @@ off for local_smtp). */
 
 if (yield == OK &&
   !transport_write_message(addr, fd,
-    (return_path_add? topt_add_return_path : 0) |
-    (delivery_date_add? topt_add_delivery_date : 0) |
-    (envelope_to_add? topt_add_envelope_to : 0 ) |
-    (ob->from_hack? topt_from_hack : 0 ) |
-    (smtp_dots? topt_smtp_dots : 0),
-    addr->errors_address, 0))
+    topt_writing_file |
+      (return_path_add? topt_add_return_path : 0) |
+      (delivery_date_add? topt_add_delivery_date : 0) |
+      (envelope_to_add? topt_add_envelope_to : 0) |
+      (ob->from_hack? topt_from_hack : 0) |
+      (ob->use_crlf? topt_use_crlf : 0) |
+      (smtp_dots? topt_smtp_dots : 0),
+    addr->errors_address, 0, tblock->add_headers, tblock->remove_headers))
   yield = DEFER;
 
 /* Now a configured suffix. */
@@ -1227,7 +1463,7 @@ if (yield == OK && ob->suffix != NULL)
 /* If local_smtp, write the terminating dot. */
 
 if (yield == OK && tblock->local_smtp != local_smtp_off &&
-  !write_block(fd, ".\n", 2)) yield = DEFER;
+  !write_string(fd, ".%s\n", cr)) yield = DEFER;
 
 /* Force out the remaining data to check for any errors */
 
@@ -1238,12 +1474,12 @@ the error, with the reason in errno. */
 
 if (yield != OK)
   {
-  /* Save the error number. It will ultimately cause a strerror() call to
-  generate some text. */
+  /* Save the error number. If positive, it will ultimately cause a strerror()
+  call to generate some text. */
 
   addr->basic_errno = errno;
 
-  /* Handle system quota excession. Set more_errno to the time since the mailbox
+  /* Handle system quota excession. Set more_errno to the time since the file
   was last read, and add an explanatory phrase for the error message, since
   some systems don't have special quota-excession errors. */
 
@@ -1252,9 +1488,9 @@ if (yield != OK)
     addr->more_errno = time(NULL) - times.actime;
     #ifndef EDQUOT
     addr->message = string_sprintf("probably user's quota exceeded while "
-      "writing to mailbox %s", filename);
+      "writing to file %s", filename);
     #endif
-    DEBUG(9) debug_printf("System quota exceeded for %s: time since mailbox "
+    DEBUG(9) debug_printf("System quota exceeded for %s: time since file "
       "read = %s\n", filename, readconf_printtime(addr->more_errno));
     }
 
@@ -1264,70 +1500,86 @@ if (yield != OK)
     {
     addr->more_errno = time(NULL) - times.actime;
     addr->message = string_sprintf("MTA-imposed quota exceeded while "
-      "writing to mailbox %s", filename);
-    DEBUG(9) debug_printf("Exim quota exceeded for %s: time since mailbox "
+      "writing to file %s", filename);
+    DEBUG(9) debug_printf("Exim quota exceeded for %s: time since file "
       "read = %s\n", filename, readconf_printtime(addr->more_errno));
     }
 
+  /* Handle a process failure while writing via a filter; the return
+  from child_close() is in more_errno. */
+
+  else if (errno == ERRNO_FILTER_FAIL)
+    addr->message = string_sprintf("filter process failure %d while writing "
+      "to %s", addr->more_errno, filename);
+
+  /* Handle failure to add a header - usually expansion failure. */
+
+  else if (errno == ERRNO_ADDHEADER_FAIL)
+    {
+    yield = PANIC;
+    addr->message = string_sprintf("failed to expand added headers while "
+      "writing to %s: %s", filename, expand_string_message);
+    }
+
   /* For other errors, a general-purpose explanation */
 
-  else addr->message = string_sprintf("error while writing to mailbox %s",
+  else addr->message = string_sprintf("error while writing to %s",
     filename);
 
-  /* For a write to a directory, remove the file. */
-
-  if (isdirectory) unlink(tempname);
-
   /* For a file, reset the file size to what it was before we started, leaving
   the last modification time unchanged, so it will get reset also. All systems
   investigated so far have ftruncate(), whereas not all have the F_FREESP
   fcntl() call (BSDI & FreeBSD do not). */
 
-  else ftruncate(fd, saved_size);
+  if (!isdirectory) ftruncate(fd, saved_size);
   }
 
-/* Handle successful writing - we want the modification time to be now.
-Remove the default backstop error number. For a directory, now is the time
-to rename the file with a unique name, constructed from the time and the file's
-inode in base 62 in the form tttttt-iiiiii. As soon as such a name appears
-it may get used by another process. */
+/* Handle successful writing - we want the modification time to be now for
+appended files. Remove the default backstop error number. For a directory, now
+is the time to rename the file with a unique name. As soon as such a name
+appears it may get used by another process, so we close the file first and
+check that all is well. */
 
 else
   {
   times.modtime = now;
   addr->basic_errno = 0;
 
+  /* Handle the case of writing to a new file in a directory. This applies
+  both to maildir and non-maildir formats. */
+
   if (isdirectory)
     {
     if (fstat(fd, &statbuf) < 0)
       {
       addr->basic_errno = errno;
       addr->message = string_sprintf("while fstatting opened message file %s",
-        tempname);
+        filename);
       yield = DEFER;
       }
 
-    /* Build the new name. It starts with 'q' for smail compatibility. */
+    else if (close(fd) < 0)
+      {
+      addr->basic_errno = errno;
+      addr->message = string_sprintf("close() error for %s", filename);
+      yield = DEFER;
+      }
 
     else
       {
-      char newname[256];
-      char *p = newname + (int)strlen(tempname);
-      strcpy(newname, tempname);
-      while (p[-1] != '/') p--;
-      *p++ = 'q';
-      strcpy(p, string_base62(time(NULL)));
-      p += (int)strlen(p);
-      sprintf(p, "-%s", string_base62(statbuf.st_ino));
-
-      if (rename(tempname, newname) < 0)
+      fd = -1;
+      if (rename(filename, newname) < 0)
         {
         addr->basic_errno = errno;
-        addr->message = string_sprintf("while renaming message file %s as %s",
-          tempname, newname);
-        unlink(tempname);
+        addr->message = string_sprintf("while renaming %s as %s",
+          filename, newname);
         yield = DEFER;
         }
+      else
+        {
+        DEBUG(9) debug_printf("renamed %s as %s\n", filename, newname);
+        filename = NULL;   /* Prevents attempt to unlink at end */
+        }
       }
     }
   }
@@ -1351,22 +1603,24 @@ DEBUG(2) debug_printf("appendfile yields %d with errno=%d more_errno=%d\n",
 
 addr->transport_return = yield;
 
-/* Close the file, which will release the fcntl lock. If everything has gone
-right but close fails, defer the message. Then unlink the lock file, if
-present. This point in the code is jumped to from a number of places when
-errors are detected, in order to get the file closed and the lock file tidied
-away. */
+/* Close the file, which will release the fcntl lock. For a directory write it
+is closed above, except in cases of error which goto RETURN, when we also need
+to remove the original file. If everything has gone right but close fails,
+defer the message. Then unlink the lock file, if present. This point in the
+code is jumped to from a number of places when errors are detected, in order to
+get the file closed and the lock file tidied away. */
 
 RETURN:
 
 if (fd >= 0 && close(fd) < 0 && yield == OK)
   {
   addr->basic_errno = errno;
-  addr->message = string_sprintf("while closing mailbox %s", filename);
+  addr->message = string_sprintf("while closing %s", filename);
   addr->transport_return = DEFER;
   }
 
 if (hd >= 0) unlink(lockname);
+if (isdirectory && filename != NULL) unlink(filename);
 }
 
 /* End of transport/appendfile.c */
