@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -21,6 +21,7 @@ typedef struct {
   int   lockfile_timeout;
   int   lock_retries;
   int   lock_interval;
+  int   maildir_retries;
   int   create_file;
   BOOL  allow_symlink;
   BOOL  check_group;
@@ -28,11 +29,15 @@ typedef struct {
   BOOL  notify_comsat;
   BOOL  require_lockfile;
   BOOL  use_lockfile;
+  BOOL  use_fcntl;
+  BOOL  use_crlf;
   BOOL  from_hack;
   BOOL  return_path_add;
   BOOL  delivery_date_add;
   BOOL  envelope_to_add;
   BOOL  file_must_exist;
+  BOOL  mode_fail_narrower;
+  BOOL  maildir_format;
 } appendfile_transport_options_block;
 
 /* Restricted creation options */
