@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -107,7 +107,7 @@ autoreply_transport_options_block *ob =
 
 /* If a fixed uid field is set, then a gid field must also be set. */
 
-if (tblock->uid_set && !tblock->gid_set)
+if (tblock->uid_set && !tblock->gid_set && tblock->expand_gid == NULL)
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "user set without group for the %s transport", tblock->name);
 }
@@ -149,7 +149,8 @@ if (ss == NULL)
 
 for (t = ss; *t != 0; t++)
   {
-  if (!isprint(*t) && *t != '\t' && (*t != '\n' || !text))
+  int c = (uschar)*t;
+  if (!mac_isprint(c) && c != '\t' && (c != '\n' || !text))
     {
     s = string_printing(s, TRUE);
     addr->transport_return = FAIL;
@@ -178,7 +179,7 @@ autoreply_transport_entry(
 {
 int fd, pid, rc;
 int log_fd = -1;
-BOOL file_expand;
+BOOL file_expand, return_message;
 char *from, *to, *cc, *bcc, *subject, *headers, *text, *file;
 char *logfile, *oncelog;
 char *now = NULL;
@@ -215,6 +216,7 @@ if (addr->reply != NULL)
   logfile = addr->reply->logfile;
   oncelog = addr->reply->oncelog;
   file_expand = addr->reply->file_expand;
+  return_message = addr->reply->return_message;
   }
 else
   {
@@ -230,6 +232,7 @@ else
   logfile = ob->logfile;
   oncelog = ob->oncelog;
   file_expand = ob->file_expand;
+  return_message = ob->return_message;
 
   if ((from  != NULL &&
         (from = checkexpand(from, addr, tblock->name, FALSE)) == NULL) ||
@@ -275,7 +278,7 @@ envisaged. */
 if (oncelog != NULL && to != NULL)
   {
   EXIM_DATUM key_datum, result_datum;
-  dbm_file = EXIM_DBOPEN(oncelog, O_RDWR|O_CREAT, ob->mode);
+  EXIM_DBOPEN(oncelog, O_RDWR|O_CREAT, ob->mode, &dbm_file);
   if (dbm_file == NULL)
     {
     addr->transport_return = DEFER;
@@ -285,6 +288,10 @@ if (oncelog != NULL && to != NULL)
     return;
     }
 
+  #ifdef EXIM_CLEAR_DATUMSTRUCT
+  memset(&key_datum, 0, sizeof(key_datum));
+  #endif
+
   EXIM_DATUM_DATA(key_datum) = to;
   EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
 
@@ -296,7 +303,7 @@ if (oncelog != NULL && to != NULL)
       {
       char *ptr = log_buffer;
       if (now == NULL) now = tod_stamp(tod_log);
-      sprintf(ptr, "%s\n  previously sent to %s\n", now, to);
+      sprintf(ptr, "%s\n  previously sent to %.200s\n", now, to);
       while(*ptr) ptr++;
       write(log_fd, log_buffer, ptr - log_buffer);
       }
@@ -323,7 +330,7 @@ if (file != NULL)
 /* Make a subprocess to send the message */
 
 pid = child_open(mailer_argv, NULL, 077, NULL, NULL, &fd,
-  (debug_file != NULL)? fileno(debug_file) : -1);
+  (debug_file != NULL)? fileno(debug_file) : -1, NULL);
 
 /* Creation of child failed; defer this delivery. */
 
@@ -358,7 +365,7 @@ if (h != NULL)
   {
   char *s = strchr(h->text, ':') + 1;
   while (isspace(*s)) s++;
-  fprintf(f, "In-Reply-To: %s\n", s);
+  fprintf(f, "In-Reply-To: %s", s);
   }
 
 /* Add any specially requested headers */
@@ -389,27 +396,30 @@ if (ff != NULL)
 /* Copy the original message if required, observing the return size
 limit. */
 
-if (ob->return_message)
+if (return_message)
   {
   if (return_size_limit > 0)
     {
     struct stat statbuf;
-    int max = (return_size_limit/DELIVER_BUFFER_SIZE + 1)*DELIVER_BUFFER_SIZE;
+    int max = (return_size_limit/DELIVER_IN_BUFFER_SIZE + 1) *
+      DELIVER_IN_BUFFER_SIZE;
     if (fstat(deliver_datafile, &statbuf) == 0 && statbuf.st_size > max)
       {
+      int size = statbuf.st_size;  /* Because might be a long */
       fprintf(f, "\n"
-"------ This is a copy of your message, including all the headers.\n"
-"------ The body of your message is %d characters long; only the first\n"
-"------ %d or so are included here.\n\n", (int)statbuf.st_size, (max/1000)*1000);
+"------ This is a copy of the message, including all the headers.\n"
+"------ The body of the message is %d characters long; only the first\n"
+"------ %d or so are included here.\n\n", size, (max/1000)*1000);
       }
     else fprintf(f, "\n"
-"------ This is a copy of your message, including all the headers. ------\n\n");
+"------ This is a copy of the message, including all the headers. ------\n\n");
     }
   else fprintf(f, "\n"
-"------ This is a copy of your message, including all the headers. ------\n\n");
+"------ This is a copy of the message, including all the headers. ------\n\n");
 
   fflush(f);
-  transport_write_message(addr, fileno(f), 0, NULL, return_size_limit);
+  transport_write_message(addr, fileno(f), 0, NULL, return_size_limit,
+    tblock->add_headers, tblock->remove_headers);
   }
 
 /* End the message and wait for the child process to end; no timeout. */
@@ -424,6 +434,10 @@ if (dbm_file != NULL)
   {
   EXIM_DATUM key_datum, value_datum;
   if (now == NULL) now = tod_stamp(tod_log);
+  #ifdef EXIM_CLEAR_DATUMSTRUCT
+  memset(&key_datum, 0, sizeof(key_datum));
+  memset(&value_datum, 0, sizeof(value_datum));
+  #endif
   EXIM_DATUM_DATA(key_datum) = to;
   EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
   EXIM_DATUM_DATA(value_datum) = now;
@@ -449,8 +463,7 @@ want to fail the delivery, since the message has been successfully sent. For
 the moment, ignore open failures. Write the log entry as a single write() to a
 file opened for appending, in order to avoid interleaving of output from
 different processes. The log_buffer can be used exactly as for main log
-writing. Oh, wouldn't it be nice if all C systems had ANSI-compliant sprintf()
-funtions? For now, code without using the result of sprintf(). */
+writing. */
 
 if (logfile != NULL)
   {
@@ -465,32 +478,38 @@ if (logfile != NULL)
     while(*ptr) ptr++;
     if (from != NULL)
       {
-      sprintf(ptr, "  From: %s\n", from);
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  From: %s\n", from);
       while(*ptr) ptr++;
       }
     if (to != NULL)
       {
-      sprintf(ptr, "  To: %s\n", to);
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  To: %s\n", to);
       while(*ptr) ptr++;
       }
     if (cc != NULL)
       {
-      sprintf(ptr, "  Cc: %s\n", cc);
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  Cc: %s\n", cc);
       while(*ptr) ptr++;
       }
     if (bcc != NULL)
       {
-      sprintf(ptr, "  Bcc: %s\n", bcc);
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  Bcc: %s\n", bcc);
       while(*ptr) ptr++;
       }
     if (subject != NULL)
       {
-      sprintf(ptr, "  Subject: %s\n", subject);
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  Subject: %s\n", subject);
       while(*ptr) ptr++;
       }
     if (headers != NULL)
       {
-      sprintf(ptr, "  %s\n", headers);
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  %s\n", headers);
       while(*ptr) ptr++;
       }
     write(log_fd, log_buffer, ptr - log_buffer);
