@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -114,7 +114,8 @@ else
   fprintf(f, "-------------------------------------------------------------------\n");
   fflush(f);
 
-  transport_write_message(addrlist, fileno(f), 0, NULL, 0);
+  transport_write_message(addrlist, fileno(f), 0, NULL, 0,
+    tblock->add_headers, tblock->remove_headers);
 
   fclose(f);
   yield = OK;
