@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -27,12 +27,16 @@ optionlist pipe_transport_options[] = {
       (void *)(offsetof(transport_instance, gid_set)) },
   { "*set_user",          opt_bool | opt_hidden | opt_public,
       (void *)(offsetof(transport_instance, uid_set)) },
+  { "allow_commands",    opt_stringptr,
+      (void *)(offsetof(pipe_transport_options_block, allow_commands)) },
   { "batch",             opt_local_batch | opt_public,
       (void *)(offsetof(transport_instance, local_batch)) },
   { "batch_max",         opt_int | opt_public,
       (void *)(offsetof(transport_instance, batch_max)) },
   { "bsmtp",             opt_local_batch | opt_public,
       (void *)(offsetof(transport_instance, local_smtp)) },
+  { "bsmtp_helo",        opt_bool | opt_public,
+      (void *)(offsetof(transport_instance, bsmtp_helo)) },
   { "command",           opt_stringptr,
       (void *)(offsetof(pipe_transport_options_block, cmd)) },
   { "current_directory", opt_stringptr | opt_public,
@@ -43,6 +47,8 @@ optionlist pipe_transport_options[] = {
       (void *)(offsetof(transport_instance, home_dir)) },
   { "envelope_to_add", opt_bool,
       (void *)(offsetof(pipe_transport_options_block, envelope_to_add)) },
+  { "freeze_exec_fail",  opt_bool,
+      (void *)(offsetof(pipe_transport_options_block, freeze_exec_fail)) },
   { "from_hack",         opt_bool,
       (void *)(offsetof(pipe_transport_options_block, from_hack)) },
   { "group",             opt_expand_gid | opt_public,
@@ -53,6 +59,8 @@ optionlist pipe_transport_options[] = {
       (void *)(offsetof(pipe_transport_options_block, ignore_status)) },
   { "initgroups",        opt_bool | opt_public,
       (void *)(offsetof(transport_instance, initgroups)) },
+  { "log_defer_output",  opt_bool | opt_public,
+      (void *)(offsetof(transport_instance, log_defer_output)) },
   { "log_fail_output",   opt_bool | opt_public,
       (void *)(offsetof(transport_instance, log_fail_output)) },
   { "log_output",        opt_bool | opt_public,
@@ -81,6 +89,10 @@ optionlist pipe_transport_options[] = {
       (void *)(offsetof(pipe_transport_options_block, timeout)) },
   { "umask",             opt_octint,
       (void *)(offsetof(pipe_transport_options_block, umask)) },
+  { "use_crlf",          opt_bool,
+      (void *)(offsetof(pipe_transport_options_block, use_crlf)) },
+  { "use_shell",         opt_bool,
+      (void *)(offsetof(pipe_transport_options_block, use_shell)) },
   { "user",              opt_expand_uid | opt_public,
       (void *)(offsetof(transport_instance, uid)) },
 };
@@ -95,6 +107,7 @@ int pipe_transport_options_count =
 
 pipe_transport_options_block pipe_transport_option_defaults = {
   NULL,           /* cmd */
+  NULL,           /* allow_commands */
   "/usr/bin",     /* path */
   "From ${if def:return_path{$return_path}{MAILER-DAEMON}} ${tod_bsdinbox}\n",
                   /* prefix */
@@ -102,12 +115,14 @@ pipe_transport_options_block pipe_transport_option_defaults = {
   022,            /* umask */
   20480,          /* max_output */
   60*60,          /* timeout */
+  FALSE,          /* freeze_exec_fail */
   FALSE,          /* from_hack */
   FALSE,          /* ignore_status */
   FALSE,          /* return_path_add */
   FALSE,          /* delivery_date_add */
   FALSE,          /* envelope_to_add */
-  FALSE           /* restrict_to_path */
+  FALSE,          /* restrict_to_path */
+  FALSE,          /* use_shell */
 };
 
 
@@ -123,10 +138,8 @@ to be set up. */
 void
 pipe_transport_init(transport_instance *tblock)
 {
-/*
 pipe_transport_options_block *ob =
   (pipe_transport_options_block *)(tblock->options_block);
-*/
 
 /* Retry_use_local_part defaults TRUE if unset */
 
@@ -142,7 +155,7 @@ if (tblock->deliver_as_creator && (tblock->uid_set || tblock->gid_set ||
 
 /* If a fixed uid field is set, then a gid field must also be set. */
 
-if (tblock->uid_set && !tblock->gid_set)
+if (tblock->uid_set && !tblock->gid_set && tblock->expand_gid == NULL)
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "user set without group for the %s transport", tblock->name);
 
@@ -163,6 +176,234 @@ if (tblock->log_output && tblock->log_fail_output)
 
 if (tblock->local_smtp != local_batch_off)
   tblock->local_batch = tblock->local_smtp;
+
+/* The restrict_to_path  and use_shell options are incompatible */
+
+if (ob->restrict_to_path && ob->use_shell)
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
+    "both restrict_to_path and use_shell set for %s transport",
+    tblock->name);
+
+/* The allow_commands and use_shell options are incompatible */
+
+if (ob->allow_commands && ob->use_shell)
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
+    "both allow_commands and use_shell set for %s transport",
+    tblock->name);
+}
+
+
+
+/*************************************************
+*          Set up direct (non-shell) command     *
+*************************************************/
+
+/* This function is called when a command line is to be parsed by the transport
+and executed directly, without the use of /bin/sh.
+
+Arguments:
+  argvptr            pointer to anchor for argv vector
+  cmd                points to the command string
+  expand_arguments   true if expansion is to occur
+  addr               chain of addresses
+  tname              the transport name
+  ob                 the transport options block
+
+Returns:             TRUE if all went well; otherwise an error will be
+                     set in the first address and FALSE returned
+*/
+
+static BOOL
+set_up_direct_command(char ***argvptr, char *cmd, BOOL expand_arguments,
+  address_item *addr, char *tname, pipe_transport_options_block *ob)
+{
+BOOL permitted = FALSE;
+char **argv;
+char buffer[64];
+
+/* Set up "transport <name>" to be put in any error messages, and then
+call the common function for creating an argument list and expanding
+the items if necessary. If it fails, this function fails (error information
+is in the addresses). */
+
+sprintf(buffer, "%.50s transport", tname);
+if (!transport_set_up_command(argvptr, cmd, expand_arguments,
+      (addr->local_part[0]== '|')? FAIL : PANIC,
+      addr, buffer))
+  return FALSE;
+
+/* Point to the set-up arguments. */
+
+argv = *argvptr;
+
+/* If allow_commands is set, see if the command is in the permitted list. */
+
+if (ob->allow_commands != NULL)
+  {
+  char *s, *p;
+  char buffer[256];
+
+  s = expand_string(ob->allow_commands);
+  if (s == NULL)
+    {
+    addr->transport_return = DEFER;
+    addr->message = string_sprintf("failed to expand string \"%s\" "
+      "for %s transport: %s", ob->allow_commands, tname, expand_string_message);
+    return FALSE;
+    }
+
+  for (p = string_nextinlist(&s, ':', buffer, sizeof(buffer));
+       p != NULL;
+       p = string_nextinlist(&s, ':', buffer, sizeof(buffer)))
+    {
+    if (strcmp(p, argv[0]) == 0) { permitted = TRUE; break; }
+    }
+  }
+
+/* If permitted is TRUE it means the command was found in the allowed list, and
+no further checks are done. If permitted = FALSE, it either means
+allow_commands wasn't set, or that the command didn't match anything in the
+list. In both cases, if restrict_to_path is set, we fail if the command
+contains any slashes, but if restrict_to_path is not set, we must fail the
+command only if allow_commands is set. */
+
+if (!permitted)
+  {
+  if (ob->restrict_to_path)
+    {
+    if (strchr(argv[0], '/') != NULL)
+      {
+      addr->transport_return = FAIL;
+      addr->message = string_sprintf("\"/\" found in \"%s\" (command for %s "
+        "transport) - failed for security reasons", cmd, tname);
+      return FALSE;
+      }
+    }
+
+  else if (ob->allow_commands != NULL)
+    {
+    addr->transport_return = FAIL;
+    addr->message = string_sprintf("\"%s\" command not permitted by %s "
+      "transport", argv[0], tname);
+    return FALSE;
+    }
+  }
+
+/* If the command is not an absolute path, search the PATH directories
+for it. */
+
+if (argv[0][0] != '/')
+  {
+  char *p;
+  char *listptr = ob->path;
+  char buffer[1024];
+
+  for (p = string_nextinlist(&listptr, ':', buffer, sizeof(buffer));
+       p != NULL;
+       p = string_nextinlist(&listptr, ':', buffer, sizeof(buffer)))
+    {
+    struct stat statbuf;
+    sprintf(big_buffer, "%.256s/%.256s", p, argv[0]);
+    if (stat(big_buffer, &statbuf) == 0)
+      {
+      argv[0] = string_copy(big_buffer);
+      break;
+      }
+    }
+  if (p == NULL)
+    {
+    addr->transport_return = FAIL;
+    addr->message = string_sprintf("\"%s\" command not found for %s transport",
+      argv[0], tname);
+    return FALSE;
+    }
+  }
+
+return TRUE;
+}
+
+
+/*************************************************
+*               Set up shell command             *
+*************************************************/
+
+/* This function is called when a command line is to be passed to /bin/sh
+without parsing inside the transport.
+
+Arguments:
+  argvptr            pointer to anchor for argv vector
+  cmd                points to the command string
+  expand_arguments   true if expansion is to occur
+  addr               chain of addresses
+  tname              the transport name
+  ob                 the transport options block
+
+Returns:             TRUE if all went well; otherwise an error will be
+                     set in the first address and FALSE returned
+*/
+
+static BOOL
+set_up_shell_command(char ***argvptr, char *cmd, BOOL expand_arguments,
+  address_item *addr, char *tname, pipe_transport_options_block *ob)
+{
+char **argv;
+
+*argvptr = argv = (char **)store_malloc((4)*sizeof(char *));
+
+argv[0] = "/bin/sh";
+argv[1] = "-c";
+
+/* We have to take special action to handle the special "variable" called
+$pipe_addresses, which is not recognized by the normal expansion function. */
+
+DEBUG(7) debug_printf("shell pipe command before expansion:\n  %s\n", cmd);
+
+if (expand_arguments)
+  {
+  char *s = cmd;
+  char *p = strstr(cmd, "pipe_addresses");
+
+  if (p != NULL && (
+         (p > cmd && p[-1] == '$') ||
+         (p > cmd + 1 && p[-2] == '$' && p[-1] == '{' && p[14] == '}')))
+    {
+    address_item *ad;
+    char *q = p + 14;
+    int size = (int)strlen(cmd) + 64;
+    int offset;
+
+    if (p[-1] == '{') { q++; p--; }
+
+    s = store_malloc(size);
+    offset = p - cmd - 1;
+    strncpy(s, cmd, offset);
+
+    for (ad = addr; ad != NULL; ad = ad->next)
+      {
+      if (ad != addr) string_cat(s, &size, &offset, " ", 1);
+      string_cat(s, &size, &offset, ad->orig, (int)strlen(ad->orig));
+      }
+
+    string_cat(s, &size, &offset, q, (int)strlen(q));
+    s[offset] = 0;
+    }
+
+  argv[2] = expand_string(s);
+  if (argv[2] == NULL)
+    {
+    addr->transport_return = search_find_defer? DEFER :
+      (addr->local_part[0] == '|')? FAIL : PANIC;
+    addr->message = string_sprintf("Expansion of command \"%s\" "
+      "in %s transport failed: %s",
+      cmd, tname, expand_string_message);
+    return FALSE;
+    }
+  DEBUG(7) debug_printf("shell pipe command after expansion:\n  %s\n", argv[2]);
+  }
+else argv[2] = cmd;
+
+argv[3] = (char *)0;
+return TRUE;
 }
 
 
@@ -180,13 +421,9 @@ pipe_transport_entry(
   address_item *addr)              /* address(es) we are working on */
 {
 pid_t pid, outpid;
-int i, fd, rc;
+int fd, rc;
 int pipefd[2];
 int envcount = 0;
-int argcount = 0;
-int address_count = 0;
-int max_args;
-address_item *ad;
 pipe_transport_options_block *ob =
   (pipe_transport_options_block *)(tblock->options_block);
 BOOL smtp_dots = FALSE;
@@ -197,7 +434,7 @@ BOOL expand_arguments;
 char **argv;
 char *envp[20];
 char *env_local_part;
-char *s, *ss, *cmd;
+char *cmd;
 
 DEBUG(2) debug_printf("%s transport entered\n", tblock->name);
 
@@ -223,208 +460,34 @@ else
   expand_arguments = TRUE;
   }
 
-/* Get store in which to build an argument list. Count the number of addresses
-supplied, and allow for that many arguments, plus an additional 60, which
-should be enough for anybody. Multiple addresses happen only when one of
-the batch options is set. */
+/* When a pipe is set up by a filter file, there may be numerical variables
+in existence. These are passed in addr->pipe_expandn for use here. */
 
-for (ad = addr; ad != NULL; ad = ad->next) address_count++;
-max_args = address_count + 60;
-argv = (char **)store_malloc((max_args+1)*sizeof(char *));
-
-/* Split the command up into arguments terminated by white space. Lose
-trailing space at the start and end. Double-quoted arguments can contain \\ and
-\" escapes; single quotes may not. Copy each argument into a new string. The
-total length can be no longer than the length of the original. */
-
-s = cmd;
-ss = store_malloc((int)strlen(s) + 1);
-while (isspace(*s)) s++;
-
-while (*s != 0 && argcount < max_args)
+if (expand_arguments && addr->pipe_expandn != NULL)
   {
-  argv[argcount++] = ss;
-
-  /* Handle single-quoted arguments */
-
-  if (*s == '\'')
+  char **ss = addr->pipe_expandn;
+  expand_nmax = -1;
+  while (*ss != NULL)
     {
-    s++;
-    while (*s != 0)
-      {
-      if (*s == '\'') break;
-      *ss++ = *s++;
-      }
-    if (*s != 0) s++;
-    }
-
-  /* Handle double-quoted arguments */
-
-  else if (*s == '\"')
-    {
-    s++;
-    while (*s != 0)
-      {
-      if (*s == '\"') break;
-
-      /* Handle escape sequences */
-
-      else if (*s == '\\')
-        {
-        if (s[1] == 0) break;
-        *ss++ = string_interpret_escape(&s);
-        s++;
-        }
-
-      /* Not an escape sequence */
-
-      else *ss++ = *s++;
-      }
-
-    /* Advance past terminator */
-
-    if (*s != 0) s++;
-    }
-
-  /* Argument not in quotes is terminated by EOL or white space */
-
-  else while (*s != 0 && !isspace(*s)) *ss++ = *s++;
-
-  /* Terminate the current argument, and skip trailing spaces. */
-
-  *ss++ = 0;
-  while (isspace(*s)) s++;
-  }
-
-argv[argcount] = (char *)0;
-
-/* If *s != 0 we have run out of argument slots. */
-
-if (*s != 0)
-  {
-  addr->transport_return = FAIL;
-  addr->message = string_sprintf("Too many arguments to command \"%s\" in "
-    "%s transport", cmd, tblock->name);
-  return;
-  }
-
-/* Expand each individual argument if required. Expansion happens for pipes set
-up in filter files and with directly-supplied commands. It does not happen if
-the pipe comes from a traditional .forward file. A failing expansion is a big
-disaster if the command came from the transport's configuration; if it came
-from a user it is just a normal failure.
-
-An argument consisting just of the text "$pipe_addresses" is treated specially.
-It is not passed to the general expansion function. Instead, it is replaced by
-a number of arguments, one for each address. This avoids problems with shell
-metacharacters and spaces in addresses. */
-
-DEBUG(7)
-  {
-  debug_printf("pipe command:\n");
-  for (i = 0; argv[i] != (char *)0; i++)
-    debug_printf("  argv[%d] = %s\n", i, string_printing(argv[i], FALSE));
-  }
-
-if (expand_arguments)
-  {
-  for (i = 0; argv[i] != (char *)0; i++)
-    {
-
-    /* Handle special fudge for passing an address list */
-
-    if (strcmp(argv[i], "$pipe_addresses") == 0 ||
-        strcmp(argv[i], "${pipe_addresses}") == 0)
-      {
-      int additional;
-
-      if (argcount + address_count - 1 > max_args)
-        {
-        addr->transport_return = FAIL;
-        addr->message = string_sprintf("Too many arguments to command \"%s\" "
-          "in %s transport", cmd, tblock->name);
-        return;
-        }
-
-      additional = address_count - 1;
-      if (additional > 0)
-        memmove(argv + i + 1 + additional, argv + i + 1,
-          (argcount - i)*sizeof(char *));
-
-      for (ad = addr; ad != NULL; ad = ad->next) argv[i++] = ad->orig;
-      i--;
-      }
-
-    /* Handle normal expansion string */
-
-    else
-      {
-      char *expanded_arg = expand_string(argv[i]);
-      if (expanded_arg == NULL)
-        {
-        addr->transport_return = (addr->local_part[0] == '|')? FAIL : PANIC;
-        addr->message = string_sprintf("Expansion of \"%s\" "
-          "from command \"%s\" in %s transport failed: %s",
-          argv[i], cmd, tblock->name, expand_string_message);
-        return;
-        }
-      argv[i] = expanded_arg;
-      }
-    }
-
-  DEBUG(7)
-    {
-    debug_printf("pipe command after expansion:\n");
-    for (i = 0; argv[i] != (char *)0; i++)
-      debug_printf("  argv[%d] = %s\n", i, string_printing(argv[i], FALSE));
+    expand_nstring[++expand_nmax] = *ss;
+    expand_nlength[expand_nmax] = (int)strlen(*ss++);
     }
   }
 
+/* The default way of processing the command is to split it up into arguments
+here, and run it directly. This offers some security advantages. However, there
+are installations that want by default to run commands under /bin/sh always, so
+there is an option to do that. */
 
-/* If restrict_to_path is set, check that the name of the command does not
-contain any slashes. */
-
-if (ob->restrict_to_path)
+if (ob->use_shell)
   {
-  if (strchr(argv[0], '/') != NULL)
-    {
-    addr->transport_return = FAIL;
-    addr->message = string_sprintf("\"/\" found in \"%s\" (command for %s "
-      "transport) - failed for security reasons", cmd, tblock->name);
-    return;
-    }
-  }
-
-/* If the command is not an absolute path, search the PATH directories
-for it. */
-
-if (argv[0][0] != '/')
-  {
-  char *p;
-  char *listptr = ob->path;
-  char buffer[1024];
-
-  for (p = string_nextinlist(&listptr, ':', buffer, sizeof(buffer));
-       p != NULL;
-       p = string_nextinlist(&listptr, ':', buffer, sizeof(buffer)))
-    {
-    struct stat statbuf;
-    sprintf(big_buffer, "%s/%s", p, argv[0]);
-    if (stat(big_buffer, &statbuf) == 0)
-      {
-      argv[0] = string_copy(big_buffer);
-      break;
-      }
-    }
-  if (p == NULL)
-    {
-    addr->transport_return = FAIL;
-    addr->message = string_sprintf("\"%s\" command not found for %s transport",
-      argv[0], tblock->name);
-    return;
-    }
+  if (!set_up_shell_command(&argv, cmd, expand_arguments, addr, tblock->name,
+    ob)) return;
   }
+else if (!set_up_direct_command(&argv, cmd, expand_arguments, addr,
+  tblock->name, ob)) return;
 
+expand_nmax = -1;   /* Reset */
 
 /* Set up the environment for the command. */
 
@@ -491,10 +554,11 @@ if (pipe(pipefd) < 0)
   }
 
 /* As this is a local transport, we are already running with the required
-uid/gid, so pass NULL to child_open to indicate no change. */
+uid/gid and current directory, so pass NULL to child_open to indicate no
+change. */
 
 if ((pid = child_open(argv, envp, ob->umask, NULL, NULL, &fd,
-     pipefd[pipe_write])) < 0)
+     pipefd[pipe_write], NULL)) < 0)
   {
   addr->transport_return = DEFER;
   addr->message = string_sprintf(
@@ -555,8 +619,8 @@ close(pipefd[pipe_read]);  /* Not used in this process */
 /* Carrying on now with the main parent process. Attempt to write the message
 to it down the pipe. It is a fallacy to think that you can detect write errors
 when the sub-process fails to read the pipe. The parent process may complete
-writing and close the pipe before the sub-process completes. Sleeping for a bit
-here lets the sub-process get going, but it may still not complete. So we
+writing and close the pipe before the sub-process completes. We could sleep a
+bit here to let the sub-process get going, but it may still not complete. So we
 ignore all writing errors. */
 
 DEBUG(2) debug_printf("Writing message to pipe\n");
@@ -571,12 +635,22 @@ if (tblock->local_smtp != local_smtp_off)
   address_item *a;
   char *sender = (addr->errors_address != NULL)?
     addr->errors_address : sender_address;
+
   smtp_dots = TRUE;
   return_path_add = delivery_date_add = envelope_to_add = FALSE;
 
+  if (tblock->bsmtp_helo)
+    {
+    write(fd, "HELO ", 5);
+    write(fd, primary_hostname, (int)strlen(primary_hostname));
+    if (ob->use_crlf) write(fd, "\r\n", 2);
+      else write(fd, "\n", 1);
+    }
+
   write(fd, "MAIL FROM: <", 12);
   write(fd, sender, (int)strlen(sender));
-  write(fd, ">\n", 2);
+  if (ob->use_crlf) write(fd, ">\r\n", 3);
+    else write(fd, ">\n", 2);
 
   for (a = addr; a != NULL; a = a->next)
     {
@@ -593,10 +667,12 @@ if (tblock->local_smtp != local_smtp_off)
       write(fd, "@", 1);
       write(fd, a->domain, (int)strlen(a->domain));
       }
-    write(fd, ">\n", 2);
+    if (ob->use_crlf) write(fd, ">\r\n", 3);
+      else write(fd, ">\n", 2);
     }
 
-  write(fd, "DATA\n", 5);
+  if (ob->use_crlf) write(fd, "DATA\r\n", 6);
+    else write(fd, "DATA\n", 5);
   }
 
 /* Now any other configured prefix. */
@@ -606,21 +682,41 @@ if (ob->prefix != NULL)
   char *prefix = expand_string(ob->prefix);
   if (prefix == NULL)
     {
-    addr->transport_return = PANIC;
+    addr->transport_return = search_find_defer? DEFER : PANIC;
     addr->message = string_sprintf("Expansion of \"%s\" (prefix for %s "
-      "transport) failed", ob->prefix, tblock->name);
+      "transport) failed: %s", ob->prefix, tblock->name, expand_string_message);
     return;
     }
   write(fd, prefix, (int)strlen(prefix));
   }
 
-(void) transport_write_message(addr, fd,
-  (return_path_add? topt_add_return_path : 0) |
-  (delivery_date_add? topt_add_delivery_date : 0) |
-  (envelope_to_add? topt_add_envelope_to : 0) |
-  (ob->from_hack? topt_from_hack : 0 ) |
-  (smtp_dots? topt_smtp_dots : 0),
-  addr->errors_address, 0);   /* no CRLF */
+if (!transport_write_message(addr, fd,
+    (return_path_add? topt_add_return_path : 0) |
+    (delivery_date_add? topt_add_delivery_date : 0) |
+    (envelope_to_add? topt_add_envelope_to : 0) |
+    (ob->from_hack? topt_from_hack : 0) |
+    (ob->use_crlf? topt_use_crlf : 0) |
+    (smtp_dots? topt_smtp_dots : 0),
+    addr->errors_address, 0, tblock->add_headers, tblock->remove_headers))
+  {
+  if (errno != EPIPE && errno != ERRNO_WRITEINCOMPLETE)
+    {
+    addr->transport_return = PANIC;
+    addr->basic_errno = errno;
+    if (errno == ERRNO_ADDHEADER_FAIL)
+      addr->message = string_sprintf("Failed to expand added headers");
+    else if (errno == ERRNO_FILTER_FAIL)
+      addr->message = string_sprintf("Filter process failure");
+    else
+      addr->message = string_sprintf("Error %d", errno);
+    return;
+    }
+  else DEBUG(9)
+    {
+    debug_printf("transport error %s ignored\n",
+      (errno == EPIPE)? "EPIPE" : "WRITEINCOMPLETE");
+    }
+  }
 
 /* Now any configured suffix */
 
@@ -629,9 +725,9 @@ if (ob->suffix != NULL)
   char *suffix = expand_string(ob->suffix);
   if (suffix == NULL)
     {
-    addr->transport_return = PANIC;
+    addr->transport_return = search_find_defer? DEFER : PANIC;
     addr->message = string_sprintf("Expansion of \"%s\" (suffix for %s "
-      "transport) failed", ob->suffix, tblock->name);
+      "transport) failed: %s", ob->suffix, tblock->name, expand_string_message);
     return;
     }
   write(fd, suffix, (int)strlen(suffix));
@@ -639,7 +735,11 @@ if (ob->suffix != NULL)
 
 /* If local_smtp, write the terminating dot. */
 
-if (tblock->local_smtp != local_smtp_off) write(fd, ".\n", 2);
+if (tblock->local_smtp != local_smtp_off)
+  {
+  if (ob->use_crlf) write(fd, ".\r\n", 3);
+    else write(fd, ".\n", 2);
+  }
 
 /* OK, the writing is now all done. Close the pipe. */
 
@@ -688,14 +788,50 @@ if ((rc = child_close(pid, ob->timeout)) != 0)
   status code to request deferral. A number of systems contain the following
   line in sysexits.h:
 
-      #define EX_TEMPFAIL 75 temp failure; user is invited to retry
+      #define EX_TEMPFAIL 75
+
+  with the description
+
+      EX_TEMPFAIL -- temporary failure, indicating something that
+	 is not really an error.  In sendmail, this means
+	 that a mailer (e.g.) could not create a connection,
+	 and the request should be reattempted later.
 
   Based on this, use exit code EX_TEMPFAIL to mean "defer" when not ignoring
-  the returned status. Otherwise fail. */
+  the returned status.
+
+  Another case that needs special treatment is if execve() failed (typically
+  the command that was given is a non-existent path). By default this is
+  treated as just another failure, but if freeze_exec_fail is set, the reaction
+  is to freeze the message rather than bounce the address. This failure is
+  signalled by EX_UNAVAILABLE which is definined in many systems as
+
+      #define EX_UNAVAILABLE  69
+
+  with the description
+
+      EX_UNAVAILABLE -- A service is unavailable.  This can occur
+	    if a support program or file does not exist.  This
+	    can also be used as a catchall message when something
+	    you wanted to do doesn't work, but you don't know why.
+
+  so this is used by the code in child.c to signal execve() failure. */
 
   else
     {
-    if (!ob->ignore_status)
+    /* Always handle execve() failure specially if requested to */
+
+    if (ob->freeze_exec_fail && (rc == EX_UNAVAILABLE))
+      {
+      addr->transport_return = DEFER;
+      addr->special_action = SPECIAL_FREEZE;
+      addr->message = string_sprintf("pipe process failed to exec \"%s\"",
+        cmd);
+      }
+
+    /* Otherwise take action only if not ignoring status */
+
+    else if (!ob->ignore_status)
       {
       int size, ptr, i;
       addr->transport_return = (rc == EX_TEMPFAIL)? DEFER : FAIL;
@@ -726,10 +862,8 @@ if ((rc = child_close(pid, ob->timeout)) != 0)
     }
   }
 
-/* The child_close() function just calls wait() until the required pid
-is returned. Therefore it might already have waited for the output handling
-process. In case it hasn't, do some more waiting here. All subprocesses
-should be done before we pass this point. */
+/* Ensure all subprocesses (in particular, the output handling process)
+are complete before we pass this point. */
 
 while (wait(&rc) >= 0);
 
