@@ -2,25 +2,29 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 199h */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
   char *cmd;
+  char *allow_commands;
   char *path;
   char *prefix;
   char *suffix;
   int   umask;
   int   max_output;
   int   timeout;
+  BOOL  freeze_exec_fail;
   BOOL  from_hack;
   BOOL  ignore_status;
   BOOL  return_path_add;
   BOOL  delivery_date_add;
   BOOL  envelope_to_add;
   BOOL  restrict_to_path;
+  BOOL  use_shell;
+  BOOL  use_crlf;
 } pipe_transport_options_block;
 
 /* Data for reading the private options. */
