@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options and other private data. */
@@ -12,6 +12,7 @@ typedef struct {
   char *fallback_hosts;
   host_item *hostlist;
   host_item *fallback_hostlist;
+  char *interface;
   char *service;
   char *mx_domains;
   char *non_mx_domains;
@@ -26,6 +27,8 @@ typedef struct {
   int   connect_timeout;
   int   data_timeout;
   int   final_timeout;
+  int   size_addition;
+  BOOL  allow_localhost;
   BOOL  gethostbyname;
   BOOL  dns_qualify_single;
   BOOL  dns_search_parents;
