@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1997 */
+/* Copyright (c) University of Cambridge 1995 - 1998 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for maintaining binary balanced trees and some associated
@@ -133,7 +133,7 @@ tree_add_unusable(host_item *h)
 {
 tree_node *node;
 char s[256];
-sprintf(s, "T:%s:%s", h->name, h->address);
+sprintf(s, "T:%.200s:%s", h->name, h->address);
 node = store_malloc(sizeof(tree_node) + (int)strlen(s));
 strcpy(node->name, s);
 node->data.val = h->why;
